// $Id: DriverLog.c 14206 2011-09-08 16:47:22Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This tool dumps the driver log.
 */

#ifdef WIN32
#include <Windows.h>
#include "platformwin.h"
#else
#include <ctype.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>

#include <sys/types.h>
#include <sys/stat.h>
#endif

#include <toolslib.h>
#include <DriverLog.h>


/* The NTCommonInterface.h header provides the interface definitions
 * for the Xyratex NTCI library */

#include <NTCommonInterface.h>

/* These headers are needed for accessing the capabilities interface: */
#include <NTCI_capabilities.h>
#include <capabilitiesstructure.h>

/* These headers are needed for CardInfo functions */
#include <cardinfo.h>
#include <NTCI_memoryaccess.h>

static char *req_mask;
static char *opt_wait;
static char *opt_clear;
static CommandLineOptions_t tools_options[] = {
  {"mask",  "Log level mask that specifies what to dump.", "64-bit mask", 1, &req_mask},
  {"wait",  "Wait for new entries.", "0|1",  0, &opt_wait},
  {"clear", "Clear the log after read.", "0|1", 0, &opt_clear},
  {NULL, NULL, NULL, 0, NULL}
};

static char tool_description[] =
{
  "Retrieve driver log entries.\n"
  "Log mask values:\n"
  " 0x01 : ERROR\n"
  " 0x02 : WARNING\n"
  " 0x04 : INFO\n"
  " 0x08 : DEBUG\n"
  " 0x10 : UPDATE IMAGE\n"
  " 0x20 : DIAGNOSTICS\n"
  " 0x40 : AVR\n"
};
static char output_description[] = { "" };

extern int toolRunning;

#ifdef WIN32
static BOOL CatchSignalCTRL_C(DWORD CtrlType)
{
	toolRunning=0;
	return TRUE;
}
#else
static void CatchSignalCTRL_C (int sig __attribute__((unused)))
{
  toolRunning=0;
}
#endif
            
int main(int argc, char *argv[])
{
  uint32_t cardIndex;
  int err;
#ifndef WIN32
  struct sigaction new;
#endif

  /* Parse the command line and initialize the needed variables. */    
  if(InitApplication(&argc, argv, FALSE, &cardIndex,
                     tools_options, tool_description,
                     output_description) == -1) {
    exit(-1);
  }
  
  DisplayProgramHeader(argv[0], PRG_DRVTOOL);

  /* Catch CTRL-C */
#ifdef WIN32
  SetConsoleCtrlHandler((PHANDLER_ROUTINE)CatchSignalCTRL_C, TRUE);
#else
	memset(&new, 0, sizeof(new));
	new.sa_handler = CatchSignalCTRL_C;
	if (sigaction(SIGINT, &new, NULL) < 0) {
    fprintf(stderr, "Failed to register sigaction.\n");
    exit(-1);
  }
#endif
  
  err=GetDriverLog(cardIndex, AsciiToInt(req_mask), AsciiToInt(opt_wait), AsciiToInt(opt_clear));

  CardShutdown(0);

  DisplayProgramFooter();

  return err;
}

