// $Id: Info.c 25600 2013-09-19 07:40:19Z mh $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Info.c: Various info functions.
 */

#ifdef WIN32
#include <windows.h>
#include <stdio.h>
#else
#include <ctype.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>

#include <sys/types.h>
#include <sys/stat.h>
#endif

#include "toolslib.h"

#include <NTCommonInterface.h>

/* These headers are needed for accessing the capabilities interface: */
#include <NTCI_capabilities.h>
#include <capabilitiesstructure.h>

/* These headers are needed for accessing the devicecontrol interface: */
#include <NTCI_devicecontrol.h>
#include <devicecontrolstructure.h>
#include <fpgainfostructure.h>

/* These headers are needed for accessing the VPD API */
#include <NTCI_vpdaccess.h>
#include <vpdstructure.h>
#include <NTCI_cardconfiguration.h>

/* These headers are needed for monitoring HW sensors. */
#include <hwmonstructure.h>
#include <NTCI_hardwaremonitor.h>

/* These headers are needed for TimeSync functions */
#include <cardconfigurationstructure.h>

#define DUMP_TO_SCREEN(__pos__, __buffer__, __size__, ...) __pos__ += WriteToBuffer(&__buffer__[__pos__], __size__-__pos__,  __VA_ARGS__);

char *str_insert(char *str, uint8_t pos, const char *ins)
{
  size_t lenstr;

  lenstr = strlen(str);
  if (lenstr > pos) {
    memmove(str+pos+1, str+pos, lenstr-pos);
    memcpy(str+pos, ins, 1);
  }
  return str;
}


uint32_t GetNtProductInfo(uint8_t card)
{
  handle_t cardHandle;
  FpgaImageHeaderType2_t flashHeaderType2;
  CardHardwareVersionCapabilitiesType3_t data3;
  VPDImageType4_t vpdData4;
  VPDImageType5_t vpdData5;
  int vpdData5_valid = 0;
  uint8_t ImageName[128];
  uint32_t result;
  int i = 0;
  int x = 0;
  char WriteBuffer[4096];
  const size_t bufferSize = sizeof(WriteBuffer);
  uint32_t writePos=0;
  CardHardwareCapabilitiesType1_t data1;
  char avrPrint[5];
  const char ins_char = '-';

  writePos=0;
  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }
  result = NTCI_ReadVitalProductData(cardHandle,
                                     0, /* ADAPTER CARD */
                                     STID_VPD_IMAGE_TYPE4,
                                     &vpdData4);

  if (result == NTCI_ERRCODE_SUCCESS) {
    DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                   "P/N                     : %s\n", vpdData4.productId);
    DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                   "Serial No               : %s\n", vpdData4.serialNumber);
    if (sizeof(vpdData4.pbaId) > 13) {
      vpdData4.pbaId[13] = '\0';
    }
    DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                   "PBA                     : %s\n", vpdData4.pbaId);
  } else {
    char temp_string[20];

    result = NTCI_ReadVitalProductData(cardHandle,
                                       0, /* ADAPTER CARD */
                                       STID_VPD_IMAGE_TYPE5,
                                       &vpdData5);
    if (result == NTCI_ERRCODE_SUCCESS) {
      vpdData5_valid = 1;
      if (vpdData5.mainProductId[3] != '-') {
        memset(temp_string, '\0', sizeof(temp_string));
        memcpy(temp_string, vpdData5.mainProductId, sizeof(vpdData5.mainProductId));
        (void)str_insert(temp_string, 3, &ins_char);
        (void)str_insert(temp_string, 8, &ins_char);
        (void)str_insert(temp_string, 11, &ins_char);
        memset(temp_string+14, '\0', 1);
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "P/N                     : %s\n", temp_string);
        memset(temp_string, '\0', sizeof(temp_string));
        memcpy(temp_string, vpdData5.mainSerialNumber, sizeof(vpdData5.mainSerialNumber));
        (void)str_insert(temp_string, 4, &ins_char);
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "Main Board Serial No    : %s\n", temp_string);
        memset(temp_string, '\0', sizeof(temp_string));
        memcpy(temp_string, vpdData5.mainPbaId, sizeof(vpdData5.mainPbaId));
        (void)str_insert(temp_string, 3, &ins_char);
        (void)str_insert(temp_string, 10, &ins_char);
        memset(temp_string+13, '\0', 1);
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "Main Board PBA          : %s\n", temp_string);
      } else {
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "P/N                     : %s\n", vpdData5.mainProductId);
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "Main Board Serial No    : %s\n", vpdData5.mainSerialNumber);
        if (sizeof(vpdData5.mainPbaId)>13) {
          vpdData5.mainPbaId[13]='\0';
        }
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "Main Board PBA          : %s\n", vpdData5.mainPbaId);
      }
    }
  }

  if((result = NTCI_GetCapabilities(cardHandle,
                                    CAPID_HARDWARE,
                                    NOT_SUB_CAPABILITIES,
                                    STID_CARD_HARDWARE_CAPABILITIES_TYPE1,
                                    &data1)) != NTCI_ERRCODE_SUCCESS) {
    WriteToLog(LOG_ERROR, "Failed to get HW capabilities. Result = 0x%x\n", result);

  } else if (data1.CardType == NTBPE) {
    FailoverConfigurationType1_t config;
    if (NTCI_GetCardConfigurationDetails(cardHandle,
                                         0 /* Not needed. */,
                                         STID_FAILOVER_CONFIGURATION_TYPE1,
                                         &config) == NTCI_ERRCODE_SUCCESS) {

      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                     "Firmware id             : %02d.%02d\n",
                     (config.fwVersion >> 8)  & 0xFF,
                     (config.fwVersion & 0xFF));
    }
  } else {
    result = NTCI_GetCapabilities(cardHandle,
                                  CAPID_VERSION,
                                  NOT_SUB_CAPABILITIES,
                                  STID_CARD_HARDWARE_VERSION_CAPABILITIES_TYPE3,
                                  &data3);

    if (result == NTCI_ERRCODE_SUCCESS)
    {
      switch (data3.AdapterCard.dwInfoType) {
      case 1:
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "CPLD                    : %02x-%02x\n",
                       (uint16_t)((data3.AdapterCard.u.InfoType1.dwCpldId & 0xFC00) >> 10),
                       (uint16_t)((data3.AdapterCard.u.InfoType1.dwCpldId & 0x03FC) >> 2));
        break;

      case 2:
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "CPLD                    : %02x-%02x\n",
                       (uint16_t)((data3.AdapterCard.u.InfoType2.dwCpldId & 0xFC00) >> 10),
                       (uint16_t)((data3.AdapterCard.u.InfoType2.dwCpldId & 0x03FC) >> 2));
        break;

      case 3:
        avrPrint[0] = (char)((data3.AdapterCard.u.InfoType3.dwAvrMain & 0xFF0000) >> 16);
        avrPrint[1] = '.';
        avrPrint[2] = (char)((data3.AdapterCard.u.InfoType3.dwAvrMain & 0xFF00) >> 8);
        //avrPrint[3] = (char)(data3.AdapterCard.u.InfoType3.dwAvrMain & 0xFF);
        avrPrint[3] = '\0';
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "Main Board AVR          : %s\n", avrPrint);
        avrPrint[0] = (char)((data3.AdapterCard.u.InfoType3.dwAvrFront & 0xFF0000) >> 16);
        avrPrint[1] = '.';
        avrPrint[2] = (char)((data3.AdapterCard.u.InfoType3.dwAvrFront & 0xFF00) >> 8);
        //avrPrint[3] = (char)(data3.AdapterCard.u.InfoType3.dwAvrFront & 0xFF);
        avrPrint[3] = '\0';
        break;

      default:
        WriteToLog(LOG_ERROR, "Unsupported NIC hardware information returned. Type = %li\n", (unsigned long)data3.AdapterCard.dwInfoType);
        break;
      }

    } else {
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                   "result = 0x%08X\n", result);
    }

    for (i=0; i<2; i++) {
      result = NTCI_ReadVitalProductData(cardHandle,
                                         i,
                                         STID_FPGA_IMAGE_HEADER_TYPE2,
                                         &flashHeaderType2);

      if (result != NTCI_ERRCODE_SUCCESS)
      {
        WriteToLog(LOG_ERROR, "Unable to read flash header for slot '%i', probably does not contain a valid FPGA image\n", i);
      } else {
        strcpy((char*)ImageName, (char*)flashHeaderType2.imageSourceFileName);
        if (flashHeaderType2.imageSourceFileName[4] == '9') {
          for(x=12; x<20; x++) {
            if (flashHeaderType2.imageSourceFileName[x] == '-') {
              ImageName[x] = '\0';
              continue;
            }
          }
        } else {
          ImageName[11] = '\0';
        }
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "FPGA flash image #%d     : %s\n", flashHeaderType2.imageNumber, ImageName);
      }
    }

    if (vpdData5_valid) {
      if (vpdData5.mainProductId[3] != '-') {
        char temp_string[20];
        memset(temp_string, '\0', sizeof(temp_string));
        memcpy(temp_string, vpdData5.frontSerialNumber, sizeof(vpdData5.frontSerialNumber));
        (void)str_insert(temp_string, 4, &ins_char);
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "Front Board Serial No   : %s\n", temp_string);
        memset(temp_string, '\0', sizeof(temp_string));
        memcpy(temp_string, vpdData5.frontPbaId, sizeof(vpdData5.frontPbaId));
        (void)str_insert(temp_string, 3, &ins_char);
        (void)str_insert(temp_string, 10, &ins_char);
        memset(temp_string+13, '\0', 1);
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "Front Board PBA         : %s\n", temp_string);
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "Front Board AVR         : %s\n", avrPrint);
      } else {
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "Front Board Serial No   : %s\n", vpdData5.frontSerialNumber);
        if (sizeof(vpdData5.frontPbaId)>13) {
          vpdData5.frontPbaId[13]='\0';
        }
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "Front Board PBA         : %s\n", vpdData5.frontPbaId);
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "Front Board AVR         : %s\n", avrPrint);
      }
    }
  }

  WriteToLog(LOG_INFO, WriteBuffer);
  return NTCI_ERRCODE_SUCCESS;
}


uint32_t _GetProductInfo(void *arglist)
{
  uint8_t  *argptr = arglist;
  uint8_t  card;
  card      = get8incr(&argptr);
  return GetNtProductInfo(card);
}


uint32_t GetAdapterInfo(uint8_t card)
{
  handle_t cardHandle;
  FpgaImageHeaderType2_t flashHeaderType2;
  CardHardwareVersionCapabilitiesType3_t data3;
  VPDImageType4_t vpdData4;
  uint8_t ImageName[128];
  uint32_t SBDFIndex=0xFFFFFFFF;
  uint32_t result;
  int i = 0;
  int x = 0;
  char WriteBuffer[4096];
  const size_t bufferSize = sizeof(WriteBuffer);
  uint32_t writePos=0;
  CardHardwareCapabilitiesType1_t data1;

  writePos=0;
  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  NTCI_GetPCISlotBusDeviceFunction(cardHandle, &SBDFIndex);

  DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                 "  PCI bus slot         : %02x:%02x:%02x.%x\n",
                 ((SBDFIndex>>24)&0xFF),
                 ((SBDFIndex>>16)&0xFF),
                 ((SBDFIndex>> 8)&0xFF),
                 ((SBDFIndex   )&0xFF));

  result = NTCI_ReadVitalProductData(cardHandle,
                                     0, /* ADAPTER CARD */
                                     STID_VPD_IMAGE_TYPE4,
                                     &vpdData4);

  if (result == NTCI_ERRCODE_SUCCESS) {
    DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                   "  Adapter type         : %s\n", vpdData4.productName);
    DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                   "  Serial number        : %s\n", vpdData4.serialNumber);
    DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                   "  PCB id               : %s\n", vpdData4.pcbId);
    DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                   "  PBA id               : %s\n", vpdData4.pbaId);
  } else {
    VPDImageType5_t vpdData5;

    result = NTCI_ReadVitalProductData(cardHandle,
                                       0, /* ADAPTER CARD */
                                       STID_VPD_IMAGE_TYPE5,
                                       &vpdData5);
    if (result == NTCI_ERRCODE_SUCCESS) {
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                     "  Main Adapter type    : %s\n", vpdData5.mainBoardType);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                     "  Main Serial number   : %s\n", vpdData5.mainSerialNumber);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                     "  Main Product Id      : %s\n", vpdData5.mainProductId);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                     "  Main PBA id          : %s\n", vpdData5.mainPbaId);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                     "  Front Adapter type   : %s\n", vpdData5.frontBoardType);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                     "  Front Serial number  : %s\n", vpdData5.frontSerialNumber);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                     "  Front Product id     : %s\n", vpdData5.frontProductId);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                     "  Front PBA id         : %s\n", vpdData5.frontPbaId);
    }
  }

  if((result = NTCI_GetCapabilities(cardHandle,
                                      CAPID_HARDWARE,
                                      NOT_SUB_CAPABILITIES,
                                      STID_CARD_HARDWARE_CAPABILITIES_TYPE1,
                                      &data1)) != NTCI_ERRCODE_SUCCESS) {
    WriteToLog(LOG_ERROR, "Failed to get HW capabilities. Result = 0x%x\n", result);

  } else if (data1.CardType == NTBPE) {
      FailoverConfigurationType1_t config;
      if (NTCI_GetCardConfigurationDetails(cardHandle,
                                           0 /* Not needed. */,
                                           STID_FAILOVER_CONFIGURATION_TYPE1,
                                           &config) == NTCI_ERRCODE_SUCCESS) {

      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                   "  Firmware id          : %02d.%02d\n",
                                        (config.fwVersion >> 8)  & 0xFF,
                                        (config.fwVersion & 0xFF));
    }
  } else {

    result = NTCI_GetCapabilities(cardHandle,
                                  CAPID_VERSION,
                                  NOT_SUB_CAPABILITIES,
                                  STID_CARD_HARDWARE_VERSION_CAPABILITIES_TYPE3,
                                  &data3);

    if (result == NTCI_ERRCODE_SUCCESS)
    {
      switch (data3.AdapterCard.dwInfoType) {
      case 1:
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "  CPLD version         : %02x-%02x\n",
                       (uint16_t)((data3.AdapterCard.u.InfoType1.dwCpldId & 0xFC00) >> 10),
                       (uint16_t)((data3.AdapterCard.u.InfoType1.dwCpldId & 0x03FC) >> 2));
        if((uint16_t)((data3.AdapterCard.u.InfoType1.qwFpgaId>>16)&0xFFFF) > 9000) {
          DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "  Active FPGA Image    : %03d-%04d-%02d-%02d-%02d\n",
                       (uint16_t)((data3.AdapterCard.u.InfoType1.qwFpgaId>>32)&0x0FFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType1.qwFpgaId>>16)&0xFFFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType1.qwFpgaId>>8)&0xFF),
                       (uint16_t)(data3.AdapterCard.u.InfoType1.qwFpgaId&0xFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType1.qwFpgaId>>44)&0x3FF));
        } else {
          DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "  Active FPGA Image    : %03d-%04d-%02d-%02d\n",
                       (uint16_t)((data3.AdapterCard.u.InfoType1.qwFpgaId>>32)&0x0FFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType1.qwFpgaId>>16)&0xFFFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType1.qwFpgaId>>8)&0xFF),
                       (uint16_t)(data3.AdapterCard.u.InfoType1.qwFpgaId&0xFF));
        }
        break;

      case 2:
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "  CPLD version         : %02x-%02x\n",
                       (uint16_t)((data3.AdapterCard.u.InfoType2.dwCpldId & 0xFC00) >> 10),
                       (uint16_t)((data3.AdapterCard.u.InfoType2.dwCpldId & 0x03FC) >> 2));
        if((uint16_t)((data3.AdapterCard.u.InfoType2.qwFpgaId>>16)&0xFFFF) > 9000) {
          DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "  Active FPGA Image    : %03d-%04d-%02d-%02d-%02d\n",
                       (uint16_t)((data3.AdapterCard.u.InfoType2.qwFpgaId>>32)&0x0FFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType2.qwFpgaId>>16)&0xFFFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType2.qwFpgaId>>8)&0xFF),
                       (uint16_t)(data3.AdapterCard.u.InfoType2.qwFpgaId&0xFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType2.qwFpgaId>>44)&0x3FF));
        } else {
          DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "  Active FPGA Image    : %03d-%04d-%02d-%02d\n",
                       (uint16_t)((data3.AdapterCard.u.InfoType2.qwFpgaId>>32)&0x0FFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType2.qwFpgaId>>16)&0xFFFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType2.qwFpgaId>>8)&0xFF),
                       (uint16_t)(data3.AdapterCard.u.InfoType2.qwFpgaId&0xFF));
        }
        break;

      case 3:
      {
        char avrPrint[5];
        avrPrint[0] = (char)((data3.AdapterCard.u.InfoType3.dwAvrMain & 0xFF0000) >> 16);
        avrPrint[1] = '.';
        avrPrint[2] = (char)((data3.AdapterCard.u.InfoType3.dwAvrMain & 0xFF00) >> 8);
        avrPrint[3] = (char)(data3.AdapterCard.u.InfoType3.dwAvrMain & 0xFF);
        avrPrint[4] = '\0';
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "  Main Board AVR       : %s\n", avrPrint);
        avrPrint[0] = (char)((data3.AdapterCard.u.InfoType3.dwAvrFront & 0xFF0000) >> 16);
        avrPrint[1] = '.';
        avrPrint[2] = (char)((data3.AdapterCard.u.InfoType3.dwAvrFront & 0xFF00) >> 8);
        avrPrint[3] = (char)(data3.AdapterCard.u.InfoType3.dwAvrFront & 0xFF);
        avrPrint[4] = '\0';
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "  Front Board AVR      : %s\n", avrPrint);
        if((uint16_t)((data3.AdapterCard.u.InfoType3.qwFpgaId>>16)&0xFFFF) > 9000) {
          DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "  Active FPGA Image    : %03d-%04d-%02d-%02d-%02d\n",
                       (uint16_t)((data3.AdapterCard.u.InfoType3.qwFpgaId>>32)&0x0FFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType3.qwFpgaId>>16)&0xFFFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType3.qwFpgaId>>8)&0xFF),
                       (uint16_t)(data3.AdapterCard.u.InfoType3.qwFpgaId&0xFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType3.qwFpgaId>>44)&0x3FF));
        } else {
          DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "  Active FPGA Image    : %03d-%04d-%02d-%02d\n",
                       (uint16_t)((data3.AdapterCard.u.InfoType3.qwFpgaId>>32)&0x0FFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType3.qwFpgaId>>16)&0xFFFF),
                       (uint16_t)((data3.AdapterCard.u.InfoType3.qwFpgaId>>8)&0xFF),
                       (uint16_t)(data3.AdapterCard.u.InfoType3.qwFpgaId&0xFF));
        }
        break;
      }

      default:
        WriteToLog(LOG_ERROR, "Unsupported NIC hardware information returned. Type = %li\n", (unsigned long)data3.AdapterCard.dwInfoType);
        break;
      }

    } else {
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                   "result = 0x%08X\n", result);
    }

    for (i=0; i<2; i++) {
      result = NTCI_ReadVitalProductData(cardHandle,
                                         i,
                                         STID_FPGA_IMAGE_HEADER_TYPE2,
                                         &flashHeaderType2);

      if (result != NTCI_ERRCODE_SUCCESS)
      {
        WriteToLog(LOG_ERROR, "Unable to read flash header for slot '%i', probably does not contain a valid FPGA image\n", i);
      } else {
        strcpy((char*)ImageName, (char*)flashHeaderType2.imageSourceFileName);
        if (flashHeaderType2.imageSourceFileName[4] == '9') {
          for(x=15; x<20; x++) {
            if (flashHeaderType2.imageSourceFileName[x] == '-') {
              ImageName[x] = '\0';
              continue;
            }
          }
        } else {
          ImageName[11] = '\0';
        }
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                       "  FPGA flash image #%d  : %s\n", flashHeaderType2.imageNumber, ImageName);
      }
    }
  }
  WriteToLog(LOG_INFO, WriteBuffer);
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t _GetAdapterInfo(void *arglist)
{
  uint8_t  *argptr = arglist;
  uint8_t  card;
  card      = get8incr(&argptr);
  return GetAdapterInfo(card);
}

uint32_t GetDriverInfo(uint8_t card)
{
  handle_t cardHandle;
  uint32_t result=NTCI_ERRCODE_SUCCESS;
  char WriteBuffer[4096];
  const size_t bufferSize = sizeof(WriteBuffer);
  uint32_t writePos=0;
  NtVersionInfoType1_t versionInfo;

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  NTCI_GetCardInfo(cardHandle, STID_CARD_INFO_VERSION_INFO_TYPE1, &versionInfo);

  DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                 "NT Adapter driver information\n");
  DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                 "  NTCommonInterface         : %3d.%03d.%03d.%03d  (%s)\n",
                 versionInfo.NTCommonInterfaceMajorVersion,
                 versionInfo.NTCommonInterfaceMinorVersion,
                 versionInfo.NTCommonInterfaceFeatureVersion,
                 versionInfo.NTCommonInterfaceBuildVersion,
                 versionInfo.NTCommonInterfaceReleasePackageInfo);

  DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                 "  Interface Protocol driver : %3d.%03d.%03d.%03d  (%s)\n",
                 versionInfo.ProtocolMajorVersion,
                 versionInfo.ProtocolMinorVersion,
                 versionInfo.ProtocolFeatureVersion,
                 versionInfo.ProtocolBuildVersion,
                 versionInfo.ProtocolReleasePackageInfo);

  DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize,
                 "  Kernel Device Driver      : %3d.%03d.%03d.%03d  (%s)\n",
                 versionInfo.DriverMajorVersion,
                 versionInfo.DriverMinorVersion,
                 versionInfo.DriverFeatureVersion,
                 versionInfo.DriverBuildVersion,
                 versionInfo.DriverReleasePackageInfo);

  WriteToLog(LOG_INFO, WriteBuffer);
  return result;
}

uint32_t _GetDriverInfo(void *arglist)
{
  uint8_t  *argptr = arglist;
  uint8_t  card;
  card      = get8incr(&argptr);
  return GetDriverInfo(card);
}


uint32_t GetSensorInfo(uint8_t card)
{
  handle_t cardHandle;
  uint32_t i;
  HardwareMonitorType99_t* sensor;
  HardwareMonitorType99_t SensorList99[50]; /* Max sensors */
  uint32_t blockSize;
  uint32_t SensorListLength;

  uint32_t result=NTCI_ERRCODE_SUCCESS;
  char WriteBuffer[4096];
  const size_t bufferSize = sizeof(WriteBuffer);
  uint32_t writePos = 0;

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  result = NTCI_GetPartSensorsBlockSize(cardHandle,
                                        STID_HARDWARE_MONITOR_TYPE99,
                                        &blockSize);

  if(result != NTCI_ERRCODE_SUCCESS) {
    DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, ">>> Error: Failed getting sensor block size.\nIs the driver running with the sensor monitoring service?\n");
    DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "Please enable the sensor monitoring service in default.cfg (SensorAlerts=1)\n"
           "and reload the driver.\n");
    return result;
  }

  if(blockSize > sizeof(SensorList99)) {
    DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, ">>> Error: Too many sensors at Adapter. Not enough memory.\n");
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  result = NTCI_GetPartSensors(cardHandle,
                               STID_HARDWARE_MONITOR_TYPE99,
                               &SensorList99,
                               blockSize);

  if(result != NTCI_ERRCODE_SUCCESS) {
    DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, ">>> Error: Failed getting sensors.\n");
    return result;
  }

  /* One entry for each sensor, plus the "all sensors" entry */
  SensorListLength = blockSize/sizeof(HardwareMonitorType99_t);

  DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%12s ", "");
  DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, " %-8s %-8s %-8s %-8s %-8s %-8s %-8s\n", "Sensor", "Low", "High", "Current", "Alarm", "Lowest", "Highest");
  DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%12s ", "");
  DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, " %-8s %-8s %-8s %-8s %-8s %-8s %-8s\n", "type", "limit", "limit", "value", "state", "value", "value");

  for (i=0; i<SensorListLength; i++) {
    sensor = (HardwareMonitorType99_t*)&SensorList99[i];
    if ((SensorList99[i].SensorType == TEMPERATURE)
     || (SensorList99[i].SensorType == FANSPEED )
     || (SensorList99[i].SensorType == VOLTAGE )
     || (SensorList99[i].SensorType == CURRENT )) {
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-14s", sensor->SensorName);
    }

    switch(sensor->SensorType) {
    case TEMPERATURE: DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8s ", "Temp."); break;
    case VOLTAGE: DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8s ", "Vol."); break;
    case FANSPEED: DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8s ", "Fan."); break;
    case CURRENT: DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8s ", "Current."); break;
    default: continue;
    }

    if ((sensor->SensorType ==CURRENT ) || (sensor->SensorType == VOLTAGE )) {
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->lowAlarmLimit/100);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->highAlarmLimit/100);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->sensorMeasurement/100);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8s ", sensor->SensorAlarm==TRUE?"Alarm!":"");
      if(((HardwareMonitorType1_t*)sensor)->StructureVersion == 1) {
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->lowestValue/100);
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->highestValue/100);
      }
    } else
    if (sensor->SensorType == FANSPEED ){
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->lowAlarmLimit);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->highAlarmLimit);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->sensorMeasurement);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8s ", sensor->SensorAlarm==TRUE?"Alarm!":"");
      if(((HardwareMonitorType1_t*)sensor)->StructureVersion == 1) {
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->lowestValue);
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->highestValue);
      }
    } else if (sensor->SensorType == TEMPERATURE) {
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->lowAlarmLimit/10);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->highAlarmLimit/10);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->sensorMeasurement/10);
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8s ", sensor->SensorAlarm==TRUE?"Alarm!":"");
      if(((HardwareMonitorType1_t*)sensor)->StructureVersion == 1) {
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->lowestValue/10);
        DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "%-8.2f ", (float)sensor->highestValue/10);
      }
    }
    if ((SensorList99[i].SensorType == TEMPERATURE)
     || (SensorList99[i].SensorType == FANSPEED )
     || (SensorList99[i].SensorType == VOLTAGE )
     || (SensorList99[i].SensorType == CURRENT )) {
      DUMP_TO_SCREEN(writePos, WriteBuffer, bufferSize, "\n");
    }
  }
  WriteToLog(LOG_INFO, WriteBuffer);
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t _GetSensorInfo(void *arglist)
{
  uint8_t  *argptr = arglist;
  uint8_t  card;
  card      = get8incr(&argptr);
  return GetSensorInfo(card);
}


/*****************************************************************************************
 *******************            Card Capabilities specifics.           *******************
 *****************************************************************************************/
#ifdef _WIN32
#define PRINTF_64BIT "%I64d"
#else
#define PRINTF_64BIT "%lld"
#endif

/* convert a boolean value to a string representation */
static const char *BoolAsString(int value)
{
  if (value)
    return "true";
  else
    return "false";
}

/* print a field of output data */
#define NAME_FIELD_WIDTH 30
static void PrintOutputField(uint32_t *writePos, char *pWriteBuffer, uint32_t bufferSize,
                             const char *name, const char *s, ...)
{
  va_list args;
  size_t i;

  DUMP_TO_SCREEN(*writePos, pWriteBuffer, bufferSize, "%s", name);

  /* fill in the remaining space after the name with spaces
   * so that the value column is aligned */

  for (i=strlen(name); i<NAME_FIELD_WIDTH; ++i)
    DUMP_TO_SCREEN(*writePos, pWriteBuffer, bufferSize, " ");

  DUMP_TO_SCREEN(*writePos, pWriteBuffer, bufferSize, ": ");

  va_start(args, s);

  *writePos += vsnprintf(&pWriteBuffer[*writePos], bufferSize-*writePos, s, args);

  DUMP_TO_SCREEN(*writePos, pWriteBuffer, bufferSize, "\n");

  va_end(args);
}

/* Print the option fields common to all capability structures */

static void PrintCommonCapabilities(CommonCapabilitiesType1_t *commonCapabilities,
                                    int32_t idx,
                                    uint32_t *writePos,
                                    char *pWriteBuffer,
                                    uint32_t bufferSize)
{
  const char *name_str;
  char index_str[10];

  /* Construct and print the name of this identifier */

  if (commonCapabilities->name[0] != 0)
    name_str = commonCapabilities->name;
  else
    name_str = "<NOT SET>";

  if (idx > 0)
    sprintf(index_str, "#%i", idx);
  else
    strcpy(index_str, "");

  PrintOutputField(writePos, pWriteBuffer, bufferSize,
                   "Name", "%s %s", name_str, index_str);

  /* Other common options */

  PrintOutputField(writePos, pWriteBuffer, bufferSize,
                   "Capabilities Identifier", "0x%08x",
                   commonCapabilities->capabilitiesId);

  PrintOutputField(writePos, pWriteBuffer, bufferSize,
                   "Indexed", "%s",
                   BoolAsString(commonCapabilities->flags & INDEXED_CAPABILITY));
  PrintOutputField(writePos, pWriteBuffer, bufferSize,
                   "Leaf", "%s", BoolAsString(commonCapabilities->flags & LEAF_CAPABILITY));
  PrintOutputField(writePos, pWriteBuffer, bufferSize,
                   "Dynamic", "%s", BoolAsString(commonCapabilities->flags & DYNAMIC_CAPABILITY));
  PrintOutputField(writePos, pWriteBuffer, bufferSize,
                   "Structure Identifier", "0x%08x", commonCapabilities->structId);
}

typedef enum
{
  PT_UNKNOWN,
  PT_CAPTURE,
  PT_INLINE
} ProdType_e;

/* Convert a enum CardType to a string */
static const char *CardTypeToString(enum CardType cardType, BusType_t busType, ProdType_e prodType)
{
  switch (prodType)
  {
  case PT_UNKNOWN:
    switch (cardType)
    {
    case XSERIES:
      return "X series";
    case SSERIES:
      return "S series";
    case GB10S:
      return "GB10S";
    case TENGIG:
      return "10Gb";
    case NT10F:
      return "NT series: NT10F";
    case NT20F:
      switch (busType)
      {
      case BUS_UNKNOWN:
        return "NT series: NT20";
      case BUS_PCI:
        return "NT series: NT20 PCI";
      case BUS_PCIX:
        return "NT series: NT20 PCI-X";
      case BUS_PCIEXPRESS:
        return "NT series: NT20 PCI Express";
      case BUS_COMPACTPCI:
        return "NT series: NT20 Compact PCI";
      }
      return "NT series: NT20";
    case NT4:
      return "NT series: NT4";
    case NT4STD:
      return "NT series: NT4STD";
    case NT20E2:
      return "NT series: NT20E2";
    case NT40E2_1:
      return "NT series: NT40E2_1";
    case NT40E2_4:
      return "NT series: NT40E2_4";
    default:
      return "unknown";
    }
    break;
  case PT_CAPTURE:
    switch (cardType)
    {
    case XSERIES:
      return "X series";
    case SSERIES:
      return "S series";
    case GB10S:
      return "GB10S";
    case TENGIG:
      return "10Gb";
    case NT10F:
      return "NT series: NT10F";
    case NT20F:
      switch (busType)
      {
      default:
        return "NT series: NT20";
      case BUS_PCIX:
        return "NT series: NT20 PCI-X";
      case BUS_PCIEXPRESS:
        return NT20E_CAP_STR;
      }
      return "NT series: NT20";
    case NT4:
      return NT4E_CAP_STR;
    case NT4STD:
      return NT4ESTD_CAP_STR;
    case NT20E2:
      return NT20E2_CAP_STR;
    case NT40E2_1:
      return NT40E2_1_CAP_STR;
    case NT40E2_4:
      return NT40E2_4_CAP_STR;
    default:
      return "unknown";
    }
    break;
  case PT_INLINE:
    switch (cardType)
    {
    case XSERIES:
      return "X series";
    case SSERIES:
      return "S series";
    case GB10S:
      return "GB10S";
    case TENGIG:
      return "10Gb";
    case NT10F:
      return "NT series: NT10F";
    case NT20F:
      switch (busType)
      {
      default:
        return "NT series: NT20";
      case BUS_PCIX:
        return "NT series: NT20 PCI-X";
      case BUS_PCIEXPRESS:
        return NT20E_INL_STR;
      }
      return "NT series: NT20";
    case NT4:
      return NT4E_INL_STR;
    case NT4STD:
      return NT4ESTD_INL_STR;
    case NT20E2:
      return NT20E2_INL_STR;
    case NT40E2_1:
      return NT40E2_1_INL_STR;
    case NT40E2_4:
      return NT40E2_4_INL_STR;
    default:
      return "unknown";
    }
    break;
  default:
    return "unknown";
  }
}

/* Convert a BusType_t to a string */
static const char *BusTypeToString(BusType_t busType)
{
  switch (busType)
  {
  default:
  case BUS_UNKNOWN:
    return "unknown";
  case BUS_PCI:
    return "PCI";
  case BUS_PCIX:
    return "PCI-X";
  case BUS_PCIEXPRESS:
    return "PCI Express";
  case BUS_COMPACTPCI:
    return "Compact PCI";
  }
}

/* Convert a BusSpeed_t to a string */
static const char *BusSpeedToString(BusSpeed_t busSpeed)
{
  switch (busSpeed)
  {
  default:
  case SPEED_UNKNOWN:
    return "unknown";
  case SPEED_33MHZ:
    return "33Mhz";
  case SPEED_66MHZ:
    return "66Mhz";
  case SPEED_100MHZ:
    return "100Mhz";
  case SPEED_133MHZ:
    return "133Mhz";
  }
}

/* Convert a BusWidth_t to a string */

static const char *BusWidthToString(BusWidth_t busWidth)
{
  switch (busWidth)
  {
  default:
  case WIDTH_UNKNOWN:
    return "unknown";
  case WIDTH_32BIT:
    return "32 bit";
  case WIDTH_64BIT:
    return "64 bit";
  }
}

/* Print the data stored in a capability structure */

static void PrintCapabilities(StructureID structId,
                              void *data,
                              int capIndex,
                              uint32_t *writePos,
                              char *pWriteBuffer,
                              uint32_t bufferSize)
{
  uint32_t idx;

  DUMP_TO_SCREEN(*writePos, pWriteBuffer, bufferSize,
                 "========================================================================\n");

  PrintCommonCapabilities((CommonCapabilitiesType1_t *)data, capIndex,
                          writePos, pWriteBuffer, bufferSize);

  /* print data appropriate to the particular structure */

  switch (structId)
  {
  case STID_COMMON_CAPABILITIES_TYPE1:
    break;

  case STID_CARD_HARDWARE_CAPABILITIES_TYPE1:
  {
    CardHardwareCapabilitiesType1_t *capabilities = (CardHardwareCapabilitiesType1_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "CardType", "%s", CardTypeToString(capabilities->CardType, capabilities->BusType, PT_UNKNOWN));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "BusType", "%s", BusTypeToString(capabilities->BusType));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "BusSpeed", "%s", BusSpeedToString(capabilities->BusSpeed));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "BusWidth", "%s", BusWidthToString(capabilities->BusWidth));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalNumberBondedCards", "%u", capabilities->totalNumberBondedCards);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "currentNumberBondedCards", "%u", capabilities->currentNumberBondedCards);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalChannelsBitmask", "0x%08x", capabilities->totalChannelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "currentChannelsBitmask", "0x%08x", capabilities->currentChannelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampType", "%u", capabilities->timestampType);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampSyncAcrossChannel", "%s", BoolAsString(capabilities->timestampSyncAcrossChannel));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampSyncAcrossCard", "%s", BoolAsString(capabilities->timestampSyncAcrossCard));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalBufferAllocated", "%d", capabilities->totalBufferAllocated);
  }
  break;

  case STID_CARD_HARDWARE_CAPABILITIES_TYPE2:
  {
    CardHardwareCapabilitiesType2_t *capabilities = (CardHardwareCapabilitiesType2_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "CardType", "%s", CardTypeToString(capabilities->type1.CardType, capabilities->type1.BusType, PT_UNKNOWN));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "BusType", "%s", BusTypeToString(capabilities->type1.BusType));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "BusSpeed", "%s", BusSpeedToString(capabilities->type1.BusSpeed));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "BusWidth", "%s", BusWidthToString(capabilities->type1.BusWidth));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalNumberBondedCards", "%u", capabilities->type1.totalNumberBondedCards);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "currentNumberBondedCards", "%u", capabilities->type1.currentNumberBondedCards);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalChannelsBitmask", "0x%08x", capabilities->type1.totalChannelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "currentChannelsBitmask", "0x%08x", capabilities->type1.currentChannelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampType", "%u", capabilities->type1.timestampType);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampSyncAcrossChannel", "%s", BoolAsString(capabilities->type1.timestampSyncAcrossChannel));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampSyncAcrossCard", "%s", BoolAsString(capabilities->type1.timestampSyncAcrossCard));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalBufferAllocated", "%d", capabilities->type1.totalBufferAllocated);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "rxPortSelectionSupported", "%s", BoolAsString(capabilities->rxPortSelectionSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "txPortSelectionSupported", "%s", BoolAsString(capabilities->txPortSelectionSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "localTransmitSupported", "%s", BoolAsString(capabilities->localTransmitSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "flowControlSupported", "%s", BoolAsString(capabilities->flowControlSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampTypesSupportedBitmask", "0x%08x", capabilities->timestampTypesSupportedBitmask);
  }
  break;
  case STID_CARD_HARDWARE_CAPABILITIES_TYPE3:
  {
    CardHardwareCapabilitiesType3_t *capabilities = (CardHardwareCapabilitiesType3_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "CardType", "%s", CardTypeToString(capabilities->type2.type1.CardType, capabilities->type2.type1.BusType, PT_UNKNOWN));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalNumberBondedCards", "%u", capabilities->type2.type1.totalNumberBondedCards);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "currentNumberBondedCards", "%u", capabilities->type2.type1.currentNumberBondedCards);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalChannelsBitmask", "0x%08x", capabilities->type2.type1.totalChannelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "currentChannelsBitmask", "0x%08x", capabilities->type2.type1.currentChannelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampType", "%u", capabilities->type2.type1.timestampType);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampSyncAcrossChannel", "%s", BoolAsString(capabilities->type2.type1.timestampSyncAcrossChannel));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampSyncAcrossCard", "%s", BoolAsString(capabilities->type2.type1.timestampSyncAcrossCard));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalBufferAllocated", "%d", capabilities->type2.type1.totalBufferAllocated);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "rxPortSelectionSupported", "%s", BoolAsString(capabilities->type2.rxPortSelectionSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "txPortSelectionSupported", "%s", BoolAsString(capabilities->type2.txPortSelectionSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "localTransmitSupported", "%s", BoolAsString(capabilities->type2.localTransmitSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "flowControlSupported", "%s", BoolAsString(capabilities->type2.flowControlSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampTypesSupportedBitmask", "0x%08x", capabilities->type2.timestampTypesSupportedBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "BusType", "%s", BusTypeToString(capabilities->type2.type1.BusType));
    if(capabilities->type2.type1.BusType == BUS_PCIEXPRESS) {
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "MaxPayload supported", "%d", capabilities->pcie.maxPayloadSupported);
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "MaxPayload negotiated", "%d", capabilities->pcie.maxPayloadNegotiated);
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Link width supported", "x%d", capabilities->pcie.linkWidthSupported);
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Link width negotiated", "x%d", capabilities->pcie.linkWidthNegotiated);
      if(capabilities->pcie.linkSpeed == PCIE_SPEED_2500MBPS) {
        PrintOutputField(writePos, pWriteBuffer, bufferSize, "Link speed negotiated", "%s", "2.5 GT/s");
      } else if(capabilities->pcie.linkSpeed == PCIE_SPEED_5000MBPS) {
        PrintOutputField(writePos, pWriteBuffer, bufferSize, "Link speed negotiated", "%s", "5.0 GT/s");
      } else {
        PrintOutputField(writePos, pWriteBuffer, bufferSize, "Link speed negotiated", "%s", "unknown");
      }
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Max expected throughput", "%d Mbps", capabilities->pcie.maxThroughput);
    } else {
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "BusSpeed", "%s", BusSpeedToString(capabilities->type2.type1.BusSpeed));
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "BusWidth", "%s", BusWidthToString(capabilities->type2.type1.BusWidth));
    }
  }
  break;
  case STID_CARD_HARDWARE_CAPABILITIES_TYPE4:
  {
    CardHardwareCapabilitiesType4_t *capabilities = (CardHardwareCapabilitiesType4_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "CardType", "%s", CardTypeToString(capabilities->type3.type2.type1.CardType, capabilities->type3.type2.type1.BusType, (capabilities->inlineCard? PT_INLINE : PT_CAPTURE)));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalNumberBondedCards", "%u", capabilities->type3.type2.type1.totalNumberBondedCards);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "currentNumberBondedCards", "%u", capabilities->type3.type2.type1.currentNumberBondedCards);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalChannelsBitmask", "0x%08x", capabilities->type3.type2.type1.totalChannelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "currentChannelsBitmask", "0x%08x", capabilities->type3.type2.type1.currentChannelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampType", "%u", capabilities->type3.type2.type1.timestampType);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampSyncAcrossChannel", "%s", BoolAsString(capabilities->type3.type2.type1.timestampSyncAcrossChannel));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampSyncAcrossCard", "%s", BoolAsString(capabilities->type3.type2.type1.timestampSyncAcrossCard));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "totalBufferAllocated", "%d", capabilities->type3.type2.type1.totalBufferAllocated);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "rxPortSelectionSupported", "%s", BoolAsString(capabilities->type3.type2.rxPortSelectionSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "txPortSelectionSupported", "%s", BoolAsString(capabilities->type3.type2.txPortSelectionSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "localTransmitSupported", "%s", BoolAsString(capabilities->type3.type2.localTransmitSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "flowControlSupported", "%s", BoolAsString(capabilities->type3.type2.flowControlSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "timestampTypesSupportedBitmask", "0x%08x", capabilities->type3.type2.timestampTypesSupportedBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "BusType", "%s", BusTypeToString(capabilities->type3.type2.type1.BusType));
    if(capabilities->type3.type2.type1.BusType == BUS_PCIEXPRESS) {
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "MaxPayload supported", "%d", capabilities->type3.pcie.maxPayloadSupported);
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "MaxPayload negotiated", "%d", capabilities->type3.pcie.maxPayloadNegotiated);
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Link width supported", "x%d", capabilities->type3.pcie.linkWidthSupported);
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Link width negotiated", "x%d", capabilities->type3.pcie.linkWidthNegotiated);
      if(capabilities->pcie.linkSpeedSupported == PCIE_SPEED_2500MBPS) {
        PrintOutputField(writePos, pWriteBuffer, bufferSize, "Link speed supported", "%s", "2.5 GT/s");
      } else if(capabilities->pcie.linkSpeedSupported == PCIE_SPEED_5000MBPS) {
        PrintOutputField(writePos, pWriteBuffer, bufferSize, "Link speed supported", "%s", "5.0 GT/s");
      } else {
        PrintOutputField(writePos, pWriteBuffer, bufferSize, "Link speed supported", "%s", "unknown");
      }
      if(capabilities->type3.pcie.linkSpeed == PCIE_SPEED_2500MBPS) {
        PrintOutputField(writePos, pWriteBuffer, bufferSize, "Link speed negotiated", "%s", "2.5 GT/s");
      } else if(capabilities->type3.pcie.linkSpeed == PCIE_SPEED_5000MBPS) {
        PrintOutputField(writePos, pWriteBuffer, bufferSize, "Link speed negotiated", "%s", "5.0 GT/s");
      } else {
        PrintOutputField(writePos, pWriteBuffer, bufferSize, "Link speed negotiated", "%s", "unknown");
      }
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Max expected throughput", "%d Mbps", capabilities->type3.pcie.maxThroughput);
    } else {
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "BusSpeed", "%s", BusSpeedToString(capabilities->type3.type2.type1.BusSpeed));
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "BusWidth", "%s", BusWidthToString(capabilities->type3.type2.type1.BusWidth));
    }
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "InlineCard", "%s", BoolAsString(capabilities->inlineCard));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "PacketTimestamp", "%s", capabilities->pktTsMethod==PACKET_TIMESTAMP_FIRST?"First octet":"Last octet");
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "Wis mode supported", "%s",  BoolAsString(capabilities->wisMode));
    PrintOutputField(writePos, pWriteBuffer, bufferSize, "Relative Tx Timing",
                     "%s",  BoolAsString(capabilities->relativeTxTiming));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "HighFrequencySampling enabled", "%s",  BoolAsString(capabilities->highFreqSampling));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "OS Timesync enabled", "%s",  BoolAsString(capabilities->osTimesync));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "OS Timesync Failover enabled", "%s",  BoolAsString(capabilities->osTimesyncFailover));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "Time stamp injection mode", "%s",
                     (capabilities->tsInjectionLegacyMode ? "legacy" :
                      capabilities->tsInjectionMode == 0 ? "start of frame" : "end of frame"));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "Time stamp injection offset", "%s",
                     (capabilities->tsInjectionOffsetMode == 0 ?
                      "start of frame" : "end of frame"));
  }
  break;

  case STID_CARD_HARDWARE_CHANNEL_CAPABILITIES_TYPE1:
  {
    CardHardwareChannelCapabilitiesType1_t *capabilities = (CardHardwareChannelCapabilitiesType1_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "supportedSpeeds", "%x", capabilities->supportedSpeeds);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "connectionType", "%u", capabilities->connectionType);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "virtualChannelBitmask", "0x%08x", capabilities->virtualChannelBitmask);
  }
  break;

  case STID_CARD_HARDWARE_CHANNEL_CAPABILITIES_TYPE2:
  {
    CardHardwareChannelCapabilitiesType2_t *capabilities = (CardHardwareChannelCapabilitiesType2_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "supportedSpeeds", "%x", capabilities->type1.supportedSpeeds);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "connectionType", "%u", capabilities->type1.connectionType);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "virtualChannelBitmask", "0x%08x", capabilities->type1.virtualChannelBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "MAC address", "%02X:%02X:%02X:%02X:%02X:%02X",
                     capabilities->macAddress[0], capabilities->macAddress[1],
                     capabilities->macAddress[2], capabilities->macAddress[3],
                     capabilities->macAddress[4], capabilities->macAddress[5]);
  }
  break;

  case STID_CARD_HARDWARE_CARD_CAPABILITIES_TYPE1:
  {
    CardHardwareCardCapabilitiesType1_t *capabilities = (CardHardwareCardCapabilitiesType1_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "bondedGroupId", "%u", capabilities->bondedGroupId);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "cardId", "%u", capabilities->cardId);

    if (capabilities->bondMaster)
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "bondMaster", "Master");
    else
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "bondMaster", "Slave");

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "virtualChannelsBitmask", "0x%08x", capabilities->virtualChannelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numberOfFPGAs", "%d", capabilities->numberOfFPGAs);
  }
  break;

  case STID_CARD_HARDWARE_CARD_CAPABILITIES_TYPE2:
  {
    CardHardwareCardCapabilitiesType2_t *capabilities = (CardHardwareCardCapabilitiesType2_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "bondedGroupId", "%u", capabilities->type1.bondedGroupId);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "cardId", "%u", capabilities->type1.cardId);

    if (capabilities->type1.bondMaster)
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "bondMaster", "Master");
    else
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "bondMaster", "Slave");

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "virtualChannelsBitmask", "0x%08x", capabilities->type1.virtualChannelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numberOfFPGAs", "%d", capabilities->type1.numberOfFPGAs);
    if(capabilities->SDRAMController)
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "SDRAM controller", "Available");
    else
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "SDRAM controller", "Not available");
    if(capabilities->SDRAMPresent) {
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "   SDRAM", "Present");
      if(capabilities->SDRAMSize & 0x3FF) {
        /* Special encoding. 64bit. */
        capabilities->SDRAMSize*=4096;
        PrintOutputField(writePos, pWriteBuffer, bufferSize,
                         "   Size (MB)", "%d", capabilities->SDRAMSize);
      } else {
        PrintOutputField(writePos, pWriteBuffer, bufferSize,
                         "   Size (MB)", "%d", capabilities->SDRAMSize/(1024*1024));
      }
    }
    else
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "   SDRAM", "Not present");
    if(capabilities->TimeSyncSupported)
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Time synchronization", "Supported");
    else
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Time synchronization", "Not supported");
  }
  break;
  case STID_CARD_HARDWARE_VERSION_CAPABILITIES_TYPE1:
  {
    CardHardwareVersionCapabilitiesType1_t *capabilities = (CardHardwareVersionCapabilitiesType1_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "BootStrapVersion", "%x", capabilities->BootStrapVersion);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "FPGAVersion", "%x", capabilities->FPGAVersion);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "ExtVersion", "%x", capabilities->ExtVersion);
  }
  break;

  case STID_CARD_HARDWARE_VERSION_CAPABILITIES_TYPE2:
  {
    CardHardwareVersionCapabilitiesType2_t *capabilities = (CardHardwareVersionCapabilitiesType2_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "BootStrapVersion", "%x", capabilities->type1.BootStrapVersion);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "FPGAVersion", "%x", capabilities->type1.FPGAVersion);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "ExtVersion", "%x", capabilities->type1.ExtVersion);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "CurrentFPGAImageLoaded", "%d", capabilities->CurrentFPGAImageLoaded);
  }
  break;

  case STID_CARD_HARDWARE_VERSION_CAPABILITIES_TYPE3:
  {
    CardHardwareVersionCapabilitiesType3_t *capabilities = (CardHardwareVersionCapabilitiesType3_t *)data;

    switch (capabilities->AdapterCard.dwInfoType) {
    case 1:
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Main board CPLD version", "%x", capabilities->AdapterCard.u.InfoType1.dwCpldId);
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Main board FPGA version", "%lx", capabilities->AdapterCard.u.InfoType1.qwFpgaId);
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Main board MAC ID"      , "%x", capabilities->AdapterCard.u.InfoType1.dwMacId);
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Main board PHY ID"      , "%x", capabilities->AdapterCard.u.InfoType1.dwPhyId);
      break;

    case 2:
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Main board CPLD version:", "%x",  capabilities->AdapterCard.u.InfoType2.dwCpldId);
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Main board FPGA version:", "%x",  capabilities->AdapterCard.u.InfoType2.qwFpgaId);
      break;

    case 3:
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Main board AVR version:", "%c.%c%c", ((capabilities->AdapterCard.u.InfoType3.dwAvrMain>>16)&0xFF),
                                                              ((capabilities->AdapterCard.u.InfoType3.dwAvrMain>>8)&0xFF),
                                                               (capabilities->AdapterCard.u.InfoType3.dwAvrMain&0xFF));

      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Front board AVR version:", "%c.%c%c", ((capabilities->AdapterCard.u.InfoType3.dwAvrFront>>16)&0xFF),
                                                              ((capabilities->AdapterCard.u.InfoType3.dwAvrFront>>8)&0xFF),
                                                               (capabilities->AdapterCard.u.InfoType3.dwAvrFront&0xFF));
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "Main board FPGA version:", "%x",  capabilities->AdapterCard.u.InfoType3.qwFpgaId);
      break;
    }
  }
  break;

  case STID_CARD_SERVICES_CAPABILITIES_TYPE1:
  {
    CardServicesCapabilitiesType1_t *capabilities = (CardServicesCapabilitiesType1_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "diagnosticSupported", "%s", BoolAsString(capabilities->diagnosticSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "hardwareMonitorSupported", "%s", BoolAsString(capabilities->hardwareMonitorSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "packetClassificationSupported", "%s", BoolAsString(capabilities->packetClassificationSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "lowLevelFilterSupported", "%s", BoolAsString(capabilities->lowLevelFilterSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "packetFeedSupported" ,"%s", BoolAsString(capabilities->packetFeedSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "statisticsSupported", "%s", BoolAsString(capabilities->statisticsSupported));
  }
  break;

  case STID_CARD_STATISTICS_CAPABILITIES_TYPE1:
  {
    CardStatisticsCapabilitiesType1_t *capabilities = (CardStatisticsCapabilitiesType1_t *)data;

    for (idx = 0; idx < capabilities->numberStructureIdSupported; ++idx)
    {
      char nameBuf[40];
      sprintf(nameBuf, "getDataStructureIdSupported[%u]", idx);

      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       nameBuf, "0x%08x", capabilities->getDataStructureIdSupported[idx]);
    }

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numberStructureIdSupported", "%u", capabilities->numberStructureIdSupported);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "hardwareRunControl", "%s", BoolAsString(capabilities->hardwareRunControl));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "softwareRunControl", "%s", BoolAsString(capabilities->softwareRunControl));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "hardwareRunControlChannelLinked", "%s", BoolAsString(capabilities->hardwareRunControlChannelLinked));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "softwareRunControlChannelLinked", "%s", BoolAsString(capabilities->softwareRunControlChannelLinked));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "crossChannelHardwareMerging", "%s", BoolAsString(capabilities->crossChannelHardwareMerging));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "crossChannelSoftwareMerging", "%s", BoolAsString(capabilities->crossChannelSoftwareMerging));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "crossCardHardwareMerging", "%s", BoolAsString(capabilities->crossCardHardwareMerging));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "crossCardSoftwareMerging", "%s", BoolAsString(capabilities->crossCardSoftwareMerging));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "startWithFeedStart", "%s", BoolAsString(capabilities->startWithFeedStart));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "stopWithFeedStop", "%s", BoolAsString(capabilities->stopWithFeedStop));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "resetWithFeedStart", "%s", BoolAsString(capabilities->resetWithFeedStart));
  }
  break;

  case STID_CARD_PACKET_FEED_CAPABILITIES_TYPE1:
  {
    CardPacketFeedCapabilitiesType1_t *capabilities = (CardPacketFeedCapabilitiesType1_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "minPacketSize", "%u", capabilities->minPacketSize);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "maxPacketSize", "%u", capabilities->maxPacketSize);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numberOfCaptureSupported", "%u", capabilities->numberOfCaptureSupported);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numberOfStreamingSupported", "%u", capabilities->numberOfStreamingSupported);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "crossChannelMerging", "%s", BoolAsString(capabilities->crossChannelMerging));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "crossCardMerging", "%s", BoolAsString(capabilities->crossCardMerging));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "internalRxSupported", "%s", BoolAsString(capabilities->internalRxSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "externalRxSupported", "%s", BoolAsString(capabilities->externalRxSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "internalTxSupported", "%s", BoolAsString(capabilities->internalTxSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "externalTxSupported", "%s", BoolAsString(capabilities->externalTxSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "channelsInUseBitmask", "0x%08x", capabilities->channelsInUseBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "currentEngineFeedCount", "%u", capabilities->currentEngineFeedCount);
  }
  break;

  case STID_CARD_PACKET_FEED_CAPABILITIES_TYPE2:
  {
    CardPacketFeedCapabilitiesType2_t *capabilities = (CardPacketFeedCapabilitiesType2_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "minPacketSize", "%u", capabilities->type1.minPacketSize);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "maxPacketSize", "%u", capabilities->type1.maxPacketSize);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numberOfCaptureSupported", "%u", capabilities->type1.numberOfCaptureSupported);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numberOfStreamingSupported", "%u", capabilities->type1.numberOfStreamingSupported);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "crossChannelMerging", "%s", BoolAsString(capabilities->type1.crossChannelMerging));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "crossCardMerging", "%s", BoolAsString(capabilities->type1.crossCardMerging));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "internalRxSupported", "%s", BoolAsString(capabilities->type1.internalRxSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "externalRxSupported", "%s", BoolAsString(capabilities->type1.externalRxSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "internalTxSupported", "%s", BoolAsString(capabilities->type1.internalTxSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "externalTxSupported", "%s", BoolAsString(capabilities->type1.externalTxSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "channelsInUseBitmask (Rx)", "0x%08x", capabilities->type1.channelsInUseBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "currentEngineFeedCount", "%u", capabilities->type1.currentEngineFeedCount);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "endianSwapSupported", "%s", BoolAsString(capabilities->endianSwapSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "ipAlignmentSupported", "%s", BoolAsString(capabilities->ipAlignmentSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "txChecksumCalculationSupported", "%s", BoolAsString(capabilities->txChecksumCalculationSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "slicingSupported", "%s", BoolAsString(capabilities->slicingSupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "channelsInUseBitmaskRx", "0x%08x", capabilities->channelsInUseBitmaskRx);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "channelsInUseBitmaskTx", "0x%08x", capabilities->channelsInUseBitmaskTx);
  }
  break;

  case STID_CARD_CAPTURE_PACKET_FEED_CAPABILITIES_TYPE1:
  {
    CardCapturePacketFeedCapabilitiesType1_t *capabilities = (CardCapturePacketFeedCapabilitiesType1_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "channelsBitmask", "0x%08x", capabilities->channelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "running", "%s", BoolAsString(capabilities->running));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "externalFeed", "%s", BoolAsString(capabilities->externalFeed));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "transmittingFeed", "%s", BoolAsString(capabilities->transmittingFeed));
  }
  break;

  case STID_CARD_STREAMING_PACKET_FEED_CAPABILITIES_TYPE1:
  {
    CardStreamingPacketFeedCapabilitiesType1_t *capabilities = (CardStreamingPacketFeedCapabilitiesType1_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "channelsBitmask", "0x%08x", capabilities->channelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "externalFeed", "%s", BoolAsString(capabilities->externalFeed));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "transmittingFeed", "%s", BoolAsString(capabilities->transmittingFeed));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "dropErroredFrames", "%s", BoolAsString(capabilities->dropErroredFrames));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "sliceLength", "%u", capabilities->sliceLength);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numSections", "%u", capabilities->numSections);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "sectionBufferSize", "0x%08x", capabilities->sectionBufferSize);
  }
  break;

  case STID_CARD_STREAMING_PACKET_FEED_CAPABILITIES_TYPE2:
  {
    CardStreamingPacketFeedCapabilitiesType2_t *capabilities = (CardStreamingPacketFeedCapabilitiesType2_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "channelsBitmask", "0x%08x", capabilities->type1.channelsBitmask);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "externalFeed", "%s", BoolAsString(capabilities->type1.externalFeed));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "transmittingFeed", "%s", BoolAsString(capabilities->type1.transmittingFeed));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "dropErroredFrames", "%s", BoolAsString(capabilities->type1.dropErroredFrames));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "sliceLength", "%u", capabilities->type1.sliceLength);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numSections", "%u", capabilities->type1.numSections);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "sectionBufferSize", "0x%08x", capabilities->type1.sectionBufferSize);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "channelsBitmaskRx", "0x%08x", capabilities->channelsBitmaskRx);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "channelsBitmaskTx", "0x%08x", capabilities->channelsBitmaskTx);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numSectionsRx", "%u", capabilities->numSectionsRx);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numSectionsTx", "%u", capabilities->numSectionsTx);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "sectionBufferSizeRx", "0x%08x", capabilities->sectionBufferSizeRx);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "sectionBufferSizeTx", "0x%08x", capabilities->sectionBufferSizeTx);
  }
  break;

  case STID_CARD_PACKET_CLASSIFICATION_CAPABILITIES_TYPE1:
  {
    CardPacketClassificationCapabilitiesType1_t *capabilities = (CardPacketClassificationCapabilitiesType1_t *)data;
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "supportedComponentTypes", "0x" PRINTF_64BIT, capabilities->supportedComponentTypes);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "supportedActionTypes", "0x" PRINTF_64BIT, capabilities->supportedActionTypes);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "dynamicOffsetSupport", "%s", BoolAsString(capabilities->dynamicOffsetSupport));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "numDynamicOffsets", "%i", capabilities->numDynamicOffsets);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "maxExpressionNesting", "%i", capabilities->maxExpressionNesting);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "maxExpressions", "%i", capabilities->maxExpressions);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "maxFunctions", "%i", capabilities->maxFunctions);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "maxPatternLength", "%i", capabilities->maxPatternLength);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "maxFixedOffset", "%i", capabilities->maxFixedOffset);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "maxSignatureMatchPatternLength", "%i", capabilities->maxSignatureMatchPatternLength);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "supportedSignatureMatchTypes", "%i", capabilities->supportedSignatureMatchTypes);
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "supportedSignatureMatchActionTypes", "%i", capabilities->supportedSignatureMatchActionTypes);

  }
  break;

  case STID_CARD_INTERFACE_SUPPORTED_TYPE1:
  {
    CardInterfaceSupportedType1_t *capabilities = (CardInterfaceSupportedType1_t *)data;

    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "StatisticsInterface", "%s", BoolAsString(capabilities->boStatisticsIFsupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "PacketFeedInterface", "%s", BoolAsString(capabilities->boPacketFeedIFsupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "VPDInterface", "%s", BoolAsString(capabilities->boVPDIFsupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "CardConfiguration", "%s", BoolAsString(capabilities->boCardConfigurationIFsupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "CardStatusInterface", "%s", BoolAsString(capabilities->boCardStatusIFsupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "CapabilitiesIFsupported", "%s", BoolAsString(capabilities->boCapabilitiesIFsupported));
    PrintOutputField(writePos, pWriteBuffer, bufferSize,
                     "CardInfoIFsupported", "%s", BoolAsString(capabilities->boCardInfoIFsupported));
  }
  break;


  case STID_CARD_INTERFACE_SUPPORTED_STID_TYPE1:
  {
    int i=0;

    while (strlen(((CardInterfaceSupportedSTIDType1_t*)data)->InterfaceSTID[i]) != 0) {
      PrintOutputField(writePos, pWriteBuffer, bufferSize,
                       "supportedSTID", "%s", ((CardInterfaceSupportedSTIDType1_t*)data)->InterfaceSTID[i]);
      i++;
    }
  }
  break;

  default:
    DUMP_TO_SCREEN(*writePos, pWriteBuffer, bufferSize,
                   "Unknown structure type, raw hex bytes:\n");

    for (idx = 0; idx < GET_STRUCTURE_SIZE(structId); ++idx)
    {
      DUMP_TO_SCREEN(*writePos, pWriteBuffer, bufferSize,
                     "%02x ", *((unsigned char *)data + idx));

      /* split line every 16 characters */

      if ((idx + 1) % 16 == 0)
        DUMP_TO_SCREEN(*writePos, pWriteBuffer, bufferSize, "\n");
    }
    DUMP_TO_SCREEN(*writePos, pWriteBuffer, bufferSize, "\n");
    break;
  }
}

/* Recursively walk all of the nodes in the capabilities tree and
 * print them */

#define MAX_CAPABILITIES_SIZE 2048
#define MAX_INDEX 32

static uint32_t WalkTree(
  handle_t cardHandle, uint32_t capabilitiesId,
  uint32_t *writePos, char *pWriteBuffer, uint32_t bufferSize)
{
  uint32_t status;
  CommonCapabilitiesType1_t commonCapabilities;
  char capabilitiesBuffer[MAX_CAPABILITIES_SIZE];
  uint32_t idx;

  status = NTCI_GetCapabilities(cardHandle,
                                capabilitiesId,
                                NOT_SUB_CAPABILITIES,
                                STID_COMMON_CAPABILITIES_TYPE1,
                                &commonCapabilities);

  if (status != NTCI_ERRCODE_SUCCESS)
  {
    WriteToLog(LOG_ERROR,
               "NTCI_GetCapabilities(cardHandle, "
               "capabilitiesId = 0x%08x, "
               "NOT_SUB_CAPABILITIES, "
               "structId = 0x%08x, "
               "&commonCapabilities) "
               "returned 0x%08x\n",
               capabilitiesId, STID_COMMON_CAPABILITIES_TYPE1, status);

    return 0;
  }

  if (GET_STRUCTURE_SIZE(commonCapabilities.structId) > MAX_CAPABILITIES_SIZE)
  {
    DUMP_TO_SCREEN(*writePos, pWriteBuffer, bufferSize,
                   "Size of capabilities to big for static buffer. Increase MAX_CAPABILITIES_SIZE.\n");
  }
  else if ((commonCapabilities.flags & INDEXED_CAPABILITY) == 0)
  {
    status = NTCI_GetCapabilities(cardHandle,
                                  capabilitiesId,
                                  NOT_SUB_CAPABILITIES,
                                  commonCapabilities.structId,
                                  capabilitiesBuffer);

    if (status != NTCI_ERRCODE_SUCCESS)
    {
      WriteToLog(LOG_ERROR,
                 "NTCI_GetCapabilities(cardHandle, "
                 "capabilitiesId = 0x%08x, "
                 "NOT_SUB_CAPABILITIES, "
                 "structId = 0x%08x, "
                 "&commonCapabilities) "
                 "returned 0x%08x\n",
                 capabilitiesId, commonCapabilities.structId, status);
      return 0;
    }

    PrintCapabilities(commonCapabilities.structId,
                      capabilitiesBuffer,
                      0,
                      writePos, pWriteBuffer, bufferSize);
  }
  if (commonCapabilities.flags & INDEXED_CAPABILITY)
  {
    for (idx = 0; idx < MAX_INDEX; ++idx)
    {
      status = NTCI_GetCapabilities(cardHandle,
                                    capabilitiesId,
                                    (uint8_t)idx,
                                    commonCapabilities.structId,
                                    capabilitiesBuffer);


      if (status == NTCI_ERRCODE_SUCCESS)
      {
        PrintCapabilities(commonCapabilities.structId,
                          capabilitiesBuffer, idx+1,
                          writePos, pWriteBuffer, bufferSize);
      }
    }
  }
  else
  {
    for (idx = 0; idx < MAX_INDEX; ++idx)
    {
      status = NTCI_GetCapabilities(cardHandle,
                                    capabilitiesId,
                                    (uint8_t)idx,
                                    STID_COMMON_CAPABILITIES_TYPE1,
                                    &commonCapabilities);

      if (status == NTCI_ERRCODE_SUCCESS)
      {
        if (!WalkTree(cardHandle, commonCapabilities.capabilitiesId,
                      writePos, pWriteBuffer, bufferSize))
        {
          return 0;
        }
      }
    }
  }

  return 1;
}


uint32_t GetCapabilities(uint8_t card)
{
  uint32_t result;
  handle_t cardHandle;
  char *pWriteBuffer;
  uint32_t writePos = 0;
  const size_t bufferSize = 80 * 1024; /* It takes up some space... Allocate enough. (80k) */

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  if ((pWriteBuffer = malloc(bufferSize)) == NULL) {
    WriteToLog(LOG_ERROR, "Failed to allocate %llu screendump buffer.\n",
               (long long unsigned)bufferSize);
    return NTCI_ERRCODE_INSUFFICIENT_MEMORY;
  }

  result = WalkTree(cardHandle, CAPID_ROOT, &writePos, pWriteBuffer, bufferSize);
  WriteToLog(LOG_INFO, pWriteBuffer);
  free(pWriteBuffer);
  return result;
}

uint32_t _GetCapabilities(void *arglist)
{
  uint8_t  *argptr = arglist;
  uint8_t  card;
  card      = get8incr(&argptr);
  GetCapabilities(card);
  return 0;
}

uint32_t GetHWMonInfo (uint8_t card)
{
  HardwareMonitorType2_t SensorList[10]; /* Max sensors */
  handle_t cardHandle=NULL;
  uint32_t blockSize;
  uint32_t SensorListLength;
  uint32_t result=0, i;
  char     writeBuffer[1024];
  uint32_t writePos=0;

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  result = NTCI_GetPartSensorsBlockSize(cardHandle, STID_HARDWARE_MONITOR_TYPE2, &blockSize);

  if(result != NTCI_ERRCODE_SUCCESS) {
    WriteToLog(LOG_ERROR, "Failed getting sensor block size.\nIs the driver running with the sensor monitoring service?\n");
    WriteToLog(LOG_ERROR, "Please enable the sensor monitoring service in default.cfg (SensorAlerts=1)\n");
    return result;
  }

  if(blockSize > sizeof(SensorList)) {
    WriteToLog(LOG_ERROR, "Adapter has too many sensors. Not enough memory.\n");
    return NTCI_ERRCODE_INSUFFICIENT_MEMORY;
  }

  result = NTCI_GetPartSensors(cardHandle,
                               STID_HARDWARE_MONITOR_TYPE2,
                               &SensorList,
                               blockSize);

  if(result != NTCI_ERRCODE_SUCCESS) {
    WriteToLog(LOG_ERROR, "Failed getting sensors.\n");
    return result;
  }

  if(((HardwareMonitorType1_t*)SensorList)->StructureVersion != 1) {
    WriteToLog(LOG_ERROR, "Only STID_HARDWARE_MONITOR_TYPE2 is supported. HW returns %d\n",
           ((HardwareMonitorType1_t*)SensorList)->StructureVersion);
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  /* One entry for each sensor, plus the "all sensors" entry */
  SensorListLength = blockSize/sizeof(HardwareMonitorType2_t);

  /* Print the temperature sensors current value */
  writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "Temperature Sensor Values:\n");
  for (i=0; i<SensorListLength; i++) {
    HardwareMonitorType2_t *sensor = &SensorList[i];
    if (sensor->SensorType == TEMPERATURE) {
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%-6s", "Temp.");
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%-10s:", sensor->SensorName);
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%4d ", sensor->sensorMeasurement/10);
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "\n");
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%-10s", "TempHigh.");
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%-10s:", sensor->SensorName);
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%4d ", sensor->highAlarmLimit/10);
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "\n");
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%-10s", "TempLow.");
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%-10s:", sensor->SensorName);
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%4d ", sensor->lowAlarmLimit/10);
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "\n");
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%-10s", "TempAlarm.");
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%-10s:", sensor->SensorName);
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "%4d ", sensor->SensorAlarm);
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "\n");
    }
  }
  WriteToLog(LOG_INFO, "%s\n", writeBuffer);

  return result;
}

uint32_t _GetHWMonInfo (void *arglist)
{
  uint8_t  *argptr = arglist;
  uint8_t  card;

  card      = get8incr(&argptr);

  return GetHWMonInfo(card);
}

uint32_t GetTimeSyncInfo (uint8_t card)
{
  handle_t                     cardHandle=NULL;
  TimeSyncConfigurationType2_t config2;
  TimeSyncConfigurationType3_t config3;
  char                         writeBuffer[1024];
  uint32_t                     writePos=0;
  uint32_t                     result=0;

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  result = NTCI_GetCardConfigurationDetails(cardHandle,
                                            0 /* Not needed. */,
                                            STID_TIMESYNC_CONFIGURATION_TYPE3,
                                            &config3);
  if(result != NTCI_ERRCODE_SUCCESS) {
    result = NTCI_GetCardConfigurationDetails(cardHandle,
                                              0 /* Not needed. */,
                                              STID_TIMESYNC_CONFIGURATION_TYPE2,
                                              &config2);
    if(result != NTCI_ERRCODE_SUCCESS) {
      WriteToLog(LOG_ERROR, "Failed reading time synchronization status. %i\n", result);
      return result;
    }
    writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "TimeSync Status:\n");
    if (config2.timesyncConfigMaster) {
      (void)WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "Adapter is time synchronization Master.\n");
    }
    else {
      writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "Adapter is time synchronization Slave.\n");
      writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos,
                                "Master present: %s\nMaster in synchronization: %s\n",
                                (config2.link_active_status ? "Yes" : "No"),
                                (config2.slave_in_sync ? "Yes" : "No"));

      if (config2.GPS_Box_is_Master) {
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "GPS Box is master\n");
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "GPS Box PPS is %s \n",
                                  (config2.GPS_Box_PPS_Ok ? "OK" : "not OK"));
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "GPS Box is %s synchronization\n",
                                  (config2.GPS_Box_In_sync ? "in" : "not in"));
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "GPS Box serial stream is %s\n",
                                  (config2.GPS_Box_Serial_Stream_OK ? "OK" : "not OK"));
      }
      if (config2.PTA_is_Master) {
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "PTA is master\n");
      }

      if (config2.OS_Mode_Sync_enabled) {
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "OS mode %s enabled\n",
                                  (config2.OS_Mode_Sync_enabled ? "is" : "is not"));
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "OS mode is %s synchronization\n",
                                  (config2.OS_Mode_Sync_enabled ? "in" : "not in"));
      }
      (void)WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "Master Id code: %d\n",
                          config2.Master_ID_Code);
    }
  }
  else {
    writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "TimeSync Status:\n");
    if (config3.timesyncConfigMaster) {
      writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "Adapter is time synchronization Master.\n");
    }
    else {
      writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "Adapter is time synchronization Slave.\n");
      writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos,
                                "Master present: %s\nMaster in synchronization: %s\n",
                                (config3.link_active_status ? "Yes" : "No"),
                                (config3.slave_in_sync ? "Yes" : "No"));

      if (config3.GPS_Box_is_Master) {
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "GPS Box is master\n");
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "GPS Box PPS is %s \n",
                                  (config3.GPS_Box_PPS_Ok ? "OK" : "not OK"));
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "GPS Box is %s synchronization\n",
                                  (config3.GPS_Box_In_sync ? "in" : "not in"));
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "GPS Box serial stream is %s\n",
                                  (config3.GPS_Box_Serial_Stream_OK ? "OK" : "not OK"));
      }
      if (config3.PTA_is_Master) {
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "PTA is master\n");
      }

      if (config3.OS_Mode_Sync_enabled) {
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "OS mode %s enabled\n",
                                  (config3.OS_Mode_Sync_enabled ? "is" : "is not"));
        writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "OS mode is %s synchronization\n",
                                  (config3.OS_Mode_Sync_enabled ? "in" : "not in"));
      }
      writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "Master Id code: %d\n",
                                config3.Master_ID_Code);
    }
    writePos += WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "External time: 0x%llX\n",
                              config3.External_Time);
    (void)WriteToBuffer(&writeBuffer[writePos], sizeof(writeBuffer)-writePos, "Adapter time:  0x%llX\n",
                        config3.Adapter_Time);
  }
  WriteToLog(LOG_INFO, "%s\n", writeBuffer);

  return result;
}

uint32_t _GetTimeSyncInfo (void *arglist)
{
  uint8_t  *argptr = arglist;
  uint8_t  card;

  card      = get8incr(&argptr);

  return GetTimeSyncInfo(card);
}
