// $Id: toolslib.c 25600 2013-09-19 07:40:19Z mh $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * toolslib.c : Napatech tools and test functions.
 *
 * NOTE: The code here is not to be used by customer applications, it is
 *       not an abstraction of the NTCI library and will change from between
 *       releases.
 */

#ifdef WIN32
// XENA and Sockets
#include <WINSOCK2.H>
#include <ws2tcpip.h>
#include <stdlib.h>
#include <stdio.h>
#include <process.h>
#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "platformwin.h"
#else
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdarg.h>
#include <unistd.h>
#include <fcntl.h>
#include <libgen.h>
#include <errno.h>

//xena defines for sockets
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <pthread.h>

// end


#endif
#include <limits.h>	
 
#include <NTCommonInterface.h>
#include <NTCI_cardconfiguration.h>
#include <NTCI_capabilities.h>
#include <NTCI_vpdaccess.h>
#include <NTCI_packetfeed.h>

/* These headers are needed for accessing the capabilities interface: */
#include <capabilitiesstructure.h>
#include <vpdstructure.h>
#include <packetfeedstructure.h>
#include <packetdescriptor.h>

#include "toolslib.h"
#include "../../config/napaversions.h"

/* Do not use the synchronous option (slow) for the log file */
#define GO_FAST 1

pid_t parrent_pid=-1;

/* Function return values */
uint64_t TestFrameLibFuncRetVal;
char TestFrameLibFuncRetValStr[USERVARSTRMAXLEN];
int g_AdapterNumber = -1;

#ifdef WIN32
WSADATA wsaData;
SOCKET ConnectSocket = INVALID_SOCKET;
struct addrinfo *result = NULL,
				*ptr = NULL,
				hints;
#else
int sockfd;
struct sockaddr_in address;
#endif



/* Windows/Linux conversion function */
#ifdef WIN32
#define FILE_FLAGS1 GENERIC_WRITE
#define FILE_RIGHTS1 OPEN_ALWAYS
#define FILE_FLAGS2 GENERIC_WRITE
#define FILE_RIGHTS2 CREATE_ALWAYS
#define FILE_FLAGS3 GENERIC_READ
#define FILE_RIGHTS3 OPEN_EXISTING
#else
#if defined(GO_FAST)
#define FILE_FLAGS1 O_WRONLY | O_CREAT | O_APPEND
#else
#define FILE_FLAGS1 O_WRONLY | O_CREAT | O_APPEND | O_SYNC
#endif
#define FILE_RIGHTS1 0666
#if defined(GO_FAST)
#define FILE_FLAGS2 O_WRONLY | O_CREAT | O_TRUNC
#else
#define FILE_FLAGS2 O_WRONLY | O_CREAT | O_TRUNC | O_SYNC
#endif
#define FILE_RIGHTS2 0666
#define FILE_FLAGS3 O_RDONLY
#define FILE_RIGHTS3 0666

#define fileopen open
#define filelseek lseek
#define fileclose close
#define fileread read
#define filewrite write
#define NT_HANDLE int
#endif


/*-------------------------------------------------------------------------*/
/**
  @brief    Skip blanks until the first non-blank character.
  @param    s   String to parse.
  @return   Pointer to char inside given string.

  This function returns a pointer to the first non-blank character in the
  given string.
 */
/*--------------------------------------------------------------------------*/
static char * strskp (char * s)
{
    char * skip = s;
    if (s==NULL) return NULL ;
    while (isspace((int)*skip) && *skip) skip++;
    return skip ;
}

/*-------------------------------------------------------------------------*/
/**
  @brief    Converts ASCII number string (hex or decimal) to integer.
  @param    str     ASCII string
  @return   Integer value. 

 */
/*--------------------------------------------------------------------------*/
uint64_t AsciiToInt (char *str)
{
  int      i=0;
  char     ch;  
  uint64_t result=0;
  uint8_t  base=10;

  if(str==NULL) {
    return 0;
  }
  
  /* Skip leading non-digit characters */
  while ( str[0] && ((str[0] < '0') || (str[0] >'9')) ) {
    str++;
  }

  if (!str[0]) {
    return 0;
  }

  while ( str[i] != '\0' && str[i] != '\n'  && str[i] != '\r' && !isspace((int)str[i]) ) {    
    ch=toupper(str[i++]);
    if ((ch >= '0') && (ch <= '9')) {
      result = result * base + (ch - '0');
      } else {
        if ((ch >= 'A') && (ch <= 'F')) {
          result = result * base + (ch  - 'A' + 10);
        } else {
          if (ch == 'X') {
            /* This is a hex number */
            result = 0;
            base = 16;
          } else {
            break;
          }
        }
      }
  }
  return result;
}

int ParseYesNo(char *str) /* Return 0 if No, 1 is Yes and -1 if neither. */
{
  if(strstr("Yes", str)) {
    return 1;
  } else if(strstr("No", str)) {
    return 0;
  } else {
    return -1;
  }
  return 0;
}

static uint32_t ResultBufferStop(void);


char cardInfo[100];
char errorBuffer[1000];

/* some of the basic card capabilities that may be useful elsewhere
 * in the program */
t_CardDetails CardDetails;

/* handle to the driver */

handle_t CardHandle;

static handle_t CardHandleTable[MAX_CARDS];
static int userSelection=0;

uint32_t OpenAnyCard(uint32_t cardType,   /* What cardtype to open. 0=any, 0x80000000=XSeries, 0x400000000=NTSeries */
                     uint8_t  cardNumber, /* What card number to open 0, 1.                      */
                     uint8_t  cardIndex)  /* What index should be used to refer to it 1..3       */
{
  handle_t card=NULL;

  // validate card number and index - before we use them as index into an array
  if( cardIndex >= MAX_CARDS || cardNumber >= MAX_CARDS ) {
    WriteToLog(LOG_ERROR, "Adapter %u: out of range: (%d-%d)\n", cardNumber, 0, MAX_CARDS-1);
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  /* Open one card only. NT or X or ANY. */
  if(CardHandleTable[cardIndex]!=NULL) {
    WriteToLog(LOG_ERROR, "CardIndex %d is already in use\n", cardIndex);
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }
  
  if((card=NTCI_OpenCard(cardType, cardNumber))) {
    CardHandleTable[cardIndex] = card;
  } else {
    return NTCI_ERRCODE_NO_DEVICE_OPEN;
  }
  return NTCI_ERRCODE_SUCCESS;  
}

uint32_t OpenCaptureCard(uint32_t cardType,   /* What cardtype to open. 0=any, 0x80000000=XSeries, 0x400000000=NTSeries */
                         uint8_t  cardNumber, /* What card number to open 0, 1.                      */
                         uint8_t  cardIndex)  /* What index should be used to refer to it 1..3       */
{
  CardServicesCapabilitiesType1_t services;
  uint32_t num=0, feedCards=0;
  handle_t card=0;
  uint32_t result=0;

  // validate card number and index - before we use them as index into an array
  if(cardIndex >= MAX_CARDS || cardNumber >= MAX_CARDS ) {
    WriteToLog(LOG_ERROR, "Adapter %u: out of range: (%d-%d)\n", cardNumber, 0, MAX_CARDS-1);
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  if (g_AdapterNumber != -1)
    cardNumber = g_AdapterNumber;

  /* Open one card only. NT or X or ANY. */
  if(CardHandleTable[cardIndex]!=NULL) {
    WriteToLog(LOG_ERROR, "CardIndex %d is already in use\n", cardIndex);
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  for(;;) {
    if((card=NTCI_OpenCard(cardType, num))) {
      /* A card has been found. Check if it has packetfeeds support. */
      if((result = NTCI_GetCapabilities(card, CAPID_SERVICES, NOT_SUB_CAPABILITIES,
                                        STID_CARD_SERVICES_CAPABILITIES_TYPE1,
                                        &services)) != NTCI_ERRCODE_SUCCESS) {
        WriteToLog(LOG_INFO, "Failed to read card services capability.\n");
        return result;
      }
      if(!services.packetFeedSupported) {
        /* Skip card it has no packetfeed support. */
      } else {
        /* The card has packetfeed support. */
        if(feedCards==cardNumber) {
          /* We have found the right card. */
          break;
        }
        feedCards++;
      }
      /* Check the next. */
      num++;
      NTCI_CloseCard(card);
      card=NULL;
    } else {
      break;
    }
  }
  if(card) {
    CardHandleTable[cardIndex] = card;
  } else {
    return NTCI_ERRCODE_NO_DEVICE_OPEN;
  }
  return NTCI_ERRCODE_SUCCESS;
}

static uint32_t DetectCard(handle_t cardHandle) {
  uint32_t result=0;
  CardHardwareVersionCapabilitiesType3_t data3;
  uint64_t fpgaId=0;

  if((result = NTCI_GetCapabilities(cardHandle,
    CAPID_VERSION,
    NOT_SUB_CAPABILITIES,
    STID_CARD_HARDWARE_VERSION_CAPABILITIES_TYPE3,
    &data3)) != NTCI_ERRCODE_SUCCESS) {
      return CARD_UNKNOWN;
  }
  if(data3.AdapterCard.dwInfoType==1) {
    fpgaId = (data3.AdapterCard.u.InfoType1.qwFpgaId&0xFFFF0000)>>16;
  }else if(data3.AdapterCard.dwInfoType==2) {
    fpgaId = (data3.AdapterCard.u.InfoType2.qwFpgaId&0xFFFF0000)>>16;
  }else if(data3.AdapterCard.dwInfoType==3) {
    fpgaId = (data3.AdapterCard.u.InfoType3.qwFpgaId&0xFFFF0000)>>16;
  }

  switch((fpgaId&0xFFFF0000) >> 16) {
  case 9008: return NTSERIES_NT20X;
  case 9014: return NTSERIES_NT20E_CAP;
  case 9114: return NTSERIES_NT20E_INL;
  case 9015: return NTSERIES_NT4E_CAP;
  case 9115: return NTSERIES_NT4E_INL;
  case 9017: return NTSERIES_NT4E_STD_CAP;
  case 9117: return NTSERIES_NT4E_STD_INL;
  case 9019: return NTSERIES_NT4PORT;
  case 9220: return NTSERIES_NT20E2;
  case 9221: return NTSERIES_NT40E2_4;
  case 9222: return NTSERIES_NT40E2_1;
  default: return CARD_UNKNOWN; 
  }
}

uint32_t OpenNotExcludedCard(uint32_t cardType,    /* What cardtype to open. 0=any, 0x80000000=XSeries, 0x400000000=NTSeries */
                             uint8_t  cardNumber,  /* What card number to open 0, 1.                      */
                             uint8_t  cardIndex,   /* What index should be used to refer to it 1..3       */
                             uint32_t excludeMask) /* Mask of cards to exclude                            */
{
  uint32_t num=0, cardsOK=0;
  handle_t card=0;

  // validate card number and index - before we use them as index into an array
  if(cardIndex >= MAX_CARDS || cardNumber >= MAX_CARDS ) {
    WriteToLog(LOG_ERROR, "Adapter %u: out of range: (%d-%d)\n", cardNumber, 0, MAX_CARDS-1);
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  if (g_AdapterNumber != -1)
    cardNumber = g_AdapterNumber;

  /* Open one card only. NT or X or ANY. */
  if(CardHandleTable[cardIndex]!=NULL) {
    WriteToLog(LOG_ERROR, "CardIndex %d is already in use\n", cardIndex);
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  for(;;) {
    if((card=NTCI_OpenCard(cardType, num))) {
      uint32_t localCardType;
      /* A card has been found. Check that it is not on the exclude list. */
      localCardType = DetectCard(card);

      if(localCardType & excludeMask) {
        /* Skip card as it is excluded. */
      } else {
        /* The card is nit excluded. */
        if(cardsOK==cardNumber) {
          /* We have found the right card. */
          break;
        }
        cardsOK++;
      }
      /* Check the next. */
      num++;
      NTCI_CloseCard(card);
      card=NULL;
    } else {
      break;
    }
  }
  if(card) {
    CardHandleTable[cardIndex] = card;
  } else {
    return NTCI_ERRCODE_NO_DEVICE_OPEN;
  }
  return NTCI_ERRCODE_SUCCESS;
}


uint32_t _OpenCaptureCard(void *arglist)
{
  uint8_t  *argptr = arglist;
  uint32_t cardType;
  uint8_t  cardNumber;
  uint8_t  cardIndex;

  cardType   = get32incr(&argptr);
  cardNumber = get8incr(&argptr);
  cardIndex  = get8incr(&argptr);

  return OpenCaptureCard(cardType, cardNumber, cardIndex);
}

uint32_t _OpenAnyCard(void *arglist)
{
  uint8_t  *argptr = arglist;
  uint32_t cardType;
  uint8_t  cardNumber;
  uint8_t  cardIndex;

  cardType   = get32incr(&argptr);
  cardNumber = get8incr(&argptr);
  cardIndex  = get8incr(&argptr);

  return OpenAnyCard(cardType, cardNumber, cardIndex);
}

uint32_t CloseCard(uint8_t cardIndex)
{
  uint32_t result=0;

  // validate card index/number - before we use them as index into an array
  if(cardIndex >= MAX_CARDS ) {
    WriteToLog(LOG_ERROR, "Adapter %u: out of range: (%d-%d)\n", cardIndex, 0, MAX_CARDS-1);
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  /* Close a specific card. */
  if(CardHandleTable[cardIndex] != NULL) {
    result = NTCI_CloseCard(CardHandleTable[cardIndex]);
    CardHandleTable[cardIndex] = NULL;
    return result;
  }
  return NTCI_ERRCODE_CARD_HANDLE_NOT_VALID;
}

uint32_t _CloseCard(void *arglist)
{
  uint8_t  *argptr = arglist;
  uint8_t  cardIndex;

  cardIndex  = get8incr(&argptr);

  return CloseCard(cardIndex);
}

void ExitProgram(void)
{
  CardShutdown(-1);
}

void CardShutdown(int returnStatus)
{
  uint32_t result;
 if(CardHandle != NULL) {
    if(returnStatus != 0) {
      NTCI_ExplainLastCardError(CardHandle, errorBuffer, sizeof(errorBuffer));
      printf("\nThe program exited erroneously because of [%s]\n", errorBuffer);
    }
  }

  {
  uint8_t i;
  for(i=0; i<(sizeof(CardHandleTable)/sizeof(handle_t)); i++) {
    if(CardHandleTable[i] != NULL) {
      result = NTCI_CloseCard(CardHandleTable[i]);
      if (result != NTCI_ERRCODE_SUCCESS) {
        fprintf(stderr, "%s", "Failed to shut down card\n");
      }
      CardHandleTable[i] = NULL;
    }
  }
  CardHandle = NULL;
  }
}

void DisplayProgramHeader (char *programName, ePrgType programType) {
  char *version_str=NULL, *package_str=(char*)"\0";
  char *package_timestamp=(char*)"\0";
  int  i;
  
  printf("\n%s ", basename(programName));

  switch (programType) {
  case PRG_DRVTOOL:
    package_str = (char*)RELEASE_PACKAGE_NAME;
    package_timestamp = (char*)RELEASE_PACKAGE_TIMESTAMP;
    break;
  case PRG_APPLICATION:
    package_str = (char*)APPLICATIONS_PACKAGE_NAME;
    package_timestamp = (char*)APPLICATIONS_PACKAGE_TIMESTAMP;
    break;
  case PRG_TOOL:
    package_str = (char*)TOOLS_PACKAGE_NAME;
    package_timestamp = (char*)TOOLS_PACKAGE_TIMESTAMP;
    break;
  }

  /*
   * Extract the version string from the package name string.
   * (Version starts at the first digit in the package string).
   */
  i = 0;
  while (package_str[i] != '\0') {
    if ((package_str[i] <= '9') && (package_str[i] >= '0')) {
      version_str = & package_str[i];
      break;
    }
    i++;
  }
  if (version_str == NULL) {
    version_str = (char*)"Unknown";
  }
  printf("(v. %s - %s)\n", version_str, package_timestamp);

  /* Print separator line */
  for (i=0; i<78; ++i)
      putchar('=');
  putchar('\n');
  putchar('\n');  
}

void DisplayProgramFooter (void) {
  printf("\n");
}

/************************************************************************/
/* cardTypeMask:                                                        */
/* = 0 (FALSE) => Use all cards                                         */
/* = 1 (TRUE)  => Use only capture cards (Supports feeds)               */
/* > 1         => Exclude mask. Exclude cards that are in the mask.     */
/*                                                                      */
/* Mask values:                                                         */
/* XSERIES_XD, XSERIES_XL2, NTSERIES_NT20X, NTSERIES_NT20E_INL,         */
/* NTSERIES_NT20E_CAP, NTSERIES_NT4E_CAP, NTSERIES_NT4E_INL,            */
/* NTSERIES_NT4E_STD_CAP, NTSERIES_NT4E_STD_INL, NTSERIES_NT4PORT       */
/************************************************************************/

int InitApplication(int *argc, char *argv[], uint32_t cardTypeMask, uint32_t *cardIndex,
                    CommandLineOptions_t *tools_options, char *tool_description,
                    char *output_description)
{
  bool_t boFoundArg=FALSE;
  bool_t boForceUseCard=FALSE;
  uint32_t cardType=NTCI_CARD_TYPE_ANY;
  bool_t boFailSettingRequired=FALSE;
  int cardToUse=0;
  int i=0, nbrCards=0, arg=0;
  VPDImageType4_t vpdData4;
  VPDImageType5_t vpdData5;
  CommandLineOptions_t *options=NULL;
  int result, cp=0, aLen=0;
  char *cc=NULL;
  unsigned int cmpLen=0;
    
  parrent_pid=getpid();

  memset(CardHandleTable, 0x00, sizeof(CardHandleTable));

  /* Star by clearing all result pointers in the tool_options. */
  options=tools_options;
  while(options->option) {
    *options->result = NULL;
    options++;
  }

  /* Skip argv[0], it is the program name. */
  for(arg=1; arg<*argc; arg++) {
    boFoundArg=FALSE;
    if(argv[arg][0] != '-') {
      printf("option %s doesn't have a \"-\" to indicate that it is an option.\n", argv[arg]);
      PrintUsage(argv[0], tools_options, tool_description, output_description);
      return -1;
    }
    /* Look for general options. */
    if(!strncasecmp("-adapter", argv[arg], 8)) {
      if(strlen(argv[arg]) == (strlen("-adapter"))) {
        /* In this case we expect that the option value is the next argument. */
        if((arg+1) <*argc) {
          if(argv[arg+1][0] == '-') {
            printf("Option '%s' doesn't have any assignment.\n", "adapter");
            return -1;
          }
          /* Ok, we can assume that the argument is the assignment. */
          arg++; /* Skip to next argument. */
          cc=argv[arg];
          aLen=strlen(argv[arg]);
        }
      } else {
        aLen=strlen(&argv[arg][strlen("-adapter")]);
        cc=&argv[arg][strlen("-adapter")];
      }
      for(cp=0;cp<aLen;cp++) {
        if(cc[cp]=='=') {
          continue;
        } else {
          char *endptr=NULL;
          /* Check for =ntxc has been used. */          
          cardToUse = strtol(&cc[cp], &endptr, 10);
          if(endptr == &cc[cp]) {
            /* We have ASCII chars in front. */
            if(((cc[cp] == 'n') || (cc[cp] == 'N')) &&
               ((cc[cp+1] == 't') || (cc[cp+1] == 'T')) &&
               ((cc[cp+2] == 'x') || (cc[cp+2] == 'X')) &&
               ((cc[cp+3] == 'c') || (cc[cp+3] == 'C'))) {
              if((cc[cp+4] < '0') || (cc[cp+4] > '9')) {
                fprintf(stderr, ">>> Error: Unknown adapter string: %s\n", &cc[cp]);
                exit(-1);                
              }
              cardType = NTCI_ANY_NAPATECH_CARD_TYPE;
              cardToUse = strtol(&cc[cp+4], &endptr, 10);
            } else {
              fprintf(stderr, ">>> Error: Unknown adapter string: %s\n", &cc[cp]);
              exit(-1);
            }
          }
        }        
        boForceUseCard=TRUE;
        boFoundArg=TRUE;
        break;
      }
      if(boFoundArg) {
        continue;
      }
    }
    if(!strcasecmp("-help", argv[arg])) {
      PrintUsage(argv[0], tools_options, tool_description, output_description);
      return -1;
    } else if (!strcasecmp("-", argv[arg])) {
      printf("Unknown option: \"%s\"\n", argv[arg]);
      PrintUsage(argv[0], tools_options, tool_description, output_description);
      return -1;
    } else {
      /* Ok look for application specific options. */
      options=tools_options;
      while(options->option) {
        /* Check for a '=' */
        cc=argv[arg];
        for(cmpLen=0;(cmpLen<strlen(&argv[arg][1])) && (cc[cmpLen] != '=');cmpLen++);
        if(cmpLen<strlen(&argv[arg][1])) {
          /* We found a '='. */
          cmpLen = cmpLen-1;
        }
        if(!strncasecmp(options->option, &argv[arg][1], cmpLen)) {
          if(*options->result != NULL) {
            printf("Option '%s' is already set.\n", options->option);
            return -1;
          } else if(options->mode==2) {
            if(strlen(options->option) != strlen(&argv[arg][1])) {
              printf("Option '%s' doesn't match %s.\n", options->option, &argv[arg][1]);
              return -1;
            } else {
              *options->result=(char*)"1"; /* This type of options are booleans. */
              boFoundArg=TRUE;                
            }
          } else {
            if(strlen(argv[arg]) == (strlen(options->option)+1)) {
              /* In this case we expect that the option value is the next argument. */
              if((arg+1) <*argc) {
                if(argv[arg+1][0] == '-') {
                  printf("Option '%s' doesn't have any assignment.\n", options->option);
                  return -1;
                }
                /* Ok, we can assume that the argument is the assignment. */
                arg++; /* Skip to next argument. */
                cc=argv[arg];
                aLen=strlen(argv[arg]);
              }
            } else {
              cc=&argv[arg][strlen(options->option)+1];
              aLen=strlen(argv[arg])-(strlen(options->option)+1);
            }
            /* Strip '=' if present. */
            for(cp=0;cp<aLen;cp++) {
              if(cc[cp]=='=') {
                continue;
              } else {
                break;
              }
            }
            *options->result=&cc[cp];             
            boFoundArg=TRUE;
            break;
          }
        }      
        options++;
      }
    }
    if(boFoundArg==FALSE) {
      printf("Unknown option: \"%s\"\n", argv[arg]);
      PrintUsage(argv[0], tools_options, tool_description, output_description);
      return -1;
    }
  }
  /* Check that all required parameters has been set. */
  options=tools_options;
  while(options->option) {
    if(options->mode==1) {
      if(*options->result == NULL) {
        printf("Required parameter: %s is not set.\n", options->option);
        boFailSettingRequired=TRUE;
      }
    }
    options++;
  }
  if(boFailSettingRequired) {
    PrintUsage(argv[0], tools_options, tool_description, output_description);
    return -1;
  }
    
  atexit(ExitProgram);    
  if(boForceUseCard==TRUE) {
    if(cardTypeMask == TRUE) {
      if(OpenCaptureCard(cardType, cardToUse, cardToUse) != NTCI_ERRCODE_SUCCESS) {
        WriteToLog(LOG_ERROR, "Could not find adapter %d.\n", cardToUse);
        exit(-1);
      }
    } else if(cardTypeMask == 0) {
      if(OpenAnyCard(cardType, cardToUse, cardToUse) != NTCI_ERRCODE_SUCCESS) {
        WriteToLog(LOG_ERROR, "Could not find adapter %d.\n", cardToUse);
        exit(-1);
      }      
    } else {
      if(OpenNotExcludedCard(cardType, cardToUse, cardToUse, cardTypeMask) != NTCI_ERRCODE_SUCCESS) {
        WriteToLog(LOG_ERROR, "Could not find adapter %d.\n", cardToUse);
        exit(-1);
      }
    }
    /* Simulate a user selection. */
    userSelection=cardToUse;
  } else {  
    /* Use must choose which card to use. Close the remaining cards */
    for(i=0; i<MAX_CARDS; i++) {
      if(cardTypeMask == TRUE) {
        if(OpenCaptureCard(cardType, i, i) != NTCI_ERRCODE_SUCCESS) {
          if(nbrCards==0) {
            WriteToLog(LOG_ERROR, "No adapters could be found.\n");
            return NTCI_ERRCODE_NO_DEVICE_OPEN;
          }
        } else {
          nbrCards++;
        }
      } else if(cardTypeMask == 0) {
        if(OpenAnyCard(cardType, i, i) != NTCI_ERRCODE_SUCCESS) {
          if(nbrCards==0) {
            WriteToLog(LOG_ERROR, "No adapters could be found.\n");
            return NTCI_ERRCODE_NO_DEVICE_OPEN;
          }
        } else {
          nbrCards++;
        }
      } else {
        if(OpenNotExcludedCard(cardType, i, i, cardTypeMask) != NTCI_ERRCODE_SUCCESS) {
          if(nbrCards==0) {
            WriteToLog(LOG_ERROR, "No adapters could be found.\n");
            return NTCI_ERRCODE_NO_DEVICE_OPEN;
          }
        } else {
          nbrCards++;
        }
      }
    }
    if(nbrCards != 1) {
      uint32_t key;
      for(i=0; i<nbrCards; i++) {
        if(NTCI_ReadVitalProductData(CardHandleTable[i], 0,STID_VPD_IMAGE_TYPE4 , &vpdData4) != 0) {
          if(NTCI_ReadVitalProductData(CardHandleTable[i], 0,STID_VPD_IMAGE_TYPE5 , &vpdData5) != 0) {
            sprintf(cardInfo, "Unknown");
          } else {
            CardHardwareVersionCapabilitiesType3_t data3;
            uint64_t fpgaId=0;

            if((result = NTCI_GetCapabilities(CardHandleTable[i],
                                              CAPID_VERSION,
                                              NOT_SUB_CAPABILITIES,
                                              STID_CARD_HARDWARE_VERSION_CAPABILITIES_TYPE3,
                                              &data3)) != NTCI_ERRCODE_SUCCESS) {
              return CARD_UNKNOWN;
            }
            if(data3.AdapterCard.dwInfoType==1) {
              fpgaId = (data3.AdapterCard.u.InfoType1.qwFpgaId&0xFFFF0000)>>16;
            }else if(data3.AdapterCard.dwInfoType==2) {
              fpgaId = (data3.AdapterCard.u.InfoType2.qwFpgaId&0xFFFF0000)>>16;
            }else if(data3.AdapterCard.dwInfoType==3) {
              fpgaId = (data3.AdapterCard.u.InfoType3.qwFpgaId&0xFFFF0000)>>16;
            }
            if(fpgaId == 9220) {
              snprintf(cardInfo, 100, "NT: NT20E2 %s", vpdData5.mainBoardType);;
            } else if(fpgaId == 9221) {
              snprintf(cardInfo, 100, "NT: NT40E2_4 %s", vpdData5.mainBoardType);;
            } else if(fpgaId == 9222) {
              snprintf(cardInfo, 100, "NT: NT40E2_1 %s", vpdData5.mainBoardType);;
            } 
          }
        } else {
          snprintf(cardInfo, 100, "NT: %s", vpdData4.productName);
        }        
        printf("%d: %s\n", i, cardInfo);      
      }
      
      for(;;) {
        printf("\rPlease choose an adapter (%d-%d): ", 0, nbrCards-1);
        key = getchar();
        if ((toupper(key) >= toupper('0')) && (toupper(key) < toupper('0'+nbrCards))) {
          userSelection = toupper(key)-'0';
          break;
        }
      }
    }
  }
  
  CardHandle = CardHandleTable[userSelection];
  if(CardHandle == NULL) {
    WriteToLog(LOG_ERROR, "Could not find any adapters.\n");
    exit(-1);
  }
  if(NTCI_ReadVitalProductData(CardHandle, 0,STID_VPD_IMAGE_TYPE4 , &vpdData4) != 0) {
    if(NTCI_ReadVitalProductData(CardHandle, 0,STID_VPD_IMAGE_TYPE5 , &vpdData5) != 0) {
      sprintf(cardInfo, "Unknown");
    } else {
      CardHardwareVersionCapabilitiesType3_t data3;
      uint64_t fpgaId=0;

      if((result = NTCI_GetCapabilities(CardHandle,
                                        CAPID_VERSION,
                                        NOT_SUB_CAPABILITIES,
                                        STID_CARD_HARDWARE_VERSION_CAPABILITIES_TYPE3,
                                        &data3)) != NTCI_ERRCODE_SUCCESS) {
        return CARD_UNKNOWN;
      }
      if(data3.AdapterCard.dwInfoType==1) {
        fpgaId = (data3.AdapterCard.u.InfoType1.qwFpgaId&0xFFFF0000)>>16;
      }else if(data3.AdapterCard.dwInfoType==2) {
        fpgaId = (data3.AdapterCard.u.InfoType2.qwFpgaId&0xFFFF0000)>>16;
      }else if(data3.AdapterCard.dwInfoType==3) {
        fpgaId = (data3.AdapterCard.u.InfoType3.qwFpgaId&0xFFFF0000)>>16;
      }
      if(fpgaId == 9220) {
        snprintf(cardInfo, 100, "NT: NT20E2 %s", vpdData5.mainBoardType);;
      } else if(fpgaId == 9221) {
        snprintf(cardInfo, 100, "NT: NT40E2_4 %s", vpdData5.mainBoardType);;
      } else if(fpgaId == 9222) {
        snprintf(cardInfo, 100, "NT: NT40E2_1 %s", vpdData5.mainBoardType);;
      } 
    }
  } else {
    snprintf(cardInfo, 100, "NT: %s", vpdData4.productName);;
  }
  
  for(i=0; i<4; i++) {
    if((i != userSelection) && (CardHandleTable[i] != NULL)) {
      NTCI_CloseCard(CardHandleTable[i]);
      CardHandleTable[i] = NULL;
    }
  }
  
  *cardIndex = userSelection;
  return 0;
} 

/* Initialise the CardDetails structure */
int CardInitCardDetails(void)
{
  int result;
  t_CardDetails cardDetails;

  result = GetCardDetails(CardHandle, &cardDetails);
  if (result != 0) {
    return -1;
  }

  // move to global structure for backwards compatibility
  CardDetails.CardType = cardDetails.CardType;
  CardDetails.AllChannelsMask = cardDetails.AllChannelsMask;
  CardDetails.NumChannels = cardDetails.NumChannels;

  return 0;
}

//
//
//
int GetCardDetails(handle_t cardHandle, t_CardDetails* cardDetails)
{
  int result;
  uint32_t i;
  CardHardwareCapabilitiesType1_t cardCapabilitiesType1;

  // find the card hardware capabilities (type 1)
  result = NTCI_GetCapabilities(cardHandle,
                                CAPID_HARDWARE,
                                NOT_SUB_CAPABILITIES,
                                STID_CARD_HARDWARE_CAPABILITIES_TYPE1,
                                &cardCapabilitiesType1);

  if (result != NTCI_ERRCODE_SUCCESS) {
    fprintf(stderr, "Error finding adapter capabilities (type 1): %i\n", result);
    return -1;
  }

  // Fill in some CardDetails fields
  cardDetails->CardType = cardCapabilitiesType1.CardType;
  cardDetails->AllChannelsMask = cardCapabilitiesType1.totalChannelsBitmask;

  /* Convert the channels mask into a count of the number
   * of channels, from the number of bits set.
   *
   * Start with the full bitmask, and shift it down by one
   * bit with every iteration, incrementing the channels count
   * whenever the lowest bit is set. Keep going until no more
   * bits are set */

  cardDetails->NumChannels = 0;
  i = cardDetails->AllChannelsMask;

  while (i != 0) {
    if (i & 0x1) {
      ++cardDetails->NumChannels;
    }
    i >>= 1;
  }

  return 0;
}


//
//
//
int ValidateCardDetailChannelBitmask(t_CardDetails* cardDetails, uint32_t cardChannelBitmask)
{
  if (cardChannelBitmask) {
    if ( cardDetails != NULL) {
      if ( (cardChannelBitmask & cardDetails->AllChannelsMask) != cardChannelBitmask ) {
        fprintf(stdout, ">>> Error: Adapter channelbitmask (0x%02x) references channels outside the available adapter channels (0x%02x).\n",
                cardChannelBitmask, cardDetails->AllChannelsMask);
        return -1;
      }
    } else {
      fprintf(stdout, ">>> Error: invalid adapter card-details\n");
      return -1;
    }
  }
  return 0;
}

//
// simple wrapper function around GetCardDetails() and ValidateCardDetailChannelBitmask()
//
int ValidateAdapterChannelBitmask(handle_t cardHandle, uint32_t cardChannelBitmask)
{
  int result;
  t_CardDetails cardDetails;

  // Get card details
  if ( (result = GetCardDetails(cardHandle, &cardDetails) != 0) ) {
    return -1;
  }

  // check channel bitmask
  if ( (result = ValidateCardDetailChannelBitmask(&cardDetails, cardChannelBitmask)) != 0) {
    return -1;
  }

  return 0;
}

//
//
//
int GetAdapterChannelBitmask(handle_t cardHandle, uint32_t* cardChannelBitmask)
{
  int result;
  t_CardDetails cardDetails;

  // Get card details
  if ( (result = GetCardDetails(cardHandle, &cardDetails) != 0) ) {
    return -1;
  }

  // return result
  if ( cardChannelBitmask != NULL ) {
    *cardChannelBitmask = cardDetails.AllChannelsMask;
  } else {
    return -1;
  }

  return 0;
}

//
//
//
#ifdef WIN32

/* Windows has 64-bit integer arithmetic built in */
#define DIV64(_dividend, _divisor, _quotient_type, _quotient, _remainder) do { \
    uint64_t quot = (uint64_t)(_dividend) / (uint64_t)(_divisor); \
	_remainder    = (_quotient_type)( (uint64_t)(_dividend) % (uint64_t)(_divisor) ); \
	_quotient     = (_quotient_type)(quot); \
} while (0)
#else
#include <sys/time.h>
#define DIV64(_dividend, _divisor, _quotient_type, _quotient, _remainder) do { \
	uint64_t __quot    = (uint64_t) _dividend; \
	uint32_t __mod     = _dividend % _divisor; \
        __quot /= _divisor; \
	_quotient          = (_quotient_type) __quot; \
	_remainder         = (_quotient_type) __mod; \
} while (0);
#endif

#define CONST64(__val) __val##ULL

struct nttimeval
{
  uint32_t tv_sec;
  uint32_t tv_usec;
};

static void ConvertNdisToPcap( uint64_t* ptime )
{
	uint64_t tmp = *ptime;
#ifdef WIN32
	uint64_t tmp2;
#else
	uint64_t tmp2 __attribute__((unused));  
#endif  
	struct nttimeval *punix_time = (struct nttimeval *)ptime;

	/* Convert from 100ns ticks from 01/01/1601 to ticks from 01/01/1970 */
	tmp -= UNIX_EPOCH_OFFSET_MAGIC;

	/* Convert u64 100ns ticks to u64 microseconds */
	DIV64(tmp, CONST64(10), uint64_t, tmp, tmp2);

	/* Convert from u64 microseconds to u32 seconds and u32 microseconds
	 * (places directly back into time pointer) */
	DIV64(tmp, CONST64(1000000), uint32_t, punix_time->tv_sec, punix_time->tv_usec);
}


static void ConvertNdisToNanoT( uint64_t* ptime )
{
	uint64_t tmp = *ptime;
	struct nttimeval *punix_time = (struct nttimeval *)ptime;

	/* Convert from 100ns ticks from 01/01/1601 to ticks from 01/01/1970 */
	tmp -= UNIX_EPOCH_OFFSET_MAGIC;

	/* Convert u64 100ns ticks to u64 nanoseconds */
	tmp *= CONST64(100);

	/* Convert from u64 microseconds to u32 seconds and u32 nanoseconds
	 * (places directly back into time pointer) */
	DIV64(tmp, CONST64(1000000000), uint32_t, punix_time->tv_sec, punix_time->tv_usec);
}


static void ConvertPcapToNdis( uint64_t* ptime )
{
	uint64_t tmp;
	struct nttimeval *punix_time = (struct nttimeval *)ptime;

	/* Convert from u32 seconds and u32 microseconds to u64 microseconds */
	tmp = (punix_time->tv_sec * CONST64(1000000)) + punix_time->tv_usec;

	/* Convert u64 microseconds to u64 100ns ticks */
	tmp *= CONST64(10);

	/* Convert from 100ns ticks from 01/01/1970 to ticks 01/01/1601 */
	tmp += UNIX_EPOCH_OFFSET_MAGIC;

	*ptime = tmp;
}


void ConvertSystemTickTime( uint64_t* ptime,
                            TimestampType_t fromTimestampType,
                            TimestampType_t toTimestampType )
{
	bool_t bStatus = TRUE;

	if ( fromTimestampType == toTimestampType ) {
		/* No conversion is required */
		return;
	}
	else if ( (fromTimestampType == TIMESTAMP_NATIVE) ||
            (toTimestampType   == TIMESTAMP_NATIVE) ) {
		/* Native timestamp conversions are not allowed */
		bStatus = FALSE;
	}
	else {
#ifdef WIN32    
		uint64_t mod;
#else
    uint64_t mod __attribute__((unused));
#endif
		switch ( fromTimestampType ) {
		case TIMESTAMP_NDIS:
			switch ( toTimestampType ) {
			case TIMESTAMP_PCAP:
				/* Convert from NDIS to PCAP */
				ConvertNdisToPcap(ptime);
				break;
			case TIMESTAMP_NATIVE_NDIS:
				/* Convert from NDIS 100ns res to native 10ns res */
				*ptime = *ptime * CONST64(10);
				break;
			case TIMESTAMP_NATIVE_UNIX:
				/* Convert NDIS to Native Unix */
				*ptime = (*ptime - UNIX_EPOCH_OFFSET_MAGIC) * CONST64(10);
				break;
			case TIMESTAMP_PCAP_NANOTIME:
				/* Convert from NDIS to PCAP NANOTIME */
				ConvertNdisToNanoT(ptime);
				break;
			default:
				bStatus = FALSE;
			}
			break;

		case TIMESTAMP_PCAP:
			switch ( toTimestampType ) {
			case TIMESTAMP_NDIS:
				/* Convert PCAP to NDIS */
				ConvertPcapToNdis(ptime);
				break;
			case TIMESTAMP_NATIVE_NDIS:
				/* Convert PCAP to NDIS */
				ConvertPcapToNdis(ptime);
				/* Convert NDIS to NATIVE_NDIS */
				*ptime = *ptime * CONST64(10);
				break;
			case TIMESTAMP_NATIVE_UNIX:
				/* Convert PCAP to Native Unix */
				ConvertPcapToNdis(ptime);
				*ptime = (*ptime - UNIX_EPOCH_OFFSET_MAGIC) * CONST64(10);
				break;
			case TIMESTAMP_PCAP_NANOTIME:
      {
        /* Convert from PCAP to PCAP NANOTIME */
        struct nttimeval *punix_time = (struct nttimeval *)ptime;
        punix_time->tv_usec *= 1000;
      }
      break;
			default:
				bStatus = FALSE;
			}
			break;
      
		case TIMESTAMP_NATIVE_NDIS:
			switch ( toTimestampType ) {
			case TIMESTAMP_NDIS:
				/* Convert from NATIVE_NDIS to NDIS */
				DIV64(*ptime, 10, uint64_t, *ptime, mod);
				break;
			case TIMESTAMP_PCAP:
				/* Convert from NATIVE_NDIS to PCAP */
				DIV64(*ptime, 10, uint64_t, *ptime, mod);
				/* Convert NDIS to PCAP */
				ConvertNdisToPcap(ptime);
				break;
			case TIMESTAMP_NATIVE_UNIX:
				/* Convert NATIVE_NDIS to Native Unix */
				*ptime = *ptime - (UNIX_EPOCH_OFFSET_MAGIC * CONST64(10));
				break;
			case TIMESTAMP_PCAP_NANOTIME:
				/* Convert from NATIVE_NDIS to NDIS */
				DIV64(*ptime, 10, uint64_t, *ptime, mod);
				/* Convert from NDIS to PCAP NANOTIME */
				ConvertNdisToNanoT(ptime);
				break;
			default:
				bStatus = FALSE;
			}
			break;

		case TIMESTAMP_NATIVE_UNIX:
			switch ( toTimestampType ) {
			case TIMESTAMP_NATIVE_NDIS:
				/* Convert from NATIVE_UNIX to Native NDIS */
				*ptime = *ptime + (UNIX_EPOCH_OFFSET_MAGIC * CONST64(10));
				break;
			case TIMESTAMP_NDIS:
				/* Convert from NATIVE_UNIX to NDIS */
				DIV64(*ptime, 10, uint64_t, *ptime, mod);
				*ptime = *ptime + UNIX_EPOCH_OFFSET_MAGIC;
				break;
			case TIMESTAMP_PCAP:
				/* Convert from NATIVE_UNIX to NDIS */
				DIV64(*ptime, 10, uint64_t, *ptime, mod);
				*ptime = *ptime + UNIX_EPOCH_OFFSET_MAGIC;
				/* Convert NDIS to PCAP */
				ConvertNdisToPcap(ptime);
				break;
			case TIMESTAMP_PCAP_NANOTIME:
				/* Convert from NATIVE_UNIX to NDIS */
				DIV64(*ptime, 10, uint64_t, *ptime, mod);
				*ptime = *ptime + UNIX_EPOCH_OFFSET_MAGIC;
				/* Convert from NDIS to PCAP NANOTIME */
				ConvertNdisToNanoT(ptime);
				break;
			default:
				bStatus = FALSE;
			}
			break;

		case TIMESTAMP_PCAP_NANOTIME:
			switch ( toTimestampType ) {
			case TIMESTAMP_NDIS:
      {
        /* Convert from PCAP NANOTIME to PCAP */
        struct nttimeval *punix_time = (struct nttimeval *)ptime;
        punix_time->tv_usec /= 1000;
        /* Convert PCAP to NDIS */
        ConvertPcapToNdis(ptime);
      }
      break;
			case TIMESTAMP_PCAP:
      {
        /* Convert from PCAP NANOTIME to PCAP */
        struct nttimeval *punix_time = (struct nttimeval *)ptime;
        punix_time->tv_usec /= 1000;
      }
      break;
			case TIMESTAMP_NATIVE_NDIS:
      {
        /* Convert from PCAP NANOTIME to PCAP */
        struct nttimeval *punix_time = (struct nttimeval *)ptime;
        punix_time->tv_usec /= 1000;
        /* Convert PCAP to NDIS */
        ConvertPcapToNdis(ptime);
        /* Convert NDIS to NDIS_NATIVE */
        *ptime = *ptime * CONST64(10);
      }
      break;
			case TIMESTAMP_NATIVE_UNIX:
      {
        /* Convert from PCAP NANOTIME to PCAP */
        struct nttimeval *punix_time = (struct nttimeval *)ptime;
        punix_time->tv_usec /= 1000;
        /* Convert PCAP to NDIS */
        ConvertPcapToNdis(ptime);
        /* Convert NDIS to NDIS_NATIVE */
        *ptime = (*ptime - UNIX_EPOCH_OFFSET_MAGIC) * CONST64(10);
      }
      break;
			default:
				bStatus = FALSE;
			}
			break;

		default:
			bStatus = FALSE;
			break;
		}
	}

	if ( ! bStatus ) {
		printf("Unsupported timestamp conversion: %u to %u\n",
           fromTimestampType, toTimestampType );
		*ptime = CONST64(0);
	}

	return;
}


#ifndef _WIN32
void ClearScreen(void)
{
  static bool_t clearScreenInitialized = FALSE;
  static char cls[FILENAME_MAX];

  if (!clearScreenInitialized) {
    /* Get POSIX clear screen control sequence. */
    FILE *f = popen("tput clear", "r");
    if (f) {
      if (fgets(cls, sizeof(cls), f))
        clearScreenInitialized = TRUE;
      (void)pclose(f);
    }
  }
  if (clearScreenInitialized)
    (void)fputs(cls, stdout);
}
#endif

#ifdef WIN32
HANDLE hLogFile;
#else
struct flock logMutex;
#endif
static NT_HANDLE logHandle=NT_OPEN_ERROR;
static char logFile[512];
static char *logStr=NULL;
static uint32_t logSize=10*1024*1024;
static bool_t logFileOnly;
uint32_t StartLog(char *filename, bool_t append, bool_t fileOnly)
{
  if(logHandle != NT_OPEN_ERROR) {
    fprintf(stderr, "Another logfile: \"%s\" is already in use.\n", logFile);
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }
  strcpy(logFile, filename);
  logFileOnly = fileOnly;
#ifdef WIN32
  hLogFile = CreateMutex(NULL, FALSE, TEXT("LogFileMutex"));
#else
  /* Create a write lock. */
  logMutex.l_type   = F_WRLCK; 
  logMutex.l_whence = SEEK_SET;
  logMutex.l_start  = 0;       
  logMutex.l_len    = 0;       
  logMutex.l_pid    = getpid();
#endif
  if (append) {
    logHandle = fileopen(logFile, FILE_FLAGS1, FILE_RIGHTS1);
  } else {
    logHandle = fileopen(logFile, FILE_FLAGS2, FILE_RIGHTS2);
  }
  
  if(logHandle == NT_OPEN_ERROR) {
    fprintf(stderr, "Failed to create file: \"%s\".\n", logFile);
    return NTCI_ERRCODE_INTERNAL_ERROR;   
  }

  if((logStr=malloc(logSize)) == NULL) {
    fprintf(stdout, "%s", "Failed to allocate log buffer.\n");
  }

  filelseek(logHandle, 0, SEEK_END);

  if (logFileOnly == FALSE)
  {
    ResultBufferStart(); /* Init the result buffer. */
    WriteToLog(LOG_INFO, "-------------------- LOG START --------------------\n");
  }
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t StopLog(char *filename)
{
  if(logHandle != NT_OPEN_ERROR) {
    if (logFileOnly == FALSE)
      WriteToLog(LOG_INFO, "-------------------- LOG END --------------------\n");
    fileclose(logHandle);
    logHandle = NT_OPEN_ERROR;
  }
  if (logFileOnly == FALSE)
  {
    fprintf(stdout, "Logfile %s closed.\n", filename);
    ResultBufferStop(); /* Stop the result buffer. */
  }

  if(logStr) {
    free(logStr);
    logStr=NULL;
  }
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t WriteToBuffer(char *pBuf, uint32_t maxBytes, const char *pInStr, ...)
{
  uint32_t bytesWritten=0;
  va_list args;

  va_start(args, pInStr);  
  bytesWritten = vsnprintf(pBuf, maxBytes, pInStr, args);
  va_end(args);
  return bytesWritten;
}


uint32_t WriteToLog(uint32_t logLevel, const char *pInStr, ...)
{
  va_list args;
  struct tm *cTime;
  time_t now;
  NOUSE int res;
  
#ifdef WIN32
  HANDLE hLogFileMutex = OpenMutex(MUTEX_ALL_ACCESS, FALSE, TEXT("LogFileMutex"));

  WaitForSingleObject(hLogFileMutex, INFINITE);
#else
  /* Take the lock. */  
  fcntl(logHandle, F_SETLKW, &logMutex);  /* set the lock, waiting if necessary */
#endif

  if(logHandle == NT_OPEN_ERROR) {    
    /* StartLog has not been called. Asume stdout is ok. */
    if(logLevel == LOG_ERROR) {
      fprintf(stderr, "%s", ">>> Error: ");
      va_start(args, pInStr);  
      vfprintf(stderr, pInStr, args);  
      va_end(args);      
    } else {
      va_start(args, pInStr);  
      vfprintf(stdout, pInStr, args);  
      va_end(args);
#if defined(GO_FAST)
      (void)fflush(stdout);
#endif
    }
  } else {
    if(logStr==NULL) {
      fprintf(stderr, "%s", ">>> Error: Log buffer not allocated!\n");
      va_start(args, pInStr);  
      vfprintf(stderr, pInStr, args);  
      va_end(args);
    } else if (logFileOnly == TRUE) {
      va_start(args, pInStr);  
      vsnprintf(logStr, logSize, pInStr, args);  
      va_end(args);
      res = filewrite(logHandle, logStr, strlen(logStr));
    } else {
      time(&now);
      cTime = localtime(&now);
      sprintf(logStr, "%02d-%02d-%04d %02d:%02d:%02d : ",
        cTime->tm_mday, cTime->tm_mon + 1, 1900+cTime->tm_year,
        cTime->tm_hour, cTime->tm_min, cTime->tm_sec);

      va_start(args, pInStr);  
      vsnprintf(&logStr[strlen(logStr)], logSize-strlen(logStr), pInStr, args);  
      va_end(args);
      
      if(logLevel == LOG_ERROR) {
        fprintf(stderr, "%s", ">>> Error: ");
        fprintf(stderr, "%s", logStr);
        if(logHandle != NT_OPEN_ERROR) {    
          res = filewrite(logHandle, logStr, strlen(logStr));
        }
      } else {
        if(logHandle == NT_OPEN_ERROR) {
          fprintf(stderr, "%s", "StartLog() has not been called. No filehandle.\n");
        } else {
          fprintf(stdout, "%s", logStr);
          res = filewrite(logHandle, logStr, strlen(logStr));
#if defined(GO_FAST)
          (void)fflush(stdout);
#endif
        }
      }
    }
  }
#if !defined(GO_FAST)
  (void)fflush(stdout);
#endif

#ifdef WIN32
  ReleaseMutex(hLogFileMutex);
#else
  /* Release the lock. */
  logMutex.l_type= F_UNLCK;
  fcntl(logHandle, F_SETLK, &logMutex); /* set the region to unlocked */
#endif
  return NTCI_ERRCODE_SUCCESS;
}

static NT_HANDLE resultHandle=NT_OPEN_ERROR;
static uint64_t resultOffset=0;
static char    *resultBuffer=NULL;
static uint64_t resultBufferSize=0;
/* Will note the current location in the logfile and base the
   results and validation based on that entry and EOF. */
uint32_t ResultBufferStart(void)
{
  if(logHandle == NT_OPEN_ERROR) {
    fprintf(stderr, "No logfile open. Cannot init result buffer.\n");
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  if(resultHandle == NT_OPEN_ERROR) {
    /* File needs to be opened. */
    resultHandle = fileopen(logFile, FILE_FLAGS3, FILE_RIGHTS3);
    
    if(resultHandle == NT_OPEN_ERROR) {
      fprintf(stderr, "Failed to create file: \"%s\".\n", logFile);
      return NTCI_ERRCODE_INTERNAL_ERROR;   
    }
  }
  /* Get the current location in the logfile. */
  resultOffset = filelseek(logHandle, 0, SEEK_CUR);
  return NTCI_ERRCODE_SUCCESS;
}

/* This will close the resultbuffer file handle. */
static uint32_t ResultBufferStop(void)
{
  if(resultHandle != NT_OPEN_ERROR) {
      fileclose(resultHandle);
  }
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t _ResultBufferStart(void *arglist __attribute__ ((unused)))
{
  return ResultBufferStart();
}
uint32_t _ResultBufferStop(void *arglist __attribute__ ((unused)))
{
  return ResultBufferStop();
}

/* Compare functions. Use to validate results. */
static uint32_t ResultBufferAllocate(void)
{
  if((resultHandle == NT_OPEN_ERROR) || (logHandle == NT_OPEN_ERROR)) {
    WriteToLog(LOG_ERROR, "Cannot do compare. No result buffer.\n");
    return NTCI_ERRCODE_SUCCESS;
  }

  /* Find the buffersize. */
  resultBufferSize = filelseek(logHandle, 0, SEEK_CUR) - resultOffset;
  /* Set offset to beginning of file buffer. */
  filelseek(resultHandle, resultOffset, SEEK_SET);

  /* Allocate a buffer and copy the content of the logfile to that buffer. */
  if(resultBufferSize != 0) {
    resultBufferSize+=1;
    if((resultBuffer=malloc(resultBufferSize)) == NULL) {
      WriteToLog(LOG_ERROR, "Failed to allocate result buffer.\n");
      return NTCI_ERRCODE_INSUFFICIENT_MEMORY;
    }
    memset(resultBuffer, 0x00, sizeof(resultBufferSize));
    /* Copy the content of the log to the buffer. */
    if(fileread(resultHandle, resultBuffer, resultBufferSize-1) <=0) {
      WriteToLog(LOG_ERROR, "Failed to read from log.\n");
      free(resultBuffer);
      resultBuffer=NULL;
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
  } else {
    resultBuffer=NULL;
    return NTCI_ERRCODE_INSUFFICIENT_MEMORY;
  }
  return NTCI_ERRCODE_SUCCESS;
}

static uint32_t ResultBufferFree(void)
{
  if(resultBuffer != NULL) {    
    free(resultBuffer);
    resultBuffer=NULL;
  }
  return NTCI_ERRCODE_SUCCESS;
}

static char *ResultBufferSearch(char *subStr, char *pattern)
{
  char *result=NULL;
  char *hayStack=resultBuffer;
  if(subStr!=NULL) {
    hayStack = subStr;
  }
  if(hayStack != NULL) {
    result = strstr(hayStack, pattern);
    if(result != NULL) {
      result+=strlen(pattern);
    }
  } else {
    WriteToLog(LOG_ERROR, "No result buffer found.\n");
    return NULL;
  }
  return result;
}

uint32_t CompareEqual(char *lineMatch, char *pattern, uint64_t expectedValue)
{
  uint32_t result;
  uint64_t value;
  char *pos=NULL;
  if((result = ResultBufferAllocate())!=NTCI_ERRCODE_SUCCESS) {
    return result;
  } else {
    /* Find the requested line. */
    if(strlen(lineMatch)) {
      if((pos = ResultBufferSearch(NULL, lineMatch)) == NULL) {
        ResultBufferFree();
        WriteToLog(LOG_ERROR, "lineMatch \"%s\" is not found in the resultbuffer.\n", lineMatch);
        return NTCI_ERRCODE_INTERNAL_ERROR;
      }
    }
    if((pos = ResultBufferSearch(pos, pattern)) == NULL) {
      ResultBufferFree();
      WriteToLog(LOG_ERROR, "Pattern \"%s\" is not found in the resultbuffer.\n", pattern);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
    value = AsciiToInt(pos);
    ResultBufferFree();
    if(value != expectedValue) {
      WriteToLog(LOG_INFO, "Pattern \"%s\" was %lld (0x%llX). Expected %lld (0x%llX).\n",
                 pattern, value, value, expectedValue, expectedValue);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
  }
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t CompareLarger(char *lineMatch, char *pattern, uint64_t expectedValue)
{
  uint32_t result;
  uint64_t value;
  char *pos=NULL;
  if((result = ResultBufferAllocate())!=NTCI_ERRCODE_SUCCESS) {
    return result;
  } else {
    /* Find the requested line. */
    if(strlen(lineMatch)) {
      if((pos = ResultBufferSearch(NULL, lineMatch)) == NULL) {
        ResultBufferFree();
        WriteToLog(LOG_ERROR, "lineMatch \"%s\" is not found in the resultbuffer.\n", lineMatch);
        return NTCI_ERRCODE_INTERNAL_ERROR;
      }
    }
    if((pos = ResultBufferSearch(pos, pattern)) == NULL) {
      ResultBufferFree();
      WriteToLog(LOG_ERROR, "Pattern \"%s\" is not found in the resultbuffer.\n", pattern);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
    value = AsciiToInt(pos);
    ResultBufferFree();
    if(value <= expectedValue) {
      WriteToLog(LOG_INFO, "Pattern \"%s\" was %lld (0x%llX). Expected %lld (0x%llX).\n",
                 pattern, value, value, expectedValue, expectedValue);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
  }
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t CompareSmaller(char *lineMatch, char *pattern, uint64_t expectedValue)
{
  uint32_t result;
  uint64_t value;
  char *pos=NULL;
  if((result = ResultBufferAllocate())!=NTCI_ERRCODE_SUCCESS) {
    return result;
  } else {
    /* Find the requested line. */
    if(strlen(lineMatch)) {
      if((pos = ResultBufferSearch(NULL, lineMatch)) == NULL) {
        ResultBufferFree();
        WriteToLog(LOG_ERROR, "lineMatch \"%s\" is not found in the resultbuffer.\n", lineMatch);
        return NTCI_ERRCODE_INTERNAL_ERROR;
      }
    }
    if((pos = ResultBufferSearch(pos, pattern)) == NULL) {
      ResultBufferFree();
      WriteToLog(LOG_ERROR, "Pattern \"%s\" is not found in the resultbuffer.\n", pattern);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
    value = AsciiToInt(pos);
    ResultBufferFree();
    if(value >= expectedValue) {
      WriteToLog(LOG_INFO, "Pattern \"%s\" was %lld (0x%llX). Expected %lld (0x%llX).\n",
                 pattern, value, value, expectedValue, expectedValue);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
  }
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t CheckValueEqual(uint64_t value1, uint64_t value2)
{
  if (value1 == value2) {
    return NTCI_ERRCODE_SUCCESS;
  }
  else {
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }
}


uint32_t CheckValueSmaller(uint64_t value1, uint64_t value2)
{
  if (value1 < value2) {
    return NTCI_ERRCODE_SUCCESS;
  }
  else {
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }
}

uint32_t CheckValueLarger(uint64_t value1, uint64_t value2)
{
  if (value1 > value2) {
    return NTCI_ERRCODE_SUCCESS;
  }
  else {
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }
}

uint32_t CheckStringEqual(char *string1, char *string2)
{
  if (strcmp(string1, string2) == 0) {
    return NTCI_ERRCODE_SUCCESS;
  }
  else {
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }
}

uint32_t Compare_String(  char *lineMatch, char *pattern, char* expectedString)
{
  uint32_t result;
  char *resultPos;
  char *pos=NULL;
  char buf[120];
  if((result = ResultBufferAllocate())!=NTCI_ERRCODE_SUCCESS) {
    return result;
  } else {
    if(strlen(lineMatch)) {
      /* Find the requested line. */
      if((pos = ResultBufferSearch(NULL, lineMatch)) == NULL) {
        ResultBufferFree();
        WriteToLog(LOG_ERROR, "lineMatch \"%s\" is not found in the resultbuffer.\n", lineMatch);
        return NTCI_ERRCODE_INTERNAL_ERROR;
      }
    }
    if((pos = ResultBufferSearch(pos, pattern)) == NULL) {
      ResultBufferFree();
      WriteToLog(LOG_ERROR, "Pattern \"%s\" is not found in the resultbuffer.\n", pattern);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
    if(sizeof(buf) < (strlen(expectedString)+1)) {
      WriteToLog(LOG_ERROR, "Expected string \"%s\" is too long.\n", expectedString);
      ResultBufferFree();
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
    /* Skip spaces. */
    pos=strskp(pos);
    snprintf(buf, strlen(expectedString)+1, "%s", pos);
    buf[strlen(expectedString)] = 0;
    resultPos = strstr(buf, expectedString);
    ResultBufferFree();
    if(resultPos == NULL) {
      WriteToLog(LOG_INFO, "Pattern \"%s\" was \"%s\". Expected \"%s\".\n",
                 pattern, buf, expectedString);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
  }
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t CompareStringLength(char *lineMatch, char *pattern, uint32_t expectedLength)
{
  uint32_t result;
  char *pos=NULL;
  uint32_t length=0;
  if((result = ResultBufferAllocate())!=NTCI_ERRCODE_SUCCESS) {
    return result;
  } else {
    if(strlen(lineMatch)) {
      /* Find the requested line. */      
      if((pos = ResultBufferSearch(NULL, lineMatch)) == NULL) {
        ResultBufferFree();
        WriteToLog(LOG_ERROR, "lineMatch \"%s\" is not found in the resultbuffer.\n", lineMatch);
        return NTCI_ERRCODE_INTERNAL_ERROR;
      }
    }
    if((pos = ResultBufferSearch(pos, pattern)) == NULL) {
      ResultBufferFree();
      WriteToLog(LOG_ERROR, "Pattern \"%s\" is not found in the resultbuffer.\n", pattern);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
    /* Skip spaces. */
    pos=strskp(pos);
    /* Calculate string length. */
    for(;;) {
      if(*pos == '\n' || *pos == '\r') {
        break;
      }
      length++;
      pos++;
    }
    ResultBufferFree();
    if(length != expectedLength) {
      WriteToLog(LOG_INFO, "Pattern \"%s\" was expected to have length %d. The length was %d.\n",
                 pattern, expectedLength, length);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
  }
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t CompareStringAndValue(  char *lineMatch, char *pattern, uint64_t expectedValue)
{
  uint32_t result;
  uint64_t value;
  char *pos=NULL;

  if((result = ResultBufferAllocate())!=NTCI_ERRCODE_SUCCESS) {
    return result;
  } else {
    if(strlen(lineMatch)) {
      /* Find the requested line. */
      if((pos = ResultBufferSearch(NULL, lineMatch)) == NULL) {
        ResultBufferFree();
        WriteToLog(LOG_ERROR, "lineMatch \"%s\" is not found in the resultbuffer.\n", lineMatch);
        return NTCI_ERRCODE_INTERNAL_ERROR;
      }
    }
    if((pos = ResultBufferSearch(pos, pattern)) == NULL) {
      ResultBufferFree();
      WriteToLog(LOG_ERROR, "Pattern \"%s\" is not found in the resultbuffer.\n", pattern);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }

    /* Skip spaces. */
    pos=strskp(pos);
    value = AsciiToInt(pos);
    if(value != expectedValue) {
      WriteToLog(LOG_INFO, "CompareStringAndValue: Pattern \"%s\" was \"0x%llX\". Expected \"0x%llX\".\n",
                 pattern, value, expectedValue);
      ResultBufferFree();
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
    ResultBufferFree();
  }
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t ReadString( char *lineMatch, char *pattern )
{
  uint32_t result,i;
  char *pos=NULL;

  if((result = ResultBufferAllocate())!=NTCI_ERRCODE_SUCCESS) {
    WriteToLog(LOG_ERROR, "lineMatc error\n");
    return result;
  } else {
    if(strlen(lineMatch)) {
      /* Find the requested line. */
      if((pos = ResultBufferSearch(NULL, lineMatch)) == NULL) {
        ResultBufferFree();
        WriteToLog(LOG_ERROR, "lineMatch \"%s\" is not found in the resultbuffer.\n", lineMatch);
        return NTCI_ERRCODE_INTERNAL_ERROR;
      }
    }
    if((pos = ResultBufferSearch(pos, pattern)) == NULL) {
      ResultBufferFree();
      WriteToLog(LOG_ERROR, "Pattern \"%s\" is not found in the resultbuffer.\n", pattern);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
    pos=strskp(pos);
    for(i=0;i<USERVARSTRMAXLEN;i++) {
      if ((*(pos+i) == '\n') || (*(pos+i) == '\r') || ( *(pos+i) == '\0')) {
	break;
      }
      TestFrameLibFuncRetValStr[i] = *(pos+i);
    }
    TestFrameLibFuncRetValStr[i]='\0';
    ResultBufferFree();
    return NTCI_ERRCODE_SUCCESS;
  }
}

uint32_t _ReadString(void *arglist)
{
  uint8_t *argptr = arglist;
  char *lineMatch;
  char *pattern;

  getCHARPincr(argptr, lineMatch);
  getCHARPincr(argptr, pattern);
  return ReadString(lineMatch, pattern);
}

uint32_t ReadnString( char *lineMatch, char *pattern, uint32_t length )
{
  uint32_t result,i;
  char *pos=NULL;

  if(length > USERVARSTRMAXLEN) {
    WriteToLog(LOG_ERROR, "Invalid line length %d\n", length);
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  if((result = ResultBufferAllocate())!=NTCI_ERRCODE_SUCCESS) {
    WriteToLog(LOG_ERROR, "lineMatc error\n");
    return result;
  } else {
    if(strlen(lineMatch)) {
      /* Find the requested line. */
      if((pos = ResultBufferSearch(NULL, lineMatch)) == NULL) {
        ResultBufferFree();
        WriteToLog(LOG_ERROR, "lineMatch \"%s\" is not found in the resultbuffer.\n", lineMatch);
        return NTCI_ERRCODE_INTERNAL_ERROR;
      }
    }
    if((pos = ResultBufferSearch(pos, pattern)) == NULL) {
      ResultBufferFree();
      WriteToLog(LOG_ERROR, "Pattern \"%s\" is not found in the resultbuffer.\n", pattern);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
    pos=strskp(pos);
    for(i=0;i<length;i++) {
      if(*(pos+i) == 0) {
	break;
      }
      TestFrameLibFuncRetValStr[i] = *(pos+i);
    }
    TestFrameLibFuncRetValStr[i]='\0';
    ResultBufferFree();
    return NTCI_ERRCODE_SUCCESS;
  }
}


uint32_t _ReadnString(void *arglist)
{
  uint8_t *argptr = arglist;
  char *lineMatch;
  char *pattern;
  uint32_t length;

  getCHARPincr(argptr, lineMatch);
  getCHARPincr(argptr, pattern);
  length = get32incr(&argptr);
  return ReadnString(lineMatch, pattern, length);
}

uint32_t _CompareEqual(void *arglist)
{
  uint8_t *argptr = arglist;
  char *lineMatch;
  char *pattern;
  uint64_t expectedValue=0;
  getCHARPincr(argptr, lineMatch);
  getCHARPincr(argptr, pattern);
  expectedValue = get64incr(&argptr);  
  return CompareEqual(lineMatch, pattern, expectedValue);
}

uint32_t _CompareLarger(void *arglist)
{
  uint8_t *argptr = arglist;
  char *lineMatch;
  char *pattern;
  uint64_t expectedValue=0;
  getCHARPincr(argptr, lineMatch);
  getCHARPincr(argptr, pattern);
  expectedValue = get64incr(&argptr);  
  return CompareLarger(lineMatch, pattern, expectedValue);
}

uint32_t _CompareSmaller(void *arglist)
{
  uint8_t *argptr = arglist;
  char *lineMatch;
  char *pattern;
  uint64_t expectedValue=0;
  getCHARPincr(argptr, lineMatch);
  getCHARPincr(argptr, pattern);
  expectedValue = get64incr(&argptr);  
  return CompareSmaller(lineMatch, pattern, expectedValue);
}

uint32_t _CompareString(void *arglist)
{
  uint8_t *argptr = arglist;
  char *lineMatch;
  char *pattern;
  char* expectedString;

  getCHARPincr(argptr, lineMatch);
  getCHARPincr(argptr, pattern);
  getCHARPincr(argptr, expectedString);
  return Compare_String(lineMatch, pattern, expectedString);
}

uint32_t _CompareStringLength(void *arglist)
{
  uint8_t *argptr = arglist;
  char *lineMatch;
  char *pattern;
  uint32_t expectedLength;

  getCHARPincr(argptr, lineMatch);
  getCHARPincr(argptr, pattern);

  expectedLength= get32incr(&argptr);
  return CompareStringLength(lineMatch, pattern, expectedLength);
}

uint32_t _CompareStringAndValue(void *arglist)
{
  uint8_t *argptr = arglist;
  char *lineMatch;
  char *pattern;
  uint64_t expectedValue;

  getCHARPincr(argptr, lineMatch);
  getCHARPincr(argptr, pattern);
  expectedValue = get64incr(&argptr);
  return CompareStringAndValue(lineMatch, pattern, expectedValue);
}

uint32_t _CheckValueEqual(void *arglist)
{
  uint8_t *argptr = arglist;
  uint64_t value1=0;
  uint64_t value2=0;
  value1 = get64incr(&argptr);
  value2 = get64incr(&argptr);  
  return CheckValueEqual(value1, value2);
}

uint32_t _CheckValueSmaller(void *arglist)
{
  uint8_t *argptr = arglist;
  uint64_t value1=0;
  uint64_t value2=0;
  value1 = get64incr(&argptr);
  value2 = get64incr(&argptr);  
  return CheckValueSmaller(value1, value2);
}

uint32_t _CheckValueLarger(void *arglist)
{
  uint8_t *argptr = arglist;
  uint64_t value1=0;
  uint64_t value2=0;
  value1 = get64incr(&argptr);
  value2 = get64incr(&argptr);  
  return CheckValueLarger(value1, value2);
}

uint32_t _CheckStringEqual(void *arglist)
{
  uint8_t *argptr = arglist;
  char *string1;
  char *string2;
  getCHARPincr(argptr, string1);
  getCHARPincr(argptr, string2);
  return CheckStringEqual(string1, string2);
}

uint32_t ReadInteger(char *lineMatch, char *pattern)
{
  uint32_t result;
  char *pos=NULL;

  TestFrameLibFuncRetVal = 0;
  if((result = ResultBufferAllocate())!=NTCI_ERRCODE_SUCCESS) {
    return result;
  } else {
    /* Find the requested line. */
    if(strlen(lineMatch)) {
      if((pos = ResultBufferSearch(NULL, lineMatch)) == NULL) {
        ResultBufferFree();
        WriteToLog(LOG_ERROR, "lineMatch \"%s\" is not found in the resultbuffer.\n", lineMatch);
        return NTCI_ERRCODE_INTERNAL_ERROR;
      }
    }
    if((pos = ResultBufferSearch(pos, pattern)) == NULL) {
      ResultBufferFree();
      WriteToLog(LOG_ERROR, "Pattern \"%s\" is not found in the resultbuffer.\n", pattern);
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
    TestFrameLibFuncRetVal = AsciiToInt(pos);
    ResultBufferFree();
  }
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t _ReadInteger(void *arglist)
{
  uint8_t *argptr = arglist;
  char *lineMatch;
  char *pattern;

  getCHARPincr(argptr, lineMatch);
  getCHARPincr(argptr, pattern);
  return ReadInteger(lineMatch, pattern);
}

uint32_t GetCardHandle(uint8_t card, handle_t *cardHandle)
{
  // validate card number/index - before we use them as index into an array
  if( card >= MAX_CARDS ) {
    WriteToLog(LOG_ERROR, "Adapter %u: out of range: (%d-%d)\n", card, 0, MAX_CARDS-1);
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  if(CardHandleTable[card] != NULL) {
    *cardHandle = CardHandleTable[card];
    return NTCI_ERRCODE_SUCCESS;
  } else {
    *cardHandle = NULL;
    return NTCI_ERRCODE_NO_DEVICE_OPEN;    
  }
}


void PrintUsage(char *cmdName, CommandLineOptions_t *tools_options,
                char *tool_description, char *output_description)
{
  CommandLineOptions_t *options=NULL;

  printf("\n%s\n%s\n", basename(cmdName), tool_description);
  printf("Syntax:\n");
  /* First pass the required parameters. */
  options=tools_options;
  printf("%s ", basename(cmdName));
  while(options->option) {
    if(options->mode==1) {
      printf("-%s", options->option);
      if (options->descriptionShort != NULL) {
        printf(" <%s>", options->descriptionShort);
      }
      printf(" ");
    }
    options++;
  }
  /* Pass the optional parameters. */
  options=tools_options;
  printf("[-help] [-adapter <adapter number>] ");
  while(options->option) {
    if(!options->mode || options->mode==2) {
      printf("[-%s", options->option);
      if (options->descriptionShort != NULL) {
        printf("<%s>", options->descriptionShort);
      }
      printf("] ");
    }
    options++;
  }
  printf("\n\n");
  printf("Reguired parameters:\n");
  /* First pass the required parameters. */
  options=tools_options;
  while(options->option) {
    if(options->mode==1) {
      printf("  %-10s:  %s\n",
             options->option,
             options->description);
    }
    options++;
  }
  /* Pass the optional parameters. */
  options=tools_options;
  printf("Optional parameters:\n");
  printf("  %-10s:  %s\n", "help", "This message.");
  printf("  %-10s:  %s\n", "adapter", "The adapter to use");
  while(options->option) {
    if(!options->mode || options->mode==2) {
      printf("  %-10s:  %s\n",
             options->option,
             options->description);
    }
    options++;
  }
  printf("\n%s\n", output_description);
}
