// $Id: toolslib.h 14206 2011-09-08 16:47:22Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * toolslib.c header file
 */
#ifndef __TOOLSLIB_H__
#define __TOOLSLIB_H__

#include <ostype.h>
#include <basetype.h>
#include <NTCommonStructures.h>
#include <NTCI_cardinfo.h>

/* These headers are needed for accessing the VPD API */
#include <NTCI_vpdaccess.h>
#include <vpdstructure.h>
#include <capabilitiesstructure.h>

#include <time.h>
#ifndef WIN32
#include <unistd.h>
#endif

#include "PacketFeedInfo.h"
#include "Statistics.h"
#include "CardConfiguration.h"
#include "Info.h"
#include "CardStatus.h"
#include "FilterTest.h"

#define MAX_CARDS            16
#define MAX_CHANNELS         8

#define PACKETFEED_MAX_FEEDS      32
#define PACKETFEED_MAX_CHANNELS   MAX_CHANNELS

#define ULL unsigned long long

#define CONST64(__val) __val##ULL

/* Offset of Unix epoch (1 Jan 1970) from Windows epoch (1 Jan 1601) in
 * 100ns (Windows system tick time) units */
#define UNIX_EPOCH_OFFSET_MAGIC CONST64(116444736000000000)


typedef enum {
  eUINT8,
  eUINT16,
  eUINT32,
  eUINT64,
  eCHARP
} eArgType;

struct _Arg_t
{
  const char *argName;
  eArgType argType;
};

#define MAX_ARGUMENTS 20
typedef struct
{
  const char *functionName;
  uint32_t (*function)(void *);
  uint8_t argCount;
  struct _Arg_t argList[MAX_ARGUMENTS];
} TestFrameWorkFunction_t;

#define USERVARSTRMAXLEN    512
uint32_t GetTestFrameWorkFunctions(TestFrameWorkFunction_t **functions);
uint32_t DumpTestFrameWorkFunctions(TestFrameWorkFunction_t *functions);

/* some of the basic card capabilities that may be useful elsewhere
 * in the program */
typedef struct 
{
    /* The type of card (X-Series, etc.) */    
    enum CardType CardType;

    /* Number of channels on the card and a bitmask to 
     * select all channels */

    uint32_t NumChannels;
    uint32_t AllChannelsMask;

} t_CardDetails;

typedef enum {
  PRG_DRVTOOL,
  PRG_APPLICATION,
  PRG_TOOL
} ePrgType;

/* Card types */
#define XSERIES_XD              0x00000002
#define XSERIES_XL2             0x00000004
#define NTSERIES_NT20X          0x00000008
#define NTSERIES_NT20E_INL      0x00000010
#define NTSERIES_NT20E_CAP      0x00000020
#define NTSERIES_NT4E_CAP       0x00000040
#define NTSERIES_NT4E_INL       0x00000080
#define NTSERIES_NT4E_STD_CAP   0x00000100
#define NTSERIES_NT4E_STD_INL   0x00000200
#define NTSERIES_NT4PORT        0x00000400
#define NTSERIES_NT20E2         0x00000800
#define NTSERIES_NT40E2_4       0x00001000
#define NTSERIES_NT40E2_1       0x00002000
#define CARD_UNKNOWN            0x80000000

extern char cardInfo[];
extern t_CardDetails CardDetails;
extern handle_t CardHandle;

extern uint64_t TestFrameLibFuncRetVal;
extern char TestFrameLibFuncRetValStr[USERVARSTRMAXLEN];

uint32_t OpenAnyCard(uint32_t cardType,   /* What cardtype to open. 0=any, 0x80000000=XSeries, 0x400000000=NTSeries */
                     uint8_t  cardNumber, /* What card number to open 0, 1.                      */
                     uint8_t  cardIndex); /* What index should be used to refer to it 1..3       */
uint32_t _OpenAnyCard(void *);

uint32_t OpenCaptureCard(uint32_t cardType,   /* What cardtype to open. 0=any, 0x1=XSeries, 0x2=NTSeries */
                         uint8_t  cardNumber, /* What card number to open 0, 1.                      */
                         uint8_t  cardIndex); /* What index should be used to refer to it 1..3       */
uint32_t OpenNotExcludedCard(uint32_t cardType,    /* What cardtype to open. 0=any, 0x80000000=XSeries, 0x400000000=NTSeries */
                             uint8_t  cardNumber,  /* What card number to open 0, 1.                      */
                             uint8_t  cardIndex,   /* What index should be used to refer to it 1..3       */
                             uint32_t excludeMask);/* Mask of cards to exclude                            */

uint32_t _OpenCaptureCard(void *);

uint32_t CloseCard(uint8_t cardIndex);
uint32_t _CloseCard(void *);

uint32_t ResultBufferStart(void);
uint32_t _ResultBufferStart(void *arglist);

uint32_t _CompareEqual(void *arglist);
uint32_t CompareEqual(char* lineMatch, char *pattern, uint64_t expectedValue);

uint32_t _CompareLarger(void *arglist);
uint32_t CompareLarger(char* lineMatch, char *pattern, uint64_t expectedValue);

uint32_t _CompareSmaller(void *arglist);
uint32_t CompareSmaller(char* lineMatch, char *pattern, uint64_t expectedValue);

uint32_t _CompareString(void *arglist);
uint32_t Compare_String(char* lineMatch, char *pattern, char* expectedString);

uint32_t CompareStringLength(char* lineMatch, char *pattern, uint32_t expectedLength);
uint32_t _CompareStringLength(void *arglist);

uint32_t _CompareStringAndValue(void *arglist);
uint32_t CompareStringAndValue(char* lineMatch, char *pattern, uint64_t expectedValue);

uint32_t _CheckValueEqual(void *arglist);
uint32_t CheckValueEqual(uint64_t value1, uint64_t value2);

uint32_t _CheckValueSmaller(void *arglist);
uint32_t CheckValueSmaller(uint64_t value1, uint64_t value2);

uint32_t _CheckValueLarger(void *arglist);
uint32_t CheckValueLarger(uint64_t value1, uint64_t value2);

uint32_t _CheckStringEqual(void *arglist);
uint32_t CheckStringEqual(char *string1, char *string2);

uint32_t _ReadInteger(void *arglist);
uint32_t ReadInteger(char* lineMatch, char *pattern);

uint32_t _ReadString(void *arglist);
uint32_t ReadString( char *lineMatch, char *pattern );
uint32_t _ReadnString(void *arglist);
uint32_t ReadnString( char *lineMatch, char *pattern, uint32_t length );


/* initialise the driver for use */
int CardInit(int *argc, char *argv[], uint32_t *cardIndex);
int CardInitCardDetails(void);
void CardShutdown(int i);

/* Search for feeds currently running and shut them down, before
 * starting the new feed */
void ShutdownAllPacketFeeds(handle_t card);

/* Convert timestamp types. */
void ConvertSystemTickTime( uint64_t* ptime,
                            TimestampType_t fromTimestampType,
                            TimestampType_t toTimestampType );


/* Clear the terminal screen without using system(); */
void ClearScreen(void);

uint32_t WriteToBuffer(char *pBuf, uint32_t maxBytes, const char *pInStr, ...);

#define LOG_INFO  0 
#define LOG_ERROR 1
uint32_t WriteToLog(uint32_t logLevel, const char *pInStr, ...);
uint32_t StartLog(char *filename, bool_t append, bool_t fileOnly);
uint32_t StopLog(char *filename);

uint32_t GetCardHandle(uint8_t card, handle_t *cardHandle);

int GetCardDetails(handle_t cardHandle, t_CardDetails* cardDetails);
int ValidateCardDetailChannelBitmask(t_CardDetails* cardDetails, uint32_t cardChannelBitmask);
int ValidateAdapterChannelBitmask(handle_t cardHandle, uint32_t cardChannelBitmask);
int GetAdapterChannelBitmask(handle_t cardHandle, uint32_t* cardChannelBitmask);

uint32_t TestFrameLibInit(void);
uint32_t TestFrameLibClose(void);


/* Notice: The macros below do not differenciate between
 *   little and big endian. Therefore don't use them to
 *   exchange datagrams between different system
 *   architectures.
 */

#define put8incr(addr, val)                                         \
{ uint8_t __val = (val);                                        \
    ((unsigned char *) (*addr))[0] = __val;                         \
    (*addr)+=1; (void) 0; }
#define put16incr(addr, val)                                        \
{ uint16_t __val = (val);                                       \
    ((unsigned char *) (*addr))[0] = (unsigned char)(__val);        \
    ((unsigned char *) (*addr))[1] = (unsigned char)(__val >> 8);   \
    (*addr)+=2; (void) 0; }
#define put32incr(addr, val)                                        \
{ uint32_t __val = (val);                                       \
    ((unsigned char *) (*addr))[0] = (unsigned char)__val;          \
    __val >>= 8;                                                    \
    ((unsigned char *) (*addr))[1] = (unsigned char)__val;          \
    __val >>= 8;                                                    \
    ((unsigned char *) (*addr))[2] = (unsigned char)__val;          \
    __val >>= 8;                                                    \
    ((unsigned char *) (*addr))[3] = (unsigned char)__val;          \
    (*addr)+=4; (void) 0; }
#define put64incr(addr, val)                                        \
{ uint64_t __val = (val);                                       \
    ((unsigned char *) (*addr))[0] = (unsigned char)__val;          \
    __val >>= 8;                                                    \
    ((unsigned char *) (*addr))[1] = (unsigned char)__val;          \
    __val >>= 8;                                                    \
    ((unsigned char *) (*addr))[2] = (unsigned char)__val;          \
    __val >>= 8;                                                    \
    ((unsigned char *) (*addr))[3] = (unsigned char)__val;          \
    __val >>= 8;                                                    \
    ((unsigned char *) (*addr))[4] = (unsigned char)__val;          \
    __val >>= 8;                                                    \
    ((unsigned char *) (*addr))[5] = (unsigned char)__val;          \
    __val >>= 8;                                                    \
    ((unsigned char *) (*addr))[6] = (unsigned char)__val;          \
    __val >>= 8;                                                    \
    ((unsigned char *) (*addr))[7] = (unsigned char)__val;          \
    (*addr)+=8; (void) 0; }
#define putCHARPincr(addr, ptr)                                     \
{ unsigned char *_ptr = (unsigned char*)ptr;                    \
    memcpy(*addr, &_ptr, sizeof(unsigned char *));                  \
    (*addr)+=sizeof(unsigned char *); (void) 0; }

#define get8incr(addr)                                          \
    ((const unsigned char *)(*addr))[0];                        \
    (*addr)+=1;
#define get16incr(addr)                                         \
    (((const unsigned char *)(*addr))[1] << 8                   \
    | ((const unsigned char *)(*addr))[0]);                     \
    (*addr)+=2;
#define get32incr(addr)                                         \
    (((((const unsigned char *)(*addr))[3] << 8                 \
    | ((const unsigned char *)(*addr))[2]) << 8                 \
    | ((const unsigned char *)(*addr))[1]) << 8                 \
    | ((const unsigned char *)(*addr))[0]);                     \
    (*addr)+=4;
#define get64incr(addr)                                         \
    ((((((((uint64_t)(((const unsigned char *)(*addr))[7] << 8  \
    | ((const unsigned char *)(*addr))[6]) << 8                 \
    | ((const unsigned char *)(*addr))[5]) << 8                 \
    | ((const unsigned char *)(*addr))[4]) << 8                 \
    | ((const unsigned char *)(*addr))[3]) << 8                 \
    | ((const unsigned char *)(*addr))[2]) << 8                 \
    | ((const unsigned char *)(*addr))[1]) << 8                 \
    | ((const unsigned char *)(*addr))[0]));                    \
    (*addr)+=8;
#define getCHARPincr(addr, ptr)                                 \
    { char *val;                                                \
    memcpy(&val, addr, sizeof(char *));                         \
    (addr)+=sizeof(char *); ptr = val; }

typedef struct {
  const char *option;
  const char *description;
  const char *descriptionShort;
  int  mode;
  char **result;
} CommandLineOptions_t;

void PrintUsage(char *cmdName, CommandLineOptions_t *tools_options,
                char *tool_description, char *output_description);

int InitApplication(int *argc, char *argv[], uint32_t boCaptureCardsOnly, uint32_t *cardIndex,
                    CommandLineOptions_t *tools_options, char *tool_description,
                    char *output_description);

void DisplayProgramHeader (char *programName, ePrgType gramType);
void DisplayProgramFooter (void);

/*-------------------------------------------------------------------------*/
/**
  @brief    Converts ASCII number string (hex or decimal) to integer.
  @param    str     ASCII string
  @return   Integer value. 

 */
/*--------------------------------------------------------------------------*/
uint64_t AsciiToInt (char *str);

int ParseYesNo(char *str); /* Return 0 if No, 1 is Yes and -1 if neither. */

#endif //__TOOLSLIB_H__
