
#ifndef NT_LEGACY_4_25

#ifndef DEPRECATED_4_25
#warning: This file was part of a Xyratex specific framework and has been retired.
#warning: It is not and has never been required for Napatech Adapter operations.
#warning: Please remove this file from your project and recompile.
#warning: See file /include/xyratex_retired_readme.txt for more information.
#endif

#else //NT_LEGACY_4_25
#ifndef _NTCI_asyncevent_H_
#define _NTCI_asyncevent_H_

#include "NTCommonInterface.h"

typedef uint32_t EventID;

#define MAKE_EVENT_ID(GroupID,EventTypeID) (((GroupID) << 24) | ((EventTypeID) & 0x00FFFFFF))
#define GET_EVENT_GROUP(EventID) ((EventID >> 24) & 0xFF)
#define GET_EVENT_TYPE(EventID)  (EventID & 0x00FFFFFF)


/* Legacy events are events defined before the GroupId bits were implemented */
#define EVENT_GROUP_LEGACY                  0
#define EVENT_ID_LEGACY(LegacyMessageId)    MAKE_EVENT_ID(EVENT_GROUP_LEGACY , 1 << LegacyMessageId)


#ifdef __cplusplus
/*  For Napatech internal use. */
// ~NITS~ return NTCOMMONLIB
// ~NITS~ return NTCOMMONCALL
#endif


/*
**      Masking an event.
**      =================
**      Special mask:   EventID     == 0        "All events in all groups"
**
**      Special mask:   GroupID     == 255      "All events in all groups"
**                      EventTypeID == 0xFFFFFF
**
**      Special mask:   GroupID     == 255      "event Id matching N in any group"
**                      EventTypeID == N
**
**      Typical mask:   GroupID     == M        "All events in group M"
**                      EventTypeID == 0xFFFFFF
**
**      Limited mask:   GroupID     == M        "Only event id matching M in group N"
**                      EventTypeID == N
*/

/*
**      Recieving an event.
**      ===================
**      GroupID         Set to id of group id for the message type
**                      Only 1 group id can be specified by an event
**
**      EventTypeId     Set bits to indicate the message type
**
**                      NOTE: Multiple bits can be set by sender.
**                      Multiple bits are only set if message data is a common value.
**                      Multiple bits usually means "same event occurred on many channels"
**
*/


typedef enum _AsyncRecipient 
{
    eAsyncRecipientDevNull         = 0,
    eAsyncRecipientCallbackType1   = 1,
    eAsyncRecipientManualReadType1 = 2,
} AsyncRecipient;

/*
**      AsyncRecipient type 0
**      Device NULL recipient
**
**      Matched messages are processed by an empty function
**
*/
typedef struct _AsyncRecipientDevNull_t
{
    handle_t hCard;
    EventID Mask;
} AsyncRecipientDevNull_t;


/*
**      AsyncRecipient type 1
**      Callback recipient (version 1)
**
**      Matched messages are passed by parameters to a user supplied callback function
**
*/
typedef  void(NTCOMMONCALL *PFN_AsyncCallback)(handle_t hCard,
                                               EventID  MessageType,
                                               uint32_t MessageLength,
                                               void *   pMessage,
                                               uint32_t UserData);

typedef struct _AsyncRecipientCallbackType1_t
{
	handle_t  		hCard;
	EventID			EventsMask;
	PFN_AsyncCallback	AsyncEventCallback;
	uint32_t		UserData;
} AsyncRecipientCallbackType1_t;


/*
**    AsyncRecipient type 2
**    Manual Read recipient (version 1)
**
**    Matched messages are stored in a local memory FIFO queue
**    Application can then read individual messages using NTCI_ManualEventRead
**    Unlike NTCI_GetAsyncEvent, ManualEventRead will return immediately even
**    when no messages available.
**
*/
typedef struct _AsyncRecipientManualReadType1_t
{
    handle_t  		hCard;
	EventID			EventsMask;    
} AsyncRecipientManualReadType1_t;



/* Use the following values in a call to NTCI_RegisterAsyncEventRecipient  */

#define STID_ASYNC_EVENT_RECIPIENT_DEV_NULL \
	MAKE_STRUCTURE_ID(eAsyncRecipientDevNull, AsyncRecipientDevNull_t)

#define STID_ASYNC_EVENT_RECIPIENT_CALLBACK_TYPE1 \
	MAKE_STRUCTURE_ID(eAsyncRecipientCallbackType1, AsyncRecipientCallbackType1_t)

#define STID_ASYNC_EVENT_RECIPIENT_MANUAL_READ_TYPE1 \
	MAKE_STRUCTURE_ID(eAsyncRecipientManualReadType1, AsyncRecipientManualReadType1_t)


/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_EnableAsyncEvents
**
**  Description    :Enable async event generation within the driver
**
**  Parameters     :IN handle_t hCard,
**                  IN bool_t   Enable
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_EnableAsyncEvents(   IN  handle_t    hCard,
                                                            IN  bool_t      Enable);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetAsyncEvent
**
**  Description    :Retrieve the next async event message block.
**                  The eventBuffer memory block may be filled with more than
**                  one message upon return.
**                  The call will fail if bufferLen is too small to receive
**                  all of the data.
**
**                  IMPORTANT: This call is synchronous. It will wait until an
**                  async event block is available before returning.
**                  If no async messages arrive, the function will return only 
**                  if the driver is closed. This is achieved by closing all
**                  of the open card handles.
**
**  Parameters     :IN handle_t hCard
**                  OUT void *  eventBuffer,
**                  IN uint32_t bufferLen,
**                  OUT uint32_t*   bytesReturned
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetAsyncEvent(   IN  handle_t    hCard,
                                                        OUT void *  eventBuffer,
                                                        IN uint32_t bufferLen,
                                                        OUT uint32_t*   bytesReturned);


/*  ----------------------------------------------------------------------------
**
**  Name           :NTCOMMONCALL NTCI_RegisterAsyncEventRecipient
**
**  Description    : Create a new recipient of async event messages.
**
**                  Each card maintains a thread that listens for async event
**                  messages. This thread uses NTCI_GetAsyncEvent internally.
**                  Each message recieved is passed on to each registered
**                  recipient that wants it.
**
**                  NTCI_GetAsyncEvent is not called until the first recipient
**                  is registered on a particular card.
**                  
**                  NOTE: Each card maintains a separate list of recipients
**
**
**  Parameters     : IN StructureID RecipientType
**                  IN void * pRecipientData
**                  OUT handle_t * pNewRecipientHandle
**
**  Returns        :uint32_t 
**
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_RegisterAsyncEventRecipient( IN StructureID RecipientType,
                                                  IN void * pRecipientData,
                                                  OUT handle_t * pNewRecipientHandle);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCOMMONCALL NTCI_UnregisterAsyncEventRecipient
**
**  Description    : Destroy an existing recipient of async event messages
**
**                  When all recipients have been destroyed, the thread that
**                  listens for messages is stopped. However, the thread will
**                  not actaully stop until after NTCI_GetAsyncEvent returns.
**
**  Parameters     :IN handle_t RecipientHandle
**
**  Returns        :uint32_t 
**
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_UnregisterAsyncEventRecipient( IN handle_t RecipientHandle);


/*  ----------------------------------------------------------------------------
**
**  Name           :NTCOMMONCALL NTCI_SimulateEvent
**
**  Description    :Send a test event to all recipients for a particular card
**                  
**                  The recipients will not know that this message is a test
**                  message. NOTE: NTCI_GetAsyncEvent does not see this event.
**
**
**  Parameters     : IN  handle_t    hCard
**                  IN  uint32_t EventDataLength
**                  IN  uint32_t EventDataType
**                  IN  void * pEventData
**
**  Returns        :uint32_t 
**
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_SimulateEvent( IN  handle_t    hCard,
                                                      IN  uint32_t EventDataLength,
                                                      IN  uint32_t EventDataType,
                                                      IN  void * pEventData);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCOMMONCALL NTCI_ManualEventRead
**
**  Description    :Reads a queued event from a specific recipient type.
**
**                  This applies to AsyncRecipientManualReadType1 only.
**                  AsyncRecipientManualReadType1 creates a FIFO queue of events
**                  in memory. This function reads and removes the next message
**                  available in the queue. Returns a errorcode if not messages
**                  have been queued.
**
**  Parameters     : IN handle_t Recipient
**                  IN uint32_t MaxBufferSize
**                  IN void * ReadBuffer
**                  OUT uint32_t * pEventDataLength
**                  OUT uint32_t * pEventDataType
**
**  Returns        :uint32_t 
**
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_ManualEventRead( IN handle_t Recipient,
                                                IN uint32_t MaxBufferSize,
                                                IN void * ReadBuffer,
                                                OUT uint32_t * pEventDataLength,
                                                OUT uint32_t * pEventDataType);




#endif
#endif //NT_LEGACY_4_25

