// $Id: NTCI_errorcode.h 17394 2012-02-17 15:05:30Z mh $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Definitions of errorcode returned from NTCI functions.
 */

#ifndef __NTCI_ERRORCODE_H__
#define __NTCI_ERRORCODE_H__

#include "NTCommonInterface.h"

//Standard NTCommonInterface error codes
//  These codes are returned directly by NTCI functions

//Function succeeded
#define NTCI_ERRCODE_SUCCESS                    0

//Dll found an internal data problem
#define NTCI_ERRCODE_INTERNAL_ERROR             1

//Dll could not reserve enough memory
#define NTCI_ERRCODE_INSUFFICIENT_MEMORY        2

//Function was given an invalid parameter
#define NTCI_ERRCODE_INVALID_PARAMETER          3

//The driver has not been opened
#define NTCI_ERRCODE_NO_DEVICE_OPEN             4

//The function failed due to low level error
#define NTCI_ERRCODE_DRIVER_ERROR               5

//The function is not supported by that specific card
#define NTCI_ERRCODE_INTERFACE_NOT_AVAILABLE    6

//The function was given a null pointer as a memory block
#define NTCI_ERRCODE_NULL_POINTER               7

//The function was given a count or structure id of zero
#define NTCI_ERRCODE_ZERO_PARAMETER             8

//The handle to the card did not match the list of open cards
#define NTCI_ERRCODE_CARD_HANDLE_NOT_VALID      9

//Failed to get semaphore
#define NTCI_ERRCODE_FASTLOCK_FAILED           30

#define NTCI_ERRCODE_HOST_BUFFER_OFFSET_TOO_BIG        31

//PACKETFEED error codes
//  These codes are returned only when the driver returns a 
//  specific low level error.
//  Other low level errors will be reported as NTCI_ERRCODE_DRIVER_ERROR

#define NTCI_ERRCODE_INVALID_PACKET_FEED_PRIORITY   10

#define NTCI_STATUSCODE_PACKET_FEED_STOPPED         11

#define NTCI_STATUSCODE_NO_SEGMENT_AVAILABLE        12
//Below define is deprecated. Wording in above define is more accurate.
/* LEGACY - PLEASE IGNORE THIS LINE */ #define NTCI_STATUSCODE_NO_PACKETS_AVAILABLE        12

#define NTCI_STATUSCODE_NO_DESCRIPTION_AVAILABLE    13

#define NTCI_STATUSCODE_PACKET_FEED_STOPPING        14

//CAPABILITIES error codes
//  These codes are returned only when the driver returns a 
//  specific low level error.
//  Other low level errors will be reported as NTCI_ERRCODE_DRIVER_ERROR

#define NTCI_STATUSCODE_CAPABILITY_NOT_FOUND        20

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_ExplainLastManagerError
**
**  Description    :Gets a textual description of the most recent error stored
**                  in the card manager. It does not reset the stored error.
**                  After calling NTCI_ExplainLastManagerError, you should call
**                  NTCI_GetManagerLastError() and 
**                  NTCI_GetManagerLastLowLevelError() to clear the error codes.
**
**  Parameters     :OUT char *  pExplanationBuffer
**                  IN uint32_t MaxBufferLength
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_ExplainLastManagerError( OUT char *      pExplanationBuffer,
                                                                IN  uint32_t    MaxBufferLength);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetManagerLastError
**
**  Description    :Get the most recent error code from the card manager
**                  The card manager will store errors for the following
**                  1) Functions that did not require IN handle_t hCard
**                  2) Card functions where hCard was not valid
**                  3) Card functions not supported by a particular card type
**                  4) Stores a copy of the last error raised by ANY card
**                  Once read back, the manager error code will be reset to 0.
**
**  Parameters     :none
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetManagerLastError(void);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetLastError
**
**  Description    :EQUIVALENT TO
**                  NTCI_GetManagerLastError()
**
**  Parameters     :none
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetLastError(void);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetManagerLastLowLevelError
**
**  Description    :Get the most recent low level error code from the card manager
**                  The card manager will store errors for the following
**                  1) Functions that did not require IN handle_t hCard
**                  2) Card functions where hCard was not valid
**                  3) Card functions not supported by a particular card type
**                  4) Stores a copy of the last error raised by ANY card
**                  Once read back, the error code will be reset to 0.
**
**  Parameters     :none
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetManagerLastLowLevelError(void);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetLastLowLevelDriverError
**
**  Description    :EQUIVALENT TO
**                  NTCI_GetManagerLastLowLevelError()
**
**  Parameters     :none
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetLastLowLevelDriverError(void);



/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_ExplainLastCardError
**
**  Description    :Gets a textual description of the most recent error stored
**                  on the specified card. It does not reset the stored error.
**                  After calling NTCI_ExplainLastCardError, you should call
**                  NTCI_GetCardLastError() and 
**                  NTCI_GetCardLastLowLevelError() to clear the error codes.
**
**  Parameters     :IN handle_t hCard
**                  OUT char *  pExplanationBuffer
**                  IN uint32_t MaxBufferLength
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_ExplainLastCardError(IN  handle_t    hCard,
                                                            OUT char *      pExplanationBuffer,
                                                            IN  uint32_t    MaxBufferLength);
/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetCardLastError
**
**  Description    :Get the most recent error code from the card itself.
**                  The card will store errors for the following
**                  1) Functions given invalid parameters, such as null pointers
**                  2) Functions that fail due to the low level device driver
**                   reporting a failure.
**                  NOTE: some parameters can only be checked by the device driver
**                  so will result in low level error codes
**                  See also NTCI_GetCardLastLowLevelError
**                  NOTE: Card errors will be copied in to the card manager
**                  Once read back, the error code will be reset to 0
**                  The managers last error is unaffected
**
**  Parameters     :IN handle_t hCard
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetCardLastError(IN handle_t hCard);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetCardLastLowLevelError
**
**  Description    :Get the low level error code that caused the last card
**                  command to fail.
**                  Potentially useful when you need a more detailed reason for
**                  a failed NTCI_* function call.
**                  Note: the act of calling this routine automatically resets
**                  the last low-level driver error to zero (i.e. success),
**                  ready for recording future errors.
**                  NOTE: Card errors will be copied in to the card manager
**                  Once read back, the error code will be reset to 0
**                  The managers last error is unaffected
**
**  Parameters     :IN handle_t hCard
**
**  Returns        :uint32_t
**
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetCardLastLowLevelError(IN handle_t hCard);

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ERRCODE_SUCCESS NTCI_ERRCODE_SUCCESS 
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ERRCODE_INTERNAL_ERROR NTCI_ERRCODE_INTERNAL_ERROR	
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ERRCODE_INSUFFICIENT_MEMORY NTCI_ERRCODE_INSUFFICIENT_MEMORY
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ERRCODE_INVALID_PARAMETER NTCI_ERRCODE_INVALID_PARAMETER
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ERRCODE_NO_DEVICE_OPEN NTCI_ERRCODE_NO_DEVICE_OPEN
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ERRCODE_DRIVER_ERROR NTCI_ERRCODE_DRIVER_ERROR
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ERRCODE_INTERFACE_NOT_AVAILABLE	NTCI_ERRCODE_INTERFACE_NOT_AVAILABLE
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ERRCODE_NULL_POINTER NTCI_ERRCODE_NULL_POINTER
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ERRCODE_ZERO_PARAMETER NTCI_ERRCODE_ZERO_PARAMETER
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ERRCODE_CARD_HANDLE_NOT_VALID NTCI_ERRCODE_CARD_HANDLE_NOT_VALID
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ERRCODE_INVALID_PACKET_FEED_PRIORITY NTCI_ERRCODE_INVALID_PACKET_FEED_PRIORITY 
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_STATUSCODE_PACKET_FEED_STOPPED NTCI_STATUSCODE_PACKET_FEED_STOPPED
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_STATUSCODE_NO_PACKETS_AVAILABLE NTCI_STATUSCODE_NO_PACKETS_AVAILABLE
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_STATUSCODE_NO_DESCRIPTION_AVAILABLE NTCI_STATUSCODE_NO_DESCRIPTION_AVAILABLE
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_STATUSCODE_CAPABILITY_NOT_FOUND NTCI_STATUSCODE_CAPABILITY_NOT_FOUND
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif

#endif  // __NTCI_ERRORCODE_H__
