// $Id: NTCI_registeraccess.h 16051 2011-11-30 12:20:27Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * C functions to access the driver interface Register Access.
 */

#ifndef __NTCI_REGISTERACCESS_H__
#define __NTCI_REGISTERACCESS_H__

#include "NTCommonInterface.h"

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetRegisterBlockInformation
**
**  Description    :Retrieves the number of available register block (==regions)
**                  Regions are number from 0.
**
**  Parameters     :IN  handle_t hCard
**                  OUT int *  pNumBlocks
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetRegisterBlockInformation( IN  handle_t    hCard,
                                                                    OUT int *   pNumBlocks);


/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_ReadRegister8
**
**  Description    :Read a register from the device.
**                  Note: 'iOffset' is the offset from the base address of
**                  the block of registers associated with the supplied register
**                  region.
**
**  Parameters     :IN  handle_t hCard
**                  IN  uint32_t  iRegisterRegion
**                  IN  uint32_t  iOffset
**                  OUT uint8_t*  piValue
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_ReadRegister8(   IN  handle_t    hCard,
                                                        IN  uint32_t    iRegisterRegion,
                                                        IN  uint32_t    iOffset,
                                                        OUT uint8_t *   piValue );

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_WriteRegister8
**
**  Description    :Write a value to a register on the device.
**                  Note: 'iOffset' is the offset from the base address of
**                  the block of registers associated with the supplied register
**                  region.
**
**  Parameters     :IN handle_t hCard
**                  IN uint32_t  iRegisterRegion
**                  IN uint32_t  iOffset
**                  IN uint8_t  iValue
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_WriteRegister8(  IN  handle_t    hCard,
                                                        IN  uint32_t    iRegisterRegion,
                                                        IN  uint32_t    iOffset,
                                                        IN  uint8_t     iValue );

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_ReadRegister16
**
**  Description    :Read a register from the device.
**                  Note: 'iOffset' is the offset from the base address of
**                  the block of registers associated with the supplied register
**                  region.
**
**  Parameters     :IN  handle_t hCard
**                  IN  uint32_t  iRegisterRegion
**                  IN  uint32_t  iOffset
**                  OUT uint16_t*  piValue
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_ReadRegister16(  IN  handle_t    hCard,
                                                        IN  uint32_t    iRegisterRegion,
                                                        IN  uint32_t    iOffset,
                                                        OUT uint16_t *  piValue );

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_WriteRegister16
**
**  Description    :Write a value to a register on the device.
**                  Note: 'iOffset' is the offset from the base address of
**                  the block of registers associated with the supplied register
**                  region.
**
**  Parameters     :IN handle_t hCard
**                  IN uint32_t  iRegisterRegion
**                  IN uint32_t  iOffset
**                  IN uint16_t  iValue
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_WriteRegister16( IN  handle_t    hCard,
                                                        IN  uint32_t    iRegisterRegion,
                                                        IN  uint32_t    iOffset,
                                                        IN  uint16_t    iValue );

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_ReadRegister
**
**  Description    :EQUIVALENT TO
**                      NTCI_ReadRegister32
**                  Supplied to allow access to the default sized registers
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_ReadRegister(    IN  handle_t    hCard,
                                                        IN  uint32_t    iRegisterRegion,
                                                        IN  uint32_t    iOffset,
                                                        OUT uint32_t *  piValue );

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_WriteRegister
**
**  Description    :EQUIVALENT TO
**                      NTCI_WriteRegister32
**                  Supplied to allow access to the default sized registers
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_WriteRegister(   IN  handle_t    hCard,
                                                        IN  uint32_t    iRegisterRegion,
                                                        IN  uint32_t    iOffset,
                                                        IN  uint32_t    iValue );

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_ReadRegister32
**
**  Description    :Read a register from the device.
**                  Note: 'iOffset' is the offset from the base address of
**                  the block of registers associated with the supplied register
**                  region.
**
**  Parameters     :IN  handle_t hCard
**                  IN  uint32_t  iRegisterRegion
**                  IN  uint32_t  iOffset
**                  OUT uint32_t*  piValue
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_ReadRegister32(  IN  handle_t    hCard,
                                                        IN  uint32_t    iRegisterRegion,
                                                        IN  uint32_t    iOffset,
                                                        OUT uint32_t *  piValue );

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_WriteRegister32
**
**  Description    :Write a value to a register on the device.
**                  Note: 'iOffset' is the offset from the base address of
**                  the block of registers associated with the supplied register
**                  region.
**
**  Parameters     :IN handle_t hCard
**                  IN uint32_t  iRegisterRegion
**                  IN uint32_t  iOffset
**                  IN uint32_t  iValue
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_WriteRegister32( IN  handle_t    hCard,
                                                        IN  uint32_t    iRegisterRegion,
                                                        IN  uint32_t    iOffset,
                                                        IN  uint32_t    iValue );

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_ReadRegister64
**
**  Description    :Read a register from the device.
**                  Note: 'iOffset' is the offset from the base address of
**                  the block of registers associated with the supplied register
**                  region.
**
**  Parameters     :IN  handle_t hCard
**                  IN  uint32_t  iRegisterRegion
**                  IN  uint32_t  iOffset
**                  OUT uint64_t*  piValue
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_ReadRegister64(  IN  handle_t    hCard,
                                                        IN  uint32_t    iRegisterRegion,
                                                        IN  uint32_t    iOffset,
                                                        OUT uint64_t *  piValue );

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_WriteRegister64
**
**  Description    :Write a value to a register on the device.
**                  Note: 'iOffset' is the offset from the base address of
**                  the block of registers associated with the supplied register
**                  region.
**
**  Parameters     :IN handle_t hCard
**                  IN uint32_t  iRegisterRegion
**                  IN uint32_t  iOffset
**                  IN uint64_t  iValue
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_WriteRegister64( IN  handle_t    hCard,
                                                        IN  uint32_t    iRegisterRegion,
                                                        IN  uint32_t    iOffset,
                                                        IN  uint64_t    iValue );

#endif // __NTCI_REGISTERACCESS_H__
