// $Id: cardconfigurationstructure.h 24835 2013-08-08 14:08:01Z fc $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Structure Definitions that are used by the external cardconfiguration.h interfaces
 */

#ifndef __NTAPI_CARDCONFIGURATIONSTRUCTURE_H__
#define __NTAPI_CARDCONFIGURATIONSTRUCTURE_H__

#include <ostype.h>
#include "basetype.h"
#include "commontypes.h"
#include "ntpackpush.h"
#include "packetfeedstructure.h"
#include "kernelif/identifiers.h"

#define   MANUAL_DUPLEX_HALF 0
#define   MANUAL_DUPLEX_FULL 1

enum
{
  eAutonegotiationModeType1,
  eFrameDecoderConfigurationType1,
  eFrameDecoderConfigurationType2,
  eFrameDecoderConfigurationType3,
  eTimestampConfigurationType1,
  eTimestampDriftConfigurationType1,
  eFrameDecoderConfigurationType4,
  eFrameDecoderConfigurationType5,
  eFeedTransmitConfigurationType1,
  eFeedTransmitConfigurationType2,
  eLocalTransmitConfigurationType1,
  eReceiveChannelPortConfigurationType1,
  eTransmitChannelPortConfigurationType1,
  eAutonegotiationModeType2,
  ePriorityControlConfigurationType1,
  ePriorityControlConfigurationType2,
  ePriorityControlConfigurationType3,
  eSDRAMConfigurationType1,
  eTimeSyncConfigurationType1,
  eSDRAMPolicyConfigurationType1,
  eAutonegotiationModeType3,
  eAutonegotiationModeType4,
  eTimestampOffsetConfigurationType1,
  eTimeSyncConfigurationType2,
  eFrameDecoderConfigurationType6,
  eTimeSyncConfigurationType3,
  eAdapterPairConfigurationType1,
  eFrameDecoderConfigurationType7,
  eAutonegotiationModeType5,
  eFailoverConfigurationType1,
  eFailoverHeartbeatSignalType1,
  eTimeSyncConfigurationType4,
  eTxLaserConfigurationType1,
  ePortIdentificationConfigurationType1,
  eAutonegotiationModeType6,
  eFrameDecoderConfigurationType8,
  eTimeSynchronizedTxConfigurationType1,
  eTimeSyncInfoType1,
  eTimeSyncInfoType2,
};

/*************************************************************/
/* Structure(s) to be used with Set/GetAutonegotiationMode() */
/*************************************************************/
typedef enum
{
  PAUSE_CTRL_MODE_NO_PAUSE,
  PAUSE_CTRL_MODE_ASYMMETRIC_PAUSE,
  PAUSE_CTRL_MODE_SYMMETRIC_PAUSE,
  PAUSE_CTRL_MODE_SYMMETRIC_AND_ASYMMETRIC_PAUSE
} PauseCtrlMode_t;

typedef enum
{
  MDI_MODE_AUTO,
  MDI_MODE_MDI,
  MDI_MODE_MDIX,
  MDI_MODE_NA
} MdiMode_t;

typedef enum
{
  MANUAL_SPEED_1000,
  MANUAL_SPEED_100,
  MANUAL_SPEED_10
}ManualSpeed_t;

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  enable DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  required DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  restartAutoNeg DEPRECATED_4_25 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise10000FullDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise10000HalfDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise1000FullDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise1000HalfDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise100FullDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise100HalfDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise10FullDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise10HalfDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} AutonegotiationModeType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_AUTONEGOTIATION_MODE_TYPE1 MAKE_STRUCTURE_ID(eAutonegotiationModeType1, AutonegotiationModeType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t enable DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t required DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t restartAutoNeg DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise10000FullDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise10000HalfDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise1000FullDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise1000HalfDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise100FullDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise100HalfDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise10FullDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  advertise10HalfDuplex DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ PauseCtrlMode_t pauseControlMode DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} AutonegotiationModeType2_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_AUTONEGOTIATION_MODE_TYPE2 MAKE_STRUCTURE_ID(eAutonegotiationModeType2, AutonegotiationModeType2_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ bool_t  write_action DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ uint32_t reg DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ uint16_t value DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} AutonegotiationModeType3_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_AUTONEGOTIATION_MODE_TYPE3 MAKE_STRUCTURE_ID(eAutonegotiationModeType3, AutonegotiationModeType3_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#endif //NT_LEGACY_4_25

typedef struct
{
  bool_t  enable;
  bool_t  required;
  bool_t  restartAutoNeg;
  bool_t  advertise10000FullDuplex;
  bool_t  advertise10000HalfDuplex;
  bool_t  advertise1000FullDuplex;
  bool_t  advertise1000HalfDuplex;
  bool_t  advertise100FullDuplex;
  bool_t  advertise100HalfDuplex;
  bool_t  advertise10FullDuplex;
  bool_t  advertise10HalfDuplex;
  PauseCtrlMode_t pauseControlMode;
  bool_t  manualFullDuplex;
  ManualSpeed_t manualSpeed;
  MdiMode_t mdiMode;
} AutonegotiationModeType4_t;

#define STID_AUTONEGOTIATION_MODE_TYPE4 \
    MAKE_STRUCTURE_ID(eAutonegotiationModeType4, AutonegotiationModeType4_t)

typedef struct
{
  bool_t  setdefault;                 /* Configure to default autonegotiation 0=disabled, 1=enabled */
  bool_t  enable;                     /* Enable autonegotiation 0=disable, 1=enabled */
  bool_t  required;                   /* Currently not used */
  bool_t  restartAutoNeg;             /* Currently not used */
  bool_t  advertise10000FullDuplex;   /* Autonegotiation advertise setting 0=disable, 1=enabled */
  bool_t  advertise10000HalfDuplex;   /* Autonegotiation advertise setting 0=disable, 1=enabled */
  bool_t  advertise1000FullDuplex;    /* Autonegotiation advertise setting 0=disable, 1=enabled */
  bool_t  advertise1000HalfDuplex;    /* Autonegotiation advertise setting 0=disable, 1=enabled */
  bool_t  advertise100FullDuplex;     /* Autonegotiation advertise setting 0=disable, 1=enabled */
  bool_t  advertise100HalfDuplex;     /* Autonegotiation advertise setting 0=disable, 1=enabled */
  bool_t  advertise10FullDuplex;      /* Autonegotiation advertise setting 0=disable, 1=enabled */
  bool_t  advertise10HalfDuplex;      /* Autonegotiation advertise setting 0=disable, 1=enabled */
  PauseCtrlMode_t pauseControlMode;   /* Currently not used */
  bool_t  manualFullDuplex;           /* Manual duplex setting 0=Half Duplex, 1=Full Duplex */
  ManualSpeed_t manualSpeed;          /* Manual speed setting 0=1000, 1=100, 2=10 */
  MdiMode_t mdiMode;                  /* Manual mdi setting 0=Auto, 1=MDI, 2=MDIX */
} AutonegotiationModeType5_t;

#define STID_AUTONEGOTIATION_MODE_TYPE5 \
    MAKE_STRUCTURE_ID(eAutonegotiationModeType5, AutonegotiationModeType5_t)

//Defines for advertise bit mask
#define ADVERTISE_10M_HALFDUPLEX   0x0001
#define ADVERTISE_10M_FULLDUPLEX   0x0002
#define ADVERTISE_100M_HALFDUPLEX  0x0004
#define ADVERTISE_100M_FULLDUPLEX  0x0008
#define ADVERTISE_1G_HALFDUPLEX    0x0010
#define ADVERTISE_1G_FULLDUPLEX    0x0020
#define ADVERTISE_10G_FULLDUPLEX   0x0040
#define ADVERTISE_40G_FULLDUPLEX   0x0080
#define ADVERTISE_100G_FULLDUPLEX  0x0100

//Change advertise fields to
typedef struct
{
  bool_t   setdefault;                  /* Configure to default autonegotiation 0=disabled, 1=enabled */
  bool_t   enable;                      /* Enable autonegotiation 0=disable, 1=enabled */
  bool_t   required;                    /* Currently not used */
  bool_t   restartAutoNeg;              /* Currently not used */
  uint32_t advertiseSpeedDuplexBitMask; /* Autonegotiation advertise setting 0=disable, 1=enabled */
  PauseCtrlMode_t pauseControlMode;     /* Currently not used */
  bool_t  manualFullDuplex;             /* Manual duplex setting 0=Half Duplex, 1=Full Duplex */
  ManualSpeed_t manualSpeed;            /* Manual speed setting 0=1000, 1=100, 2=10 */
  MdiMode_t mdiMode;                    /* Manual mdi setting 0=Auto, 1=MDI, 2=MDIX */
} AutonegotiationModeType6_t;

#define STID_AUTONEGOTIATION_MODE_TYPE6 \
    MAKE_STRUCTURE_ID(eAutonegotiationModeType6, AutonegotiationModeType6_t)

/******************************************************************/
/* Structure(s) to be used with Set/GetCardConfigurationDetails() */
/******************************************************************/
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef enum {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FEED_MODE_INLINE = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FEED_MODE_END_STATION,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FEED_MODE_CUSTOM_CONFIGURATION,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} FeedMode_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FeedMode_t    feedMode DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t      enableGlobalHardwareCalculations DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} FeedTransmitConfigurationType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_FEED_TRANSMIT_CONFIGURATION_TYPE1 MAKE_STRUCTURE_ID(eFeedTransmitConfigurationType1, FeedTransmitConfigurationType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef enum {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FLOW_CONTROL_RESPONSE_MODE_NONE = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FLOW_CONTROL_RESPONSE_MODE_OUR_MAC,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FLOW_CONTROL_RESPONSE_MODE_ANY_MAC,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} FlowControlResponseMode_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef enum {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FLOW_CONTROL_GENERATION_MODE_NONE = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FLOW_CONTROL_GENERATION_MODE_BUFFER_LEVEL_LOW,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FLOW_CONTROL_GENERATION_MODE_BUFFER_LEVEL_HIGH,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} FlowControlGenerationMode_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef enum {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  PREAMBLE_MODE_4_BYTES = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  PREAMBLE_MODE_5_BYTES,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  PREAMBLE_MODE_6_BYTES,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  PREAMBLE_MODE_7_BYTES,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} PreambleMode_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FlowControlResponseMode_t flowControlResponseMode DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FlowControlGenerationMode_t flowControlGenerationMode DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t            enableErroredFrameRemovalFromRetransmit DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  PreambleMode_t        preambleMode DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  uint32_t          globalIPG DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t            enableDescriptorControlledIPG DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t            enableGlobalHardwareCalculations DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} FeedTransmitConfigurationType2_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_FEED_TRANSMIT_CONFIGURATION_TYPE2 MAKE_STRUCTURE_ID(eFeedTransmitConfigurationType2, FeedTransmitConfigurationType2_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define MAX_LOCAL_TRANSMIT_CHANNELS   8
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  uint32_t  localTransmitMappingBitmask[ MAX_LOCAL_TRANSMIT_CHANNELS ]  DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} LocalTransmitConfigurationType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_LOCAL_TRANSMIT_CONFIGURATION_TYPE1 MAKE_STRUCTURE_ID(eLocalTransmitConfigurationType1, LocalTransmitConfigurationType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define MAX_LOGICAL_CHANNELS        8
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  uint32_t  channelPortMappingBitmask[ MAX_LOGICAL_CHANNELS ]  DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} ReceiveChannelPortConfigurationType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_RECEIVE_CHANNEL_PORT_CONFIGURATION_TYPE1 MAKE_STRUCTURE_ID(eReceiveChannelPortConfigurationType1, ReceiveChannelPortConfigurationType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  uint32_t  channelPortMappingBitmask[ MAX_LOGICAL_CHANNELS ]  DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} TransmitChannelPortConfigurationType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_TRANSMIT_CHANNEL_PORT_CONFIGURATION_TYPE1 MAKE_STRUCTURE_ID(eTransmitChannelPortConfigurationType1, TransmitChannelPortConfigurationType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t    enableISL  DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t    enableVLAN  DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t    enableJUMBO  DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t    allowISLVLAN DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t    allowISLJUMBO DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t    allowJUMBOVLAN DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t    ISL64IsSmall DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  bool_t    VLAN64IsSmall DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} FrameDecoderConfigurationType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_FRAME_DECODER_CONFIGURATION_TYPE1 MAKE_STRUCTURE_ID(eFrameDecoderConfigurationType1, FrameDecoderConfigurationType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#endif //NT_LEGACY_4_25

typedef enum
{
  DECODER_MODE_USE_LEGACY=0,           /* Setting this, sets the internal definition of Jumbo to validate
                                          against VLAN and ISL frames values inconjunction with enabling of
                                          VLAN/ISL/JUMBO, this also sets the internal definition of Large
                                          to take into account and VLAN or ISL or JUMBO frame sizes.
                                          This is the default mode of operation.
                                          This can also be viewed as the legacy mode of operation */

  DECODER_MODE_USE_RAW_DISTRIBUTION=1,  /* Setting this, sets the internal definition of JUMBO to all
                                           frames above 1518, assuming Jumbo is enabled,
                                           this also sets the internal definition of Large to the value
                                           supplied by way of 'maxNormalFrameSize'
                                           When using this mode you can take advantage of the new statistic counters
                                           available in foo */
} DecoderMode_t ;

typedef struct
{
  uint32_t  maxNormalFrameSize;     /* This value can be used to change the default frame size at
                                       which Large frames are defined. i.e. if you set this value to 2000
                                       the Large frame detection will only kick in once a frame > 2000 bytes.
                                       The valid range for this value is 1518->'max frame size' as determined
                                       from the cardcapabilities interface
                                       NOTE: This value ONLY affects the definition of Large, not jumbo */
  DecoderMode_t decoderMode;
} FrameDecoderConfigurationType2_t;

#define STID_FRAME_DECODER_CONFIGURATION_TYPE2 \
    MAKE_STRUCTURE_ID(eFrameDecoderConfigurationType2, FrameDecoderConfigurationType2_t)

typedef struct
{
  uint32_t  maxNormalFrameSize;     /* This value can be used to change the default frame size at
                                      which Large frames are defined. i.e. if you set this value to 2000
                                      the Large frame detection will only kick in once a frame > 2000 bytes.
                                      The valid range for this value is 1518->'max frame size' as determined
                                      from the cardcapabilities interface
                                      NOTE: This value ONLY affects the definition of Large, not jumbo */
  DecoderMode_t decoderMode;

  bool_t    enableISL;            /* Enables detection of ISL frames. */
  bool_t    enableVLAN;           /* Enables detection of VLAN tagged frames. */
  bool_t    enableJUMBO;          /* Enables detection of Jumbo frames. */
  bool_t    allowISLVLAN;         /* Enables VLAN tagged frame detection within ISL frames. */
  bool_t    allowISLJUMBO;        /* Enables JUMBO frame detection within ISL frames. */
  bool_t    allowJUMBOVLAN;       /* Enables JUMBO frame detection for VLAN tagged frames. */
  bool_t    ISL64IsSmall;         /* Changes the small packet decode criteria for ISL frames. */
  bool_t    VLAN64IsSmall;        /* Changes the small packet decode criteria for VLAN frames */
} FrameDecoderConfigurationType3_t;

#define STID_FRAME_DECODER_CONFIGURATION_TYPE3 \
    MAKE_STRUCTURE_ID(eFrameDecoderConfigurationType3, FrameDecoderConfigurationType3_t)

typedef struct
{
  bool_t  enableEndianSwap;                /* This enables endian swapping of packet data */
  bool_t  enableIpAlignment;               /* This enables pre-pending of padding bytes to the packet data to provide alignment of IP data. */
                                           /* The amount of padding is dependent on the layer 2 protocol and layer 2 encapsulation detected */
                                           /* within the packet */
  bool_t  enableMPLS;                      /* Enables detection of MPLS frames. */
}FrameDecoderConfigurationType4_t;

#define STID_FRAME_DECODER_CONFIGURATION_TYPE4 \
    MAKE_STRUCTURE_ID(eFrameDecoderConfigurationType4, FrameDecoderConfigurationType4_t)

typedef struct
{
  bool_t  enableEndianSwap;         /* This enables endian swapping of packet data */
  bool_t  enableIpAlignment;        /* This enables pre-pending of padding bytes to the packet data to provide alignment of IP data.
                                       The amount of padding is dependent on the layer 2 protocol and layer 2 encapsulation detected
                                       within the packet */
  uint32_t  maxNormalFrameSize;     /* This value can be used to change the default frame size at
                                       which Large frames are defined. i.e. if you set this value to 2000
                                       the Large frame detection will only kick in once a frame > 2000 bytes.
                                       The valid range for this value is 1518->'max frame size' as determined
                                       from the cardcapabilities interface
                                       NOTE: This value ONLY affects the definition of Large, not jumbo */
  DecoderMode_t decoderMode;
  bool_t    enableMPLS;             /* Enables detection of MPLS frames. */
  bool_t    enableISL;              /* Enables detection of ISL frames. */
  bool_t    enableVLAN;             /* Enables detection of VLAN tagged frames. */
  bool_t    enableJUMBO;            /* Enables detection of Jumbo frames. */
  bool_t    allowISLVLAN;           /* Enables VLAN tagged frame detection within ISL frames. */
  bool_t    allowISLJUMBO;          /* Enables JUMBO frame detection within ISL frames. */
  bool_t    allowJUMBOVLAN;         /* Enables JUMBO frame detection for VLAN tagged frames. */
  bool_t    ISL64IsSmall;           /* Changes the small packet decode criteria for ISL frames. */
  bool_t    VLAN64IsSmall;          /* Changes the small packet decode criteria for VLAN frames */
}FrameDecoderConfigurationType5_t;

#define STID_FRAME_DECODER_CONFIGURATION_TYPE5 \
    MAKE_STRUCTURE_ID(eFrameDecoderConfigurationType5, FrameDecoderConfigurationType5_t)

/* Type 6 is to be used for NT Series cards. */
typedef struct
{
  DecoderMode_t decoderMode;
  uint32_t  maxNormalFrameSize;   /* This value can be used to change the default frame size at
                                     which Large frames are defined. i.e. if you set this value to 2000
                                     the Large frame detection will only kick in once a frame > 2000 bytes.
                                     The valid range for this value is 1518->'max frame size' as determined
                                     from the cardcapabilities interface
                                     NOTE This value ONLY affects the definition of Large, not jumbo */

  bool_t    enableISL;            /* Enables detection of ISL frames. */
  bool_t    enableVLAN;           /* Enables detection of VLAN tagged frames. */
  bool_t    enableMPLS;           /* Enables detection of MPLS frames. */
  bool_t    enableJUMBO;          /* Enables detection of Jumbo frames. */
  bool_t    allowISLVLAN;         /* Enables VLAN tagged frame detection within ISL frames. */
  bool_t    allowISLJUMBO;        /* Enables JUMBO frame detection within ISL frames. */
  bool_t    allowJUMBOVLAN;       /* Enables JUMBO frame detection for VLAN tagged frames. */
  bool_t    allowJUMBOMPLS;       /* Enables JUMBO frame detection for MPLS frames. */
  bool_t    ISL64IsSmall;         /* Changes the small packet decode criteria for ISL frames. */
  bool_t    VLAN64IsSmall;        /* Changes the small packet decode criteria for VLAN frames */
} FrameDecoderConfigurationType6_t;

#define STID_FRAME_DECODER_CONFIGURATION_TYPE6 \
    MAKE_STRUCTURE_ID(eFrameDecoderConfigurationType6, FrameDecoderConfigurationType6_t)

/* Type 7 is to be used for NT Series cards. */
typedef struct
{
  DecoderMode_t decoderMode;
  uint32_t  maxNormalFrameSize;   /* This value can be used to change the default frame size at
                                     which Large frames are defined. i.e. if you set this value to 2000
                                     the Large frame detection will only kick in once a frame > 2000 bytes.
                                     The valid range for this value is 1518->'max frame size' as determined
                                     from the cardcapabilities interface
                                     NOTE This value ONLY affects the definition of Large, not jumbo */

  bool_t    enableISL;            /* Enables detection of ISL frames. */
  bool_t    enableVLAN;           /* Enables detection of VLAN tagged frames. */
  bool_t    enableMPLS;           /* Enables detection of MPLS frames. */
  bool_t    enableJUMBO;          /* Enables detection of Jumbo frames. */
  bool_t    allowISLVLAN;         /* Enables VLAN tagged frame detection within ISL frames. */
  bool_t    allowISLJUMBO;        /* Enables JUMBO frame detection within ISL frames. */
  bool_t    allowJUMBOVLAN;       /* Enables JUMBO frame detection for VLAN tagged frames. */
  bool_t    allowJUMBOMPLS;       /* Enables JUMBO frame detection for MPLS frames. */
  bool_t    ISL64IsSmall;         /* Changes the small packet decode criteria for ISL frames. */
  bool_t    VLAN64IsSmall;        /* Changes the small packet decode criteria for VLAN frames */
  bool_t    MPLS64IsSmall;        /* Changes the small packet decode criteria for MPLS frames */
} FrameDecoderConfigurationType7_t;

#define STID_FRAME_DECODER_CONFIGURATION_TYPE7 \
    MAKE_STRUCTURE_ID(eFrameDecoderConfigurationType7, FrameDecoderConfigurationType7_t)

/* Type 8 is to be used for NT Series cards. */
typedef struct
{
  DecoderMode_t decoderMode;
  uint32_t  maxNormalFrameSize;   /* This value can be used to change the default frame size at
                                     which Large frames are defined. i.e. if you set this value to 2000
                                     the Large frame detection will only kick in once a frame > 2000 bytes.
                                     The valid range for this value is 1518->'max frame size' as determined
                                     from the cardcapabilities interface
                                     NOTE This value ONLY affects the definition of Large, not jumbo */

  bool_t    enableISL;            /* Enables detection of ISL frames. */
  bool_t    enableVLAN;           /* Enables detection of VLAN tagged frames. */
  bool_t    enableMPLS;           /* Enables detection of MPLS frames. */
  bool_t    enableJUMBO;          /* Enables detection of Jumbo frames. */
  bool_t    allowISLVLAN;         /* Enables VLAN tagged frame detection within ISL frames. */
  bool_t    allowISLJUMBO;        /* Enables JUMBO frame detection within ISL frames. */
  bool_t    allowJUMBOVLAN;       /* Enables JUMBO frame detection for VLAN tagged frames. */
  bool_t    allowJUMBOMPLS;       /* Enables JUMBO frame detection for MPLS frames. */
  bool_t    ISL64IsSmall;         /* Changes the small packet decode criteria for ISL frames. */
  bool_t    VLAN64IsSmall;        /* Changes the small packet decode criteria for VLAN frames */
  bool_t    MPLS64IsSmall;        /* Changes the small packet decode criteria for MPLS frames */
  uint16_t  altTPID;              /* Alternative VLAN TPID */
} FrameDecoderConfigurationType8_t;

#define STID_FRAME_DECODER_CONFIGURATION_TYPE8 \
    MAKE_STRUCTURE_ID(eFrameDecoderConfigurationType8, FrameDecoderConfigurationType8_t)

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FeedHandle      feedHandle DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  uint32_t      dataSetSize DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  uint32_t      preloadValue DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  uint32_t      periodicUpdateFrequency DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  uint32_t      baselineThreshold DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} PriorityControlConfigurationType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_PRIORITY_CONTROL_CONFIGURATION_TYPE1 MAKE_STRUCTURE_ID(ePriorityControlConfigurationType1, PriorityControlConfigurationType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef enum {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  PRIORITY_CONTROL_MODE_INSTANT_UPDATE = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  PRIORITY_CONTROL_MODE_PAUSE_PERIODIC_UPDATE,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  PRIORITY_CONTROL_MODE_RESUME_PERIODIC_UPDATE,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} PriorityControlMode_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FeedHandle        feedHandle DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  PriorityControlMode_t priorityControlMode DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} PriorityControlConfigurationType2_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_PRIORITY_CONTROL_CONFIGURATION_TYPE2 MAKE_STRUCTURE_ID(ePriorityControlConfigurationType2, PriorityControlConfigurationType2_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  FeedHandle      feedHandle DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  uint32_t      hwSamplePeriod DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */} PriorityControlConfigurationType3_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#define STID_PRIORITY_CONTROL_CONFIGURATION_TYPE3 MAKE_STRUCTURE_ID(ePriorityControlConfigurationType3, PriorityControlConfigurationType3_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */#endif //NT_LEGACY_4_25

typedef struct
{
  FeedHandle feedHandle;  /* The feed handle of a packet feed that is
                             to make use the policy control feature          */
  uint32_t BlockLevel;    /* When the SDRAM is filled above this level,
                             all traffic is discarded regardless of channel
                             policy setting. Level is in bytes.              */
  uint32_t PoliceLevel;   /* When the SDRAM is filled above this level the
                             policing start and discard packets according to
                             the channel policy settings.
                             Level is in bytes.                              */
  uint32_t RestartLevel;  /* If BlockLevel is reached all packets will be
                             discarded until the level drops below the
                             restart level. (Hysteresis)
                             Level is in bytes.                              */
  uint8_t EnableLevels;   /* Bit 0: Enable restartlevel.
                                    If not enabled only data above Block level
                                    will be discarded. No hysteresis.
                             Bit 1: Enale policing level.
                                    If not enabled the police level will be
                                    ignored.
                             Bit 2: Enable block level.
                                    If not set the block level is ignored and
                                    packets will be discarded when the SDRAM
                                    is full.                                 */
  struct {
    uint32_t BucketSize;  /* Size of the bucket. Equivalent to the number of
                             bytes that can be received before packets are
                             discarded.                                      */
    uint32_t Bandwidth;   /* The channel bandwidth in kbps. If the SDRAM reach
                             police level everything above this bandwidth is
                             discarded.                                      */
  } channelPolicy[8];     /* Max 8 channles.                                 */
} SDRAMPolicyConfigurationType1_t;

#define STID_SDRAM_POLICY_CONFIGURATION_TYPE1 \
    MAKE_STRUCTURE_ID(eSDRAMPolicyConfigurationType1, SDRAMPolicyConfigurationType1_t)

typedef struct
{
  bool_t   sdramEnable;             /* Set to TRUE to enable the SDRAM controller.
                                       NOTE: It can only be set to TRUE. */
} SDRAMConfigurationType1_t;

#define STID_SDRAM_CONFIGURATION_TYPE1 \
    MAKE_STRUCTURE_ID(eSDRAMConfigurationType1, SDRAMConfigurationType1_t)

typedef enum
{
  TIMESYNC_COMMAND_CONFIGURE = 0,
  TIMESYNC_COMMAND_RESET_STAT_COUNTER,
  TIMESYNC_COMMAND_PPS_INITIALIZE,
  TIMESYNC_COMMAND_PPS_MEASURE_ADJUST
} TimeSyncCommand_t;

typedef struct
{
  TimeSyncCommand_t  timesyncCommandType;      /* Configuration command type */
  bool_t             timesyncConfigMaster;     /* Configuration parameter.
                                                  TRUE : adapter is configured as TimeSync Master.
                                                  FALSE: adapter is configured as TimeSync Slave */
  uint32_t           timesyncStatus;           /* Status return value */
} TimeSyncConfigurationType1_t;

#define STID_TIMESYNC_CONFIGURATION_TYPE1 \
    MAKE_STRUCTURE_ID(eTimeSyncConfigurationType1, TimeSyncConfigurationType1_t)

/****************************************************************/
/* Structure(s) to be used with Set/GetTimestampConfiguration() */
/****************************************************************/
typedef struct
{
  uint64_t        timestampValue;       /* The timestamp value as used in the hardware */
  TimestampType_t timestampType;        /* see commontypes.h for a list of timestamp types. */
} TimestampConfigurationType1_t;

#define STID_TIMESTAMP_CONFIGURATION_TYPE1 \
    MAKE_STRUCTURE_ID(eTimestampConfigurationType1, TimestampConfigurationType1_t)

typedef struct
{
  TimeSyncCommand_t  timesyncCommandType;      /* Configuration command type */
  bool_t             timesyncConfigMaster;     /* Configuration parameter.
                                                  TRUE : adapter is configured as TimeSync Master.
                                                  FALSE: adapter is configured as TimeSync Slave */
  bool_t link_active_status;                   /* TRUE : Link is active
                                                  FALSE: Link is inactive */
  bool_t slave_in_sync;                        /* TRUE : Slave is in synchronization
                                                  FALSE: Slave is NOT synchronization */
  bool_t GPS_Box_PPS_Ok;                       /* TRUE : PPS is OK
                                                  FALSE: PPS is NOT OK*/
  bool_t GPS_Box_In_sync;                      /* TRUE : GPS box is in synchronization
                                                  FALSE: GPS box is NOT in synchronization */
  bool_t GPS_Box_Serial_Stream_OK;             /* TRUE : Serial stream is OK
                                                  FALSE: Serial stream is not OK */
  bool_t OS_Mode_Sync_enabled;                 /* TRUE : OS mode is enabled
                                                  FALSE: OS mode is not enabled */
  bool_t OS_Mode_in_Sync;                      /* TRUE : OS mode is in synchronization
                                                  FALSE: OS mode is not in synchronization */
  bool_t GPS_Box_is_Master;                    /* TRUE : GPS box is master
                                                  FALSE: GPS box is not master */
  bool_t PTA_is_Master;                        /* TRUE : PTA is master
                                                  FALSE: PTA is not master */
  uint8_t Master_ID_Code;                      /* ID code for the Master */
} TimeSyncConfigurationType2_t;

#define STID_TIMESYNC_CONFIGURATION_TYPE2 \
    MAKE_STRUCTURE_ID(eTimeSyncConfigurationType2, TimeSyncConfigurationType2_t)

typedef struct
{
  TimeSyncCommand_t  timesyncCommandType;      /* Configuration command type */
  bool_t             timesyncConfigMaster;     /* Configuration parameter.
                                                  TRUE : adapter is configured as TimeSync Master.
                                                  FALSE: adapter is configured as TimeSync Slave */
  bool_t link_active_status;                   /* TRUE : Link is active
                                                  FALSE: Link is inactive */
  bool_t slave_in_sync;                        /* TRUE : Slave is in synchronization
                                                  FALSE: Slave is NOT synchronization */
  bool_t GPS_Box_PPS_Ok;                       /* TRUE : PPS is OK
                                                  FALSE: PPS is NOT OK*/
  bool_t GPS_Box_In_sync;                      /* TRUE : GPS box is in synchronization
                                                  FALSE: GPS box is NOT in synchronization */
  bool_t GPS_Box_Serial_Stream_OK;             /* TRUE : Serial stream is OK
                                                  FALSE: Serial stream is not OK */
  bool_t OS_Mode_Sync_enabled;                 /* TRUE : OS mode is enabled
                                                  FALSE: OS mode is not enabled */
  bool_t OS_Mode_in_Sync;                      /* TRUE : OS mode is in synchronization
                                                  FALSE: OS mode is not in synchronization */
  bool_t GPS_Box_is_Master;                    /* TRUE : GPS box is master
                                                  FALSE: GPS box is not master */
  bool_t PTA_is_Master;                        /* TRUE : PTA is master
                                                  FALSE: PTA is not master */
  uint8_t Master_ID_Code;                      /* ID code for the Master */
  bool_t valid_external_source;                /* TRUE : The external source is valid
                                                  FALSE: The external source is NOT valid */
  uint64_t External_Time;                      /* Last received External time sampled at PPS */
  uint64_t Adapter_Time;                       /* Adapter time sampled at PPS */
} TimeSyncConfigurationType3_t;

#define STID_TIMESYNC_CONFIGURATION_TYPE3 \
    MAKE_STRUCTURE_ID(eTimeSyncConfigurationType3, TimeSyncConfigurationType3_t)

typedef struct
{
  TimeSyncCommand_t  timesyncCommandType;      /* Configuration command type */
  bool_t             timesyncConfigMaster;     /* Configuration parameter.
                                                  TRUE : adapter is configured as TimeSync Master.
                                                  FALSE: adapter is configured as TimeSync Slave */
  bool_t link_active_status;                   /* TRUE : Link is active
                                                  FALSE: Link is inactive */
  bool_t slave_in_sync;                        /* TRUE : Slave is in synchronization
                                                  FALSE: Slave is NOT synchronization */
  bool_t GPS_Box_PPS_Ok;                       /* TRUE : PPS is OK
                                                  FALSE: PPS is NOT OK*/
  bool_t GPS_Box_In_sync;                      /* TRUE : GPS box is in synchronization
                                                  FALSE: GPS box is NOT in synchronization */
  bool_t GPS_Box_Serial_Stream_OK;             /* TRUE : Serial stream is OK
                                                  FALSE: Serial stream is not OK */
  bool_t OS_Mode_Sync_enabled;                 /* TRUE : OS mode is enabled
                                                  FALSE: OS mode is not enabled */
  bool_t OS_Mode_in_Sync;                      /* TRUE : OS mode is in synchronization
                                                  FALSE: OS mode is not in synchronization */
  bool_t GPS_Box_is_Master;                    /* TRUE : GPS box is master
                                                  FALSE: GPS box is not master */
  bool_t PTA_is_Master;                        /* TRUE : PTA is master
                                                  FALSE: PTA is not master */
  uint8_t Master_ID_Code;                      /* ID code for the Master */
  bool_t valid_external_source;                /* TRUE : The external source is valid
                                                  FALSE: The external source is NOT valid */
  uint64_t External_Time;                      /* Last received External time sampled at PPS */
  uint64_t Adapter_Time;                       /* Adapter time sampled at PPS */
  uint64_t Reference_Time;                     /* External time device time at PPS */
  int64_t Clock_Skew;                          /* Clock Skew */
} TimeSyncConfigurationType4_t;

#define STID_TIMESYNC_CONFIGURATION_TYPE4 \
    MAKE_STRUCTURE_ID(eTimeSyncConfigurationType4, TimeSyncConfigurationType4_t)

/****************************************************************/
/* Structure(s) to be used with Set/GetTimestampConfiguration() */
/****************************************************************/
typedef struct
{
  uint32_t    cableLength;        /* The cable length in meters */
  uint32_t    numberOfGPSBoxes;   /* The number of GPS boxes    */
} TimestampOffsetConfigurationType1_t;

#define STID_TIMESTAMP_OFFSET_CONFIGURATION_TYPE1 \
    MAKE_STRUCTURE_ID(eTimestampOffsetConfigurationType1, TimestampOffsetConfigurationType1_t)

/****************************************************************/
/* Structure(s) to be used with GetTimestampConfiguration() */
/****************************************************************/
typedef struct
{
  uint32_t  timeSyncProtocol;            /* 0 = NT timesync, 1 = OS TimesyncTS, 2 = PPS Timesync,  */
                                         /* 3 = IEEE1588.                                          */
  uint32_t  timeSyncConnectorIn;         /* 0 = External, 1 = Internal1, 2 = Internal2, 3 = None   */
  uint32_t  timeSyncConnectorOut;        /* 0 = None, 1 = External, 2 = Internal1, 4 = Internal2   */
  uint32_t  timeSyncConnectorRepeat;     /* 0 = None, 1 = External, 2 = Internal1, 4 = Internal2   */
  uint32_t  timeSyncTimeJumpThreshold;   /* [0..3600]                                              */
  uint32_t  timeSyncTimeOffset;          /* [0..255]                                               */
  uint32_t  highFreqSampling;            /* 0 = disabled, 1 = enabled.                             */
  uint32_t  osTimeSyncFailover;          /* 0 = disabled, 1 = enabled.                             */
} TimeSyncInfoType1_t;

#define STID_TIME_SYNC_INFO_TYPE1 \
    MAKE_STRUCTURE_ID(eTimeSyncInfoType1, TimeSyncInfoType1_t)

typedef struct
{
  uint32_t  timeSyncProtocol;            /* 0 = NT timesync, 1 = OS TimesyncTS, 2 = PPS Timesync,  */
                                         /* 3 = IEEE1588.                                          */
  uint32_t  timeSyncConnectorIn;         /* 0 = External, 1 = Internal1, 2 = Internal2, 3 = None   */
  uint32_t  timeSyncConnectorOut;        /* 0 = None, 1 = External, 2 = Internal1, 4 = Internal2   */
  uint32_t  timeSyncConnectorRepeat;     /* 0 = None, 1 = External, 2 = Internal1, 4 = Internal2   */
  uint32_t  timeSyncTimeJumpThreshold;   /* [0..3600]                                              */
  uint32_t  timeSyncTimeOffset;          /* [0..255]                                               */
  uint32_t  highFreqSampling;            /* 0 = disabled, 1 = enabled.                             */
  uint32_t  osTimeSyncFailover;          /* 0 = disabled, 1 = enabled.                             */
  uint32_t  hardSyncCounter;             /* The number of NT-TS hard synchronization events        */
} TimeSyncInfoType2_t;

#define STID_TIME_SYNC_INFO_TYPE2 \
    MAKE_STRUCTURE_ID(eTimeSyncInfoType2, TimeSyncInfoType2_t)

/****************************************************************/
/* Structure(s) used to toggle tx laser                         */
/****************************************************************/
typedef struct
{
  uint32_t    txLaserDisable; /* 0:disable tx laser, 1:enable tx laser */
} TxLaserConfigurationType1_t;

#define STID_TX_LASER_CONFIGURATION_TYPE1 \
    MAKE_STRUCTURE_ID(eTxLaserConfigurationType1, TxLaserConfigurationType1_t)

/****************************************************************/
/* Structure(s) used to enable port identification              */
/****************************************************************/
typedef struct
{
  uint32_t    portIdentificationEnableMask;  /* Enable port identification bitmask       */
                                             /* 0: port identification will be disabled  */
                                             /* 1: linkled on port will blink            */
} PortIdentificationConfigurationType1_t;

#define STID_PORT_IDENTIFICATION_CONFIGURATION_TYPE1 \
    MAKE_STRUCTURE_ID(ePortIdentificationConfigurationType1, PortIdentificationConfigurationType1_t)

/****************************************************************/
/* Structure(s) used for Failover (Bypass Adapter) configuration*/
/****************************************************************/
typedef struct
{
  uint8_t   currentMode;           /* Get   0 = Normal Mode,  1 = Bypass Mode,  2=Erroneous Mode (Switch pos inconsistent) */
  uint8_t   powerOnMode;           /* Get   0 = Normal Mode,  1 = Bypass Mode                                              */
  uint8_t   heartBeatTimeout;      /* Get   0 = Heartbeat disabled,  1-100 = heartbeat timeout value in 10 ms steps        */
  uint8_t   powerLossDetect;       /* Get   0 = Disable,  1 = Enable                                                       */
  uint8_t   powerLossSwitchDelay;  /* Get   0 = Immediate.   1-100 = delay in 100 ms steps                                 */
  uint8_t   autonomousMode;        /* Get   0 = Off.  1 = On    (Autonomous Mode is controlled by Dip Switch on PCB)       */
  uint32_t  fwVersion;             /* Get   Firmware version ID                                                            */
} FailoverConfigurationType1_t;

#define STID_FAILOVER_CONFIGURATION_TYPE1 \
    MAKE_STRUCTURE_ID(eFailoverConfigurationType1, FailoverConfigurationType1_t)

/***********************************************************************/
/* Structure(s) used for Failover (Bypass Adapter) heartbeat signaling */
/***********************************************************************/
typedef struct
{
  uint32_t    action;              /* 1 = Heartbeat */
} FailoverHeartbeatSignalType1_t;

#define STID_FAILOVER_SIGNAL_TYPE1 \
    MAKE_STRUCT_ID(CARD_CONFIGURATION_IF_ID, CC_FAILOVER_HEARTBEAT_SIGNAL, eFailoverHeartbeatSignalType1, FailoverHeartbeatSignalType1_t)

/****************************************************************/
/* Structure(s) used for adapter pairing features               */
/****************************************************************/
typedef struct
{
  uint32_t    action;
} AdapterPairConfigurationType1_t;

#define STID_ADAPTER_PAIR_CONFIGURATION_TYPE1 \
    MAKE_STRUCT_ID(CARD_CONFIGURATION_IF_ID, CC_ADAPTER_PAIR_CONFIGURATION, eAdapterPairConfigurationType1, AdapterPairConfigurationType1_t)

/****************************************************************/
/* Structure(s) used to enable port identification              */
/****************************************************************/
typedef struct
{
  uint32_t    timeSynchronizedTxPortEnableMask; /* Enable coordinated time synchronized transmit port bitmask */
                                                /* 0: Disabled                                               */
                                                /* 1: Enabled                                                */
  uint64_t    timeSynchronizedTxOffset;         /* Time offset between timestamp in first frame and nic time */
  bool_t      timeSynchronizedTxOffsetUpdate;   /* Set to TRUE to update the offset value                    */
} TimeSynchronizedTxConfigurationType1_t;

#define STID_TIME_SYNCHRONIZED_TX_CONFIGURATION_TYPE1 \
    MAKE_STRUCTURE_ID(eTimeSynchronizedTxConfigurationType1, TimeSynchronizedTxConfigurationType1_t)

#include "ntpackpop.h"

#endif // __NTAPI_CARDCONFIGURATIONSTRUCTURE_H__
