// $Id: cardinfo.h 25387 2013-09-10 08:16:36Z vmi $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This file implements the NTCI card and driver info ioctl functions.
 */

#ifndef __NTAPI_CARDINFO_H__
#define __NTAPI_CARDINFO_H__

#include <ostype.h>
#include "basetype.h"
#include "memoryaccess.h"
#include "ntpackpush.h"

#ifdef WIN32
#pragma warning( once : 4214 ) // warning C4214: bit-field types other than int
#endif

/* The following enum block relates to the structure ID groups */
enum
{
  eVersionInfo,
  eVersionInfoType1,
  eLogType1
};

#ifndef NT_LEGACY_4_25
typedef struct _NTCIVersionInfo {
    uint32_t reserved0;
    uint32_t reserved1;
    uint32_t reserved2;
    uint32_t reserved3;
    uint16_t ProtocolMajorVersion;
    uint16_t ProtocolMinorVersion;
    uint16_t ProtocolFeatureVersion;
    uint16_t ProtocolBuildVersion;
    uint16_t MiniportMajorVersion;
    uint16_t MiniportMinorVersion;
    uint16_t MiniportFeatureVersion;
    uint16_t MiniportBuildVersion;
} NTCI_VERSION_INFO;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #else //NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct _XYCIVersionInfo {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint32_t XYCommonInterfaceDLLMajorVersion DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint32_t XYCommonInterfaceDLLInterfacesVersion DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint32_t XYCommonInterfaceDLLStructuresVersion DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint32_t XYCommonInterfaceDLLBuildVersion;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint16_t ProtocolMajorVersion;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint16_t ProtocolMinorVersion;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint16_t ProtocolFeatureVersion;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint16_t ProtocolBuildVersion;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint16_t MiniportMajorVersion;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint16_t MiniportMinorVersion;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint16_t MiniportFeatureVersion;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint16_t MiniportBuildVersion;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } XYCI_VERSION_INFO  DEPRECATED_4_25, *PXYCI_VERSION_INFO  DEPRECATED_4_25, NTCI_VERSION_INFO;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

#define STID_CARD_INFO_VERSION_INFO\
    MAKE_STRUCTURE_ID(eVersionInfo, NTCI_VERSION_INFO)

typedef struct _NTCIVersionInfoType1 {
  uint32_t NTCommonInterfaceMajorVersion;
  uint32_t NTCommonInterfaceMinorVersion;
  uint32_t NTCommonInterfaceFeatureVersion;
  uint32_t NTCommonInterfaceBuildVersion;
  char     NTCommonInterfaceReleasePackageInfo[100];

  uint32_t ProtocolMajorVersion;
  uint32_t ProtocolMinorVersion;
  uint32_t ProtocolFeatureVersion;
  uint32_t ProtocolBuildVersion;
  char     ProtocolReleasePackageInfo[100];

  uint32_t DriverMajorVersion;
  uint32_t DriverMinorVersion;
  uint32_t DriverFeatureVersion;
  uint32_t DriverBuildVersion;
  char     DriverReleasePackageInfo[100];
} NtVersionInfoType1_t;

#define STID_CARD_INFO_VERSION_INFO_TYPE1\
    MAKE_STRUCTURE_ID(eVersionInfoType1, NtVersionInfoType1_t)

/*-----------------------------------------------------------------------------
 * Definition of log levels.
 *---------------------------------------------------------------------------*/
typedef enum
{
  NT_LOG_ERROR=0x01,
  NT_LOG_WARNING=0x02,
  NT_LOG_INFO=0x04,
  NT_LOG_DEBUG=0x08,
  NT_LOG_UPDATEIMAGE=0x10,
  NT_LOG_DIAGNOSTICS=0x20,
  NT_LOG_AVR=0x40,
} eLogType;

#define LOG_ENTRY_SIZE_TYPE1 200 /* 200 chars to be on the safe size. */

typedef struct {
  uint64_t logType;                    /* eLogType.                             */
  uint64_t timestamp;                  /* 31:00 Seconds since 1. jan 1970.      */
                                       /* 63:32 uSecs                           */
  uint8_t logStr[LOG_ENTRY_SIZE_TYPE1];/* Log message.                          */
} CardInfoLogEntryType1_t;

typedef struct {
  uint32_t waitForNewEntry:1;    /* Wait until a new message is available?      */
  uint32_t clearAfterRead:1;     /* Should the log be cleared after the read.
                                    NOTE: This will have global effect on all apps
                                    getting logs.                               */
  uint32_t reserved:30;          /* Reserved.                                   */
  uint32_t numEntries;           /* Number of log entries.                      */
  ExternalBufferInfoType1_t log; /* The logs. (CardInfoLogEntryType1_t list)    */
} CardInfoLogType1_t;

#define STID_CARD_INFO_LOG_TYPE1 \
    MAKE_STRUCTURE_ID(eLogType1, CardInfoLogType1_t)

#include "ntpackpop.h"

#endif // __NTAPI_CARDINFO_H__
