#ifndef NT_LEGACY_4_25

#ifndef DEPRECATED_4_25
#warning: This file was part of a Xyratex specific framework and has been retired.
#warning: It is not and has never been required for Napatech Adapter operations.
#warning: Please remove this file from your project and recompile.
#warning: See file /include/xyratex_retired_readme.txt for more information.
#endif

#else //NT_LEGACY_4_25
#ifndef  __INCLUDED_EXTENDEDDESCRIPTOR01_H__
#define  __INCLUDED_EXTENDEDDESCRIPTOR01_H__

#include "basetype.h"

#include "ntpackpush.h"

#ifdef __cplusplus 
extern "C" {
#endif

#define EXTENDED_DESCRIPTOR_01_KNOWN

/* Uncomment the following line to enable the debug display function */
/*#define EXTENDED_DESCRIPTOR_01_DISPLAY(s,t) ext01_CheckNDisplayContents(s,t)*/


/* The following are the values to use with CreatePacketFeed in order to request this
    full extended descriptor*/

/* The following definitions should be used when creating a packet feed with one of the
	'FeedConfigSampledType' structures
  For creating a feed with this structure you need to set the 'descriptorType' (if applicable)
   to EXTENDED_DESCRIPTOR
  Use the following with the 'extendedDescriptorFormat' attribute */
#define EXTENDED_DESCRIPTOR_01_TYPE	0x01
/* Use the following with the 'extendedDescriptorLength' attribute 
    The extension length is 3x64bits, you can choose reduce the etended length by
    multiples of 64-bits -  note you will have to handle the structure overlay correctly*/
#define EXTENDED_DESCRIPTOR_01_LENGTH	(3)

/* Use the following with the 'extendedDescriptorConfig' attribute if applicable */
#define EXTENDED_DESCRIPTOR_01_CONFIG 0x017


/*
** This is the extended frame descriptor.  The frame data follows this extended descriptor.
** NOTE that the position of the frame data following this descriptor will be dependent on the
**  length of the extended descriptor requested.
*/
typedef struct tagPacketExtDescriptorType01
{
    uint32_t HashA;                         /* Hash caclulation on IP protocol, IPv4 address and udp/tcp port pairs bi-dir flow */
    uint32_t HashB;                         /* Hash caclulation on IP protocol, IPv4 address and udp/tcp port pairs uni-dir flow */

    uint32_t Layer2Encapsulation:4;         /* ext01_Layer2EncapsulationType_t */
    uint32_t Layer2Protocol:2;              /* ext01_Layer2ProtocolType_t */
    uint32_t Layer2AddressType:2;           /* ext01_Layer2AddressType_t */
    uint32_t Layer2HeaderOffset:8;          /* Offset in bytes to the Layer 2 Header (MAC Address) - compensates for encapsulation */

    uint32_t Layer3Protocol:4;              /* ext01_Layer3ProtocolType_t */
    uint32_t MACAddressSwap:1;              /* Indicates that the destination MAC address is a greater value than the source MAC address */
    uint32_t Reserved:1;
    uint32_t Layer3IPv4Checksum:2;          /* ext01_ChecksumType_t */
    uint32_t Layer3HeaderOffset:8;          /* Offset in bytes to the Layer 3 Header (IP, IPX, ...) - compensates for encapsulation */

    uint32_t Layer4Protocol:4;              /* ext00_Layer4ProtocolType_t */
    uint32_t IPFragmentation:2;             /* Indicates that the frame is part of a fragmented packet ext00_Layer4FragmentType_t */
    uint32_t Layer4IPv4Checksum:2;          /* ext00_ChecksumType_t */
    uint32_t Layer4HeaderOffset:8;          /* Offset in bytes to the Layer 4 Header (TCP, UDP, ...) - compensates for IP Header Options
                                               For an IPv4 frame then this is valid on the FIRST frame of a fragmented IP frame or non
                                               fragmented frame */

    uint32_t HashAValid:1;                  /* HashA calculation is valid */
    uint32_t HashBValid:1;                  /* HashB calculation is valid */
    uint32_t Reserved2:2;
    uint32_t IPAddressSwap:1;               /* If set the destination address is greater than the source address */
    uint32_t Reserved3:3;
    uint32_t HighLayerOffset:8;             /* Offset in bytes to higher layer TCP/UDP Data - Compensates for TCP/UDP Headers (if present) */

    uint64_t GroupMatch;                    /* 1 bit per Group Match Filter Definition, each of the 64 group matches will cause the appropriate
                                               bit to be set in this field if this match triggers a group match */
} PacketExtDescriptorType01_t;

/* Layer2 Encapsulation Type is a bitwise field, thus more 
   than one bit may be set at any one time */
typedef enum
{
	EXT01_VLAN	  = 1,
  EXT01_MPLS    = 2,
	EXT01_ISL	    = 4,
} ext01_Layer2EncapsulationType_t;

typedef enum
{
	EXT01_EII	= 0,
	EXT01_LLC   = 1,
	EXT01_SNAP	= 2,
	EXT01_RAW   = 3,
} ext01_Layer2ProtocolType_t;

typedef enum
{
	EXT01_UNICAST	= 0,
	EXT01_MULTICAST = 1,
	EXT01_BROADCAST	= 2,
} ext01_Layer2AddressType_t;

typedef enum
{
	EXT01_OTHER	= 0,
	EXT01_IPv4  = 1,
	EXT01_IPv6	= 2,
	EXT01_IPX   = 3,
	EXT01_ARP   = 4,
} ext01_Layer3ProtocolType_t;

typedef enum
{
	EXT01_CHKSUM_NOT_CHECKED = 0,
	EXT01_CHKSUM_GOOD        = 1,
	EXT01_CHKSUM_BAD	     = 2,
} ext01_ChecksumType_t;

typedef enum
{
	EXT01_TCP   = 1,
	EXT01_UDP	= 2,
	EXT01_ICMP  = 3,
} ext01_Layer4ProtocolType_t;

typedef enum
{
	EXT01_NO_FRAGMENT    = 0,
	EXT01_FIRST_FRAGMENT = 1,
	EXT01_FRAGMENT       = 2,
	EXT01_LAST_FRAGMENT  = 3,
} ext01_Layer4FragmentType_t;

/* --------------------------------------------------------------- */
/* Debug display to verify the contents of the extended descriptor */

#ifdef EXTENDED_DESCRIPTOR_01_DISPLAY
static void ext01_DisplayContents(void* data)
{
	PacketExtDescriptorType01_t* descriptorData = (PacketExtDescriptorType01_t*)data;

	printf( "-------------------------\n");
	printf( "GroupMatch               : 0x%X\n", descriptorData->GroupMatch );
	printf( "Layer2Encapsulation      : " );
		switch(descriptorData->Layer2Encapsulation)
		{
			case EXT01_VLAN: printf("VLAN\n"); break;
			case EXT01_ISL: printf("ISL\n"); break;
			default: printf("Other\n"); break;
		}	
	printf( "Layer2Protocol           : " );
		switch(descriptorData->Layer2Protocol)
		{
			case EXT01_EII: printf("EII\n"); break;
			case EXT01_LLC: printf("LLC\n"); break;
			case EXT01_SNAP: printf("SNAP\n"); break;
			case EXT01_RAW: printf("Raw\n"); break;
			default: printf("Other\n"); break;
		}	
	printf( "Layer2AddressType        : " );
		switch(descriptorData->Layer2AddressType)
		{
			case EXT01_UNICAST: printf("Unicast\n"); break;
			case EXT01_MULTICAST: printf("Multicast\n"); break;
			case EXT01_BROADCAST: printf("Broadcst\n"); break;
			default: printf("Other\n"); break;
		}	
	printf( "MACAddress swap flag     : " );
	    if(descriptorData->MACAddressSwap == 1)
	    {
	        printf( "Set\n" );
	    }
	    else
	    {
	        printf( "Not set\n" );
	    }
	printf( "Layer2HeaderOffset       : %d\n", descriptorData->Layer2HeaderOffset );
	printf( "Layer3Protocol           : ");
		switch(descriptorData->Layer3Protocol)
		{
			case EXT01_IPv4: printf("IPv4\n"); break;
			case EXT01_IPv6: printf("IPv6\n"); break;
			case EXT01_IPX: printf("IPX\n"); break;
			case EXT01_ARP: printf("ARP\n"); break;
			default: printf("Other\n"); break;
		}	
	printf( "Layer3IPv4Checksum       : " );
		switch(descriptorData->Layer3IPv4Checksum)
		{
			case EXT01_CHKSUM_NOT_CHECKED: printf("Not checked\n"); break;
			case EXT01_CHKSUM_GOOD: printf("GOOD\n"); break;
			case EXT01_CHKSUM_BAD: printf("BAD\n"); break;
			default: printf("NA\n"); break;
		}	
	printf( "Layer3HeaderOffset       : %d\n", descriptorData->Layer3HeaderOffset );
	printf( "Layer4Protocol           : " );
		switch(descriptorData->Layer4Protocol)
		{
			case EXT01_TCP: printf("TCP\n"); break;
			case EXT01_UDP: printf("UDP\n"); break;
			case EXT01_ICMP: printf("ICMP\n"); break;
			default: printf("Other\n"); break;
		}	
        printf( "IPv4 Fragmentation       : " );
		switch(descriptorData->IPFragmentation)
		{
			case EXT01_FIRST_FRAGMENT: printf("First fragment\n"); break;
			case EXT01_FRAGMENT: printf("Fragment\n"); break;
			case EXT01_LAST_FRAGMENT: printf("Last fragment\n"); break;
			default: printf("Not fragmented\n"); break;
		}	
	printf( "Layer4IPv4Checksum       : " );
		switch(descriptorData->Layer4IPv4Checksum)
		{
			case EXT01_CHKSUM_NOT_CHECKED: printf("Not checked\n"); break;
			case EXT01_CHKSUM_GOOD: printf("GOOD\n"); break;
			case EXT01_CHKSUM_BAD: printf("BAD\n"); break;
			default: printf("NA\n"); break;
		}	
	printf( "Layer4HeaderOffset       : %d\n", descriptorData->Layer4HeaderOffset );
	printf( "HighLayerOffset          : %d\n", descriptorData->HighLayerOffset );
    printf( "IPAddress swap flag      : " );
	    if(descriptorData->IPAddressSwap == 1)
	    {
	        printf( "Set\n" );
	    }
	    else
	    {
	        printf( "Not set\n" );
	    }
	printf( "HashA                    : 0x%4x - ", descriptorData->HashA );
	    if(descriptorData->HashAValid == 1)
		{
		    printf( "valid\n" );
		}
		else
		{
		    printf( "not valid\n" );
		}
	printf( "HashB                    : 0x%4x - ", descriptorData->HashB );
	    if(descriptorData->HashBValid == 1)
		{
		    printf( "valid\n" );
		}
		else
		{
		    printf( "not valid\n" );
		}
	printf( "-------------------------\n");
}

static void ext01_CheckNDisplayContents(int type, void* data)
{
	if (type == EXTENDED_DESCRIPTOR_01_TYPE)
	{
		ext01_DisplayContents(data);
	}
}
#endif

#ifdef __cplusplus 
}
#endif

#include "ntpackpop.h"

#endif /* __INCLUDED_EXTENDEDDESCRIPTOR01_H__ */
#endif //NT_LEGACY_4_25
