// $Id: extendeddescriptor09.h 16051 2011-11-30 12:20:27Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This file defines the format (structure definition) of the extended
 * descriptor with a type field of 0x09.
 * This structure definition can be overlaid onto the data that immeadiatly
 * follows the standard descriptor assuming that the user has created a packet
 * feed for which this extended descriptor has been requested.
 */

#ifndef  __NTAPI_EXTENDEDDESCRIPTOR09_H__
#define  __NTAPI_EXTENDEDDESCRIPTOR09_H__

#include "basetype.h"
#include "ntpackpush.h"

#define EXTENDED_DESCRIPTOR_09_KNOWN

/* The following are the values to use with CreatePacketFeed in order to request this
    full extended descriptor*/

/* The following definitions should be used when creating a packet feed with one of the
	'FeedConfigSampledType' structures
  For creating a feed with this structure you need to set the 'descriptorType' (if applicable)
   to EXTENDED_DESCRIPTOR
  Use the following with the 'extendedDescriptorFormat' attribute */
#define EXTENDED_DESCRIPTOR_09_TYPE	0x09

#define EXTENDED_DESCRIPTOR_09_LENGTH 0x03

// This structure represent RX fields in the Extended Descriptor Format 9.
// Access it through below union PacketExtDescriptorType09.rx mapped over the extended descriptor offset of each packet descriptor.
struct PacketExtDescriptorType09Rx
{
  /* 32bit Word0. */
  uint32_t hash:24;                 /* 23:00 - Hash value. */
  uint32_t hashType:5;              /* 28:24 - Hash type. */
  uint32_t Reserved0:2;             /* 30:29 - Reserved. */
  uint32_t hashValid:1;             /*    31 - Hash valid. zero if hash-config is None or inbound frame invalid for hash-calc, one otherwise. */
  /* 32bit Word1. */
  uint32_t jumbo:1;                 /*    00 - Jumbo frame. 1 when PktSz > 1518 (+ISL, VLAN, MPLS if set via CardConfigDecoderSet()) */
  uint32_t broadcastDest:1;         /*    01 - Destination MAC address is broadcast. (DMAC are all 1's) */
  uint32_t l4PortType:4;            /* 05:02 - Layer4 port type.  See ext09_Layer4PortType_t*/
  uint32_t l4FrameType:4;           /* 09:06 - Layer4 frame type. See ext09_Layer4FrameType_t; */
  uint32_t l3FrameType:3;           /* 12:10 - Layer3 type. 0:IPv4, 1:IPv6, 2:IPX, 3:Other*/
  uint32_t l2FrameType:2;           /* 14:13 - Layer2 type. (0:EtherII, 1:LLC, 2:SNAP, 3:Novell RAW) */
  uint32_t l4Size:4;                /* 18:15 - Layer4 header length (in units of 32bits) */
  uint32_t l3Size:7;                /* 25:19 - Layer3 header length (in units of 32bits). Note: Valid for L3=IPv4 and IPv6. */
  uint32_t mplsCount:3;             /* 28:26 - Number of MPLS shim labels present. */
  uint32_t vlanCount:2;             /* 30:29 - Number of VLANs present. */
  uint32_t islPresent:1;            /*    31 - ISL encapsulation present. */
  /* 32bit Word2. */
  uint32_t l3UnmatchedFragFeed:8;   /* 07:00 - Feed where unmatched fragments are stored */
  uint32_t Reserved1:4;             /* 11:08 - Reserved */
  uint32_t l3UnmatchedFragFlag:1;   /*    12 - L3 Unmatched fragment flag. Requires IPF engine on (see config file).
                                               If 1, and IP frame is a first fragment, then all subsequent fragments for this frame
                                                 are moved to the l3UnmatchedFragFeed. (FPGA table is full and cannot track more fragments).
                                               If 1, and IP frame is a non-first-fragment, then this fragment could not be located in
                                                 the FPGA table indicating first-fragment has not yet been received. Use l3UnmatchedFragFeed
                                                 field to locate the feed container where fragments arrive (except first-fragment once seen) */
  uint32_t l3LastFragment:1;        /*    13 - L3 Last Fragment flag (More-bit=0 && offset!=0) */
  uint32_t Reserved2:3;             /* 16:14 - Reserved */
  uint32_t frameProtSmall:1;        /*    17 - Frame is "protocol small". */
  uint32_t frameLarge:1;            /*    18 - Large frame. 1 when PktSz > MaxSz as configured with CardConfigMaxFrameSize() */
  uint32_t frameSmall:1;            /*    19 - Small frame. 1 when PktSz < 64 (+ISL and/or VLAN if set via CardConfigDecoderSet()) */
  uint32_t ipv6FragmentHeader:1;    /*    20 - IPv6 Fragment header present */
  uint32_t ipv6RoutingHeader:1;     /*    21 - IPv6 Routing header present */
  uint32_t l4ProtocolNumber:8;      /* 29:22 - Layer 4 Protocol Number (TCP:, UDP, SCTP etc.)*/
  uint32_t l3Fragmented:1;          /*    30 - L3 fragmented frame. (More-bit=1 || offset!=0) Note: Only valid for IPv4. */
  uint32_t l3FirstFragment:1;       /*    31 - L3 First fragment. (offset=0) Note: Only valid for IPv4. This bit is always set on IPv6. IPV6 use the ipv6FragmentHeader. */
  /* 32bit Word3. */                   
  uint32_t color:6;                 /* 05:00 - Color */
  uint32_t Reserved3:1;             /*    06 - Reserved */
  uint32_t l5Offset:9;              /* 15:07 - Layer5 offset in packet, offset is from end of extension header.
                                               Note: Valid when L4 is TCP/UDP and L3 is IPv4 or IPv6, otherwise zero! */
  uint32_t l4Offset:9;              /* 24:16 - Offset to the layer4 header. Offset is from end of extension header. */
  uint32_t l3Offset:7;              /* 31:25 - Offset to the layer3 header. Offset is from end of extension header. */
  /* 32bit Word4. */
  uint32_t dedupCrc:24;             /* 23:00 - Deduplication CRC */
  uint32_t Reserved4:8;             /* 31:24 - Reserved */
  /* 32bit Word5. */ 
  uint32_t Reserved5;               /* 31:00 - Reserved */
};

// This structure represent TX fields in the Extended Descriptor Format 9.
// Access it through below union PacketExtDescriptorType09.tx mapped over the extended descriptor offset of each packet descriptor.
struct PacketExtDescriptorType09Tx
{
  /* 32bit Word0. */
  uint32_t Unused0:32;              /* 31:00 - Unused */
  /* 32bit Word1. */
  uint32_t Unused1:6;               /* 05:00 - Unused */
  uint32_t l4FrameType:4;           /* 09:06 - Layer4 frame type. See ext09_Layer4FrameType_t; */
  uint32_t l3FrameType:3;           /* 12:10 - Layer3 type. 0:IPv4, 1:IPv6, 2:IPX, 3:Other*/
  uint32_t l2FrameType:2;           /* 14:13 - Layer2 type. (0:EtherII, 1:LLC, 2:SNAP, 3:Novell RAW) */
  uint32_t l4Size:4;                /* 18:15 - Layer4 header length (in units of 32bits). */
  uint32_t l3Size:7;                /* 25:19 - Layer3 header length (in units of 32bits). Note: Valid for L3=IPv4 and IPv6. */
  uint32_t mplsCount:3;             /* 28:26 - Number of MPLS shim labels present. */
  uint32_t vlanCount:2;             /* 30:29 - Number of VLANs present. */
  uint32_t islPresent:1;            /*    31 - ISL encapsulation present. */
  /* 32bit Word2. */
  uint32_t Unused2:14;              /* 13:00 - Unused */
  uint32_t txTsInject:1;            /*    14 - Inject TX timestamp.
                                               Note: The offset where the timestamp should be injected is located
                                                     at txTsInjectOffset/layer5HeaderOffset. */
  uint32_t calcUDPTCPChecksum:1;    /*    15 - This bit requests that the Tx function recalculates the UDP/TCP checksum (NT v41+ FPGA only) */
  uint32_t calcIPChecksum:1;        /*    16 - This bit requests that the Tx function recalculates the IP checksum      (NT v41+ FPGA only) */
  uint32_t Unused3:15;              /* 31:17 - Unused */
  /* 32bit Word3. */                    
  uint32_t Unused4:7;               /* 06:00 - Unused */
  uint32_t txTsInjectOffset:9;      /* 15:07 - TX timestamp inject offset. If txTsInject is set this is the
                                               offset where the adapter will inject a timestamp upon TX. */
  uint32_t Unused5:16;              /* 31:16 - Unused */
  /* 32bit Word4. */
  uint32_t Unused6;                 /* 31:00 - Unused */
  /* 32bit Word5. */
  uint32_t Unused7;                 /* 31:00 - Unused */
};

// Use this union to decode Extended Descriptor Format 9 for RX and TX directions alike.
typedef union PacketExtDescriptorType09 {
  struct PacketExtDescriptorType09Rx rx;
  struct PacketExtDescriptorType09Tx tx;
} PacketExtDescriptorType09_t;

/* Layer 2 types. */
typedef enum
{
  EXT09_L2_ETHER_II=0,
  EXT09_L2_LLC,
  EXT09_L2_SNAP,
  EXT09_L2_NOVELL_RAW
} ext09_Layer2FrameType_t;

/* Layer 3 types. */
typedef enum
{
  EXT09_L3_IPv4=0,
  EXT09_L3_IPv6,
  EXT09_L3_IPX,
  EXT09_L3_OTHER,
} ext09_Layer3FrameType_t;

/* Layer 4 frame types. */
typedef enum
{
  EXT09_L4_TCP=0,
  EXT09_L4_UDP,
  EXT09_L4_ICMP,
  EXT09_L4_OTHER,
  EXT09_L4_GRE,
  EXT09_L4_SCTP,
} ext09_Layer4FrameType_t;

/* Layer 4 port types. */
typedef enum
{
  EXT09_L4_PORT_OTHER=0,
  EXT09_L4_PORT_GTPV0_U,
  EXT09_L4_PORT_GTPV1V2_C,
  EXT09_L4_PORT_GTPV1_U_SIGNAL,
  EXT09_L4_PORT_GTPV1_U_GPDU
} ext09_Layer4PortType_t;

#include "ntpackpop.h"

#endif // __NTAPI_EXTENDEDDESCRIPTOR09_H__
