// $Id: cardconfigurationinterface.h 14870 2011-10-13 07:23:23Z bk $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This file contains the common types that can be used by any interface.
 */

#ifndef __CARDCONFIGURATIONINTERFACE_H__
#define __CARDCONFIGURATIONINTERFACE_H__

#include "ostype.h"
#include "basetype.h"

#if _NT_OS_TYPE == _NT_OS_TYPE_FREEBSD
  #define CARD_CONFIGURATION_INTERFACE_VERSION        2
#elif _NT_OS_TYPE == _NT_OS_TYPE_LINUX
  #define CARD_CONFIGURATION_INTERFACE_VERSION        1
#elif _NT_OS_TYPE == _NT_OS_TYPE_WIN
  #define CARD_CONFIGURATION_INTERFACE_VERSION        1
#endif

typedef struct cardConfigurationInterface
{
	uint16_t size;
	uint16_t version;
	void *context;

	NT_STATUS (*SetPortBypassMode)(void *context,
									uint32_t bypassDeviceId,
									uint32_t mode );

	NT_STATUS (*GetPortBypassMode)(void *Context,
									uint32_t bypassDeviceId,
									uint32_t *pMode);

	NT_STATUS (*ReadNetworkInterfaceModuleData)(void *context,
												 uint32_t channels,
												 uint16_t address,
												 uint8_t *data,
												 uint16_t *length);

	NT_STATUS (*WriteNetworkInterfaceModuleData)(void *context,
												  uint32_t channels, 
												  uint16_t address, 
												  uint8_t *data, 
												  uint16_t *length);

	NT_STATUS (*SetAutonegotiationMode)(void *context,
										 uint32_t channels, 
										 StructureID structid, 
										 void *data ) ;

	NT_STATUS (*GetAutonegotiationMode)(void *context,
										 uint32_t channels, 
										 StructureID structid, 
										 void *data ) ;

    NT_STATUS (*SetCardConfigurationDetails)(void* context, 
                                              uint32_t channels, 
                                              StructureID structid, 
                                              void* data);

    NT_STATUS (*GetCardConfigurationDetails)(void* context, 
                                              uint32_t channels, 
                                              StructureID structid, 
                                              void* data);

    NT_STATUS (*SetTimestampConfiguration)(void* context, 
                                            uint32_t deviceid, 
                                            StructureID structid, 
                                            void* data);

    NT_STATUS (*GetTimestampConfiguration)(void* context, 
                                            uint32_t deviceid, 
                                            StructureID structId, 
                                            void* data);

    NT_STATUS (*GetTxDelay)(void     *context,
                           uint32_t deviceid,
                           StructureID structid,
                           void     *data);

    NT_STATUS (*SetTxDelay)(void     *context,
                           uint32_t deviceid,
                           StructureID structid,
                           void     *data);
} CARD_CONFIGURATION_INTERFACE;


#endif // __CARDCONFIGURATIONINTERFACE_H__
