// $Id: kernelif.h 14206 2011-09-08 16:47:22Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This file contains the common types that can be used by any interface.
 */

#ifndef __KERNELIF_H__
#define __KERNELIF_H__

#include "ostype.h"
#include "basetype.h"
#include "NTCommonStructures.h"

#define INTERFACE_ACCESS_VERSION 1

typedef struct {
  uint32_t     Version;
  void*        Context;
  uint32_t OUT (*QueryControlInterface)(void* Context,
                                    uint32_t interfaceId, 
                                    void *interface);
  uint32_t OUT (*BindEventInterface)(void* Context,
                                     uint32_t interfaceId,
                                     void *eventInterface);
} NT_INTERFACE_ACCESS;

/*-----------------------------------------------------------------------------
  Function that external kernel kode must call release its bindings with the
  card.
  The function will close the card in use and return it to a know reset state.
  -----------------------------------------------------------------------------*/
uint32_t                                 /* Ret Result.                        */
NTSeries_ExtendedInterfaceAttach(
  uint32_t             dwDeviceIndex,    /* In  Device index.                  */
  NT_INTERFACE_ACCESS *pInterfaceAccess);/* In  Interface access structure.    */

/*-----------------------------------------------------------------------------
  Function that external kernel kode must call to get access to interface
  binding functions exposing the kernel level NTCI functions.
  The function will open an unused card and return binding functions and a
  card handle.
  ----------------------------------------------------------------------------*/
uint32_t                                 /* Ret Result.                       */
NTSeries_ExtendedInterfaceDetach(
  NT_INTERFACE_ACCESS *pInterfaceAccess);/* In  Interface access structure.   */

#endif // __KERNELIF_H__
