// $Id: packetfeedinterface.h 17405 2012-02-20 08:51:21Z bk $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This file contains the common types that can be used by any interface.
 */

#ifndef __PACKETFEEDINTERFACE_H__
#define __PACKETFEEDINTERFACE_H__

#include "ostype.h"
#include "basetype.h"

#ifdef _NT_DRIVER
  #define PACKET_FEED_INTERFACE_VERSION 2
#else
  #define PACKET_FEED_INTERFACE_VERSION 1
#endif

typedef struct packetfeedInterface
{
	uint16_t size;
	uint16_t version;
	void *context;

	/* Interface callbacks */
	NT_STATUS (*SetupPacketFeedEngine)( void *context, uint32_t numFeeds,
                                       uint32_t *priorities );
	NT_STATUS (*CreatePacketFeed)( void *context, uint32_t feedIndex, uint32_t channelBitmask,
                                  uint32_t Reserved0, uint32_t Reserved1, StructureID idFeedConfig, void *feedConfig, FeedHandle *pFeedHandle );
	NT_STATUS (*DestroyPacketFeed)( void *context, FeedHandle feedHandle );
#if _NT_OS_TYPE==_NT_OS_TYPE_WIN
	NT_STATUS (*StartPacketFeed)( void *context, FeedHandle feed, uint64_t event);
#else
  NT_STATUS (*StartPacketFeed)( void *context, FeedHandle feed);
#endif
	NT_STATUS (*GetPacketFeedBuffers)( void *context, FeedHandle feed,
                                      StructureID idFeedBufferInformation, void *feedBufferInformation );
	NT_STATUS (*StopPacketFeed)( void *context, FeedHandle feed);
	NT_STATUS (*GetNextBuffer)( void *context, FeedHandle feed,
                               StructureID idBufferRequest, void *bufferRequest, StructureID idBufferInformation, void *bufferInformation );
	NT_STATUS (*ReleaseBuffer)( void *context, FeedHandle feed,
                               StructureID idBufferInformation, void *bufferInformation );
	NT_STATUS (*GetPacketFeedCount)( void *context, uint32_t *numFeeds );
	NT_STATUS (*GetPacketFeedDescription)( void *context, uint32_t feedIndex,
                                          StructureID idFeedDescription, void *feedDescription );
	NT_STATUS (*GetPacketFeedStatus)( void *context, FeedHandle feed,
                                     StructureID idFeedStatus, void *feedStatus );
#if _NT_OS_TYPE==_NT_OS_TYPE_WIN
  NT_STATUS (*DataReadyEvent)(void *context, uint32_t cmd, void *datareadyEvents);
#endif

} PACKET_FEED_INTERFACE;

typedef struct packetfeedInterface_v2
{
	uint16_t size;
	uint16_t version;
	void *context;

	/* Interface callbacks */
	NT_STATUS (*SetupPacketFeedEngine)( void *context, uint32_t numFeeds,
                                       uint32_t *priorities );
	NT_STATUS (*CreatePacketFeed)( void *context, uint32_t feedIndex, uint32_t channelBitmask,
                                  uint32_t Reserved0, uint32_t Reserved1, StructureID idFeedConfig, void *feedConfig, FeedHandle *pFeedHandle );
	NT_STATUS (*DestroyPacketFeed)( void *context, FeedHandle feedHandle );
#if _NT_OS_TYPE==_NT_OS_TYPE_WIN
  NT_STATUS (*StartPacketFeed)( void *context, FeedHandle feed, uint64_t event);
#else
	NT_STATUS (*StartPacketFeed)( void *context, FeedHandle feed);
#endif
	NT_STATUS (*GetPacketFeedBuffers)( void *context, FeedHandle feed,
                                      StructureID idFeedBufferInformation, void *feedBufferInformation );
	NT_STATUS (*StopPacketFeed)( void *context, FeedHandle feed);
	NT_STATUS (*GetNextBuffer)( void *context, FeedHandle feed,
                               StructureID idBufferRequest, void *bufferRequest, StructureID idBufferInformation, void *bufferInformation );
	NT_STATUS (*ReleaseBuffer)( void *context, FeedHandle feed,
                               StructureID idBufferInformation, void *bufferInformation );
	NT_STATUS (*GetPacketFeedCount)( void *context, uint32_t *numFeeds );
	NT_STATUS (*GetPacketFeedDescription)( void *context, uint32_t feedIndex,
                                          StructureID idFeedDescription, void *feedDescription );
	NT_STATUS (*GetPacketFeedStatus)( void *context, FeedHandle feed,
                                     StructureID idFeedStatus, void *feedStatus );

  NT_STATUS (*SetPacketFeedChannelControl)(void *context, FeedHandle  feed,
                                            uint32_t    channelsBitmask, StructureID portControlId,
                                            void *      portControlInformation);
  
  NT_STATUS (*GetPacketFeedChannelControl)(void *context, FeedHandle  feed,
                                            uint32_t    channelBitmask, StructureID portControlId,
                                            void *      portControlInformation);
  
/* Transmit */
  NT_STATUS (*ReleaseTxBuffer)( void *context, FeedHandle feed,
                                 StructureID idBufferRequest, void *bufferRequest, StructureID idBufferInformation, void *bufferInformation );
	NT_STATUS (*SendTxBuffer)( void *context, FeedHandle feed,
                              StructureID idBufferInformation, void *bufferInformation );

  NT_STATUS (*FlushTxPorts)( void *context, uint32_t channelBitmask, bool_t enable);
} PACKET_FEED_INTERFACE_V2;

#endif // __PACKETFEEDINTERFACE_H__
