// $Id: memoryaccess.h 16051 2011-11-30 12:20:27Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This file implements the memory access specifics
 */

#ifndef __NTAPI_MEMORYACCESS_H__
#define __NTAPI_MEMORYACCESS_H__

#include <ostype.h>
#include "basetype.h"
#include "ntpackpush.h"

/* The following enum block relates to the structure ID groups */
enum
{
  eMemoryAllocDmaType1,
  eMemoryFreeDmaType1,
  eMemoryMapDmaType1,
  eMemoryUnmapDmaType1,
};

struct _meminfo_type1 {
  union {
    void     *pVirt;
    uint64_t dataAddress;
  } u;                    /* Virtual address.             */
  uint64_t    custom;     /* Private data.                */
};

typedef struct
{
  struct _meminfo_type1 driver; /* Driver private.                     */
  struct _meminfo_type1 user;   /* User private.                       */
  uint64_t    physicalAddress;  /* Physical address of the buffer.     */
  uint64_t    bufferSize;       /* Size of the buffer.                 */
} ExternalBufferInfoType1_t;

typedef struct {
  ExternalBufferInfoType1_t dmaBuffer;
} MemoryDMAType1_t;

#define STID_MEMORY_ALLOC_DMA_TYPE1 \
    MAKE_STRUCTURE_ID(eMemoryAllocDmaType1, MemoryDMAType1_t)

#define STID_MEMORY_FREE_DMA_TYPE1 \
    MAKE_STRUCTURE_ID(eMemoryFreeDmaType1, MemoryDMAType1_t)

#define STID_MEMORY_MAP_DMA_TYPE1 \
  MAKE_STRUCTURE_ID(eMemoryMapDmaType1, MemoryDMAType1_t)

#define STID_MEMORY_UNMAP_DMA_TYPE1 \
  MAKE_STRUCTURE_ID(eMemoryUnmapDmaType1, MemoryDMAType1_t)

#include "ntpackpop.h"

#endif // __NTAPI_MEMORYACCESS_H__
