// $Id: packetdescriptor.h 17384 2012-02-16 14:48:40Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Structure describing the descriptor that appears before each
 * frame in the analyser packet buffer memory.
 */

#ifndef __NTAPI_PACKETDESCRIPTOR_H__
#define __NTAPI_PACKETDESCRIPTOR_H__

#include "ntpackpush.h"

/**
 * This descriptor is placed in front of all packets being received by the adapter when
 * the adapter is operating in STANDARD or EXTENDED mode. One of the two modes are set
 * when creating packet feeds through NTCI_CreatePacketFeed() API call.
 * Note: The descriptor is used for both RX and TX segments but not all fields apply
 *       in both directions. See DN-0059 Programmers Guide for mode information.
 */
typedef struct PacketDescriptorNtSeriesType1
{
  /* Offset 0x00. */
  uint64_t timestamp;             /* RX & TX. 64 Bit timestamp. Type set via NTCI_CreatePacketFeed(). */
  /* Offset 0x08. */
  uint32_t storedLength:16;       /* RX & TX. Length of stored data incl. 8 bytes alignment padding. */
  uint32_t crcError:1;            /* RX & TX. Indicates that the frame has a Ethernet CRC Error. */
  uint32_t TCPCsumOk:1;           /* RX & TX. Indicates TCP checksum is correct. (v40+ FPGA only) (Gen2 v42+ for TX) */
  uint32_t UDPCsumOk:1;           /* RX & TX. Indicates UDP checksum is correct. (v40+ FPGA only) (Gen2 v42+ for TX) */
  uint32_t IPCsumOk:1;            /* RX & TX. Indicates IP checksum is correct.  (v40+ FPGA only) (Gen2 v42+ for TX) */
  uint32_t txCrcOverride:1;       /* RX & TX. 0=Do not recalc crc. 1=Recalc the MAC frame crc. */
  uint32_t cvError:1;             /* RX & TX. Indicates that this frame had a code violation. */
#define txSetClock rxIgnore
  uint32_t rxIgnore:1;            /* RX & TX. RX: Used for keepalive packets. Indicates the packet was not received from the wire.
                                              TX: txSetClock: Set the FPGA transmission clock equal to the time stamp of this packet. */
  uint32_t frameSliced:1;         /* RX & TX. Indicates that the current frame has been sliced (either soft or hard). */
  uint32_t channel:5;             /* RX Only. The channel that received the current frame. */
  uint32_t hardSlice:1;           /* RX & TX. The packet length is longer than the HW can handle and has been hard sliced. */
  uint32_t txNow:1;               /* TX Only. 0=Preserve original IFG between frames. 1=Transmit /wo preserving original IFG. Preset depending on filter. Default to 0. */
  uint32_t txIgnore:1;            /* TX Only. Setting this bit to 1 will prevent the frame from being transmitted. Preset to 0. */
  /* Offset 0x0C. */
  uint32_t wireLength:16;         /* RX & TX. Length of frame on the wire. */
  uint32_t txChannel:5;           /* RX & TX. The channel that should transmit the frame */
  uint32_t TCPFrame:1;            /* RX Only. Indicates current frame is TCP. (NT v40+ FPGA only) */
  uint32_t UDPFrame:1;            /* RX Only. Indicates current frame is UDP. (NT v40+ FPGA only) */
  uint32_t IPFrame:1;             /* RX Only. Indicates current frame is IP.  (NT v40+ FPGA only) */
  uint32_t descriptorType:1;      /* RX & TX. 0=PCAP or 1=STANDARD/EXTENDED. Must be 1 for TX. */
  uint32_t extensionLength:3;     /* RX & TX. Extended header length in qwords. Must be 0 or 2 for TX. */
  uint32_t extensionFormat:4;     /* RX & TX. Extension format type. */
} PacketDescriptorNtSeriesType1_t;
#define tagPacketDescriptorNtSeriesType1 PacketDescriptorNtSeriesType1  //Legacy struct name

/**
 * This descriptor is placed in front of all packets being received by the adapter when
 * the adapter is operating in PCAP mode. PCAP mode is activated when creating packet
 * feeds through NTCI_CreatePacketFeed() API call.
 * Note: The descriptor is used only for RX feeds. TX mode must be formatted using
 *       STANDARD_DESCRIPTOR or EXTENDED_DESCRIPTOR types 6 or greater.
 */
typedef struct PcapPacketDescriptorType1
{
  uint64_t Timestamp;           /* RX Only. 64 Bit timestamp format can be NDIS, FreeBSD(PCAP), Native. */
  uint32_t StoredLength:16;     /* RX Only. Length of stored data + padding. */
  uint32_t Reserved1:16;        /* RX Only. Reserved for future use. */
  uint32_t WireLength:16;       /* RX Only. Length of frame on the wire. */
  uint32_t Reserved2:16;        /* RX Only. Reserved for future use. */
} PcapPacketDescriptorType1_t;
#define tagPcapPacketDescriptorType1 PcapPacketDescriptorType1  //legacy struct name



/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct tagPacketDescriptorType2 {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint64_t Timestamp DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t StoredLength:16 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t CRCError:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t TCPCsumOK:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t UDPCsumOK:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t IPCsumOK:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t IPFrame:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t CVError:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t DropIndicator:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t FrameSliced:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t Channel:2 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t IPGOverride:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t CRCOverride:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t Reserved1:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t HardSlice:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t TruncatedDescriptor:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t TransmitIgnore:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t WireLength:16 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t SequenceDefinition:4 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t IPAlignPad:2 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t PacketDataIPAligned:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t PacketDataEndianMode:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t DescriptorType:1 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t ExtensionLength:3 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t ExtensionFormat:4 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } PacketDescriptorType2_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef enum {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	SEQDEF_NORMAL_PACKET			= 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	SEQDEF_CV_OUTSIDE_PACKET		= 1,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	SEQDEF_AUTONEGOTIATION_WORD		= 2,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	SEQDEF_IDLE_STATE_RESTORED		= 3,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	SEQDEF_ILLEGAL_PACKET			= 4,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	SEQDEF_LINK_DOWN				= 5,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	SEQDEF_LINK_UP					= 6,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } SequenceDefinition;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct tagAutoNegotiationPacketData{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint64_t RawChar1       : 10 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint64_t RawChar2       : 10 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint64_t RawChar3       : 10 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint64_t RawChar4       : 10 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint64_t DecodedData1   : 8 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint64_t DecodedData2   : 8 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint64_t Reserved       : 8 DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } AutoNegotiationPacketData;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25


#include "ntpackpop.h"

#endif // __PACKETDESCRIPTOR_H__
