// $Id: packetfeedstatsstructure.h 16051 2011-11-30 12:20:27Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Structure Definitions that are used by the external interface 
 */

#ifndef __NTAPI_PACKETFEEDSTATSSTRUCTURE_H__
#define __NTAPI_PACKETFEEDSTATSSTRUCTURE_H__

#include "basetype.h"
#include "commontypes.h"
#include "ntpackpush.h"

enum 
{
  ePacketFeedStatisticsType1,
  ePacketFeedStatisticsType2,
};

typedef struct _packetFeedStatsStructureType1
{
	/* Small frame drop counter	
		Number of frames discarded by the phy due to being smaller than minimum supported size of 17 bytes 
		(these are excluded from all other statistics). */
	uint64_t    dwlSmallFramesDropped;
	/* Bandwidth Drop Count	
		Number of frames discarded due to Bandwidth issues on the phy interface. 
		The count is effected by very small frames with very small inter-packet gap. 
		Note that these frames are excluded from all other statistics. */
	uint64_t    dwlBandwidthDropped;
	/* Packet stream - filtered drop counts	
		The number of frames from channel x that were dropped by the Packet Stream due filtering. */
	uint64_t    dwlPacketFeedFilteredDropped;
	/* Packet stream - Bandwidth drop counts	
		The number of frames from channel x that were dropped by the Packet Stream due to bandwidth reasons. */
	uint64_t    dwlPacketFeedBandwidthDropped;
	/* Packet stream - streamed frame count	
		The number of frames from channel x that have been received and passed to the Packet Stream. */
	uint64_t    dwlPacketFeedFrames;
	/* Frames that are dropped, outside the frame size denoted by the card capabilities */
	uint64_t    dwlMaxFrameSizeDropped; 
	/* Frames that are hard sliced, outside the frame size denoted by the card capabilities */
	uint64_t    dwlHardSlicedFrames;	
	/* Timestamp type that is supplied with 'TimeValidAt' typically in the same format as the packet feed */
	TimestampType_t timestampType;
	/* Timestamp when these values were extracted from the hardware counters */
	uint64_t	dwlTimeValidAt;			
    /* Counter overflow Set when HW counter overflows */
	bool_t      overflow;               
} PacketFeedStatisticsType1_t;

#define STID_PACKET_FEED_STATISTICS_TYPE1 \
    MAKE_STRUCTURE_ID(ePacketFeedStatisticsType1, PacketFeedStatisticsType1_t)

#include "ntpackpop.h"

#endif // __NTAPI_PACKETFEEDSTATSSTRUCTURE_H__
