// $Id: packetfeedstructure.h 25341 2013-09-06 13:25:57Z am $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Packet Feed Structure definitions for use with the
 * internal/external packet feed interface.
 */
#ifndef __NTAPI_PACKETFEEDSTRUCTURE_H__
#define __NTAPI_PACKETFEEDSTRUCTURE_H__

#include "basetype.h"
#include "commontypes.h"
#include "ntpackpush.h"
#include "memoryaccess.h"

/* Opaque feed handle */
#ifndef _NT_FEED_HANDLE
#define _NT_FEED_HANDLE
typedef uint32_t FeedHandle;
#endif

#ifdef WIN32
#pragma warning( once : 4214 ) // warning C4214: bit-field types other than int
#endif

enum _packetFeedStructureType
{
  eFeedEngineCapabilitiesType1          = 0,
  eFeedConfigSampledType1               = 1,
  eFeedConfigCaptureType1               = 2,
  eFeedDescriptionType1                 = 3,
  eFeedBufferInformationType1           = 4,
  eBufferRequestSampledType1            = 5,
  eBufferRequestCaptureType1            = 6,
  eBufferInformationSectionType1        = 7,
  eBufferInformationDescriptorListType1 = 8,
  eFeedStatusSampledType1               = 9,

  eFeedStatusCaptureType1               = 10,
  eFeedStatusLegacyType1                = 11,
  eFeedConfigSampledType2               = 12,
  eBufferRequestSampledType2            = 13,
  eBufferRequestCaptureType2            = 14,
  eBufferInformationDescriptorListType2 = 15,
  eFeedConfigSampledType3               = 16,
  eFeedConfigCaptureType2               = 17,
  eFeedConfigSampledType4               = 18,
  eBufferInformationSectionType2        = 19,

  eFeedStatusSampledType2               = 20,
  eFeedConfigSampledType5               = 21,
  eBufferRequestSampledType3            = 22,
  eFeedStatusSampledType3               = 23,
  eBufferInformationSectionType3        = 24,
  eBufferInformationSectionType4        = 25,
  eFeedConfigSampledType6               = 26,
  eBufferInformationSectionType5        = 27,
  eFeedChannelControlSlicingType1       = 28,
  eFeedChannelControlTrafficType1       = 29,

  eFeedBufferInformationType2           = 30,
  eFeedConfigSampledType7               = 31,
  eBufferInformationSectionType6        = 32,
  eBufferInformationSectionType7        = 33,
  eFeedStatusSampledType4               = 34,
  eBufferInformationSectionType8        = 35,
  eFeedDescriptionType2                 = 36,
  eWorklistConfigType1                  = 37,
  eWorklistBufferRequestType1           = 38,
  eWorklistBufferInformationType1       = 39,
  eFeedConfigSampledType8               = 40,
  eBufferInformationSectionType9        = 41,
};

/******************************************************************************
 * Types and data structures to be used when creating packet feeds.
 ******************************************************************************/
#define SLICE_NONE                                0
#define MAX_PACKETS_POSSIBLE_IN_SECTION_DELIVERY  0
#define NO_TIMEOUT_IN_SECTION_DELIVERY            0
#define MAX_VIRTUAL_TRANSMIT_CHANNELS             8

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef enum{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   PACKET_FEED_SOURCE_MIN = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   PACKET_FEED_SOURCE_SAMPLED = PACKET_FEED_SOURCE_MIN,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   PACKET_FEED_SOURCE_CAPTURE,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   PACKET_FEED_SOURCE_MAX = PACKET_FEED_SOURCE_CAPTURE
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedSource;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef enum{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   PACKET_FEED_DESTINATION_MIN = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   PACKET_FEED_DESTINATION_INTERNAL = PACKET_FEED_DESTINATION_MIN,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   PACKET_FEED_DESTINATION_EXTERNAL,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   PACKET_FEED_DESTINATION_MAX = PACKET_FEED_DESTINATION_EXTERNAL
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedDestination;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef enum{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BUFFER_SECTION = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BUFFER_DESCRIPTOR_LIST
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } BufferType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef enum{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   LARGE_CONTIGUOUS_SAMPLES = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   SMALL_REGULAR_SAMPLES,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   SMALL_REGULAR_SAMPLES_NEVER_TOO_OLD,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } BufferBehaviour;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

/* Capture type */
typedef enum
{
  CAPTURE_STOP_WHEN_FULL,                   /* Stop capture when internal buffer gets full */
  CAPTURE_WRAP,                             /* Wrap internal buffer if it gets full (loose old unprocessed packets) */
} CaptureType;

/* Descriptor type */
typedef enum DescriptorType
{
  PCAP_DESCRIPTOR,                          /* Prepend each packet with a PCAP header */
  STANDARD_DESCRIPTOR,                      /* Prepend each packet with a Standard descriptor header */
  EXTENDED_DESCRIPTOR,                      /* Prepend each packet with an Extended descriptor header (Multiple formats of this type) */
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   GB10S_DESCRIPTOR=3,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25
} DescriptorType;

/* Feed configuration structure to be used when creating a sampled packet feed */
#ifndef NT_LEGACY_4_25
typedef struct
{
  uint32_t        sliceLength;              /* Apply fixed slicing from first octet received. If zero, receive 0 up to 9600 octet frames */
  uint32_t        latency;                  /* Maximum time before a segment returns if no data has been received.
                                               Units in microseconds. Recommended default is 10000 (10 msec).
                                               Valid range for inline (usec): 100, 200, 300, ..., 200,000
                                               Valid range for capture (usec): 1000, 1200, 1400, ..., 1000,000 */

  uint32_t        reserved0;                /* reserved field for obsoleted feature */
  TimestampType_t timestampType;            /* Timestamp format required. See TimestampType_t for types */
  uint8_t         reserved1;                /* reserved field for obsoleted feature */
} FeedConfigSampledType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #else //NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    sliceLength;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    latency;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferType    bufferType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   TimestampType_t timestampType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t      dropErroredPacketsFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedConfigSampledType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

#define STID_FEED_CONFIG_SAMPLED_TYPE1 \
    MAKE_STRUCTURE_ID(eFeedConfigSampledType1, FeedConfigSampledType1_t)

#ifndef NT_LEGACY_4_25
typedef struct
{
  uint32_t        reserved0;                /* reserved field for obsoleted feature */
  TimestampType_t timestampType;            /* Timestamp format required. See TimestampType_t for types */
  uint8_t         reserved1;                /* reserved field for obsoleted feature */
  uint32_t        sliceLength;              /* Apply fixed slicing from first octet received. If zero, receive 0 up to 9600 octet frames */
  uint32_t        reserved2;                /* reserved field for obsoleted feature */
} FeedConfigSampledType2_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #else //NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferType    bufferType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   TimestampType_t timestampType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t      dropErroredPacketsFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    sliceLength;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferBehaviour bufferBehaviour DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedConfigSampledType2_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

#define STID_FEED_CONFIG_SAMPLED_TYPE2 \
    MAKE_STRUCTURE_ID(eFeedConfigSampledType2, FeedConfigSampledType2_t)

#ifndef NT_LEGACY_4_25
typedef struct
{
  uint32_t        reserved0;                /* reserved field for obsoleted feature */
  TimestampType_t timestampType;            /* Timestamp format required. See TimestampType_t for types */
  uint8_t         reserved1;                /* reserved field for obsoleted feature */
  uint32_t        sliceLength;              /* Apply fixed slicing from first octet received. If zero, receive 0 up to 9600 octet frames */
  uint32_t        reserved2;                /* reserved field for obsoleted feature */
  DescriptorType  descriptorType;           /* Select packet descriptor to return. Valid options are:
                                               PCAP_DESCRIPTOR, used for PCAP type capture applications.
                                               STANDARD_DESCRIPTOR, used for common capture applications.
                                               EXTENDED_DESCRIPTOR, gives additional information. See below 'extendedDescriptorFormat' field. */
  uint32_t        extendedDescriptorFormat; /* 'Version' of extended packet descriptor requested.
                                               E.g. To receive NT Extended Packet descriptor Format 7, set this field to EXTENDED_DESCRIPTOR_07_TYPE. */
  uint32_t        extendedDescriptorLength; /* Length in qwords of the extended header. For above example, set this field to EXTENDED_DESCRIPTOR_07_LENGTH.
                                               Buffer is padded if Format chosen is less than this length.
                                               Note: Must be <= 6 */
  uint32_t        reserved3;                /* reserved field for obsoleted feature */
  uint32_t        maxLatency;               /* Maximum time before a segment returns if no data has been received.
                                               Units in microseconds. Recommended default is 10000 (10 msec).
                                               Valid range for inline (usec): 100, 200, 300, ..., 200,000
                                               Valid range for capture (usec): 1000, 1200, 1400, ..., 1000,000 */
} FeedConfigSampledType3_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #else //NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferType    bufferType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   TimestampType_t timestampType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t      dropErroredPacketsFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    sliceLength;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferBehaviour bufferBehaviour DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   DescriptorType  descriptorType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorFormat;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorLength;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    maxPacketsPerSection DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    maxLatency;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedConfigSampledType3_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

#define STID_FEED_CONFIG_SAMPLED_TYPE3 \
    MAKE_STRUCTURE_ID(eFeedConfigSampledType3, FeedConfigSampledType3_t)

typedef struct
{
  union {
    void*     virtualAddress;               /* Address useable from userspace */
    uint64_t  __alignment;
  } u;
  uint64_t    physicalAddress;              /* Address reserved for kernelspace */
  uint32_t    bufferSize;                   /* Size of the buffer this structure points to */
  uint32_t    customID;                     /* User-configurable value that, although, not be directly used
                                             * by the packet feed system, will be preserved so that
                                             * callers of GetNextBuffer can identify this external buffer
                                             * as it becomes available to them */
} ExternalBufferInfo_t;

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   TimestampType_t timestampType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t      dropErroredPacketsFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    sliceLength DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferBehaviour bufferBehaviour DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   DescriptorType  descriptorType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorFormat DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorLength DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    maxPacketsPerSection DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    maxLatency DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    numExternalBuffers DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   ExternalBufferInfo_t* externalBufferArray DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedConfigSampledType4_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_FEED_CONFIG_SAMPLED_TYPE4 MAKE_STRUCTURE_ID(eFeedConfigSampledType4, FeedConfigSampledType4_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

#ifndef NT_LEGACY_4_25
typedef struct
{
  uint32_t        reserved0;                /* reserved field for obsoleted feature */
  TimestampType_t timestampType;            /* Timestamp format required. See TimestampType_t for types */
  uint8_t         reserved1;                /* reserved field for obsoleted feature */
  uint32_t        sliceLength;              /* Apply fixed slicing from first octet received. If zero, receive 0 up to 9600 octet frames */
  uint32_t        reserved2;                /* reserved field for obsoleted feature */
  DescriptorType  descriptorType;           /* Select packet descriptor to return. Valid options are:
                                               PCAP_DESCRIPTOR, used for PCAP type capture applications.
                                               STANDARD_DESCRIPTOR, used for common capture applications.
                                               EXTENDED_DESCRIPTOR, gives additional information. See below 'extendedDescriptorFormat' field. */
  uint32_t        extendedDescriptorFormat; /* 'Version' of extended packet descriptor requested.
                                               E.g. To receive NT Extended Packet descriptor Format 7, set this field to EXTENDED_DESCRIPTOR_07_TYPE. */
  uint32_t        extendedDescriptorLength; /* Length in qwords of the extended header. For above example, set this field to EXTENDED_DESCRIPTOR_07_LENGTH.
                                               Buffer is padded if Format chosen is less than this length.
                                               Note: Must be <= 6 */
  uint32_t        reserved3;                /* reserved field for obsoleted feature */
  uint32_t        maxLatency;               /* Maximum time before a segment returns if no data has been received.
                                               Units in microseconds. Recommended default is 10000 (10 msec).
                                               Valid range for inline (usec): 100, 200, 300, ..., 200,000
                                               Valid range for capture (usec): 1000, 1200, 1400, ..., 1000,000 */
  uint32_t        reserved4;                /* reserved field for obsoleted feature */
  uint32_t        reserved5;                /* reserved field for obsoleted feature */
  uint32_t        reserved6[8];             /* reserved field for obsoleted feature */
  uint32_t        numTransmitSections;      /* Arbitrary number of 'transmit-only' sections.  Zero is permitted. */
  uint32_t        reserved7;                /* reserved field for obsoleted feature */
} FeedConfigSampledType5_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #else //NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferType    bufferType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   TimestampType_t timestampType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t      dropErroredPacketsFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    sliceLength;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferBehaviour bufferBehaviour DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   DescriptorType  descriptorType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorFormat;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorLength;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    maxPacketsPerSection DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    maxLatency;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    channelReceiveBitmask DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    channelTransmitBitmask DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    virtualChannelTransmitMappingBitmask[ MAX_VIRTUAL_TRANSMIT_CHANNELS ] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    numTransmitSections;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    percentageTransmitBufferAllocation DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedConfigSampledType5_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

#define STID_FEED_CONFIG_SAMPLED_TYPE5 \
    MAKE_STRUCTURE_ID(eFeedConfigSampledType5, FeedConfigSampledType5_t)

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferType    bufferType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   TimestampType_t timestampType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t      dropErroredPacketsFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    sliceLength DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferBehaviour bufferBehaviour DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   DescriptorType  descriptorType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorFormat DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorLength DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    maxPacketsPerSection DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    maxLatency DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t      flushInternalBuffersOnStopFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    sectionAlignmentByteBoundary DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    sectionGranularityByteSize DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedConfigSampledType6_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_FEED_CONFIG_SAMPLED_TYPE6 MAKE_STRUCTURE_ID(eFeedConfigSampledType6, FeedConfigSampledType6_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

#ifndef NT_LEGACY_4_25
typedef struct
{
  uint32_t        reserved0;                /* reserved field for obsoleted feature */
  TimestampType_t timestampType;            /* Timestamp format required. See TimestampType_t for types */
  uint8_t         reserved1;                /* reserved field for obsoleted feature */
  uint32_t        sliceLength;              /* Apply fixed slicing from first octet received. If zero, receive 0 up to 9600 octet frames */
  uint32_t        reserved2;                /* reserved field for obsoleted feature */
  DescriptorType  descriptorType;           /* Select packet descriptor to return. Valid options are:
                                               PCAP_DESCRIPTOR, used for PCAP type capture applications.
                                               STANDARD_DESCRIPTOR, used for common capture applications.
                                               EXTENDED_DESCRIPTOR, gives additional information. See below 'extendedDescriptorFormat' field. */
  uint32_t        extendedDescriptorFormat; /* 'Version' of extended packet descriptor requested.
                                               E.g. To receive NT Extended Packet descriptor Format 7, set this field to EXTENDED_DESCRIPTOR_07_TYPE. */
  uint32_t        extendedDescriptorLength; /* Length in qwords of the extended header. For above example, set this field to EXTENDED_DESCRIPTOR_07_LENGTH.
                                               Buffer is padded if Format chosen is less than this length.
                                               Note: Must be <= 6 */
  uint32_t        maxLatency;               /* Maximum time before a segment returns if no data has been received.
                                               Units in microseconds. Recommended default is 10000 (10 msec).
                                               Valid range for inline (usec): 100, 200, 300, ..., 200,000
                                               Valid range for capture (usec): 1000, 1200, 1400, ..., 1000,000 */
  uint32_t        numSegments;              /* Number of seqments to allocate for this feed. */
  uint32_t        segmentSize;              /* Segment size in kBytes. 1024, 2048, 4096 are supported */
} FeedConfigSampledType7_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #else //NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferType    bufferType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   TimestampType_t timestampType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t      dropErroredPacketsFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    sliceLength;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferBehaviour bufferBehaviour DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   DescriptorType  descriptorType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorFormat;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorLength;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    maxLatency;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    numSegments;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    segmentSize;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedConfigSampledType7_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

#define STID_FEED_CONFIG_SAMPLED_TYPE7 \
    MAKE_STRUCTURE_ID(eFeedConfigSampledType7, FeedConfigSampledType7_t)

typedef struct
{
  TimestampType_t timestampType;            /* Timestamp format required. See TimestampType_t for types */
  uint32_t        sliceLength;              /* Apply fixed slicing from first octet received. If zero, receive 0 up to 9600 octet frames */
  DescriptorType  descriptorType;           /* Select packet descriptor to return. Valid options are:
                                               PCAP_DESCRIPTOR, used for PCAP type capture applications.
                                               STANDARD_DESCRIPTOR, used for common capture applications.
                                               EXTENDED_DESCRIPTOR, gives additional information. See below 'extendedDescriptorFormat' field. */
  uint32_t        extendedDescriptorFormat; /* 'Version' of extended packet descriptor requested.
                                               E.g. To receive NT Extended Packet descriptor Format 7, set this field to EXTENDED_DESCRIPTOR_07_TYPE. */
  uint32_t        extendedDescriptorLength; /* Length in qwords of the extended header. For above example, set this field to EXTENDED_DESCRIPTOR_07_LENGTH.
                                               Buffer is padded if Format chosen is less than this length.
                                               Note: Must be <= 6 */
  uint32_t        maxLatency;               /* Maximum time before a segment returns if no data has been received.
                                               Units in microseconds. Recommended default is 10000 (10 msec).
                                               Valid range for inline (usec): 100, 200, 300, ..., 200,000
                                               Valid range for capture (usec): 1000, 1200, 1400, ..., 1000,000 */
  uint32_t        numTransmitOnlySegments;  /* The number of numSegments that are used for transmit only.
                                               Note: numTransmitOnlySegments must be less or equal to numSegments */
  uint32_t        numSegments;              /* Number of seqments to allocate for this feed. */
  uint32_t        segmentSize;              /* Segment size in kBytes. 1024, 2048, 4096 are supported */
} FeedConfigSampledType8_t;

#define STID_FEED_CONFIG_SAMPLED_TYPE8 \
    MAKE_STRUCTURE_ID(eFeedConfigSampledType8, FeedConfigSampledType8_t)

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   CaptureType   captureType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    overrunCounter DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferType    bufferType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   TimestampType_t timestampType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t      dropErroredPacketsFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    sliceLength DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedConfigCaptureType1_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_FEED_CONFIG_CAPTURE_TYPE1  MAKE_STRUCTURE_ID(eFeedConfigCaptureType1, FeedConfigCaptureType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   CaptureType   captureType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    overrunCounter DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferType    bufferType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   TimestampType_t timestampType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t      dropErroredPacketsFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    sliceLength DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   DescriptorType  descriptorType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorFormat DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    extendedDescriptorLength DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedConfigCaptureType2_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_FEED_CONFIG_CAPTURE_TYPE2 MAKE_STRUCTURE_ID(eFeedConfigCaptureType2, FeedConfigCaptureType2_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

/******************************************************************************
 * Types and data structures to be used to get information about a packet feed.
 ******************************************************************************/
#ifndef NT_LEGACY_4_25
typedef struct
{
  FeedHandle       handle;                  /* Handle to feed */
  uint32_t         reserved0;               /* reserved field for obsoleted feature */
  uint32_t         reserved1;               /* reserved field for obsoleted feature */
  uint32_t         reserved2;               /* reserved field for obsoleted feature */
} FeedDescriptionType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #else //NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FeedHandle    handle;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FeedSource    source DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FeedDestination destination DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t    priority DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedDescriptionType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

#define STID_FEED_DESCRIPTION_TYPE1 \
    MAKE_STRUCTURE_ID(eFeedDescriptionType1, FeedDescriptionType1_t)

#ifndef NT_LEGACY_4_25
typedef struct
{
  FeedHandle      handle;                   /* Handle to feed */
  uint32_t        reserved0;                /* reserved field for obsoleted feature */
  uint32_t        reserved1;                /* reserved field for obsoleted feature */
  uint32_t        reserved2;                /* reserved field for obsoleted feature */
  DescriptorType  descriptorType;           /* Packet descriptor configured on feed. Valid options are:
                                               PCAP_DESCRIPTOR, used for PCAP type capture applications.
                                               STANDARD_DESCRIPTOR, used for common capture applications.
                                               EXTENDED_DESCRIPTOR, gives additional information. See below 'extendedDescriptorFormat' field. */
  uint32_t        extendedDescriptorType;   /* 'Version' of extended packet descriptor requested.
                                               E.g. EXTENDED_DESCRIPTOR_07_TYPE is returned for format NT Extended Packet descriptor Format 7 */
  uint32_t        extendedDescriptorLength; /* Length in qwords of the extended header.
                                               Buffer is padded if Format chosen is less than this length.
                                               Note: Must be <= 6 */
  TimestampType_t timestampType;            /* Timestamp format required. See TimestampType_t for types */
  uint32_t        maxLatency;               /* Maximum time before a segment returns if no data has been received.
                                               Units in microseconds. Recommended default is 10000 (10 msec).
                                               Valid range for inline (usec): 100, 200, 300, ..., 200,000
                                               Valid range for capture (usec): 1000, 1200, 1400, ..., 1000,000 */
  uint32_t        segmentCount;             /* Number of segments allocated to this feed */
  uint32_t        segmentSize;              /* Size of each segment */
  uint32_t        channelsBitmask;          /* Channels or Ports for which this feed receives trafic */
} FeedDescriptionType2_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #else //NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FeedHandle      handle;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FeedSource      source DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FeedDestination destination DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        priority DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        descriptorType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        extendedDescriptorType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        extendedDescriptorLength;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        timestampType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        maxLatency;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        segmentCount;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        segmentSize;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        channelsBitmask;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedDescriptionType2_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

#define STID_FEED_DESCRIPTION_TYPE2 \
    MAKE_STRUCTURE_ID(eFeedDescriptionType2, FeedDescriptionType2_t)

/* Packet buffer memory. All references to packets will be given as an offset from this base address */
typedef struct
{
  void*       bufferBaseAddress;                /* Contains the base address of hostbuffer containing a group of segments. */
  //If compat & 32bit platform then the above void* must be a 64bit pointer container
#ifdef NT_COMPAT_32BIT
#if (_NT_HOST_CPU_POINTER_SIZE == _NT_HOST_CPU_POINTER_SIZE_32)
  uint32_t _res1_bufferBaseAddress_64bit_align; /* Structure padding for COMPAT_32BIT */
#endif // _NT_HOST_CPU_POINTER_SIZE == _NT_HOST_CPU_POINTER_SIZE_32
#endif // NT_COMPAT_32BIT
  uint32_t    bufferSize;                       /* Size of the segment in bytes */
} FeedBufferInformationType1_t;

#define STID_FEED_BUFFER_INFORMATION_TYPE1 \
    MAKE_STRUCTURE_ID(eFeedBufferInformationType1, FeedBufferInformationType1_t)

#if (_NT_OS_TYPE == _NT_OS_TYPE_WIN)
  /* 32bit apps on 64bit windows */
typedef struct
{
  void*__ptr32 bufferBaseAddress;           /* Contains the base address of the segment. All packets are offset from this address */
  uint32_t     bufferSize;                  /* Size of the segment in bytes */
} FeedBufferInformationType1COMPAT32BIT_t;

#define STID_FEED_BUFFER_INFORMATION_TYPE1_COMPAT32BIT \
    MAKE_STRUCTURE_ID(eFeedBufferInformationType1, FeedBufferInformationType1COMPAT32BIT_t)
#endif // _NT_OS_TYPE == _NT_OS_TYPE_WIN

/* Packet buffer info. Returns info on how many buffers the current feed has
   and how large the buffers are.
*/
typedef struct
{
  uint32_t    numBuffers;                   /* Number of segments allocated to this feed */
  uint32_t    bufferSize;                   /* Size of each segment */
} FeedBufferInformationType2_t;

#define STID_FEED_BUFFER_INFORMATION_TYPE2 \
    MAKE_STRUCTURE_ID(eFeedBufferInformationType2, FeedBufferInformationType2_t)

/******************************************************************************
 * Types and data structures to be used when requesting sections or descriptor
 * lists via the GetNextBuffer() function.
 ******************************************************************************/

#define GET_NEXT_BUFFER_POLL                     0            /* Return from GetNextBuffer() even if no segment are ready for processing (non-blocking).
                                                                   Returns NTCI_STATUSCODE_NO_SEGMENT_AVAILABLE when no segment is ready for processing.
                                                                     Do call NTCI_ReleaseBuffer() when status NTCI_STATUSCODE_NO_SEGMENT_AVAILABLE is returned. Simply retry operation.
                                                                   Returns NTCI_ERRCODE_SUCCESS when a segment is returned for processing.
                                                                     Remember to release segment back to driver after processing. */
#define GET_NEXT_BUFFER_BLOCKING                 0xFFFFFFFF   /* Wait until next segment is ready for processing (Upper bound is 'maxLatency').
                                                                   Returns NTCI_ERRCODE_SUCCESS when a segment is returned for processing.
                                                                     Remember to release segment back to driver after processing. */

#define GET_NEXT_BUFFER_INFINITE_WAIT            0xFFFFFFFF   /* Obsolete, Please use GET_NEXT_BUFFER_BLOCKING */

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t  waitForBufferFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } BufferRequestCommonLegacy;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

typedef struct
{
  uint32_t waitTimeoutMS;                   /* Amount of time to wait for a segment. Supported values are GET_NEXT_BUFFER_POLL and GET_NEXT_BUFFER_BLOCKING. See above defines. */
} BufferRequestCommon;

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferRequestCommonLegacy common DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } BufferRequestSampledType1_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_BUFFER_REQUEST_SAMPLED_TYPE1 MAKE_STRUCTURE_ID(eBufferRequestSampledType1, BufferRequestSampledType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

typedef struct
{
  BufferRequestCommon common;               /* See BufferRequestCommon for field description */
} BufferRequestSampledType2_t;

#define STID_BUFFER_REQUEST_SAMPLED_TYPE2 \
    MAKE_STRUCTURE_ID(eBufferRequestSampledType2, BufferRequestSampledType2_t)

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferRequestCommon common DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t        forceReceiveBufferDeliveryFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } BufferRequestSampledType3_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_BUFFER_REQUEST_SAMPLED_TYPE3 MAKE_STRUCTURE_ID(eBufferRequestSampledType3, BufferRequestSampledType3_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferRequestCommonLegacy common DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t numStartPacket DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t numPackets DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } BufferRequestCaptureType1_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_BUFFER_REQUEST_CAPTURE_TYPE1 MAKE_STRUCTURE_ID(eBufferRequestCaptureType1, BufferRequestCaptureType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   BufferRequestCommon common DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t numStartPacket DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t numPackets DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } BufferRequestCaptureType2_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_BUFFER_REQUEST_CAPTURE_TYPE2 MAKE_STRUCTURE_ID(eBufferRequestCaptureType2, BufferRequestCaptureType2_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

/******************************************************************************
 * Types and data structures used to describe how to access the sections or
 * descriptor lists that are made available via the GetNextBuffer() function.
 ******************************************************************************/

typedef struct
{
  uint32_t  sectionOffset;                  /* Relative address of a segment. bufferBaseAddress + sectionOffset field gives actual address of segment. */
                                            /* See STID_FEED_BUFFER_INFORMATION_TYPE1 for information about how to retrieve the bufferBaseAddress. */
  uint32_t  numBytesAvailable;              /* Number of bytes filled in a segment. This value includes packet descriptors. */
  uint32_t  numDescriptorsAvailable;        /* Number of packets descriptors in a segment. (i.e. number of packets available) */
  uint64_t  numDroppedFrames;               /* Number of packets dropped since last GetNextBuffer() call. */
                                            /* Note: Sampling is done asynchronously, so this number may be inaccurate in high-load situations */
} BufferInformationSectionType1_t;

/* This structure type is for receive-only sections */
#define STID_BUFFER_INFORMATION_SECTION_TYPE1 \
    MAKE_STRUCTURE_ID(eBufferInformationSectionType1, BufferInformationSectionType1_t)

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        reserved DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   ExternalBufferInfo_t  externalBuffer DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        numBytesAvailable DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t        numDescriptorsAvailable DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint64_t        numDroppedFrames DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } BufferInformationSectionType2_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_BUFFER_INFORMATION_SECTION_TYPE2 MAKE_STRUCTURE_ID(eBufferInformationSectionType2, BufferInformationSectionType2_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

typedef struct
{
  uint32_t  sectionOffset;                  /* Relative address of a segment. bufferBaseAddress + sectionOffset field gives actual address of segment. */
                                            /* See STID_FEED_BUFFER_INFORMATION_TYPE1 for information about how to retrieve the bufferBaseAddress. */
  uint32_t  numBytesAvailable;              /* When the section is acquired via GetNextBuffer(), */
                                            /* this value will indicate the maximum size of the */
                                            /* section.  The caller is then expected to insert */
                                            /* packets into the section and adjust this value on */
                                            /* release so that the driver will know how much */
                                            /* data to transmit */
  bool_t    transmitFlag;                   /* Set to 1 to to transmit on ReleaseBuffer() */
} BufferInformationSectionType3_t;

/* This structure type is for transmit-only sections */
#define STID_BUFFER_INFORMATION_SECTION_TYPE3 \
    MAKE_STRUCTURE_ID(eBufferInformationSectionType3, BufferInformationSectionType3_t)

typedef struct
{
  uint32_t  sectionOffset;                  /* Relative address of a segment. bufferBaseAddress + sectionOffset field gives actual address of segment. */
                                            /* See STID_FEED_BUFFER_INFORMATION_TYPE1 for information about how to retrieve the bufferBaseAddress. */
  uint32_t  numBytesAvailable;              /* When the section is acquired via GetNextBuffer(), */
                                            /* this value will indicate the maximum size of the */
                                            /* section.  The caller is then expected to insert */
                                            /* packets into the section and adjust this value on */
                                            /* release so that the driver will know how much */
                                            /* data to transmit */
  uint32_t  numDescriptorsAvailable;        /* Number of packets descriptors in a segment. (i.e. number of packets available) */
  uint64_t  numDroppedFrames;               /* Number of packets dropped since last GetNextBuffer() call. */
                                            /* Note: Sampling is done asynchronously, so this number may be inaccurate in high-load situations */
  bool_t    transmitFlag;                   /* Set to 1 to to transmit on ReleaseBuffer() */
} BufferInformationSectionType4_t;

/* This structure type is for receive sections that can be transmitted on release */
#define STID_BUFFER_INFORMATION_SECTION_TYPE4 \
    MAKE_STRUCTURE_ID(eBufferInformationSectionType4, BufferInformationSectionType4_t)

#define USE_SINGLE_PACKET_BUFFER 0          /* One packet transmit is made */
#define USE_MULTI_PACKET_BUFFER  1          /* Multi packet transmit is made */

typedef struct
{
  uint32_t              channelBitmask;     /* What channel/port should transmit this buffer. */
  ExternalBufferInfo_t  externalBuffer;     /* Buffer to transmit. */
} BufferInformationSectionType5_t;

/* This structure type is for transmit-only sections.  NOTE: ONLY VALID FOR USE IN KERNEL SPACE */
#define STID_BUFFER_INFORMATION_SECTION_TYPE5 \
    MAKE_STRUCTURE_ID(eBufferInformationSectionType5, BufferInformationSectionType5_t)

typedef struct
{
  uint64_t sectionOffset;                   /* Offset to the segment. */
  uint32_t numBytesAvailable;               /* Data size of the segment. */
  uint32_t numDescriptorsAvailable;         /* Descriptors in the segment. */
  uint64_t segmentTimestamp;                /* The time when the segment was done. */
  uint64_t lastPacketTimestamp;             /* Timestamp of the last packet in the segment. */
  uint32_t numDroppedFrames;                /* Number of dropped frames since last GetNextBuffer call. */
  uint8_t  segmentTimeout;                  /* 1 if the segment timed out instead of being released because it is full. */
  uint8_t  segmentCV;                       /* Segment contain packet(s) with Coding violations. */
  uint8_t  segmentCRC;                      /* Segment contain packet(s) with CRC errors. */
  uint8_t  segmentHardSliced;               /* Segment contain packet(s) that has been hardsliced. */
} BufferInformationSectionType6_t;

#define STID_BUFFER_INFORMATION_SECTION_TYPE6 \
    MAKE_STRUCTURE_ID(eBufferInformationSectionType6, BufferInformationSectionType6_t)

typedef struct
{
  ExternalBufferInfo_t externalBuffer;      /* Segment info. */
  uint32_t numBytesAvailable;               /* Data size of the segment. */
  uint32_t numDescriptorsAvailable;         /* Descriptors in the segment. */
  uint64_t segmentTimestamp;                /* The time when the segment was done. */
  uint64_t lastPacketTimestamp;             /* Timestamp of the last packet in the segment. */
  uint32_t numDroppedFrames;                /* Number of dropped frames since last GetNextBuffer call. */
  uint8_t  segmentTimeout;                  /* 1 if the segment timed out instead of being released because it is full. */
  uint8_t  segmentCV;                       /* Segment contain packet(s) with Coding violations. */
  uint8_t  segmentCRC;                      /* Segment contain packet(s) with CRC errors. */
  uint8_t  segmentHardSliced;               /* Segment contain packet(s) that has been hardsliced. */
} BufferInformationSectionType7_t;

#define STID_BUFFER_INFORMATION_SECTION_TYPE7 \
    MAKE_STRUCTURE_ID(eBufferInformationSectionType7, BufferInformationSectionType7_t)

typedef struct
{
  ExternalBufferInfoType1_t externalBuffer; /* Segment info. */
  uint32_t numBytesAvailable:30;            /* Data size of the segment. */
  uint32_t boTransmit:1;                    /* Should the segment be transmitted. */
  uint32_t numDescriptorsAvailable;         /* Descriptors in the segment. */
  uint64_t segmentTimestamp;                /* The time when the segment was done. */
  uint64_t lastPacketTimestamp;             /* Timestamp of the last packet in the segment. */
  uint32_t numDroppedFrames;                /* Number of dropped frames since last GetNextBuffer call. */
  uint8_t  segmentTimeout;                  /* 1 if the segment timed out instead of being released because it is full. */
  uint8_t  segmentCV;                       /* Segment contain packet(s) with Coding violations. */
  uint8_t  segmentCRC;                      /* Segment contain packet(s) with CRC errors. */
  uint8_t  segmentHardSliced;               /* Segment contain packet(s) that has been hardsliced. */
} BufferInformationSectionType8_t;

#define STID_BUFFER_INFORMATION_SECTION_TYPE8 \
    MAKE_STRUCTURE_ID(eBufferInformationSectionType8, BufferInformationSectionType8_t)

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t  descriptorListOffset DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t  numDescriptorsAvailable DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t  numDroppedFrames DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } BufferInformationDescriptorListType1_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_BUFFER_INFORMATION_DESCRIPTOR_LIST_TYPE1 MAKE_STRUCTURE_ID(eBufferInformationDescriptorListType1, BufferInformationDescriptorListType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t  descriptorListOffset DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t  numDescriptorsAvailable DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint64_t  numDroppedFrames DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } BufferInformationDescriptorListType2_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_BUFFER_INFORMATION_DESCRIPTOR_LIST_TYPE2 MAKE_STRUCTURE_ID(eBufferInformationDescriptorListType2, BufferInformationDescriptorListType2_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

typedef struct
{
  uint64_t  sectionOffset;                  /* Relative address of a segment. bufferBaseAddress + sectionOffset field gives actual address of segment. */
                                            /* See STID_FEED_BUFFER_INFORMATION_TYPE1 for information about how to retrieve the bufferBaseAddress. */
  uint32_t  numBytesAvailable;              /* When the section is acquired via GetNextBuffer(), */
  /* this value will indicate the maximum size of the */
  /* section.  The caller is then expected to insert */
  /* packets into the section and adjust this value on */
  /* release so that the driver will know how much */
  /* data to transmit */
  bool_t    transmitFlag;                   /* Set to 1 to to transmit on ReleaseBuffer() */
} BufferInformationSectionType9_t;

/* This structure type is for transmit-only sections */
#define STID_BUFFER_INFORMATION_SECTION_TYPE9 \
  MAKE_STRUCTURE_ID(eBufferInformationSectionType9, BufferInformationSectionType9_t)

/******************************************************************************
 * Types and data structures to be used when determining the status of a
 * packet feed.
 ******************************************************************************/

/* Feed status: sampled mode */
typedef enum
{
  SAMPLED_FEED_STOPPED,                       /* No packets are being sampled.  Call */
                                              /* StartPacketFeed() to progress to */
                                              /* one of the other modes */

  SAMPLED_FEED_STARTED_PACKETS_AVAILABLE,     /* The system is actually sampling packets at */
                                              /* this instance in time, and has made */
                                              /* packets available via GetNextBuffer() */

  SAMPLED_FEED_STARTED_NO_PACKETS_AVAILABLE,  /* The system is actually sampling packets at */
                                              /* this instance in time, but none are */
                                              /* available via GetNextBuffer() */

  SAMPLED_FEED_STARTED_PACKETS_PENDING        /* The system is actually sampling packets at */
                                              /* this instance in time, and has packets to */
                                              /* process in its system, but none are yet */
                                              /* available via GetNextBuffer() */
} FeedStatusSampledMode;

typedef struct
{
  uint32_t  numBytesPendingTransmit;
  uint64_t  numPacketsTransmitted;
} FeedStatusSampledType3_t;

/* This structure type is only valid for use with transmit capable feeds */
#define STID_FEED_STATUS_SAMPLED_TYPE3 \
    MAKE_STRUCTURE_ID(eFeedStatusSampledType3, FeedStatusSampledType3_t)

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef enum{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   CAPTURE_FEED_NOT_STARTED = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   CAPTURE_FEED_WAITING_FOR_TRIGGER,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   CAPTURE_FEED_CAPTURING,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   CAPTURE_FEED_COMPLETE,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedStatusCaptureMode;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FeedStatusSampledMode mode DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedStatusSampledType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_FEED_STATUS_SAMPLED_TYPE1 MAKE_STRUCTURE_ID(eFeedStatusSampledType1, FeedStatusSampledType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FeedStatusSampledMode mode DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t  numBytesPending DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t  numDescriptorsPending DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t  numBytesAvailable DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t  numDescriptorsAvailable DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedStatusSampledType2_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_FEED_STATUS_SAMPLED_TYPE2 MAKE_STRUCTURE_ID(eFeedStatusSampledType2, FeedStatusSampledType2_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

typedef struct
{
  FeedStatusSampledMode mode;               /* Packetfeed state. */
  uint32_t  adapterFillLevel;               /* How much of the onboard memory is utilized in percent. */
  uint32_t  numSegmentsReady;               /* Number of segments ready via GetNextBuffer. */
  uint32_t  numSegmentsBusy;                /* Number of segments taken by the application. */
  uint32_t  numSegmentsIdle;                /* Number of segments waiting for adapter free slots. */
  uint32_t  numSegmentsAdapter;             /* Number of segments held by the adapter. */
} FeedStatusSampledType4_t;

/* This structure type is only valid for use with feeds delivering sections */
#define STID_FEED_STATUS_SAMPLED_TYPE4 \
    MAKE_STRUCTURE_ID(eFeedStatusSampledType4, FeedStatusSampledType4_t)

typedef struct {
  uint32_t sliceLength;                     /* The slice length for the channel(s) set in channelsBitmask/channelBitmask. */
} FeedChannelControlSlicingType1_t;

/* Napatech internal stid.. use NTPL to configure slicing */
#define STID_FEED_CHANNEL_CONTROL_SLICING_TYPE1 \
    MAKE_STRUCTURE_ID(eFeedChannelControlSlicingType1, FeedChannelControlSlicingType1_t)

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FeedStatusCaptureMode mode DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t bufferRawByteSize DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t bufferCaptureByteSize DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t currentBytePosition DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t   bufferWrappedFlag DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint64_t numPacketsDroppedDueToBufferWrap DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint64_t numPacketsDroppedDueToFiltering DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t numNormalPacketsAvailable DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint64_t numAutoNegPacketsSeen DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedStatusCaptureType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_FEED_STATUS_CAPTURE_TYPE1 MAKE_STRUCTURE_ID(eFeedStatusCaptureType1, FeedStatusCaptureType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef enum{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FEED_STATUS_SAMPLED = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FEED_STATUS_CAPTURE = 1,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedStatusType;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   FeedStatusType feedStatusType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   union{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     FeedStatusSampledType1_t sampled DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     FeedStatusCaptureType1_t capture DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   } type DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedStatusLegacyType1_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_FEED_STATUS_LEGACY_TYPE1 MAKE_STRUCTURE_ID(eFeedStatusLegacyType1, FeedStatusLegacyType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   bool_t enable DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } FeedChannelControlTrafficType1_t DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_FEED_CHANNEL_CONTROL_TRAFFIC_TYPE1 MAKE_STRUCTURE_ID(eFeedChannelControlTrafficType1, FeedChannelControlTrafficType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25


#include "ntpackpop.h"

#endif // __NTAPI_PACKETFEEDSTRUCTURE_H__
