// $Id: vpdstructure.h 17384 2012-02-16 14:48:40Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Structure Definitions that are used by the external interface for
 * VPD Information
 */

#ifndef __NTAPI_VPDSTRUCTURE_H__
#define __NTAPI_VPDSTRUCTURE_H__

#include <ostype.h>
#include "basetype.h"
#include "capabilitiesstructure.h"
#include "ntpackpush.h"

#define VPD_CARD_IDENTIFIER_LENGTH 20
#define VPD_CARD_TYPE_LENGTH 6
#define VPD_CARD_MODEL_NUMBER_LENGTH 4
#define VPD_CARD_SERIAL_NUMBER_LENGTH 8
#define VPD_CARD_PRODUCT_PART_NUMBER_LENGTH 3
#define VPD_CARD_EC_NUMBER_LENGTH 3
#define VPD_CARD_DATE_LENGTH 3
#define VPD_CARD_DATE_LENGTH2 4
#define VPD_CARD_MAC_ADDRESS_LENGTH 6
#define VPD_CARD_CARD_DESCRIPTION_LENGTH 43

#define VPD_CARD_TYPE_LENGTH_TYPE2 10
#define VPD_CARD_MODEL_NUMBER_LENGTH_TYPE2 9
#define VPD_CARD_CARD_DESCRIPTION_LENGTH_TYPE2 50
#define VPD_CARD_PLUGGABLE_CHANNEL_CONFIG_BITM_LENGTH 4
#define VPD_CARD_FIXED_CHANNEL_CONFIG_BITM_LENGTH 4

#define VPD_USER_IMAGE_LENGTH 400 /* In bytes */

enum
{
  eVPDImageType1,         /* Retired. Do not use. */
  eVPDImageType2,         /* Retired. Do not use. */
  eVPDGenericImageType1,  /* Retired. Do not use. */
  eVPDImageType3_t,       /* Retired. Do not use. */
  eVPDUserImageType1,     /* User provided VPD data */
  eVPDImageType4,         /* Use for NT cards */
  eVPDImageType5,         /* Use for NT cards with AVR */
  eVPDImageType6,         /* Use for NT cards with AVR */
};

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		StructureVersion DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardIdentifier[VPD_CARD_IDENTIFIER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ProductPartNumber[VPD_CARD_PRODUCT_PART_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ProductPartNumberRevision DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint8_t		DateOfManufacture[VPD_CARD_DATE_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ManufactureLocation DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		NumberOfChannels DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } VPDGenericImageType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_VPD_GENERIC_IMAGE_TYPE1  MAKE_STRUCTURE_ID(eVPDGenericImageType1, VPDGenericImageType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		StructureVersion DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardType[VPD_CARD_TYPE_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardSeries DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardModelNumber[VPD_CARD_MODEL_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardSerialNumber[VPD_CARD_SERIAL_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ProductPartNumber[VPD_CARD_PRODUCT_PART_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ProductPartNumberRevision DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ECNumber[VPD_CARD_EC_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		MACAddress[VPD_CARD_MAC_ADDRESS_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		NumberOfChannels DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardRamFitted DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardDescription[VPD_CARD_CARD_DESCRIPTION_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ManufactureLocation DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } VPDImageType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define ST_VPD_IMAGE VPDImageType1_t
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_VPD_IMAGE_TYPE1  MAKE_STRUCTURE_ID(eVPDImageType1, VPDImageType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		StructureVersion DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardIdentifier[VPD_CARD_IDENTIFIER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardType[VPD_CARD_TYPE_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardSeries DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardModelNumber[VPD_CARD_MODEL_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardSerialNumber[VPD_CARD_SERIAL_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ProductPartNumber[VPD_CARD_PRODUCT_PART_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ProductPartNumberRevision DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ECNumber[VPD_CARD_EC_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint8_t		DateOfManufacture[VPD_CARD_DATE_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ManufactureLocation DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		NumberOfChannels DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		MACAddress[VPD_CARD_MAC_ADDRESS_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardDescription[VPD_CARD_CARD_DESCRIPTION_LENGTH_TYPE2] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardRamFitted DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		PluggableChannelConfigurationBitmask[VPD_CARD_PLUGGABLE_CHANNEL_CONFIG_BITM_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		FixedChannelConfigurationBitmask[VPD_CARD_FIXED_CHANNEL_CONFIG_BITM_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		PluggableChannelConfigurationBitmaskPhysical[VPD_CARD_PLUGGABLE_CHANNEL_CONFIG_BITM_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		FixedChannelConfigurationBitmaskPhysical[VPD_CARD_FIXED_CHANNEL_CONFIG_BITM_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } VPDImageType2Preliminary_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_VPD_IMAGE_TYPE2_PRELMIN  MAKE_STRUCTURE_ID(eVPDImageType2, VPDImageType2Preliminary_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		StructureVersion DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardIdentifier[VPD_CARD_IDENTIFIER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardType[VPD_CARD_TYPE_LENGTH_TYPE2] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardSeries DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardModelNumber[VPD_CARD_MODEL_NUMBER_LENGTH_TYPE2] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardSerialNumber[VPD_CARD_SERIAL_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ProductPartNumber[VPD_CARD_PRODUCT_PART_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ProductPartNumberRevision DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ECNumber[VPD_CARD_EC_NUMBER_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint8_t		DateOfManufacture[VPD_CARD_DATE_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		ManufactureLocation DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		NumberOfChannels DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		MACAddress[VPD_CARD_MAC_ADDRESS_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardDescription[VPD_CARD_CARD_DESCRIPTION_LENGTH_TYPE2] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		CardRamFitted DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		PluggableChannelConfigurationBitmask[VPD_CARD_PLUGGABLE_CHANNEL_CONFIG_BITM_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		FixedChannelConfigurationBitmask[VPD_CARD_FIXED_CHANNEL_CONFIG_BITM_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		PluggableChannelConfigurationBitmaskPhysical[VPD_CARD_PLUGGABLE_CHANNEL_CONFIG_BITM_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */  	uint8_t		FixedChannelConfigurationBitmaskPhysical[VPD_CARD_FIXED_CHANNEL_CONFIG_BITM_LENGTH] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } VPDImageType2_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_VPD_IMAGE_TYPE2  MAKE_STRUCTURE_ID(eVPDImageType2, VPDImageType2_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint8_t  StructureVersion DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint8_t  ManufactureLocation DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint8_t  CardType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint8_t  HWRevisionNumber DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint8_t  ModelNumber[20] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint8_t  PartNumber[20] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint8_t  SerialNumber[20] DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint32_t DCMPhaseShift DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } VPDImageType3_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_VPD_IMAGE_TYPE3 MAKE_STRUCTURE_ID(eVPDImageType3_t,VPDImageType3_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

/* NT VPD manufacturing info. */
#define VPD_NT_BOARD_TYPE_LENGTH              32
#define VPD_NT_PRODUCT_ID_LENGTH              32
#define VPD_NT_PRODUCTNAME_LENGTH            128
#define VPD_NT_PCB_ID_LENGTH                  32
#define VPD_NT_PBA_ID_LENGTH                  32
#define VPD_NT_FPGA_TYPE_LENGTH               32
#define VPD_NT_SERIAL_NUMBER_LENGTH           32

typedef struct
{
  uint8_t  StructureVersion; /* = 0x04 */
  /* Manufacturing info. */
  uint8_t  boardType[VPD_NT_BOARD_TYPE_LENGTH];
  uint8_t  productId[VPD_NT_PRODUCT_ID_LENGTH];
  uint8_t  productName[VPD_NT_PRODUCTNAME_LENGTH];
  uint8_t  pcbId[VPD_NT_PCB_ID_LENGTH];
  uint8_t  pbaId[VPD_NT_PBA_ID_LENGTH];
  uint8_t  fpgaType[VPD_NT_FPGA_TYPE_LENGTH];
  uint8_t  serialNumber[VPD_NT_SERIAL_NUMBER_LENGTH];
  uint32_t manufacturingDate;
  /* MAC addresses. */
  uint8_t  macAddressCount;
  uint8_t  macAddress[8][VPD_CARD_MAC_ADDRESS_LENGTH];
} VPDImageType4_t;

/* Definition of the STRUCTID to be used at the application layer */
#define STID_VPD_IMAGE_TYPE4 \
    MAKE_STRUCTURE_ID(eVPDImageType4, VPDImageType4_t)

/* NT VPD manufacturing info. */
#define VPD5_NT_BOARD_TYPE_LENGTH              14
#define VPD5_NT_PRODUCT_ID_LENGTH              13
#define VPD5_NT_PBA_ID_LENGTH                  16
#define VPD5_NT_SERIAL_NUMBER_LENGTH           10

typedef struct
{
  uint8_t  StructureVersion; /* = 5 */
  /* Manufacturing info. */
  uint8_t  mainPsuHwRevision;                                /* Main boars PSU revision */
  uint8_t  mainBoardType[VPD5_NT_BOARD_TYPE_LENGTH+1];       /* Main board type eg NTMAINB1E2 */
  uint8_t  mainProductId[VPD5_NT_PRODUCT_ID_LENGTH+1];       /* Product ID 801, 810 number at production time*/
  uint8_t  mainPbaId[VPD5_NT_PBA_ID_LENGTH+1];               /* Main board PBA number  */
  uint8_t  mainSerialNumber[VPD5_NT_SERIAL_NUMBER_LENGTH+1]; /* Main board serial number */
  uint8_t  frontPsuHwRevision;                               /* Front board PSU revision */
  uint8_t  frontBoardType[VPD5_NT_BOARD_TYPE_LENGTH+1];      /* Front Type in Ascii string */
  uint8_t  frontProductId[VPD5_NT_PRODUCT_ID_LENGTH+1];      /* Product ID 801 or 810 number at production time */
  uint8_t  frontPbaId[VPD5_NT_PBA_ID_LENGTH+1];              /* Front board PBA number */
  uint8_t  frontSerialNumber[VPD5_NT_SERIAL_NUMBER_LENGTH+1];/* Front serial number */
  uint8_t  productFamily; //1:Capture, 2:Inline              /* Product famile, Capture or Inline */
  /* MAC addresses. */
  uint8_t  macAddressCount;                                  /* Numer of MAC address. Same as no of ports */
  uint8_t  macAddress[VPD_CARD_MAC_ADDRESS_LENGTH];          /* Port 0 Mac address. MAC address for the following ports
                                                                can be found by incrementing Port 0 MAC address. */
} VPDImageType5_t;

/* Definition of the STRUCTID to be used at the application layer */
#define STID_VPD_IMAGE_TYPE5 \
    MAKE_STRUCTURE_ID(eVPDImageType5, VPDImageType5_t)

/* NT VPD manufacturing info. */
#define VPD6_NT_BOARD_TYPE_LENGTH              14
#define VPD6_NT_PRODUCT_ID_LENGTH              13
#define VPD6_NT_PBA_ID_LENGTH                  16
#define VPD6_NT_SERIAL_NUMBER_LENGTH           10

typedef struct
{
  uint8_t  StructureVersion; /* = 6*/
  /* Manufacturing info. */
  uint8_t  mainPsuHwRevision;                                /* Main boars PSU revision */
  uint8_t  mainBoardType[VPD6_NT_BOARD_TYPE_LENGTH+1];       /* Main board type eg NTMAINB1E2 */
  uint8_t  mainProductId[VPD6_NT_PRODUCT_ID_LENGTH+1];       /* Product ID 801, 810 number at production time*/
  uint8_t  mainPbaId[VPD6_NT_PBA_ID_LENGTH+1];               /* Main board PBA number  */
  uint8_t  mainSerialNumber[VPD6_NT_SERIAL_NUMBER_LENGTH+1]; /* Main board serial number */
  uint8_t  frontPsuHwRevision;                               /* Front board PSU revision */
  uint8_t  frontBoardType[VPD6_NT_BOARD_TYPE_LENGTH+1];      /* Front Type in Ascii string */
  uint8_t  frontProductId[VPD6_NT_PRODUCT_ID_LENGTH+1];      /* Product ID 801 or 810 number at production time */
  uint8_t  frontPbaId[VPD6_NT_PBA_ID_LENGTH+1];              /* Front board PBA number */
  uint8_t  frontSerialNumber[VPD6_NT_SERIAL_NUMBER_LENGTH+1];/* Front serial number */
  uint8_t  productFamily; //1:Capture, 2:Inline              /* Product famile, Capture or Inline */
  /* MAC addresses. */
  uint8_t  macAddressCount;                                  /* Numer of MAC address. Same as no of ports */
  uint8_t  macAddress[8][VPD_CARD_MAC_ADDRESS_LENGTH];       /* Mac address. */
} VPDImageType6_t;

#define STID_VPD_IMAGE_TYPE6 \
    MAKE_STRUCTURE_ID(eVPDImageType6, VPDImageType6_t)


/* VPDUserImageType1_t - Used to provide VPD data for customer area */
typedef struct
{
    uint8_t UserData[VPD_USER_IMAGE_LENGTH];
}VPDUserImageType1_t;

#define STID_VPD_USER_IMAGE_TYPE1 \
    MAKE_STRUCTURE_ID(eVPDUserImageType1, VPDUserImageType1_t)

#include "ntpackpop.h"

#endif // __NTAPI_VPDSTRUCTURE_H__
