#ifndef _INCLUDE_PCAP_NAPATECH_H
#define _INCLUDE_PCAP_NAPATECH_H

#include "pcap.h"

int is_napatech_device(const char *device);
int is_napatech_x_device(const char *device);

/* napatech_pcap_open_live and napatech_x_pcap_open_live
 *
 * Parameters
 * device  - the name of the device to read from
 * snaplen - maximum size of packet to capture
 * promisc - flag to indicate whether read is in promiscuous mode (ignored)
 * to_ms   - read timeout in milliseconds
 * ebuf    - buffer to put error messages in
 * Returns pointer to new pcap_t structure, or NULL on failure
 */
pcap_t *napatech_pcap_open_live( const char *device, int snaplen, int promisc,
                                 int to_ms, char *ebuf );
pcap_t *napatech_x_pcap_open_live( const char *device, int snaplen, int promisc,
                                   int to_ms, char *ebuf );

/* napatech_pcap_findalldevs and napatech_x_pcap_findalldevs
 *
 * Populate device list with napatech devices. Create the list from the
 * device names returned by NTCI_Get....Device. 
 *
 * Parameters
 * alldevsp - list to put devices in
 * errbuf   - error reporting buffer
 * Returns 0 on success, -1 on failure
 */
int napatech_platform_finddevs( pcap_if_t **devlistp, char *errbuf );


pcap_t *napatech_create(const char *device, char *ebuf, int *is_ours);


#endif /* _INCLUDE_PCAP_NAPATECH_H */
