// $Id: NtplTool.c 24128 2013-06-06 12:07:27Z lm $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * NtplTool: Passes NTPL expressions to the adapter driver
 */

#ifdef WIN32
#include <Windows.h>
#include "platformwin.h"
#else
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#endif

#include "toolslib.h"
#include "FilterTest.h"

static bool_t   UserBreakReceived=FALSE;

static char *expr_str_option;
static char *input_file_option;

static CommandLineOptions_t tools_options[] = {
  {"expr", "NTPL expression string", "\"string\"",  0, &expr_str_option},
  {"file", "Input text file", "filename",  0, &input_file_option},
  {NULL, NULL, NULL, 0, NULL}
};

static char tool_description[] = {"Pass NTPL expression(s) to an Adapter.\n"};
static char output_description[] = {"\n"};

static char * strskp (char * s)
{
    char * skip = s;
    if (s==NULL) return NULL ;
    while (isspace((int)*skip) && *skip) skip++;
    return skip ;
}


#ifdef WIN32
static BOOL CatchSignalCTRL_C(DWORD CtrlType)
{
  static uint8_t  breakcnt=0;

  UserBreakReceived = TRUE;

  if (++breakcnt >= 3) {
    /* User tried to break 3 times. Something must be hanging... Just exit */
    exit(0);
  }  
  return TRUE;
}
#else
static void CatchSignalCTRL_C (int sig __attribute__((unused)))
{
  static uint8_t  breakcnt=0;
  
  UserBreakReceived = TRUE;

  if (++breakcnt >= 3) {
    /* User tried to break 3 times. Something must be hanging... Just exit */
    exit(0);
  }  
}
#endif


#define LINEMAXLEN         4096

int main(int argc, char *argv[])
{
  uint32_t  cardIndex;
  char      *ProgramName = argv[0];
  char      line[LINEMAXLEN+1];
  FILE      *filehandle=NULL;
  char      *firstchar;
  int       returnCode=0;
  char      *ptr;
  char      crlf[3] = {10, 13, 0};

  /* Parse the command line and initialize the needed variables. */    
  if(InitApplication(&argc, argv, NTSERIES_NT4PORT /* Exclude NT4PORT */, &cardIndex, 
                     tools_options, tool_description,
                     output_description) == -1) {
    exit(-1);
  }

  DisplayProgramHeader(ProgramName, PRG_TOOL);

#ifdef WIN32
  SetConsoleCtrlHandler((PHANDLER_ROUTINE)CatchSignalCTRL_C, TRUE);
#else
  signal(SIGINT, CatchSignalCTRL_C);
#endif

  if (input_file_option != NULL) {
    if ((filehandle = fopen(input_file_option, "r"))==NULL) {
      WriteToLog(LOG_ERROR, ">>> Error: Could not open file \"%s\"\n", input_file_option);
      goto errorHandling;
    }

    while ((fgets(line, LINEMAXLEN, filehandle) != NULL) && !UserBreakReceived) {
      /* Remove comments starting with "//" from the line */
      if ((firstchar = strstr(line, "//"))) {
        *firstchar = '\0';
      }
      firstchar = strskp(line);
      /* Skip comment lines and empty lines */
      if ((*firstchar != '#') && (*firstchar != '\0')) {      
        /* Remove trailing CR and LF */
        ptr = strpbrk(line, crlf);
        if (ptr != NULL) {
          *ptr = '\0';
        }
        if (PassFilter(cardIndex, line) == NTCI_ERRCODE_SUCCESS) {
          WriteToLog(LOG_INFO, ">>> Info: Line \"%s\" executed successfully\n", line);
        } else {
          returnCode=-1;
          break;
        }
      }
    }
    fclose(filehandle); 
  } else if (expr_str_option != NULL) {
    if (PassFilter(cardIndex, expr_str_option) == NTCI_ERRCODE_SUCCESS) {
      WriteToLog(LOG_INFO, ">>> Info: Expression \"%s\" executed successfully\n", expr_str_option);
    } else {
      returnCode=-1;
    }    
  } else {
    WriteToLog(LOG_ERROR, ">>> Error: You need to specify at least one options\n           Write NtplTool -help for list of options\n");
  }
  
errorHandling:  
  /* shutdown card */  
  CardShutdown(0);

  DisplayProgramFooter();
  
  return returnCode;
}
