// $Id: PacketfeedStatus.c 14206 2011-09-08 16:47:22Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This tool demonstrates dumping of the packet feed status counters.
 */

#ifdef _WIN32
#include <windows.h>
#include "platformwin.h"
#else
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <signal.h>
#include <time.h>
#include <termios.h>
#include <sys/time.h>
#include <errno.h>
#include <unistd.h>

#endif

#include <toolslib.h>

/* The NTCommonInterface.h header provides the interface definitions
 * for the Napatech NTCI library */

#include <NTCommonInterface.h>

#include <packetfeedstructure.h>

static char *req_feed=NULL;

static CommandLineOptions_t tools_options[] = {
  {"feed", "Feed to query.", "0..31",  1, &req_feed},
  {NULL, NULL, NULL, 0, NULL}
};

static char tool_description[] = {"Display feed status.\n"};
static char output_description[] = {"\n"};



int main(int argc, char *argv[])
{
  uint32_t cardIndex;
  uint32_t result;
  uint8_t  feed=0;
  handle_t cardHandle;

  /* Parse the command line and initialize the needed variables. */    
  if(InitApplication(&argc, argv, TRUE /* capture capable cards only. */, &cardIndex,
                     tools_options, tool_description,
                     output_description) == -1) {
    exit(-1);
  }

  if((result = GetCardHandle(cardIndex, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    fprintf(stderr, ">>> Error: Failed to get card handle.\n");
    exit(-1);
  }
  
  if (req_feed != NULL) {
    feed = AsciiToInt(req_feed);
  }
  
  DisplayProgramHeader(argv[0], PRG_TOOL);

    PacketFeedStatus(cardIndex, feed, 4);
  /* shutdown card */
  CardShutdown(0);

  
  DisplayProgramFooter();

  return 0;
}


