// $Id: CardConfiguration.c 25520 2013-09-16 12:16:58Z fc $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * CardConfiguration.c : Contain functions to get/set card configurations
 */

#ifdef WIN32
#include <Windows.h>
#include <stdio.h>
#else
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#endif

/* These headers are needed when monitoring sensors. */
#include <hwmonstructure.h>
#include <NTCI_hardwaremonitor.h>

/* The NTCommonInterface.h header provides the interface definitions
 * for the NTCI library */

#include <NTCommonInterface.h>

/* These headers are needed for accessing the capabilities interface: */

#include <NTCI_capabilities.h>
#include <capabilitiesstructure.h>

/* These headers are needed for the statistics API */

#include <NTCI_cardconfiguration.h>
#include <cardconfigurationstructure.h>
#include <NTCI_packetfeed.h>
#include <packetfeedstructure.h>

#include "toolslib.h"
#include "Statistics.h"

static char WriteBuffer[2048];
static uint32_t writePos=0;
#define DUMP_TO_SCREEN(...) writePos += WriteToBuffer(&WriteBuffer[writePos], sizeof(WriteBuffer)-writePos,  __VA_ARGS__);

uint32_t CardConfigMaxFrameSize(uint8_t card, uint32_t channelsBitmask, uint32_t maxFrameSize)
{
  uint32_t result;
  handle_t cardHandle;
  FrameDecoderConfigurationType2_t config;

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  config.maxNormalFrameSize = maxFrameSize;
  config.decoderMode = 1;
  
  return NTCI_SetCardConfigurationDetails(cardHandle,
                                          channelsBitmask,
                                          STID_FRAME_DECODER_CONFIGURATION_TYPE2,
                                          &config);
}

uint32_t CardConfigDecoderSet(uint8_t card, uint32_t channelsBitmask,
                              uint8_t decoderMode, uint32_t maxFrameSize,
                              uint8_t enableISL, uint8_t enableVLAN, uint8_t enableJUMBO, uint8_t enableMPLS,
                              uint8_t allowISLVLAN, uint8_t allowISLJUMBO, uint8_t allowJUMBOVLAN, uint8_t allowJUMBOMPLS,
                              uint8_t ISL64IsSmall, uint8_t VLAN64IsSmall)
{
  uint32_t result;
  handle_t cardHandle;
  void *pConfig;
  uint32_t stid;
  FrameDecoderConfigurationType6_t config6;
  
  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }
 
  config6.decoderMode = decoderMode;
  config6.maxNormalFrameSize = maxFrameSize;
  config6.enableISL = enableISL;
  config6.enableVLAN = enableVLAN;
  config6.enableJUMBO = enableJUMBO;
  config6.enableMPLS = enableMPLS;
  config6.allowISLVLAN = allowISLVLAN;
  config6.allowISLJUMBO = allowISLJUMBO;
  config6.allowJUMBOVLAN = allowJUMBOVLAN;
  config6.allowJUMBOMPLS = allowJUMBOMPLS;
  config6.ISL64IsSmall = ISL64IsSmall;
  config6.VLAN64IsSmall = VLAN64IsSmall;
  stid = STID_FRAME_DECODER_CONFIGURATION_TYPE6;
  pConfig = &config6;
  return NTCI_SetCardConfigurationDetails(cardHandle,
                                          channelsBitmask,
                                          stid,
                                          pConfig);
}

uint32_t CardConfigDecoderSet7(uint8_t card, uint32_t channelsBitmask,
                              uint8_t decoderMode, uint32_t maxFrameSize,
                              uint8_t enableISL, uint8_t enableVLAN, uint8_t enableJUMBO, uint8_t enableMPLS,
                              uint8_t allowISLVLAN, uint8_t allowISLJUMBO, uint8_t allowJUMBOVLAN, uint8_t allowJUMBOMPLS,
                              uint8_t ISL64IsSmall, uint8_t VLAN64IsSmall, uint8_t MPLS64IsSmall)
{
  uint32_t result;
  handle_t cardHandle;
  void *pConfig;
  uint32_t stid;
  FrameDecoderConfigurationType7_t config7;
  
  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }
 
  config7.decoderMode = decoderMode;
  config7.maxNormalFrameSize = maxFrameSize;
  config7.enableISL = enableISL;
  config7.enableVLAN = enableVLAN;
  config7.enableJUMBO = enableJUMBO;
  config7.enableMPLS = enableMPLS;
  config7.allowISLVLAN = allowISLVLAN;
  config7.allowISLJUMBO = allowISLJUMBO;
  config7.allowJUMBOVLAN = allowJUMBOVLAN;
  config7.allowJUMBOMPLS = allowJUMBOMPLS;
  config7.ISL64IsSmall = ISL64IsSmall;
  config7.VLAN64IsSmall = VLAN64IsSmall;
	config7.MPLS64IsSmall = MPLS64IsSmall;
  stid = STID_FRAME_DECODER_CONFIGURATION_TYPE7;
  pConfig = &config7;
  return NTCI_SetCardConfigurationDetails(cardHandle,
                                          channelsBitmask,
                                          stid,
                                          pConfig);
}

uint32_t CardConfigDecoderSet8(uint8_t card, uint32_t channelsBitmask,
                               uint8_t decoderMode, uint32_t maxFrameSize,
                               uint8_t enableISL, uint8_t enableVLAN, uint8_t enableJUMBO, uint8_t enableMPLS,
                               uint8_t allowISLVLAN, uint8_t allowISLJUMBO, uint8_t allowJUMBOVLAN, uint8_t allowJUMBOMPLS,
                               uint8_t ISL64IsSmall, uint8_t VLAN64IsSmall, uint8_t MPLS64IsSmall, uint16_t altTPID)
{
  uint32_t result;
  handle_t cardHandle;
  void *pConfig;
  uint32_t stid;
  FrameDecoderConfigurationType8_t config8;
  
  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }
 
  config8.decoderMode = decoderMode;
  config8.maxNormalFrameSize = maxFrameSize;
  config8.enableISL = enableISL;
  config8.enableVLAN = enableVLAN;
  config8.enableJUMBO = enableJUMBO;
  config8.enableMPLS = enableMPLS;
  config8.allowISLVLAN = allowISLVLAN;
  config8.allowISLJUMBO = allowISLJUMBO;
  config8.allowJUMBOVLAN = allowJUMBOVLAN;
  config8.allowJUMBOMPLS = allowJUMBOMPLS;
  config8.ISL64IsSmall = ISL64IsSmall;
  config8.VLAN64IsSmall = VLAN64IsSmall;
	config8.MPLS64IsSmall = MPLS64IsSmall;
  config8.altTPID = altTPID;
  stid = STID_FRAME_DECODER_CONFIGURATION_TYPE8;
  pConfig = &config8;
  return NTCI_SetCardConfigurationDetails(cardHandle,
                                          channelsBitmask,
                                          stid,
                                          pConfig);
}

uint32_t _CardConfigMaxFrameSize(void *arglist)
{
  uint8_t *argptr = arglist;
  uint8_t card;
  uint32_t channelsBitmask;
  uint32_t maxFrameSize;

  card            = get8incr(&argptr);
  channelsBitmask = get32incr(&argptr);
  maxFrameSize    = get32incr(&argptr);
  return CardConfigMaxFrameSize(card, channelsBitmask, maxFrameSize);
}


uint32_t _CardConfigDecoderSet(void *arglist)
{
  uint8_t *argptr = arglist;
  uint8_t card;
  uint32_t channelsBitmask;
  uint8_t decoderMode;
  uint32_t maxFrameSize;
  uint8_t enableISL;
  uint8_t enableVLAN;
  uint8_t enableJUMBO;
  uint8_t enableMPLS;
  uint8_t allowISLVLAN;
  uint8_t allowISLJUMBO;
  uint8_t allowJUMBOVLAN;
  uint8_t allowJUMBOMPLS;
  uint8_t ISL64IsSmall;
  uint8_t VLAN64IsSmall;

  card             = get8incr(&argptr);
  channelsBitmask  = get32incr(&argptr);
  decoderMode      = get8incr(&argptr);
  maxFrameSize     = get32incr(&argptr);
  enableISL        = get8incr(&argptr);
  enableVLAN       = get8incr(&argptr);
  enableJUMBO      = get8incr(&argptr);
  enableMPLS       = get8incr(&argptr);
  allowISLVLAN     = get8incr(&argptr);
  allowISLJUMBO    = get8incr(&argptr);
  allowJUMBOVLAN   = get8incr(&argptr);
  allowJUMBOMPLS   = get8incr(&argptr);
  ISL64IsSmall     = get8incr(&argptr);
  VLAN64IsSmall    = get8incr(&argptr);

  return CardConfigDecoderSet(card, channelsBitmask, decoderMode, maxFrameSize,
                              enableISL, enableVLAN, enableJUMBO, enableMPLS,
                              allowISLVLAN, allowISLJUMBO, allowJUMBOVLAN, allowJUMBOMPLS,
                              ISL64IsSmall, VLAN64IsSmall);
}


uint32_t _CardConfigDecoderSet7(void *arglist)
{
  uint8_t *argptr = arglist;
  uint8_t card;
  uint32_t channelsBitmask;
  uint8_t decoderMode;
  uint32_t maxFrameSize;
  uint8_t enableISL;
  uint8_t enableVLAN;
  uint8_t enableJUMBO;
  uint8_t enableMPLS;
  uint8_t allowISLVLAN;
  uint8_t allowISLJUMBO;
  uint8_t allowJUMBOVLAN;
  uint8_t allowJUMBOMPLS;
  uint8_t ISL64IsSmall;
  uint8_t VLAN64IsSmall;
  uint8_t MPLS64IsSmall;

  card             = get8incr(&argptr);
  channelsBitmask  = get32incr(&argptr);
  decoderMode      = get8incr(&argptr);
  maxFrameSize     = get32incr(&argptr);
  enableISL        = get8incr(&argptr);
  enableVLAN       = get8incr(&argptr);
  enableJUMBO      = get8incr(&argptr);
  enableMPLS       = get8incr(&argptr);
  allowISLVLAN     = get8incr(&argptr);
  allowISLJUMBO    = get8incr(&argptr);
  allowJUMBOVLAN   = get8incr(&argptr);
  allowJUMBOMPLS   = get8incr(&argptr);
  ISL64IsSmall     = get8incr(&argptr);
  VLAN64IsSmall    = get8incr(&argptr);
  MPLS64IsSmall    = get8incr(&argptr);

  return CardConfigDecoderSet7(card, channelsBitmask, decoderMode, maxFrameSize,
                              enableISL, enableVLAN, enableJUMBO, enableMPLS,
                              allowISLVLAN, allowISLJUMBO, allowJUMBOVLAN, allowJUMBOMPLS,
                              ISL64IsSmall, VLAN64IsSmall, MPLS64IsSmall);
}


uint32_t _CardConfigDecoderSet8(void *arglist)
{
  uint8_t *argptr = arglist;
  uint8_t card;
  uint32_t channelsBitmask;
  uint8_t decoderMode;
  uint32_t maxFrameSize;
  uint8_t enableISL;
  uint8_t enableVLAN;
  uint8_t enableJUMBO;
  uint8_t enableMPLS;
  uint8_t allowISLVLAN;
  uint8_t allowISLJUMBO;
  uint8_t allowJUMBOVLAN;
  uint8_t allowJUMBOMPLS;
  uint8_t ISL64IsSmall;
  uint8_t VLAN64IsSmall;
  uint8_t MPLS64IsSmall;
  uint16_t altTPID;

  card             = get8incr(&argptr);
  channelsBitmask  = get32incr(&argptr);
  decoderMode      = get8incr(&argptr);
  maxFrameSize     = get32incr(&argptr);
  enableISL        = get8incr(&argptr);
  enableVLAN       = get8incr(&argptr);
  enableJUMBO      = get8incr(&argptr);
  enableMPLS       = get8incr(&argptr);
  allowISLVLAN     = get8incr(&argptr);
  allowISLJUMBO    = get8incr(&argptr);
  allowJUMBOVLAN   = get8incr(&argptr);
  allowJUMBOMPLS   = get8incr(&argptr);
  ISL64IsSmall     = get8incr(&argptr);
  VLAN64IsSmall    = get8incr(&argptr);
  MPLS64IsSmall    = get8incr(&argptr);
  altTPID          = get16incr(&argptr);

  return CardConfigDecoderSet8(card, channelsBitmask, decoderMode, maxFrameSize,
                               enableISL, enableVLAN, enableJUMBO, enableMPLS,
                               allowISLVLAN, allowISLJUMBO, allowJUMBOVLAN, allowJUMBOMPLS,
                               ISL64IsSmall, VLAN64IsSmall, MPLS64IsSmall, altTPID);
}


uint32_t CardConfigPolicySet(uint8_t card, uint32_t feedIndex, uint32_t channelsBitmask, 
                             uint32_t blockLevel, uint32_t policeLevel, uint32_t restartLevel, uint8_t enableLevels,
                             uint32_t chan0BucketSize, uint32_t chan0Bandwidth,
                             uint32_t chan1BucketSize, uint32_t chan1Bandwidth)
{
  uint32_t result;
  handle_t cardHandle;
  SDRAMPolicyConfigurationType1_t policy;
  FeedDescriptionType1_t description;

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  if((result = NTCI_GetPacketFeedDescription(cardHandle, 
                                             feedIndex,
                                             STID_FEED_DESCRIPTION_TYPE1,
                                             &description)) != NTCI_ERRCODE_SUCCESS) {
    WriteToLog(LOG_ERROR, "CardConfigPolicySet: Invalid feedIndex.\n");
    return result;
  }
    
  memset(&policy, 0x00, sizeof(policy));
  policy.feedHandle = description.handle;
  policy.BlockLevel = blockLevel;
  policy.PoliceLevel= policeLevel;
  policy.RestartLevel=restartLevel;
  policy.EnableLevels=enableLevels;
  policy.channelPolicy[0].Bandwidth  = chan0Bandwidth;
  policy.channelPolicy[0].BucketSize = chan0BucketSize;
  policy.channelPolicy[1].Bandwidth  = chan1Bandwidth;
  policy.channelPolicy[1].BucketSize = chan1BucketSize;

  return NTCI_SetCardConfigurationDetails(cardHandle,
                                          channelsBitmask,
                                          STID_SDRAM_POLICY_CONFIGURATION_TYPE1,
                                          &policy);
}

uint32_t _CardConfigPolicySet(void *arglist)
{
  uint8_t *argptr = arglist;
  uint8_t card;
  uint32_t channelsBitmask;
  uint32_t feedIndex;
  uint32_t blockLevel;
  uint32_t policeLevel;
  uint32_t restartLevel;
  uint8_t enableLevels;
  uint32_t chan0BucketSize;
  uint32_t chan0Bandwidth;
  uint32_t chan1BucketSize;
  uint32_t chan1Bandwidth;

  card            = get8incr(&argptr);
  feedIndex       = get32incr(&argptr);
  channelsBitmask = get32incr(&argptr);
  blockLevel      = get32incr(&argptr);
  policeLevel     = get32incr(&argptr);
  restartLevel    = get32incr(&argptr);
  enableLevels    = get8incr(&argptr);
  chan0BucketSize = get32incr(&argptr);
  chan0Bandwidth  = get32incr(&argptr);
  chan1BucketSize = get32incr(&argptr);
  chan1Bandwidth  = get32incr(&argptr);

  return CardConfigPolicySet(card, feedIndex, channelsBitmask,
                             blockLevel, policeLevel, restartLevel, enableLevels,
                             chan0BucketSize, chan0Bandwidth,
                             chan1BucketSize, chan1Bandwidth);
}


static const char *YesOrNo(uint32_t val)
{
    if (val)
      return "Yes";
    else
      return "No ";
}

static const char *DuplexTxt(uint32_t val)
{
    if (val)
        return "Full-duplex";
    else
        return "Half-duplex";
}

static const char *MdiModeTxt(uint32_t val)
{
	switch (val) 
	{
	case MDI_MODE_AUTO:
		return("AutoMDI/MDIX");
	case MDI_MODE_MDI: 
		return("MDI");
	case MDI_MODE_MDIX:
		return("MDIX");
	case MDI_MODE_NA:
		return("N/A");    
	default:
		return("reserved");
	}
}

static const char *SpeedTxt(uint32_t val)
{
	switch (val) 
	{
	case MANUAL_SPEED_1000:
		return("1000");
	case MANUAL_SPEED_100:
		return("100");
	case MANUAL_SPEED_10:
		return("10");
	default:
		return("reserved");
	}
}

uint32_t GetAutonegotiationMode(uint8_t card, uint32_t channelsBitmask)
{
  handle_t cardHandle;
  AutonegotiationModeType6_t conf6;
  AutonegotiationModeType5_t conf;
  AutonegotiationModeType4_t conf4;
  uint32_t modeType = 0, result;

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  result = NTCI_GetAutonegotiationMode(cardHandle,
                                       channelsBitmask,
                                       STID_AUTONEGOTIATION_MODE_TYPE6,
                                       &conf6);
  if(result == NTCI_ERRCODE_SUCCESS) {
    modeType = 6;
  } else {
    result = NTCI_GetAutonegotiationMode(cardHandle,
                                         channelsBitmask,
                                         STID_AUTONEGOTIATION_MODE_TYPE5,
                                         &conf);
    if(result == NTCI_ERRCODE_SUCCESS) {
      modeType = 5;
    } else {
      result = NTCI_GetAutonegotiationMode(cardHandle,
                                           channelsBitmask,
                                           STID_AUTONEGOTIATION_MODE_TYPE4,
                                           &conf4);
       modeType = 4;
       conf.enable                   = conf4.enable;
       conf.required                 = conf4.required;
       conf.mdiMode                  = conf4.mdiMode;
       conf.advertise10FullDuplex    = conf4.advertise10FullDuplex;
       conf.advertise10HalfDuplex    = conf4.advertise10HalfDuplex;
       conf.advertise100FullDuplex   = conf4.advertise100FullDuplex;
       conf.advertise100HalfDuplex   = conf4.advertise100HalfDuplex;
       conf.advertise1000FullDuplex  = conf4.advertise1000FullDuplex;
       conf.advertise1000HalfDuplex  = conf4.advertise1000HalfDuplex;
       conf.advertise10000FullDuplex = conf4.advertise10000FullDuplex;
       conf.advertise10000HalfDuplex = conf4.advertise10000HalfDuplex;
       conf.manualSpeed              = conf4.manualSpeed;
       conf.manualFullDuplex         = conf4.manualFullDuplex;
    }

    if (result != NTCI_ERRCODE_SUCCESS) {
      fprintf(stderr, ">>> ERROR: Failed getting autonegotiation mode for channel bitmask 0x%02X. Errorcode: 0x%02X\n",  channelsBitmask, result);            
      return result;
    }
  }

  if ((modeType == 4) || (modeType == 5)) {
    writePos=0;
    DUMP_TO_SCREEN("Autonegotiation configuration for channelmask: 0x%02X\n", channelsBitmask);
    DUMP_TO_SCREEN("  Autonegotiation enabled : %s\n", YesOrNo(conf.enable));
    DUMP_TO_SCREEN("  Autonegotiation required: %s\n", YesOrNo(conf.required));
    DUMP_TO_SCREEN("  Cable detect            : %s\n", MdiModeTxt(conf.mdiMode));
    DUMP_TO_SCREEN("  Advertise:\n");
    DUMP_TO_SCREEN("  10 MBit :   Full duplex: %s   Half duplex: %s\n",
                   YesOrNo(conf.advertise10FullDuplex),
                   YesOrNo(conf.advertise10HalfDuplex));
    DUMP_TO_SCREEN("  100 MBit:   Full duplex: %s   Half duplex: %s\n",
                   YesOrNo(conf.advertise100FullDuplex),
                   YesOrNo(conf.advertise100HalfDuplex));
    DUMP_TO_SCREEN("  1 GBit  :   Full duplex: %s   Half duplex: %s\n",
                   YesOrNo(conf.advertise1000FullDuplex),
                   YesOrNo(conf.advertise1000HalfDuplex));
    DUMP_TO_SCREEN("  10 GBit :   Full duplex: %s   Half duplex: %s\n",
                   YesOrNo(conf.advertise10000FullDuplex),
                   YesOrNo(conf.advertise10000HalfDuplex));
    DUMP_TO_SCREEN("\n");
    if (conf.enable == 0) {
      DUMP_TO_SCREEN("Manual configuration (active when Autonegotiation is disabled):\n");
      DUMP_TO_SCREEN("  Current speed:   %s\n", SpeedTxt(conf.manualSpeed));
      DUMP_TO_SCREEN("  Current duplex:  %s\n", DuplexTxt(conf.manualFullDuplex));
      DUMP_TO_SCREEN("\n");
    }
    WriteToLog(LOG_INFO, WriteBuffer);
  } else if(modeType == 6) {
    writePos=0;
    DUMP_TO_SCREEN("Autonegotiation configuration for channelmask: 0x%02X\n", channelsBitmask);
    DUMP_TO_SCREEN("  Autonegotiation enabled : %s\n", YesOrNo(conf6.enable));
    DUMP_TO_SCREEN("  Autonegotiation required: %s\n", YesOrNo(conf6.required));
    DUMP_TO_SCREEN("  Cable detect            : %s\n", MdiModeTxt(conf6.mdiMode));
    DUMP_TO_SCREEN("  Advertise:\n");
    DUMP_TO_SCREEN("  10 MBit  :   Full duplex: %s   Half duplex: %s\n",
                   YesOrNo(conf6.advertiseSpeedDuplexBitMask & ADVERTISE_10M_FULLDUPLEX),
                   YesOrNo(conf6.advertiseSpeedDuplexBitMask & ADVERTISE_10M_HALFDUPLEX));
    DUMP_TO_SCREEN("  100 MBit :   Full duplex: %s   Half duplex: %s\n",
                   YesOrNo(conf6.advertiseSpeedDuplexBitMask & ADVERTISE_100M_FULLDUPLEX),
                   YesOrNo(conf6.advertiseSpeedDuplexBitMask & ADVERTISE_100M_HALFDUPLEX ));
    DUMP_TO_SCREEN("  1 GBit   :   Full duplex: %s   Half duplex: %s\n",
                   YesOrNo(conf6.advertiseSpeedDuplexBitMask & ADVERTISE_1G_FULLDUPLEX  ),
                   YesOrNo(conf6.advertiseSpeedDuplexBitMask & ADVERTISE_1G_HALFDUPLEX ));
    DUMP_TO_SCREEN("  10 GBit  :   Full duplex: %s\n",
                   YesOrNo(conf6.advertiseSpeedDuplexBitMask & ADVERTISE_10G_FULLDUPLEX));
    DUMP_TO_SCREEN("  40 GBit  :   Full duplex: %s\n",
                   YesOrNo(conf6.advertiseSpeedDuplexBitMask & ADVERTISE_40G_FULLDUPLEX));
    DUMP_TO_SCREEN("  100 GBit :   Full duplex: %s\n",
                   YesOrNo(conf6.advertiseSpeedDuplexBitMask & ADVERTISE_100G_FULLDUPLEX));
    DUMP_TO_SCREEN("\n");

    if (conf6.enable == 0) {
      DUMP_TO_SCREEN("Manual configuration (active when Autonegotiation is disabled):\n");
      DUMP_TO_SCREEN("  Current speed:   %s\n", SpeedTxt(conf6.manualSpeed));
      DUMP_TO_SCREEN("  Current duplex:  %s\n", DuplexTxt(conf6.manualFullDuplex));
      DUMP_TO_SCREEN("\n");
    }
    WriteToLog(LOG_INFO, WriteBuffer);
  }
  return 0;
}

uint32_t _GetAutonegotiationMode(void *arglist)
{
  uint8_t *argptr = arglist;
  uint8_t card;
  uint32_t channelsBitmask;

  card                    = get8incr(&argptr);
  channelsBitmask         = get32incr(&argptr);

  return GetAutonegotiationMode(card, channelsBitmask);
}


uint32_t SetAutonegotiationMode(uint8_t card, uint32_t channelBitmask,
                                uint8_t enable, uint8_t required, uint8_t restartAutoNeg,
                                uint8_t advertise10000FullDuplex, NOUSE uint8_t	advertise10000HalfDuplex,
                                uint8_t	advertise1000FullDuplex, uint8_t	advertise1000HalfDuplex,
                                uint8_t	advertise100FullDuplex, uint8_t	advertise100HalfDuplex,
                                uint8_t	advertise10FullDuplex, uint8_t	advertise10HalfDuplex,
                                uint8_t pauseControlMode, uint8_t	manualFullDuplex,
                                uint8_t	manualSpeed, uint8_t mdiMode,
                                uint8_t setdefault)
{
  handle_t cardHandle;
  AutonegotiationModeType6_t conf;
  uint32_t result;

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  conf.enable = enable;
  conf.required = required;
  conf.restartAutoNeg = restartAutoNeg;
  conf.advertiseSpeedDuplexBitMask = 0;

  if(advertise10000FullDuplex) {
    conf.advertiseSpeedDuplexBitMask |=  ADVERTISE_10G_FULLDUPLEX;
  }
  if(advertise1000FullDuplex) {
    conf.advertiseSpeedDuplexBitMask |=  ADVERTISE_1G_FULLDUPLEX;
  }
  if(advertise1000HalfDuplex) {
    conf.advertiseSpeedDuplexBitMask |=  ADVERTISE_1G_HALFDUPLEX;
  }
  if(advertise100FullDuplex) {
    conf.advertiseSpeedDuplexBitMask |=  ADVERTISE_100M_FULLDUPLEX;
  }
  if(advertise100HalfDuplex) {
    conf.advertiseSpeedDuplexBitMask |=  ADVERTISE_100M_HALFDUPLEX;
  }
  if(advertise10FullDuplex) {
    conf.advertiseSpeedDuplexBitMask |=  ADVERTISE_10M_FULLDUPLEX;
  }
  if(advertise10HalfDuplex) {
    conf.advertiseSpeedDuplexBitMask |=  ADVERTISE_10M_HALFDUPLEX;
  }
  conf.pauseControlMode        =pauseControlMode;
  conf.manualFullDuplex        =manualFullDuplex;
  conf.manualSpeed             =manualSpeed;
  conf.mdiMode                 =mdiMode;
  conf.setdefault              =setdefault;

  result = NTCI_SetAutonegotiationMode(cardHandle,
                                       channelBitmask,
                                       STID_AUTONEGOTIATION_MODE_TYPE6,
                                       &conf);
                                       
  if (result != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  return 0;
}

uint32_t _SetAutonegotiationMode(void *arglist)
{
  uint8_t *argptr = arglist;
  uint8_t card;
  uint32_t channelBitmask;
  uint8_t enable;
  uint8_t required;
  uint8_t restartAutoNeg;
  uint8_t advertise10000FullDuplex;
  uint8_t	advertise10000HalfDuplex;
  uint8_t	advertise1000FullDuplex;
  uint8_t	advertise1000HalfDuplex;
  uint8_t	advertise100FullDuplex;
  uint8_t	advertise100HalfDuplex;
  uint8_t	advertise10FullDuplex;
  uint8_t	advertise10HalfDuplex;
  uint8_t pauseControlMode;
  uint8_t	manualFullDuplex;
  uint8_t	manualSpeed;
  uint8_t mdiMode;
  uint8_t setdefault = 0;  

  card                    = get8incr(&argptr);
  channelBitmask          = get32incr(&argptr);
  enable                  = get8incr(&argptr);
  required                =get8incr(&argptr);
  restartAutoNeg          =get8incr(&argptr);
  advertise10000FullDuplex=get8incr(&argptr);
  advertise10000HalfDuplex=get8incr(&argptr);
  advertise1000FullDuplex =get8incr(&argptr);
  advertise1000HalfDuplex =get8incr(&argptr);
  advertise100FullDuplex  =get8incr(&argptr);
  advertise100HalfDuplex  =get8incr(&argptr);
  advertise10FullDuplex   =get8incr(&argptr);
  advertise10HalfDuplex   =get8incr(&argptr);
  pauseControlMode        =get8incr(&argptr);
  manualFullDuplex        =get8incr(&argptr);
  manualSpeed             =get8incr(&argptr);
  mdiMode                 =get8incr(&argptr);

  return SetAutonegotiationMode(card, channelBitmask, enable, required, restartAutoNeg,
                                advertise10000FullDuplex, advertise10000HalfDuplex,
                                advertise1000FullDuplex, advertise1000HalfDuplex,
                                advertise100FullDuplex, advertise100HalfDuplex,
                                advertise10FullDuplex, advertise10HalfDuplex,
                                pauseControlMode, manualFullDuplex,
                                manualSpeed, mdiMode,
                                setdefault);
}


uint32_t CardConfigTimeSync(uint8_t card, char *action)
{
  uint32_t result;
  handle_t cardHandle;
  int i;
  bool_t foundCapability=FALSE;
  TimeSyncConfigurationType1_t config1;
  TimeSyncConfigurationType2_t config2;
  TimeSyncConfigurationType3_t config3;
  TimeSyncConfigurationType4_t config4;
  CardHardwareCardCapabilitiesType2_t capability;
  CardHardwareCapabilitiesType1_t data1;  
  TimestampConfigurationType1_t conf;
  enum { CONFIGURE, STATUS, RESET } mode=0;
  
  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  if(!strcmp(action, "master") || !strcmp(action, "slave")) {
    mode = CONFIGURE;
  } else if (!strcmp(action, "status")) {
    mode = STATUS;
  } else if (!strcmp(action, "reset")) {
    mode = RESET;
  } else {
    WriteToLog(LOG_ERROR, "CardConfigTimeSync: Unknown action: \"%s\".\n", action);
    return NTCI_ERRCODE_INVALID_PARAMETER;
  }
    
  for(i=0; i<MAX_CAPABILITIES_SUBTREES; i++) {
    result = NTCI_GetCapabilities(cardHandle,
                                  CAPID_CARD_INDEXED,
                                  i,
                                  STID_COMMON_CAPABILITIES_TYPE1,
                                  &capability);    
    if(result != NTCI_ERRCODE_SUCCESS) {
      fprintf(stderr, "Error finding HW capabilities: %i\n", result);      
      break;
    }

    result = NTCI_GetCapabilities(cardHandle,
                                  CAPID_HARDWARE,
                                  NOT_SUB_CAPABILITIES,
                                  STID_CARD_HARDWARE_CAPABILITIES_TYPE1,
                                  &data1); 
    if(result != NTCI_ERRCODE_SUCCESS) {
      fprintf(stderr, "Error finding HW capabilities: %i\n", result);        
      break;
    }
    
    if(capability.type1.common.structId == STID_CARD_HARDWARE_CARD_CAPABILITIES_TYPE2) {
      result = NTCI_GetCapabilities(cardHandle,
                                    CAPID_CARD_INDEXED,
                                    i,
                                    STID_CARD_HARDWARE_CARD_CAPABILITIES_TYPE2,
                                    &capability);
      if(result != NTCI_ERRCODE_SUCCESS) {
        WriteToLog(LOG_ERROR, "Failed getting capability\n");
        return result;
      }
      foundCapability = TRUE;
      break;
    }
  }

  if(foundCapability != TRUE) {
    WriteToLog(LOG_ERROR, "Failed getting capability\n");
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }

  if(!capability.TimeSyncSupported) {
    if (!((data1.CardType == NT4STD) && (mode == STATUS))) {
      WriteToLog(LOG_ERROR, "CardConfigTimeSync: The requested action is not supported on this adapter\n");
      return NTCI_ERRCODE_INTERNAL_ERROR;
    }
  }
  
  /* Perform operation */  
  switch (mode)
  {
  case CONFIGURE:
  {
    config1.timesyncCommandType   = TIMESYNC_COMMAND_CONFIGURE;
    config1.timesyncConfigMaster  = !strcmp(action, "master") ? TRUE : FALSE;
    
    result = NTCI_SetCardConfigurationDetails(cardHandle,
                                              0 /* Not needed. */,
                                              STID_TIMESYNC_CONFIGURATION_TYPE1,
                                              &config1);
    if(result != NTCI_ERRCODE_SUCCESS) {
      WriteToLog(LOG_ERROR, "Failed writing Timesync configuration.\n",
                 result);
      WriteToLog(LOG_INFO, "Please make sure the adapter is not configured for OS Timesync.\n");
      WriteToLog(LOG_INFO, "Please make sure the adapter is not configured for High Frequency Sampling\n");
      return result;
    }
    WriteToLog(LOG_INFO, "Time synchronization configuration (%s) succeeded\n",
               config1.timesyncConfigMaster == TRUE ? "Master" : "Slave");
    break;
  }
  case STATUS:
  {     
    /* Check if type4 is supported. */
    result = NTCI_GetCardConfigurationDetails(cardHandle,
                                              0 /* Not needed. */,
                                              STID_TIMESYNC_CONFIGURATION_TYPE4,
                                              &config4);
    if(result != NTCI_ERRCODE_SUCCESS) {
      /* Check if type3 is supported. */
      result = NTCI_GetCardConfigurationDetails(cardHandle,
                                                0 /* Not needed. */,
                                                STID_TIMESYNC_CONFIGURATION_TYPE3,
                                                &config3);          
      if(result != NTCI_ERRCODE_SUCCESS) {
        /* Use type2. */
        result = NTCI_GetCardConfigurationDetails(cardHandle,
                                                  0 /* Not needed. */,
                                                  STID_TIMESYNC_CONFIGURATION_TYPE2,
                                                  &config2);          
        if(result != NTCI_ERRCODE_SUCCESS) {
          WriteToLog(LOG_ERROR, "Failed reading time synchronization status. %i\n",
                     result);
          return result;
        }
        WriteToLog(LOG_INFO, "Adapter is synchronization Master: %s\n", (config2.timesyncConfigMaster ? "YES" : "NO"));      
        WriteToLog(LOG_INFO, "Master is present                : %s\n", (config2.link_active_status ? "YES" : "NO"));
        WriteToLog(LOG_INFO, "Adapter is in synchronization    : %s\n", (config2.slave_in_sync? "YES" : "NO"));
        if (!config2.timesyncConfigMaster) {
          WriteToLog(LOG_INFO, "PTA is Master                    : %s\n", (config2.PTA_is_Master ? "YES" : "NO"));            
          WriteToLog(LOG_INFO, "GPS box is Master                : %s\n", (config2.GPS_Box_is_Master ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "GPS box PPS is ok                : %s\n", (config2.GPS_Box_PPS_Ok ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "GPS box is in synchronization    : %s\n", (config2.GPS_Box_In_sync ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "GPS box serial stream is ok      : %s\n", (config2.GPS_Box_Serial_Stream_OK ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "OS timesync is enabled           : %s\n", (config2.OS_Mode_Sync_enabled ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "OS timesync is synchronizing     : %s\n", (config2.OS_Mode_in_Sync ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "Master ID code                   : %d\n", config2.Master_ID_Code);
          switch(config2.Master_ID_Code)
          {
          case 0x00:
            WriteToLog(LOG_INFO, "  NT adapter detected\n");
            break;
          case 0x01:
            WriteToLog(LOG_INFO, "  NT-TSU detected\n");
            break;
          case 0x02:
            WriteToLog(LOG_INFO, "  NT adapter detected (Gen2 no PTP support)\n");
            break;
          case 0x06:
            WriteToLog(LOG_INFO, "  NT adapter detected (with PTP support)\n");
            break;
          case 0x21:
          case 0x25:
            WriteToLog(LOG_INFO, "  NT-TSE detected\n");
            break;
          case 0x11:
            WriteToLog(LOG_INFO, "  NT-TSU detected\n");
            break;
          default:
            WriteToLog(LOG_INFO, "  Unknown Master ID\n");
            break;
          }
        }
        
      } else {
        result = NTCI_GetTimestampConfiguration(cardHandle,
                                                0,
                                                STID_TIMESTAMP_CONFIGURATION_TYPE1,
                                                &conf);
        
        if (result != NTCI_ERRCODE_SUCCESS)
        {
          WriteToLog(LOG_ERROR, "Unable to read timestamp configuration: %i\n",
                     result);
          return result;
        }
        
        WriteToLog(LOG_INFO, "Adapter is synchronization Master: %s\n", (config3.timesyncConfigMaster ? "YES" : "NO"));      
        WriteToLog(LOG_INFO, "Master is present                : %s\n", (config3.link_active_status ? "YES" : "NO"));
        WriteToLog(LOG_INFO, "Adapter is in synchronization    : %s\n", (config3.slave_in_sync? "YES" : "NO"));
        if (!config3.timesyncConfigMaster) {
          WriteToLog(LOG_INFO, "PTA is Master                    : %s\n", (config3.PTA_is_Master ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "The external source is valid     : %s\n", (config3.valid_external_source ? "YES" : "NO"));      
          WriteToLog(LOG_INFO, "GPS box is Master                : %s\n", (config3.GPS_Box_is_Master ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "GPS box PPS is ok                : %s\n", (config3.GPS_Box_PPS_Ok ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "GPS box is in synchronization    : %s\n", (config3.GPS_Box_In_sync ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "GPS box serial stream is ok      : %s\n", (config3.GPS_Box_Serial_Stream_OK ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "OS timesync is enabled           : %s\n", (config3.OS_Mode_Sync_enabled ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "OS timesync is synchronizing     : %s\n", (config3.OS_Mode_in_Sync ? "YES" : "NO"));
          WriteToLog(LOG_INFO, "Master ID code                   : %d\n", config3.Master_ID_Code);
          switch(config3.Master_ID_Code)
          {
          case 0x00:
            WriteToLog(LOG_INFO, "  NT adapter detected\n");
            break;
          case 0x01:
            WriteToLog(LOG_INFO, "  NT-TSU detected\n");
            break;
          case 0x02:
            WriteToLog(LOG_INFO, "  NT adapter detected (Gen2 no PTP support)\n");
            break;
          case 0x06:
            WriteToLog(LOG_INFO, "  NT adapter detected (with PTP support)\n");
            break;
          case 0x21:
          case 0x25:
            WriteToLog(LOG_INFO, "  NT-TSE detected\n");
            break;
          case 0x11:
            WriteToLog(LOG_INFO, "  NT-TSU detected\n");
            break;
          default:
            WriteToLog(LOG_INFO, "  Unknown Master ID\n");
            break;
          }
          
          if ((conf.timestampType == TIMESTAMP_PCAP) || (conf.timestampType == TIMESTAMP_PCAP_NANOTIME)) {
            WriteToLog(LOG_INFO, "External time                    : %08X:%08X\n",(uint32_t)((config3.External_Time & 0xFFFFFFFF00000000LL) >> 32), (uint32_t)(config3.External_Time & 0x00000000FFFFFFFFLL));
            WriteToLog(LOG_INFO, "Adapter time                     : %08X:%08X\n",(uint32_t)((config3.Adapter_Time & 0xFFFFFFFF00000000LL) >> 32), (uint32_t)(config3.Adapter_Time & 0x00000000FFFFFFFFLL));
          } else {
            WriteToLog(LOG_INFO, "External time                    : %llX\n",(ULL)config3.External_Time);
            WriteToLog(LOG_INFO, "Adapter time                     : %llX\n",(ULL)config3.Adapter_Time);
          }
        }
      }
    } else {
      result = NTCI_GetTimestampConfiguration(cardHandle,
                                              0,
                                              STID_TIMESTAMP_CONFIGURATION_TYPE1,
                                              &conf);
      
      if (result != NTCI_ERRCODE_SUCCESS)
      {
        WriteToLog(LOG_ERROR, "Unable to read timestamp configuration: %i\n",
                   result);
        return result;
      }
      
      WriteToLog(LOG_INFO, "Adapter is synchronization Master: %s\n", (config4.timesyncConfigMaster ? "YES" : "NO"));      
      WriteToLog(LOG_INFO, "Master is present                : %s\n", (config4.link_active_status ? "YES" : "NO"));
      WriteToLog(LOG_INFO, "Adapter is in synchronization    : %s\n", (config4.slave_in_sync? "YES" : "NO"));
      if (!config4.timesyncConfigMaster) {
        WriteToLog(LOG_INFO, "PTA is Master                    : %s\n", (config4.PTA_is_Master ? "YES" : "NO"));
        WriteToLog(LOG_INFO, "The external source is valid     : %s\n", (config4.valid_external_source ? "YES" : "NO"));      
        WriteToLog(LOG_INFO, "GPS box is Master                : %s\n", (config4.GPS_Box_is_Master ? "YES" : "NO"));
        WriteToLog(LOG_INFO, "GPS box PPS is ok                : %s\n", (config4.GPS_Box_PPS_Ok ? "YES" : "NO"));
        WriteToLog(LOG_INFO, "GPS box is in synchronization    : %s\n", (config4.GPS_Box_In_sync ? "YES" : "NO"));
        WriteToLog(LOG_INFO, "GPS box serial stream is ok      : %s\n", (config4.GPS_Box_Serial_Stream_OK ? "YES" : "NO"));
        WriteToLog(LOG_INFO, "OS timesync is enabled           : %s\n", (config4.OS_Mode_Sync_enabled ? "YES" : "NO"));
        WriteToLog(LOG_INFO, "OS timesync is synchronizing     : %s\n", (config4.OS_Mode_in_Sync ? "YES" : "NO"));
        WriteToLog(LOG_INFO, "Master ID code                   : %X\n", config4.Master_ID_Code);
        switch(config4.Master_ID_Code)
        {
          case 0x00:
            WriteToLog(LOG_INFO, "  NT adapter detected\n");
            break;
          case 0x01:
            WriteToLog(LOG_INFO, "  NT-TSU detected\n");
            break;
          case 0x02:
            WriteToLog(LOG_INFO, "  NT adapter detected (Gen2 no PTP support)\n");
            break;
          case 0x06:
            WriteToLog(LOG_INFO, "  NT adapter detected (with PTP support)\n");
            break;
          case 0x21:
          case 0x25:
            WriteToLog(LOG_INFO, "  NT-TSE detected\n");
            break;
          case 0x11:
            WriteToLog(LOG_INFO, "  NT-TSU detected\n");
            break;
          default:
            WriteToLog(LOG_INFO, "  Unknown Master ID\n");
            break;
        }
        if ((conf.timestampType == TIMESTAMP_PCAP) || (conf.timestampType == TIMESTAMP_PCAP_NANOTIME)) {
          WriteToLog(LOG_INFO, "External time                    : %08X:%08X\n",(uint32_t)((config4.External_Time & 0xFFFFFFFF00000000LL) >> 32), (uint32_t)(config4.External_Time & 0x00000000FFFFFFFFLL));
          WriteToLog(LOG_INFO, "Adapter time                     : %08X:%08X\n",(uint32_t)((config4.Adapter_Time & 0xFFFFFFFF00000000LL) >> 32), (uint32_t)(config4.Adapter_Time & 0x00000000FFFFFFFFLL));
        } else {
          WriteToLog(LOG_INFO, "External time                    : %llX\n",(ULL)config4.External_Time);
          WriteToLog(LOG_INFO, "Adapter time                     : %llX\n",(ULL)config4.Adapter_Time);
        }
      } else {
        WriteToLog(LOG_INFO, "Reference time                   : %llX\n",(ULL)config4.Reference_Time);
        WriteToLog(LOG_INFO, "Clock skew                       : %lli\n",(int64_t)config4.Clock_Skew);
      }
    }
    break;
  }
  case RESET:
  {
    config1.timesyncCommandType = TIMESYNC_COMMAND_RESET_STAT_COUNTER;
    
    result = NTCI_SetCardConfigurationDetails(cardHandle,
                                              0 /* Not needed. */,
                                              STID_TIMESYNC_CONFIGURATION_TYPE1,
                                              &config1);
    if(result != NTCI_ERRCODE_SUCCESS) {
      WriteToLog(LOG_ERROR, "Command failed. %i\n",
                 result);
      return result;
    }
    WriteToLog(LOG_INFO, "Reset command succeeded\n");

    break;
  }
  }
  
  return NTCI_ERRCODE_SUCCESS;
}


uint32_t _CardConfigTimeSync(void *arglist)
{
  uint8_t *argptr = arglist;
  uint8_t card;
  char *action;

  card   = get8incr(&argptr);
  getCHARPincr(argptr, action);
  return CardConfigTimeSync(card, action);
}


uint32_t GetTimeSyncConfig(uint8_t card)
{
  union {
    TimeSyncInfoType1_t config1;
    TimeSyncInfoType2_t config2;
  } config;
  handle_t cardHandle;
  uint32_t result;
  int stid2; /* boolean */

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  WriteToLog(LOG_INFO, "TimeSync Configuration:\n");

  /* Check if type2 is supported. */
  result = NTCI_GetCardConfigurationDetails(cardHandle,
                                            0 /* Not needed. */,
                                            STID_TIME_SYNC_INFO_TYPE2,
                                            &config.config2);
  if (result == NTCI_ERRCODE_SUCCESS) {
    stid2 = 1;
  } else {
    stid2 = 0;
    /* Check if type1 is supported. */
    result = NTCI_GetCardConfigurationDetails(cardHandle,
                                              0 /* Not needed. */,
                                              STID_TIME_SYNC_INFO_TYPE1,
                                              &config.config1);
    if (result != NTCI_ERRCODE_SUCCESS) {
      WriteToLog(LOG_INFO, "  STID_TIME_SYNC_INFO_TYPE1 not supported (%d)\n", result);
      return NTCI_ERRCODE_SUCCESS;
    }
  }
  WriteToLog(LOG_INFO, "  TimeSyncProtocol          : %d\n", config.config1.timeSyncProtocol);
  WriteToLog(LOG_INFO, "  TimeSyncConnectorIn       : %d\n", config.config1.timeSyncConnectorIn);
  WriteToLog(LOG_INFO, "  TimeSyncConnectorOut      : %d\n", config.config1.timeSyncConnectorOut);
  WriteToLog(LOG_INFO, "  TimeSyncConnectorRepeat   : %d\n", config.config1.timeSyncConnectorRepeat);
  WriteToLog(LOG_INFO, "  timeSyncTimeJumpThreshold : %d\n", config.config1.timeSyncTimeJumpThreshold);
  WriteToLog(LOG_INFO, "  timeSyncTimeOffset        : %d\n", config.config1.timeSyncTimeOffset);
  WriteToLog(LOG_INFO, "  highFreqSampling          : %d\n", config.config1.highFreqSampling);
  WriteToLog(LOG_INFO, "  osTimeSyncFailover        : %d\n", config.config1.osTimeSyncFailover);
  if (stid2) {
    WriteToLog(LOG_INFO, "  hardSyncCounter           : %d\n", config.config2.hardSyncCounter);
  }

  return NTCI_ERRCODE_SUCCESS;
}


uint32_t _GetTimeSyncConfig(void *arglist)
{
  const uint8_t *argptr = arglist;
  const uint8_t card = get8incr(&argptr);

  return GetTimeSyncConfig(card);
}


timestampTypes_s timestampTypes[] = {
  {"native","64bit 10ns resolution timer from a base of 0", TIMESTAMP_NATIVE},
  {"ndis","64bit 100ns resolution timer from a base of January 1, 1601", TIMESTAMP_NDIS},
  {"pcap","32bits seconds and 32bits usecs from a base of January 1, 1970", TIMESTAMP_PCAP},
  {"native-ndis","64bit 10ns resolution timer from a base of January 1, 1601", TIMESTAMP_NATIVE_NDIS},
  {"native-unix","64bit 10ns resolution timer from a base of January 1, 1970", TIMESTAMP_NATIVE_UNIX},
  {"pcap-nanotime","32bits seconds and 32bits nsecs from a base of January 1, 1970", TIMESTAMP_PCAP_NANOTIME},
};

int numTimestampTypes = sizeof(timestampTypes) / sizeof(*timestampTypes);

const char *TimestampTypeToString(TimestampType_t tsType)
{
    switch (tsType)
    {
        case TIMESTAMP_NATIVE:
            return "Native";
        case TIMESTAMP_NDIS:
            return "NDIS";
        case TIMESTAMP_PCAP:
            return "PCAP";
        case TIMESTAMP_NATIVE_NDIS:
            return "Native NDIS ";
        case TIMESTAMP_NATIVE_UNIX:
            return "Native Unix";
        case TIMESTAMP_PCAP_NANOTIME:
            return "PCAP (Nanoseconds)";
        default:
            break;
    }

    return NULL;
}

uint32_t CardPrintTimestampConfig(uint8_t card, uint8_t justSet)
{
    TimestampConfigurationType1_t conf;
    char *tsName;
    uint32_t result;
    handle_t cardHandle;
    uint64_t decimal;
    uint64_t nanotime=0;
    uint32_t unit=0;
    TimeSyncConfigurationType2_t config2;    

    if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
      return result;
    }

    result = NTCI_GetTimestampConfiguration(cardHandle,
                                            0,
                                            STID_TIMESTAMP_CONFIGURATION_TYPE1,
                                            &conf);

    if (result != NTCI_ERRCODE_SUCCESS)
    {
        WriteToLog(LOG_ERROR, "Unable to read timestamp configuration: %i\n",
                result);
        return result;
    }

    writePos=0;
    if (justSet)
    {
        DUMP_TO_SCREEN("New Timestamp configuration with refreshly read value:\n");
    }
    else
    {
        DUMP_TO_SCREEN("Timestamp configuration:\n");
    }

    tsName = (char*)TimestampTypeToString(conf.timestampType);
    DUMP_TO_SCREEN("\tTimestamp type      : ");
    if (tsName != NULL) {
        DUMP_TO_SCREEN("%s\n", tsName);
    } else {
        DUMP_TO_SCREEN("unknown(%i)\n", conf.timestampType);
    }

    
    if (conf.timestampType == TIMESTAMP_PCAP_NANOTIME || conf.timestampType == TIMESTAMP_PCAP) {
      if (conf.timestampType == TIMESTAMP_PCAP_NANOTIME) {
        decimal=(uint64_t)((uint32_t)((conf.timestampValue & 0xFFFFFFFF00000000LL) >> 32))+((uint64_t)(conf.timestampValue & 0x00000000FFFFFFFFLL)*1000000000);
        DUMP_TO_SCREEN("\tTime value (decimal): %llu nsec\n", decimal);
        nanotime=(uint64_t)((uint32_t)((conf.timestampValue & 0xFFFFFFFF00000000LL) >> 32));        
      } else if (conf.timestampType == TIMESTAMP_PCAP) {
        decimal=(uint64_t)((uint32_t)((conf.timestampValue & 0xFFFFFFFF00000000LL) >> 32))+((uint64_t)(conf.timestampValue & 0x00000000FFFFFFFFLL)*1000000);
        DUMP_TO_SCREEN("\tTime value (decimal): %llu usec\n", decimal);
        nanotime=(uint64_t)((uint32_t)((conf.timestampValue & 0xFFFFFFFF00000000LL) >> 32))*1000;                
      }
      DUMP_TO_SCREEN("\tTime value (Hex)    : 0x%08X:0x%08X\n",(uint32_t)((conf.timestampValue & 0xFFFFFFFF00000000LL) >> 32), (uint32_t)(conf.timestampValue & 0x00000000FFFFFFFFLL));                    
    } else {
      if (conf.timestampType == TIMESTAMP_NATIVE || conf.timestampType == TIMESTAMP_NATIVE_NDIS || conf.timestampType == TIMESTAMP_NATIVE_UNIX) {
        /* convert 10ns ticks to nanoseconds */
        unit=10;
        nanotime=conf.timestampValue*unit;        
      } else if (conf.timestampType == TIMESTAMP_NDIS) {
        /* convert 100ns ticks to nanoseconds */
        unit=100;                
        nanotime=conf.timestampValue*unit;
      }
      DUMP_TO_SCREEN("\tTime value (decimal): %llu nsec\n ", (ULL)(conf.timestampValue*unit));
      DUMP_TO_SCREEN("\tTime value (Hex)    : 0x%llX nsec\n",(ULL)(conf.timestampValue*unit));            
    }

    if (conf.timestampType != TIMESTAMP_NATIVE) {
      uint64_t timeValue;
      time_t   timeT;
      struct tm *ptm;
      char wday[7][4]={"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
      char wmon[12][4]={"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
      
      timeValue = conf.timestampValue;
      ConvertSystemTickTime(&timeValue, conf.timestampType, TIMESTAMP_NATIVE_UNIX);
      timeT = timeValue/100000000;
      ptm = localtime(&timeT);

      result = NTCI_GetCardConfigurationDetails(cardHandle,
                                                0 /* Not needed. */,
                                                STID_TIMESYNC_CONFIGURATION_TYPE2,
                                                &config2);                

      if ((conf.timestampType == TIMESTAMP_NATIVE_NDIS) && config2.GPS_Box_is_Master) {
        DUMP_TO_SCREEN("\tNOTE >> Timeformat NDIS-NATIVE is not supported by Master\n");
      } else if ((conf.timestampType == TIMESTAMP_NATIVE_NDIS || conf.timestampType == TIMESTAMP_NDIS) && (conf.timestampValue < UNIX_EPOCH_OFFSET_MAGIC)) {
        DUMP_TO_SCREEN("\tNOTE >> Cannot diplay NDIS-NATIVE or NDIS time\n");
      } else {
        if (ptm != NULL) {
          DUMP_TO_SCREEN("\tTime value          : %s %d-%s-%d %02d:%02d:%02d.%09lld\n",
                         wday[ptm->tm_wday], ptm->tm_mday, wmon[ptm->tm_mon], ptm->tm_year+1900,
                         ptm->tm_hour, ptm->tm_min, ptm->tm_sec, ((unsigned long long)nanotime%1000000000));                       
        } else {
          DUMP_TO_SCREEN("\tTime value          : Time is illegal. Unable to calculate the value\n");
        }
      }
    }
    DUMP_TO_SCREEN("\n");
    WriteToLog(LOG_INFO, WriteBuffer);

    return NTCI_ERRCODE_SUCCESS;
}

uint32_t _CardPrintTimestampConfig(void *arglist)
{
  uint8_t  *argptr = arglist;
  uint8_t  card;
  uint8_t justSet;

  card        = get8incr(&argptr);
  justSet     = get8incr(&argptr);

  return CardPrintTimestampConfig(card, justSet);
}

uint32_t CardSetTimestampOffsetConfig(uint8_t card, uint32_t cablelength, uint32_t numberofgpsboxes)
{
    TimestampOffsetConfigurationType1_t conf;
    uint32_t result;
    handle_t cardHandle;

    if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
      return result;
    }

    if (((int32_t)cablelength >= 0) || (cablelength <= 100)) {    
      conf.cableLength = cablelength;
      conf.numberOfGPSBoxes = numberofgpsboxes;

      result = NTCI_SetTimestampConfiguration(cardHandle,
                                              0,
                                              STID_TIMESTAMP_OFFSET_CONFIGURATION_TYPE1,
                                              &conf);

      if (result != NTCI_ERRCODE_SUCCESS)
      {
        WriteToLog(LOG_ERROR, "Unable to set timestamp offset. Error code %i\n", result);
        WriteToLog(LOG_INFO, "Please make sure the adapter is not configured as Timesync Master.\n");
        WriteToLog(LOG_INFO, "Please make sure the adapter has timesync support\n");
        return result;
      }

      WriteToLog(LOG_INFO, "Cablelength set to                     : %d meter(s)\n", cablelength);
      WriteToLog(LOG_INFO, "Number of Timesync Units in daisy chain: %d\n", numberofgpsboxes);            
      
    } else {
      WriteToLog(LOG_ERROR, "Invalid cable length specified.\n");
    }    

    return NTCI_ERRCODE_SUCCESS;
}

uint32_t _CardSetTimestampOffsetConfig(void *arglist)
{
  uint8_t  *argptr = arglist;
  uint8_t  card;
  uint32_t cablelength;
  uint32_t numberofgpsboxes;

  card             = get8incr(&argptr);
  cablelength      = get32incr(&argptr);
  numberofgpsboxes = get32incr(&argptr);

  return CardSetTimestampOffsetConfig(card, cablelength, numberofgpsboxes);
}

uint32_t CardSetTimestampConfig(uint8_t card, char *timestampType, uint64_t value)
{
    TimestampConfigurationType1_t conf;
    uint32_t result;
    int i;
    handle_t cardHandle;
    TimeSyncConfigurationType2_t config2;    


    
    if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
      return result;
    }
    
    /* check all types */

    result = NTCI_GetCardConfigurationDetails(cardHandle,
                                              0 /* Not needed. */,
                                              STID_TIMESYNC_CONFIGURATION_TYPE2,
                                              &config2);                   

    for (i=0; i<numTimestampTypes; ++i)
    {
      if (!strcmp(timestampType, timestampTypes[i].Name))
      {
        if ((timestampTypes[i].TimestampType == TIMESTAMP_NATIVE_NDIS) && config2.GPS_Box_is_Master) {
          WriteToLog(LOG_ERROR, "Master does not support timestamp format NATIVE-NDIS\n");
          return NTCI_ERRCODE_SUCCESS;
        } else {        
          conf.timestampType = timestampTypes[i].TimestampType;            
          break;
        }
      }
    }

    if (i > numTimestampTypes)
    {
        int typeNumber;

        /* possibly specifying a numerical value? */
        
        i = sscanf(timestampType, "%i", &typeNumber);

        if (i <= 0)
        {
            WriteToLog(LOG_ERROR, "Invalid time value specified.\n");
            return NTCI_ERRCODE_INVALID_PARAMETER;
        }

        conf.timestampType = typeNumber;
    }

    /* set the configuration */
    conf.timestampValue = value;

    result = NTCI_SetTimestampConfiguration(cardHandle,
                                            0,
                                            STID_TIMESTAMP_CONFIGURATION_TYPE1,
                                            &conf);

    if (result != NTCI_ERRCODE_SUCCESS)
    {
        WriteToLog(LOG_ERROR, "Unable to set timestamp configuration: %i\n",
                result);
        WriteToLog(LOG_ERROR, "timeStampType=%d\n", conf.timestampType);
        return result;
    }

    CardPrintTimestampConfig(card, 1);

    return NTCI_ERRCODE_SUCCESS;
}

uint32_t _CardSetTimestampConfig(void *arglist)
{
  uint8_t  *argptr = arglist;
  uint8_t  card;
  char     *timestampType;
  uint64_t value;

  card          = get8incr(&argptr);
  getCHARPincr(argptr, timestampType);
  value         = get64incr(&argptr);

  return CardSetTimestampConfig(card, timestampType, value);
}

uint32_t CardConfigFailover(uint8_t card, char *action)
{
  uint32_t                     result;
  handle_t                     cardHandle;
  FailoverConfigurationType1_t config;
  CardHardwareCapabilitiesType1_t data1;
  const char                   *modeStr[] = {"Normal", "Bypass", "Undefined"};
  
  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  if((result = NTCI_GetCapabilities(cardHandle,
                                    CAPID_HARDWARE,
                                    NOT_SUB_CAPABILITIES,
                                    STID_CARD_HARDWARE_CAPABILITIES_TYPE1,
                                    &data1)) != NTCI_ERRCODE_SUCCESS) {
      WriteToLog(LOG_ERROR, "CardConfigFailover: Failed to get HW capabilities. Result = 0x%x\n", result);
  }
  if (data1.CardType != NTBPE) {
    WriteToLog(LOG_ERROR, "CardConfigFailover: Invalid Adapter type 0x%x (Not NT Bypass Adapter)\n", data1.CardType);
  }
  

  if(!strcmp(action, "status")) {

    result = NTCI_GetCardConfigurationDetails(cardHandle,
                                              0 /* Not needed. */,
                                              STID_FAILOVER_CONFIGURATION_TYPE1,
                                              &config);
    if(result != NTCI_ERRCODE_SUCCESS) {
        WriteToLog(LOG_ERROR, "Failed reading Failover configuration. %i\n",
                   result);
        return result;
    }

    WriteToLog(LOG_INFO, "------------  Failover Status  -----------\n");
    WriteToLog(LOG_INFO, "Failover Current Mode           : %s\n",  modeStr[config.currentMode]);
    WriteToLog(LOG_INFO, "Failover PwrOn Mode             : %s\n",  modeStr[config.powerOnMode]);
    WriteToLog(LOG_INFO, "Failover Heartbeat Value        : %d\n",  config.heartBeatTimeout);
    WriteToLog(LOG_INFO, "Failover PowerLoss Detect       : %s\n",  config.powerLossDetect == 0 ? "Disabled" : "Enabled");
    WriteToLog(LOG_INFO, "Failover PowerLoss Switch delay : %d\n",  config.powerLossSwitchDelay);
    WriteToLog(LOG_INFO, "Failover Autonomous Mode        : %s\n",  config.autonomousMode == 0 ? "Off" : "On");
    WriteToLog(LOG_INFO, "------------------------------------------\n");
  } else {
    WriteToLog(LOG_ERROR, "CardConfigFailover: Unknown action: \"%s\".\n", action);
    return NTCI_ERRCODE_INVALID_PARAMETER;
  }
    
  return NTCI_ERRCODE_SUCCESS;
}


uint32_t _CardConfigFailover(void *arglist)
{
  uint8_t *argptr = arglist;
  uint8_t card;
  char *action;

  card   = get8incr(&argptr);
  getCHARPincr(argptr, action);
  return CardConfigFailover(card, action);
}

uint32_t FailoverHeartbeatSignal(uint8_t card)
{
  uint32_t result;
  handle_t cardHandle;
  FailoverHeartbeatSignalType1_t failoverHeartbeatType1;
  
  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }

  failoverHeartbeatType1.action = 1;
  result = NTCI_Command(cardHandle, STID_FAILOVER_SIGNAL_TYPE1,  &failoverHeartbeatType1);
  if(result != NTCI_ERRCODE_SUCCESS) {
      WriteToLog(LOG_ERROR, "Failed sending failover heartbeat (%i)\n",
                 result);
      return result;
  }
    
  return NTCI_ERRCODE_SUCCESS;
}

uint32_t _FailoverHeartbeatSignal(void *arglist)
{
  uint8_t *argptr = arglist;
  uint8_t card;

  card   = get8incr(&argptr);
  return FailoverHeartbeatSignal(card);
}

uint32_t CardSetTemperatureThreshold(uint8_t card, char *sensor, char *highOrLow, uint32_t value)
{
  uint32_t result, i;
  handle_t cardHandle;

  HardwareMonitorType2_t SensorList2[50]; /* Max sensors */
  uint32_t blockSize;
  uint32_t SensorListLength;

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }
  
  result = NTCI_GetSensorsBlockSize(cardHandle,
                                    &blockSize);
  if(result != NTCI_ERRCODE_SUCCESS) {
    printf(">>> Error: Failed getting sensor block size.\n");      
    return result;
  }
  
  if(blockSize > sizeof(SensorList2)) {
    printf(">>> Error: Too many sensors at Adapter. Not enough memory.\n");
    return NTCI_ERRCODE_INTERNAL_ERROR;
  }
  
  result = NTCI_GetSensors(cardHandle,
                           STID_HARDWARE_MONITOR_TYPE2,
                           &SensorList2,
                           blockSize);
  
  if(result != NTCI_ERRCODE_SUCCESS) {
    printf(">>> Error: Failed getting sensors.\n");
    return result;
  }
  
  /* One entry for each sensor, plus the "all sensors" entry */
  SensorListLength = blockSize/sizeof(HardwareMonitorType2_t);    
 
  //Find sensor
  for (i=0; i<SensorListLength; i++) {
    if(!strcmp((char *)SensorList2[i].SensorName, sensor)) {
      if(!strcmp(highOrLow, "LOW") || !strcmp(highOrLow, "low") ) {
        SensorList2[i].lowAlarmLimit=value;
      }
      else if(!strcmp(highOrLow, "HIGH") || !strcmp(highOrLow, "high" )) {
        SensorList2[i].highAlarmLimit=value;
      } else {
        printf(">>> Error: Invalid HighLow parameter.\n");
        return NTCI_ERRCODE_INTERNAL_ERROR;
      }
      result = NTCI_SetSensors(cardHandle,
      			 STID_HARDWARE_MONITOR_TYPE2,
      			 &SensorList2,
      			 blockSize);
      if(result != NTCI_ERRCODE_SUCCESS) {
        printf(">>> Error: Failed setting sensors.\n");
      }
      return result;
    }
  }
  
  printf(">>> Error: Sensor %s not found\n", sensor);
  return NTCI_ERRCODE_INTERNAL_ERROR;
}

uint32_t _CardSetTemperatureThreshold(void *arglist)
{
  uint8_t *argptr = arglist;
  uint8_t card;
  char *sensor;
  char *highOrLow;
  uint32_t value;

  card   = get8incr(&argptr);
  getCHARPincr(argptr, sensor);
  getCHARPincr(argptr, highOrLow);
  value = get32incr(&argptr);
  return CardSetTemperatureThreshold(card, sensor, highOrLow, value);
}

uint32_t GetPortIdentificationStatus(uint8_t card, uint32_t channelsBitmask)
{
  handle_t cardHandle;
  uint32_t result = 0;
  PortIdentificationConfigurationType1_t portIDStatus;

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }  
  result = NTCI_GetCardConfigurationDetails(cardHandle,
                                            channelsBitmask,
                                            STID_PORT_IDENTIFICATION_CONFIGURATION_TYPE1,
                                            &portIDStatus);
  
  printf("portIdentificationMask = 0x%08X\n", portIDStatus.portIdentificationEnableMask);

  return result;
}

uint32_t SetPortIdentificationStatus(uint8_t card, uint32_t channelsBitmask, uint32_t enable)
{
  handle_t cardHandle;
  uint32_t result;
  PortIdentificationConfigurationType1_t portIDStatus;    

  if((result = GetCardHandle(card, &cardHandle)) != NTCI_ERRCODE_SUCCESS) {
    return result;
  }
  
  result = NTCI_GetCardConfigurationDetails(cardHandle,
                                            channelsBitmask,
                                            STID_PORT_IDENTIFICATION_CONFIGURATION_TYPE1,
                                            &portIDStatus);
  printf("Old portIdentificationMask = 0x%08X\n", portIDStatus.portIdentificationEnableMask);
  
  if (enable) {
    portIDStatus.portIdentificationEnableMask = channelsBitmask;
  } else {
    portIDStatus.portIdentificationEnableMask = portIDStatus.portIdentificationEnableMask & ~channelsBitmask;
  }
    
  printf("New portIdentificationMask = 0x%08X\n", portIDStatus.portIdentificationEnableMask);    
  result = NTCI_SetCardConfigurationDetails(cardHandle,
                                            channelsBitmask,
                                            STID_PORT_IDENTIFICATION_CONFIGURATION_TYPE1,
                                            &portIDStatus);
  return result;
}
