// $Id: NTCI_cardconfiguration.h 16051 2011-11-30 12:20:27Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * C functions to access the driver interface Card Configuration
 */

#ifndef __NTCI_CARDCONFIGURATION_H__
#define __NTCI_CARDCONFIGURATION_H__

#include "NTCommonInterface.h"

/*-----------------------------------------------------------------------------
 * The following function is used to read data from interface modules.
 *---------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL       /* Return code (status/error).       */
NTCI_ReadNetworkInterfaceModuleData(
  IN  handle_t      hCard,              /* Pointer to the cardhandle provided
                                           by the NTCI_OpenCard call.        */
  IN  uint32_t      ChannelBitmask,     /* Selects which interface to read
                                           from.                             */
  IN  uint16_t      address,            /* The address to read from.         */
  OUT uint8_t *     pData,              /* Pointer to the destination buffer.*/
  IN OUT uint16_t * MaxLengthActualLength);
                                        /* The requested number of bytes to
                                           read and returns the actual number
                                           of bytes read.                    */


/*-----------------------------------------------------------------------------
 * The following function is used to write data to interface modules.
 *---------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL       /* Return code (status/error).       */
NTCI_WriteNetworkInterfaceModuleData(
  IN  handle_t      hCard,              /* Pointer to the cardhandle provided
                                           by the NTCI_OpenCard call.        */
  IN  uint32_t      ChannelBitmask,     /* Selects which interface to write
                                           to.                               */
  IN  uint16_t      address,            /* The address to write to.          */
  OUT uint8_t *     pData,              /* Pointer to the source buffer.     */
  IN OUT uint16_t * MaxLengthActualLength);
                                        /* The requested number of bytes to
                                           write and returns the actual number
                                           of bytes written.                 */


/*-----------------------------------------------------------------------------
 * The following function is used to set the speed and duplex settings on
 * the interfaces.
 *---------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL
NTCI_SetAutonegotiationMode(
  IN  handle_t    hCard,
  IN  uint32_t    ChannelsBitmask, 
  IN  StructureID structId, 
  IN  void *      pAutoNegMode );


NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetAutonegotiationMode(  IN  handle_t    hCard,
                                                                IN  uint32_t    ChannelBitmask, 
                                                                IN  StructureID structId, 
                                                                OUT void *      pAutoNegMode );

NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_SetPortBypassMode(   IN  handle_t    hCard,
                                                            IN  uint32_t bypassDeviceId,
                                                            IN  uint32_t    mode );

NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetPortBypassMode(   IN  handle_t    hCard,
                                                            IN  uint32_t bypassDeviceId,
                                                            OUT uint32_t *  pMode );

NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_SetCardConfigurationDetails(IN  handle_t    hCard,
                                                                   IN  uint32_t ChannelsBitmask,
                                                                   IN  StructureID structId,
                                                                   IN  void * pCardConfigurationDetails);

NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetCardConfigurationDetails(IN  handle_t    hCard,
                                                                   IN  uint32_t ChannelsBitmask,
                                                                   IN  StructureID structId,
                                                                   OUT void * pCardConfigurationDetails);

NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_SetTimestampConfiguration(IN  handle_t    hCard,
                                                                 IN  uint32_t DeviceID,
                                                                 IN  StructureID structId,
                                                                 IN  void * pTimestampConfiguration);

NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetTimestampConfiguration(IN  handle_t    hCard,
                                                                 IN  uint32_t DeviceID,
                                                                 IN  StructureID structId,
                                                                 OUT void * pTimestampConfiguration);

#endif // __NTCI_CARDCONFIGURATION_H__
