// $Id: NTCI_cardinfo.h 17384 2012-02-16 14:48:40Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * C functions to get extra information about an opened driver
 */

#ifndef __NTCI_CARDINFO_H__
#define __NTCI_CARDINFO_H__

/* NOTE:  Compiler safe, circular #include sequence.                     */
/*     This file requires definitions found within NTCommonInterface.h   */
/*     Since this file contains values needed by most applications       */
/*     NTCommonInterface.h includes it automatically                     */
/*     The following include prevents compile errors if application code */
/*     does not directly include NTCommonInterface.h                     */

#include "NTCommonInterface.h"
#include "cardinfo.h"

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetVersionInfo
**
**  Description    :Get version information from the device driver and the dll.
**
**  Parameters     :IN  handle_t hCard
**                  OUT XYCI_VERSION_INFO* pVersionInfo
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetVersionInfo(  IN  handle_t            hCard,
                                                        OUT NTCI_VERSION_INFO * pVersionInfo );

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetAbstractDeviceDriver
**
**  Description    :C++ interface hook
**                 For Napatech internal use
**
**  Parameters     :none
**
**  Returns        :handle_t
**
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB handle_t NTCOMMONCALL NTCI_GetAbstractDeviceDriver(void);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetDeviceName
**
**  Description    :Copies out the Device Name of the supplied card up to the
**                  length limit supplied. The Device name usually refers to
**                  the name of the type of card.               
**
**  Parameters     :IN  handle_t    hCard, 
**                  OUT char *      pNameBuffer,
**                  IN  uint32_t    MaxBufferLength
**
**  Returns        :uint32_t
**
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetDeviceName(   IN  handle_t    hCard, 
                                                        OUT char *      pNameBuffer,
                                                        IN  uint32_t    MaxBufferLength);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetConnectionName
**
**  Description    :Copies out the Connection Name of the supplied card up to the
**                  length limit supplied. The Connection Name usually refers to
**                  the name of the network connection that the card is used for.  
**                  If the connection name can not be determined, it will return
**                  the same as if NTCI_GetConnectionGuid was called.          
**
**  Parameters     :IN  handle_t    hCard, 
**                  OUT char *      pNameBuffer,
**                  IN  uint32_t    MaxBufferLength
**
**  Returns        :uint32_t
**
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetConnectionName(   IN  handle_t    hCard, 
                                                            OUT char *      pNameBuffer,
                                                            IN  uint32_t    MaxBufferLength);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetConnectionGuid
**
**  Description    :Copies out the Connection Guid of the supplied card up to the
**                  length limit supplied. The Connection Guid is a symbolic link
**                  to the name of the network connection that the card is used for.            
**
**  Parameters     :IN  handle_t    hCard, 
**                  OUT char *      pNameBuffer,
**                  IN  uint32_t    MaxBufferLength
**
**  Returns        :uint32_t
**
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetConnectionGuid(   IN  handle_t    hCard, 
                                                            OUT char *      pNameBuffer,
                                                            IN  uint32_t    MaxBufferLength);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetServiceName
**
**  Description    :Copies out the Service Name of the supplied card up to the
**                  length limit supplied. The Service Name is a the name of the
**                  device drivers network service.            
**
**  Parameters     :IN  handle_t    hCard, 
**                  OUT char *      pNameBuffer,
**                  IN  uint32_t    MaxBufferLength
**
**  Returns        :uint32_t
**
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetServiceName(  IN  handle_t    hCard,
                                                        OUT char *      pNameBuffer,
                                                        IN  uint32_t    MaxBufferLength);

/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetPCISlotBusDeviceFunction
**
**  Description    :Attempts to get the PCI location of the supplied card.
**                  If successfull the returned value will be of the format
**                  0xSSBBDDFF      SS Slot number
**                                  BB Bus number
**                                  DD Device number
**                                  FF Function number           
**                  NOTE: This function only works under Windows NT/2000/XP.
**
**  Parameters     :IN  handle_t    hCard, 
**                  OUT uint32_t *  pSBDFIndex
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetPCISlotBusDeviceFunction( IN  handle_t    hCard,
                                                                    OUT uint32_t *  pSBDFIndex);

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_CardIdentification(IN handle_t hCard) DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif


/*  ----------------------------------------------------------------------------
**
**  Name           :NTCI_GetCardInfo
**
**  Description    :Get information about the drivers etc.
**
**  Parameters     :IN  handle_t    hCard
**                  IN  StructureID idCardInfo
**                  OUT void*       cardInfo
**
**  Returns        :uint32_t
**
**  --------------------------------------------------------------------------*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_GetCardInfo(IN  handle_t     hCard,
                                                   IN  StructureID  idCardInfo,
                                                   OUT void *       cardInfo);

#endif // __NTCI_CARDINFO_H__
