// $Id: NTCommonInterface.h 17384 2012-02-16 14:48:40Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 */

#ifndef __NTAPI_NTCOMMONINTERFACE_H__
#define __NTAPI_NTCOMMONINTERFACE_H__

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #include <NTCI_functions.h>
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

#include <ostype.h>
//Header file include basetype.h
//   Defines basic integer types such as uint32_t
#include "basetype.h"
#include "cardinfo.h"

//Definition of calling conventions and exporting options
#if _NT_OS_TYPE==_NT_OS_TYPE_LINUX
#define NTCOMMONLIB
#define NTCOMMONCALL
#elif _NT_OS_TYPE==_NT_OS_TYPE_FREEBSD
#define NTCOMMONLIB
#define NTCOMMONCALL
#elif defined(WIN32)
#define NTCOMMONLIB 
#define NTCOMMONCALL
#else
#    ifdef NTCOMMONINTERFACE_EXPORTS
#    define NTCOMMONLIB __declspec(dllexport)
#    else
#    define NTCOMMONLIB __declspec(dllimport)
#    endif
#    define NTCOMMONCALL __stdcall
#endif

//IN and OUT are used to decorate NTCI_ function prototypes. The indicate how
//the function will use the supplied parameters.
//
//OUT parameters will be overwritten with returned data
// IN parameters will usually just be read or scanned

#ifndef IN
#define IN
#endif

#ifndef OUT
#define OUT
#endif

//Header file include NTCI_errorcode.h
//   defines all NTCI_ error codes, such as NTCI_ERRCODE_SUCCESS
//   defines functions for retrieving and explaining the 
//   most recently returned last error code
#include "NTCI_errorcode.h"

//Header file include NTCI_cardinfo.h
//   defines functions for retrieving device names and the
//   version information about the drivers they use
#include "NTCI_cardinfo.h"

// Card name strings to use
#define NT4E_CAP_STR            "NT4E-CAP Capture Network Adapter PCIe 4x1Gb"
#define NT4E_INL_STR            "NT4E-INL In-line Network Adapter PCIe 4x1Gb"
#define NT4ESTD_CAP_STR         "NT4E-STD-CAP Standard Capture Network Adapter PCIe 4x1Gb"
#define NT4ESTD_INL_STR         "NT4E-STD-INL Standard In-line Network Adapter PCIe 4x1Gb"
#define NT20E_CAP_STR           "NT20E-CAP Capture Network Adapter PCIe 2x10Gb XFP"
#define NT20E_INL_STR           "NT20E-INL In-line Network Adapter PCIe 2x10Gb XFP"
#define NT20E2_CAP_STR          "NT20E2-CAP Capture Network Adapter PCIe Gen2 2x10Gb SFP+"
#define NT20E2_INL_STR          "NT20E2-INL In-line Network Adapter PCIe Gen2 2x10Gb SFP+"
#define NT40E2_1_CAP_STR        "NT40E2_1-CAP Capture Network Adapter PCIe Gen2 1x40Gb QSFP"
#define NT40E2_1_INL_STR        "NT40E2_1-INL In-line Network Adapter PCIe Gen2 1x40Gb QSFP"
#define NT40E2_4_CAP_STR        "NT40E2_4-CAP Capture Network Adapter PCIe Gen2 4x10Gb SFP+"
#define NT40E2_4_INL_STR        "NT40E2_4-INL In-line Network Adapter PCIe Gen2 4x10Gb SFP+"

// First Byte (0-7): CARD type
// Second Byte (8-15): Napatech cards
// Second Byte (16-23): Unused
// Second Byte (24-31): Napatch card

//** Card types used with NTCI_OpenCard *
#define NTCI_CARD_TYPE_ANY                0         //<! Open any Napatech recognized cards
#define NTCI_ANY_NAPATECH_CARD_TYPE      (1<<29)    //<! Open any Napatech card

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define NTCI_ANY_EXTENDED_CARD_TYPE      (1<<31)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define NTCI_EXTENDED_CARD_TYPE_XSERIES  (1<<0)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XY_CARD_TYPE                     (0)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define NT_CARD_TYPE                     (1)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_CARD_TYPE_ANY NTCI_CARD_TYPE_ANY
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ANY_EXTENDED_CARD_TYPE NTCI_ANY_EXTENDED_CARD_TYPE
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define XYCI_ANY_NAPATECH_CARD_TYPE NTCI_ANY_NAPATECH_CARD_TYPE
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

/*
    NTCI_OpenCard ( )

Synopsis:
    Open a card from a list of cards generated by the Protocol Driver.

Operation:
    If the Protocol Driver has not been opened, will be opened. It will then
    generate a list of the cards it is attached to. This list may change as and
    when the Protocol Driver attaches or disconnects from cards.

    This function will open a card from that list.
    
    Once opened, the card handle is stored by the Card Manager. The Card
    Manager keeps track of resources used by each card. It also keeps the
    last error code generated by any of the NTCI_ APIs.
    If you open a card that is already being managed, you will receive the
    same handle of the original NTCI_OpenCard. The Manager will count the
    number of times it has been opened.
    
    Call NTCI_CloseCard when you have finished using the card.
    If NTCI_CloseCard has been called for every call to NTCI_OpenCard the Card
    Manager will then release any resources that it was tracking.
    If all cards have been closed, the Card Manager will then close the
    Protocol Driver itself.

    If the Protocol Driver is disconnected from your choosen card, any APIs
    called for that card will return NTCI_ERRCODE_CARD_HANDLE_NOT_VALID. The
    Card Manager will not get a chance to release resources. Using a handle for
    a disconnected card causes most apis to fail with NTCI_ERRCODE_CARD_HANDLE_NOT_VALID.


Parameter CardTypeMask: 
    Each card the list belongs to at least one of the NTCI_CARD_TYPE groups above.
    CardTypeMask is the bitmask of one or more card types.
    The function will only return a card handle for a card that matches with at
    least one bit of your mask.

Parameter CardInstance:
    Select which of the matching cards to open through CardInstance.
    The first matched card is CardInstance 0, the next match is instance 1 ...
*/

NTCOMMONLIB handle_t NTCOMMONCALL NTCI_OpenCard(IN uint32_t CardTypeMask,
                                                IN uint32_t CardInstance );

/*
    NTCI_GetCardHandle ( )

    Get the handle for a card that the Card Manager is already managing.
    
    If the Protocol Driver has been opened, the list of cards is examined
    as per NTCI_OpenCard.
    If the matching card is being managed, the return value will be the same
    as returned by NTCI_OpenCard. The Card Manager does not count the number
    of times this function is called, so you do not need to call NTCI_CloseCard
    
    If the card has not been opened, the return value will be zero.
*/
NTCOMMONLIB handle_t NTCOMMONCALL NTCI_GetCardHandle(   IN uint32_t CardTypeMask,
                                                        IN uint32_t CardInstance );

/*
    NTCI_CloseCard ( )
    
    Closes a card previously opened with NTCI_OpenCard.
    The Card Manager counts the number of times NTCI_OpenCard and 
    NTCI_CloseCard have been called. The card is only closed for real when
    they match.
    When closed for real, any open hardware resources are closed and the card
    is no longer managed.
    When all managed cards are closed, the Protocol Driver is then closed.
    
    Once closed for real, using the card handle will cause most APIs to return
    the value NTCI_ERRCODE_CARD_HANDLE_NOT_VALID.
*/

NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_CloseCard( IN handle_t hCard );

/*
    NTCI_CloseAllCards ( )
    Closes all cards previously opened with NTCI_OpenCard.
    Since all cards will be closed, the Protocol Driver will then be
    closed.
*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_CloseAllCards( void );

/*
    NTCI_Command ( )
    Generic common interface function.
*/
NTCOMMONLIB uint32_t NTCOMMONCALL NTCI_Command( IN  handle_t    hCard,
                                                IN StructID structid,
                                                OUT void*  data);

#endif  // __NTAPI_NTCOMMONINTERFACE_H__
