// $Id: basetype.h 25309 2013-09-05 13:49:51Z vmi $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Type definitions for size specific types. Apart from the boolean and
 * handle types these are as per the C99 stdint.h definitions.es
 */

#ifndef __NTAPI_BASETYPE_H__
#define __NTAPI_BASETYPE_H__

#include "ostype.h"

#if defined(WIN32) || defined(_WIN32_WINNT) || _NT_OS_TYPE==_NT_OS_TYPE_WIN || defined(_WIN64)
typedef unsigned char      uint8_t;
typedef signed char	       int8_t;
typedef unsigned short     uint16_t;
typedef signed short       int16_t;
typedef unsigned long      uint32_t; /* Safe on windows */

#define NT_FILE HANDLE

#include <stddef.h>					 /* for intptr_t and uintptr_t */
/* Unfortunately you need a new compiler for stddef to define intptr_t and
 * uintptr_t. If they are not defined after including stddef.h define them
 * assuming a 32bit platform. This should be safe as only older stddef.h
 * files are missing these definitions
 */
#ifndef _INTPTR_T_DEFINED
typedef int            intptr_t;
#define _INTPTR_T_DEFINED
#endif

#ifndef _UINTPTR_T_DEFINED
typedef unsigned int   uintptr_t;
#define _UINTPTR_T_DEFINED
#endif


#if !defined(__BIT_TYPES_DEFINED__)	 /* Berkeley DB defines this (see below) under windows */
  #ifndef _BITTYPES_H /* only if <bittypes.h> was not already included */
    typedef signed long        int32_t;  /* Safe on windows */
  #endif
#endif // __BIT_TYPES_DEFINED__      /* Berkeley DB defines this under windows */
typedef unsigned __int64   uint64_t;
typedef signed __int64     int64_t;
typedef void*              handle_t;
typedef unsigned int       bool_t;
typedef unsigned int       boolean_t;	/* legacy support - use bool_t for new attributes */
#else
typedef unsigned int       OS_STATUS;
#endif /* _NT_OS_TYPE_WIN */



#if _NT_OS_TYPE==_NT_OS_TYPE_LINUX
#ifdef __KERNEL__
#include <linux/version.h>

/* Kernel build uses the kernel typedefs */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,24)
typedef signed long intptr_t;
typedef unsigned long uintptr_t;
#endif  // LINUX_VERSION_CODE < KERNEL_VERSION 2,6,24
#include <linux/types.h>
#else // ! __KERNEL__
/* User space build uses the standard C typedefs */
#include <stdint.h>
#endif

typedef void*           handle_t;
#ifndef TRUE
typedef unsigned int	bool_t;
#define TRUE		1
#define FALSE		0
#endif // ! TRUE

typedef int NT_FILE;

#define NT_OPEN_ERROR -1
#define THREAD_API void  *
#define THREAD_RETURN_OK return NULL;
#endif // _NT_OS_TYPE_LINUX

#if _NT_OS_TYPE==_NT_OS_TYPE_FREEBSD
#include <sys/types.h>
typedef void*           handle_t;
typedef unsigned int	bool_t;
#define TRUE		1
#define FALSE		0
typedef int NT_FILE;

#define NT_OPEN_ERROR -1
#define THREAD_API void  *
#define THREAD_RETURN_OK return NULL;
#endif // _NT_OS_TYPE_FREEBSD

#if _NT_OS_TYPE==_NT_OS_TYPE_WIN
#define TRUE		1
#define FALSE		0
#endif

/*
 * STRUCTID macros used for creating and examining our global structure identifiers.
 */
typedef uint32_t StructureID;
typedef uint64_t StructID;
//Construct a unique token defining a structure. This token guards against changes to size, padding, etc.
#define MAKE_STRUCT_ID(__structureFuncGroup, __structureFunc, __structureType, __structureName ) \
  ((StructID)(((StructID)__structureFuncGroup << 40) |                                           \
              ((StructID)__structureFunc << 32) |                                                \
              (__structureType << 16) |                                                          \
              sizeof(__structureName)))

//Obtain type of structure given a structID token
#define GET_STRUCT_TYPE( __structID)            \
  ((__structID >> 16) & 0xFFFF)

//Obtain size of structure given a structID token
#define GET_STRUCT_SIZE( __structID)            \
  (__structID & 0xFFFF)

//Obtain function group of structure given a structID token
#define GET_STRUCT_FUNC_GROUP( __structID)      \
  (((StructID)__structID >> 40) & 0xFF)

//Obtain function of structure given a structID token
#define GET_STRUCT_FUNC( __structID)            \
  (((StructID)__structID >> 32) & 0xFF)

//Construct a unique token defining a strukture (less rigid than the above MAKE_STRUCT_ID)
#define MAKE_STRUCTURE_ID( __structureType, __structureName ) \
			((StructureID)((__structureType << 16) | sizeof(__structureName)))

//Obtain type of structure given a structID token
#define GET_STRUCTURE_TYPE( __structureID) \
      ((__structureID >> 16) & 0xFFFF)

//Obtain size of structure given a structID token
#define GET_STRUCTURE_SIZE( __structureID) \
			(__structureID & 0xFFFF)

#endif // __NTAPI_BASETYPE_H__
