// $Id: capabilitiesstructure.h 17384 2012-02-16 14:48:40Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Structure Definitions that are used by the external card capabilities
 * interfaces
 */

#ifndef __NTAPI_CAPABILITIESSTRUCTURE_H__
#define __NTAPI_CAPABILITIESSTRUCTURE_H__

#include "basetype.h"
#include "commontypes.h"
#include "ntpackpush.h"

/*
 * MAX_CAPABILITIES_SUBTREES is the number of subnodes that can be connected
 * to a node. This is also limits the index of any indexed node.
 */
#define MAX_CAPABILITIES_SUBTREES 32

/* MAX_CAPABILITIES_TREE_DEPTH is the sub tree depth allowed */
#define MAX_CAPABILITIES_TREE_DEPTH 8

/* Max length of the capabilities name string */
#define MAX_CAPABILITIES_NAME_LEN 16

/* Used in the subCapabilitiesIndex parameter when no sub node is required */
#define NOT_SUB_CAPABILITIES 0xff

/* The following enum block defines the capabilities identifiers */
enum
{
  CAPID_ROOT                                    = 0x00000000,
    CAPID_SERVICES                              = 0x00100000,
      CAPID_PACKET_FEED                         = 0x00110000,
        CAPID_STREAMING_INDEXED                 = 0x00111000,
        CAPID_CAPTURE_INDEXED                   = 0x00112000,
      CAPID_STATISTICS                          = 0x00120000,
      CAPID_PACKET_CLASSIFICATION               = 0x00130000,
    CAPID_HARDWARE                              = 0x00200000,
      CAPID_CHANNEL_INDEXED                     = 0x00210000,
      CAPID_CARD_INDEXED                        = 0x00220000,
      CAPID_VERSION                             = 0x00230000,
    CAPID_INTERFACE                             = 0x00300000,
      CAPID_STATISTICS_IF                       = 0x00310000,
      CAPID_PACKET_FEED_IF                      = 0x00320000,
      CAPID_VITAL_PRODUCT_DATA_IF               = 0x00330000,
      CAPID_CARD_CONFIGURATION_IF               = 0x00340000,
      CAPID_CARD_STATUS_IF                      = 0x00350000,
      CAPID_CAPABILITIES_IF                     = 0x00360000,
      CAPID_CARD_INFO_IF                        = 0x00370000,
      CAPID_HARDWARE_MONITOR_IF                 = 0x00380000,
      CAPID_PACKET_CLASSIFICATION_IF            = 0x00390000,
      CAPID_MEMORY_ACCESS_IF                    = 0x003A0000,
};

/* The following enum block relates to the structure ID groups */
enum
{
  eCommonCapabilitiesType1,
  eCardHardwareCapabilitiesType1,
  eCardHardwareChannelCapabilitiesType1,
  eCardHardwareCardCapabilitiesType1,
  eCardServicesCapabilitiesType1,
  eCardStatisticsCapabilitiesType1,
  eCardPacketFeedCapabilitiesType1,
  eCardCapturePacketFeedCapabilitiesType1,
  eCardStreamingPacketFeedCapabilitiesType1,
  eCardHardwareVersionCapabilitiesType1,
  eCardPacketClassificationCapabilitiesType1,
  eCardPacketFeedCapabilitiesType2,
  eCardStreamingPacketFeedCapabilitiesType2,
  eCardHardwareVersionCapabilitiesType2,
  eCardHardwareCapabilitiesType2,
  eCardHardwareVersionCapabilitiesType3,
  eCardInterfaceSupportedType1,
  eCardInterfaceSupportedSTIDType1,
  eCardHardwareChannelCapabilitiesType2,
  eCardHardwareCardCapabilitiesType2,
  eCardHardwareCapabilitiesType3,
  eCardHardwareCapabilitiesType4,
};

/*
 * Common Capabilities...
 */

/* Flags */
#define INDEXED_CAPABILITY 0x0001 /* Set when indexed node */
#define LEAF_CAPABILITY    0x0002 /* Set when no sun-node */
#define DYNAMIC_CAPABILITY 0x0004 /* Set when node is to be updated dynamically */
#define NODE_CAPABILITY    0x0008 /* Is this a capability node. */

typedef struct
{
  CapabilitiesId_t capabilitiesId;
  char name[MAX_CAPABILITIES_NAME_LEN];
  uint32_t flags;
  StructureID structId;       /* Of the capabilities held at this node */
} CommonCapabilitiesType1_t;

#define STID_COMMON_CAPABILITIES_TYPE1\
    MAKE_STRUCTURE_ID(eCommonCapabilitiesType1, CommonCapabilitiesType1_t)

/*
 * Card Hardware...
 */
/* Enums removed from struct for portability between C and C++ */
typedef enum CardType {
  XSERIES = 0,
  GB10S,
  SSERIES,
  TENGIG,
  NT4E,
  NT4F,
  NT10F,
  NT20F,
  NT4,
  NT4STD,
  NTBPE,
  NT20E2,
  NT40E2_4,
  NT40E2_1,
} CardType_t;

typedef enum {
  BUS_UNKNOWN = 0,
  BUS_PCI,
  BUS_PCIX,
  BUS_PCIEXPRESS,
  BUS_COMPACTPCI
} BusType_t;
#ifndef NT_LEGACY_4_25
typedef enum {
  SPEED_UNKNOWN = 0,
  SPEED_33MHZ,
  SPEED_66MHZ,
  SPEED_100MHZ,
  SPEED_133MHZ
} BusSpeed_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #else //NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef enum {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   SPEED_UNKNOWN = 0,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   SPEED_33MHZ,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   SPEED_66MHZ,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   SPEED_100MHZ,
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   SPEED_133MHZ
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } BusSpeed_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25
typedef enum {
  WIDTH_UNKNOWN = 0,
  WIDTH_32BIT,
  WIDTH_64BIT
} BusWidth_t;

typedef struct
{
  CommonCapabilitiesType1_t common;
  uint8_t  CardType;                      /* enum CardType */
  uint8_t  BusType;                       /* BusType_t */
  uint8_t  BusSpeed;                      /* BusSpeed_t */
  uint8_t  BusWidth;                      /* BusWidth_t */
  uint32_t totalNumberBondedCards;
  uint32_t currentNumberBondedCards;
  uint32_t totalChannelsBitmask;
  uint32_t currentChannelsBitmask;
  uint8_t  timestampType;                 /* TimestampType_t */
  bool_t   timestampSyncAcrossChannel;
  bool_t   timestampSyncAcrossCard;
  uint32_t totalBufferAllocated;
} CardHardwareCapabilitiesType1_t;

#define STID_CARD_HARDWARE_CAPABILITIES_TYPE1                       \
    MAKE_STRUCTURE_ID(eCardHardwareCapabilitiesType1,               \
                      CardHardwareCapabilitiesType1_t)

typedef struct
{
  CardHardwareCapabilitiesType1_t type1;
  bool_t   rxPortSelectionSupported;
  bool_t   txPortSelectionSupported;
  bool_t   localTransmitSupported;
  bool_t   flowControlSupported;
  bool_t   timestampTypesSupportedBitmask;
  uint32_t portsSupportedBitmask;
  uint32_t rxChannelsSupportedBitmask;
  uint32_t txChannelsSupportedBitmask;
} CardHardwareCapabilitiesType2_t;

#define STID_CARD_HARDWARE_CAPABILITIES_TYPE2                       \
    MAKE_STRUCTURE_ID(eCardHardwareCapabilitiesType2,               \
                      CardHardwareCapabilitiesType2_t)

typedef enum {
  PCIE_SPEED_UNKNOWN = 0,
  PCIE_SPEED_2500MBPS, /* 2.5Gb/s */
  PCIE_SPEED_5000MBPS /* 5.0Gb/s */
} PCIE_LinkSpeed_t;

typedef struct
{
  CardHardwareCapabilitiesType2_t type2;
  /* PCI-Express specifics. */
  struct {
    uint32_t maxPayloadSupported;
    uint32_t maxPayloadNegotiated;
    uint32_t maxThroughput;        /* Max throughput measured by the driver. */
    uint8_t  linkWidthSupported;
    uint8_t  linkWidthNegotiated;
    uint8_t  linkSpeed;            /* PCIE_LinkSpeed_t */
  }pcie;
} CardHardwareCapabilitiesType3_t;

#define STID_CARD_HARDWARE_CAPABILITIES_TYPE3                       \
    MAKE_STRUCTURE_ID(eCardHardwareCapabilitiesType3,               \
                      CardHardwareCapabilitiesType3_t)

typedef struct
{
  CardHardwareCapabilitiesType3_t type3;
  struct {
    uint8_t  linkSpeedSupported;
    uint8_t  reserved1;
    uint8_t  reserved2;
    uint8_t  reserved3;
  }pcie;
  uint32_t inlineCard:1;
  uint32_t pktTsMethod:1; /**
                           * 0: Packets are timestamp when the last octet is received
                           * 1: Packets are timestamp when the first octet is received
                           */
    #define PACKET_TIMESTAMP_LAST 0
    #define PACKET_TIMESTAMP_FIRST 1
  uint32_t wisMode:1;
  uint32_t keepAlivePkts:1;
  uint32_t highFreqSampling:1;
  /* tsInjectionLegacyMode == 1:
     Injected time stamp corresponds to the position of the time stamp itself;
     tsInjectionLegacyMode == 0:
     Injected time stamp corresponds to either start or end of frame
     as specified by the tsInjectionMode field (see below). */
  uint32_t tsInjectionLegacyMode:1;
#define TIME_STAMP_LEGACY_MODE_DEFAULT_VALUE 1
  /* tsInjectionMode == 0: Injected time stamp corresponds to start of frame.
     tsInjectionMode == 1: Injected time stamp corresponds to end of frame. */
  uint32_t tsInjectionMode:1;
#define TIME_STAMP_INJECTION_VALUE_START_OF_FRAME 0
#define TIME_STAMP_INJECTION_VALUE_END_OF_FRAME 1
#define TIME_STAMP_INJECTION_DEFAULT_VALUE 0
  /* tsInjectionOffsetMode == 0:
     Injected time stamp offset base is start of frame.
     tsInjectionOffsetMode == 1:
     Injected time stamp offset base is end of frame. */
  uint32_t tsInjectionOffsetMode:1;
#define TIME_STAMP_INJECTION_OFFSET_START_OF_FRAME 0
#define TIME_STAMP_INJECTION_OFFSET_END_OF_FRAME 1
#define TIME_STAMP_INJECTION_OFFSET_DEFAULT_VALUE 0
  uint32_t osTimesync:1;
  uint32_t osTimesyncFailover:1;
  /* TX only. 1 (default) => accurate IFG; 0 => accurate transmission time */
  uint32_t relativeTxTiming:1;
#define RELATIVE_TX_TIMING_MODE_VALUE 1
#define ABSOLUTE_TX_TIMING_MODE_VALUE 0
#define RELATIVE_TX_TIMING_MODE_DEFAULT_VALUE 0
  uint32_t reserved0:21;           /* Reserved for future use */
  uint32_t reserved1;              /* Reserved for future use */
  uint32_t reserved2;              /* Reserved for future use */
  uint32_t reserved3;              /* Reserved for future use */
  uint32_t reserved4;              /* Reserved for future use */
} CardHardwareCapabilitiesType4_t;

#define STID_CARD_HARDWARE_CAPABILITIES_TYPE4                       \
    MAKE_STRUCTURE_ID(eCardHardwareCapabilitiesType4,               \
                      CardHardwareCapabilitiesType4_t)

/*
 * Card Hardware Channel...
 */

/* Enums removed from struct for portability between C and C++ */
typedef enum {
  UNKNOWN_CON,
  NONE,
  PLUGGABLE_SFP,
  PLUGGABLE_XFP,
  FIXED_RJ45,
  FIXED_MSA,
  PLUGGABLE_XPAK,
  PLUGGABLE_SFP_PLUS,
  PLUGGABLE_QSFP_PLUS,
} ConnectionType_t;

typedef enum {
  SPEED_10MB   = 0x01,
  SPEED_100MB  = 0x02,
  SPEED_1GB    = 0x04,
  SPEED_10GB   = 0x08,
  SPEED_40GB   = 0x10,
  SPEED_100GB  = 0x20
} SupportedSpeeds_t;

typedef struct
{
  CommonCapabilitiesType1_t common;
  uint32_t supportedSpeeds;               /* SupportedSpeeds_t */
  uint8_t  connectionType;                /* ConnectionType_t */
  uint32_t virtualChannelBitmask;
} CardHardwareChannelCapabilitiesType1_t;

#define STID_CARD_HARDWARE_CHANNEL_CAPABILITIES_TYPE1               \
    MAKE_STRUCTURE_ID(eCardHardwareChannelCapabilitiesType1,        \
                      CardHardwareChannelCapabilitiesType1_t)

typedef struct
{
  CardHardwareChannelCapabilitiesType1_t type1;
  uint8_t  macAddress[6];                 /* Mac address.     */
} CardHardwareChannelCapabilitiesType2_t;

#define STID_CARD_HARDWARE_CHANNEL_CAPABILITIES_TYPE2               \
    MAKE_STRUCTURE_ID(eCardHardwareChannelCapabilitiesType2,        \
                      CardHardwareChannelCapabilitiesType2_t)

/*
 * Card Hardware Card...
 */
#define MAX_NUM_FPGA_OPTIONS 2
typedef struct
{
  CommonCapabilitiesType1_t common;
  uint32_t bondedGroupId;
  uint32_t cardId;
  bool_t   bondMaster;
  uint32_t virtualChannelsBitmask;
  uint8_t  numberOfFPGAs;
} CardHardwareCardCapabilitiesType1_t;

#define STID_CARD_HARDWARE_CARD_CAPABILITIES_TYPE1                  \
    MAKE_STRUCTURE_ID(eCardHardwareCardCapabilitiesType1,           \
                      CardHardwareCardCapabilitiesType1_t)

typedef struct
{
  CardHardwareCardCapabilitiesType1_t type1;
  bool_t   SDRAMController;    /* True if the card has a SDRAM controller
                                  available.                                     */
  bool_t   SDRAMPresent;       /* True if a SDRAM module is present.             */
  uint32_t SDRAMSize;          /* SDRAM size in bytes.                           */
  bool_t   TimeSyncSupported;  /* True if adapter supports time synchronization. */
} CardHardwareCardCapabilitiesType2_t;

#define STID_CARD_HARDWARE_CARD_CAPABILITIES_TYPE2                  \
    MAKE_STRUCTURE_ID(eCardHardwareCardCapabilitiesType2,           \
                      CardHardwareCardCapabilitiesType2_t)

/*
 * Card Hardware Version...
 */
typedef struct
{
  CommonCapabilitiesType1_t common;
  uint32_t BootStrapVersion;
  uint32_t FPGAVersion;
  uint32_t ExtVersion;
} CardHardwareVersionCapabilitiesType1_t;

#define STID_CARD_HARDWARE_VERSION_CAPABILITIES_TYPE1                  \
    MAKE_STRUCTURE_ID(eCardHardwareVersionCapabilitiesType1,           \
                      CardHardwareVersionCapabilitiesType1_t)

typedef struct
{
  CardHardwareVersionCapabilitiesType1_t type1;
  uint32_t CurrentFPGAImageLoaded;
} CardHardwareVersionCapabilitiesType2_t;

#define STID_CARD_HARDWARE_VERSION_CAPABILITIES_TYPE2                  \
    MAKE_STRUCTURE_ID(eCardHardwareVersionCapabilitiesType2,           \
                      CardHardwareVersionCapabilitiesType2_t)

/*-----------------------------------------------------------------------------
 * The following data structures are used to obtain NIC hardware information.
 *---------------------------------------------------------------------------*/
typedef struct {
  /* Type2 is for backwards compability. */
  CardHardwareVersionCapabilitiesType2_t type2;
  struct {
    uint32_t          dwInfoType;
    uint32_t          dwAlignmentUnused;

    union {
      /*-----------------------------------------------------------------------
       * Main board info type 1 is used for the PCI-X NT4E and NT4F hardware
       * platforms.
       *---------------------------------------------------------------------*/
      struct {
        uint32_t    dwType;
        uint32_t    dwCpldId;
        uint64_t    qwFpgaId;
        uint32_t    dwFpgaFreq;
        uint32_t    dwMacId;
        uint32_t    dwPhyId;
      } InfoType1;

      /*-----------------------------------------------------------------------
       * Main board info type 2 is used for NT hardware platform.
       *---------------------------------------------------------------------*/
      struct {
        uint32_t    dwType;
        uint32_t    dwCpldId;
        uint64_t    qwFpgaId;
        uint32_t    dwFpgaFreq;
      } InfoType2;

      /*-----------------------------------------------------------------------
       * Main board info type 3 is used for NT hardware platform with AVR.
       *---------------------------------------------------------------------*/
      struct {
        uint32_t    dwType;
        uint32_t    dwReserved;
        uint64_t    qwFpgaId;
        uint32_t    dwFpgaFreq;
        uint32_t    dwAvrMain;
        uint32_t    dwAvrFront;
      } InfoType3;
    } u;
  } AdapterCard;

  struct {
    uint32_t          dwInfoType;
    union {
      /*-----------------------------------------------------------------------
       * Extension board info type 1 is used for NT hardware platforms.
       *---------------------------------------------------------------------*/
      struct {
        uint32_t    dwType;
        uint32_t    dwPhyId;
      } InfoType1;
    } u;
  } ExpansionCard;
} CardHardwareVersionCapabilitiesType3_t;

#define STID_CARD_HARDWARE_VERSION_CAPABILITIES_TYPE3                  \
    MAKE_STRUCTURE_ID(eCardHardwareVersionCapabilitiesType3,           \
                      CardHardwareVersionCapabilitiesType3_t)

/*
 * Card Services...
 */
typedef struct
{
  CommonCapabilitiesType1_t common;
  bool_t diagnosticSupported;
  bool_t hardwareMonitorSupported;
  bool_t packetClassificationSupported;
  bool_t lowLevelFilterSupported;
  bool_t packetFeedSupported;
  bool_t statisticsSupported;
} CardServicesCapabilitiesType1_t;

#define STID_CARD_SERVICES_CAPABILITIES_TYPE1                       \
    MAKE_STRUCTURE_ID(eCardServicesCapabilitiesType1,               \
                      CardServicesCapabilitiesType1_t)

/*
 * Card Statistics...
 */
#define MAX_STRUCTURE_ID_SUPPORTED 10
typedef struct
{
  CommonCapabilitiesType1_t common;
  uint32_t getDataStructureIdSupported[MAX_STRUCTURE_ID_SUPPORTED];
  uint32_t numberStructureIdSupported;
  bool_t   hardwareRunControl;
  bool_t   softwareRunControl;
  bool_t   hardwareRunControlChannelLinked;
  bool_t   softwareRunControlChannelLinked;
  bool_t   crossChannelHardwareMerging;
  bool_t   crossChannelSoftwareMerging;
  bool_t   crossCardHardwareMerging;
  bool_t   crossCardSoftwareMerging;
  bool_t   startWithFeedStart;
  bool_t   stopWithFeedStop;
  bool_t   resetWithFeedStart;
} CardStatisticsCapabilitiesType1_t;

#define STID_CARD_STATISTICS_CAPABILITIES_TYPE1                     \
    MAKE_STRUCTURE_ID(eCardStatisticsCapabilitiesType1,             \
                      CardStatisticsCapabilitiesType1_t)

/*
 * Card Packet Feed...
 */
typedef struct
{
  CommonCapabilitiesType1_t common;
  uint32_t minPacketSize;
  uint32_t maxPacketSize;
  uint32_t numberOfCaptureSupported;
  uint32_t numberOfStreamingSupported;
  uint32_t currentEngineFeedCount:8; /* we do not get to use 32 bit, so 24 has been reserved for future use */
  uint32_t txBoundToPort:1;          /* 1: Tx channels are per port. All packets going to feed 0 are send onto port 0, Feed1 = Port1 etc */
                                     /* 0: Tx channels are merged. The packet descriptor determine which port should transmit the packet */
  uint32_t sharedFifo:1;             /* 0: Adapter only support syscall based segment exchange */
                                     /* 1: Adapter supports shared fifo segment exchange */
  uint32_t reserved:22;
  bool_t   crossChannelMerging;
  bool_t   crossCardMerging;
  bool_t   internalRxSupported;
  bool_t   externalRxSupported;
  bool_t   internalTxSupported;
  bool_t   externalTxSupported;
  uint32_t channelsInUseBitmask;     /* Receive channels only.  In use across all packet feeds */
} CardPacketFeedCapabilitiesType1_t;

#define STID_CARD_PACKET_FEED_CAPABILITIES_TYPE1                    \
    MAKE_STRUCTURE_ID(eCardPacketFeedCapabilitiesType1,             \
                      CardPacketFeedCapabilitiesType1_t)

typedef struct
{
  CardPacketFeedCapabilitiesType1_t type1;
  bool_t   endianSwapSupported;
  bool_t   ipAlignmentSupported;
  bool_t   txChecksumCalculationSupported;
  bool_t   slicingSupported;
  uint32_t channelsInUseBitmaskRx;        /* Rx channels in use across all packet feeds */
  uint32_t channelsInUseBitmaskTx;        /* Tx channels in use across all packet feeds */
}CardPacketFeedCapabilitiesType2_t;

#define STID_CARD_PACKET_FEED_CAPABILITIES_TYPE2                    \
    MAKE_STRUCTURE_ID(eCardPacketFeedCapabilitiesType2,             \
                      CardPacketFeedCapabilitiesType2_t)

/*
 * Card Capture Packet Feed
 */
typedef struct
{
  CommonCapabilitiesType1_t common;
  uint32_t channelsBitmask;
  bool_t   running;
  bool_t   externalFeed;
  bool_t   transmittingFeed;
} CardCapturePacketFeedCapabilitiesType1_t;

#define STID_CARD_CAPTURE_PACKET_FEED_CAPABILITIES_TYPE1            \
    MAKE_STRUCTURE_ID(eCardCapturePacketFeedCapabilitiesType1,      \
                      CardCapturePacketFeedCapabilitiesType1_t)

/*
 * Card Streaming Packet Feed
 */
typedef struct
{
  CommonCapabilitiesType1_t common;
  uint32_t channelsBitmask;           /* Receive channels only */
  bool_t externalFeed;
  bool_t transmittingFeed;
  bool_t dropErroredFrames;
  uint32_t sliceLength;
  uint32_t numSections;               /* Receive only */
  uint32_t sectionBufferSize;         /* Receive buffer only */
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifndef NT_LEGACY_4_25
  uint32_t Reserved0;                 /* reserved field. Do not use. */
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #else
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */   uint32_t bufferBehaviour DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif
} CardStreamingPacketFeedCapabilitiesType1_t;

#define STID_CARD_STREAMING_PACKET_FEED_CAPABILITIES_TYPE1          \
    MAKE_STRUCTURE_ID(eCardStreamingPacketFeedCapabilitiesType1,    \
                      CardStreamingPacketFeedCapabilitiesType1_t)


typedef struct
{
  CardStreamingPacketFeedCapabilitiesType1_t type1;
  uint32_t channelsBitmaskRx;
  uint32_t channelsBitmaskTx;
  uint32_t numSectionsRx;
  uint32_t numSectionsTx;
  uint32_t sectionBufferSizeRx;
  uint32_t sectionBufferSizeTx;
} CardStreamingPacketFeedCapabilitiesType2_t;

#define STID_CARD_STREAMING_PACKET_FEED_CAPABILITIES_TYPE2          \
    MAKE_STRUCTURE_ID(eCardStreamingPacketFeedCapabilitiesType2,    \
                      CardStreamingPacketFeedCapabilitiesType2_t)

/*
 * Packet Classification Capabilities
 */
#define MAX_DYNAMIC_OFFSET_TYPES 32
typedef struct
{
  CommonCapabilitiesType1_t common;
  uint64_t supportedComponentTypes;
  uint64_t supportedActionTypes;
  bool_t   dynamicOffsetSupport;
  uint8_t  dynamicOffsetsSupported[MAX_DYNAMIC_OFFSET_TYPES];
  uint32_t numDynamicOffsets;
  uint32_t maxExpressionNesting;  /* Attribute value is a guide as the number is dynamic depending on how resources are used */
  uint32_t maxExpressions;        /* Attribute value is a guide as the number is dynamic depending on how resources are used */
  uint32_t maxFunctions;          /* Attribute value is a guide as the number is dynamic depending on how resources are used */
  uint32_t maxPatternLength;
  uint32_t maxFixedOffset;
  uint32_t maxSignatureMatchPatternLength;
  uint64_t supportedSignatureMatchTypes;
  uint64_t supportedSignatureMatchActionTypes;
} CardPacketClassificationCapabilitiesType1_t;

#define STID_CARD_PACKET_CLASSIFICATION_CAPABILITIES_TYPE1            \
    MAKE_STRUCTURE_ID(eCardPacketClassificationCapabilitiesType1,     \
                      CardPacketClassificationCapabilitiesType1_t)

/*
 * Card Capabilities Interface STID's supported...
 */
#define MAX_STID_NAME_SUPPORTED 25
#define MAX_STID_NAME_LENGTH 70

typedef struct
{
  CommonCapabilitiesType1_t common;
  char InterfaceSTID[MAX_STID_NAME_SUPPORTED+1][MAX_STID_NAME_LENGTH];
} CardInterfaceSupportedSTIDType1_t;

#define STID_CARD_INTERFACE_SUPPORTED_STID_TYPE1                     \
    MAKE_STRUCTURE_ID(eCardInterfaceSupportedSTIDType1,              \
                      CardInterfaceSupportedSTIDType1_t)

typedef struct
{
  CommonCapabilitiesType1_t common;
  bool_t boStatisticsIFsupported;
  bool_t boPacketFeedIFsupported;
  bool_t boVPDIFsupported;
  bool_t boCardConfigurationIFsupported;
  bool_t boCardStatusIFsupported;
  bool_t boCapabilitiesIFsupported;
  bool_t boCardInfoIFsupported;
} CardInterfaceSupportedType1_t;

#define STID_CARD_INTERFACE_SUPPORTED_TYPE1                     \
    MAKE_STRUCTURE_ID(eCardInterfaceSupportedType1,             \
                      CardInterfaceSupportedType1_t)

#include "ntpackpop.h"

#endif // __NTAPI_CAPABILITIESSTRUCTURE_H__
