// $Id: cardstatusstructure.h 16051 2011-11-30 12:20:27Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Structure Definitions that are used by the external card status interfaces.
 */

#ifndef __NTAPI_CARDSTATUSSTRUCT_H__
#define __NTAPI_CARDSTATUSSTRUCT_H__

#include <ostype.h>
#include "basetype.h"
#include "portbypassmodes.h"
#include "ntpackpush.h"

/* The following enum block relates to the structure ID groups */
enum
{
  eChannelsLinkStatusType1, /* Used with GetChannelsLinkStatus() */
  eChannelStatusType1,      /* Used with GetChannelStatus() */
  eChannelStatusType2,      /* Used with GetChannelStatus() */
  eChannelStatusType3,      /* Used with GetChannelStatus() */
  eChannelStatusType4,      /* Used with GetChannelStatus() */
};

#define LINK_SPEED_10M     0x01
#define LINK_SPEED_100M    0x02
#define LINK_SPEED_1G      0x04
#define LINK_SPEED_10G     0x08
#define LINK_SPEED_40G     0x10
#define LINK_SPEED_100G    0x20


enum PORT_TYPES {
  PORT_TYPE_NOT_AVAILABLE,    /* No physical channel */
  PORT_TYPE_NOT_RECOGNISED,   /* Physical channel is available but the type cannot be identified */
  PORT_TYPE_RJ45,             /* Hardwired RJ45 connector */
  PORT_TYPE_SFP_NOT_PRESENT,  /* SFP card cage available but no SFP present */
  PORT_TYPE_SFP_SX,
  PORT_TYPE_SFP_SX_DD,        /* SX SFP with digital diagnostic */
  PORT_TYPE_SFP_LX,
  PORT_TYPE_SFP_LX_DD,
  PORT_TYPE_SFP_ZX,
  PORT_TYPE_SFP_ZX_DD,
  PORT_TYPE_SFP_CU,           /* Copper SFP */
  PORT_TYPE_SFP_CU_DD,
  PORT_TYPE_SFP_NOT_RECOGNISED,
  PORT_TYPE_XFP,
  PORT_TYPE_XPAK,
  PORT_TYPE_SFP_CU_TRI_SPEED,
  PORT_TYPE_SFP_CU_TRI_SPEED_DD,
  PORT_TYPE_SFP_PLUS,         /* SFP+ port */
  PORT_TYPE_QSFP_PLUS,        /* QSFP+ port */
};

enum LINK_QUALITY {
  LINK_QUALITY_GOOD     = 1,    /* Link up with good quality */
  LINK_QUALITY_POOR     = 2,    /* Link up with a bad quality optical level */
  LINK_QUALITY_BAD      = 3,    /* Link Down or optical light level's too low to register*/
  LINK_QUALITY_ERROR    = 4,    /* Link Down caused by error */
};

/* Used with GetChannelsLinkStatus() */
typedef struct
{
  uint32_t    numberOfChannelsAvailable;    /* number of physically available channels - 2 for 'S' series cards */
  uint32_t    channelsAvailableBitmask;     /* bitmask to determine what the available channels are - always 0x3 for 'S' series cards */
  uint32_t    linkUpBitmask;                /* bitmask to show which of the available channels above have a good link up status */
} ChannelsLinkStatusType1_t;

#define STID_CHANNELS_LINK_STATUS_TYPE1 \
    MAKE_STRUCTURE_ID(eChannelsLinkStatusType1, ChannelsLinkStatusType1_t)

/* Used with GetChannelStatus() */
typedef struct
{
  enum PORT_TYPES     portType;           /* status of what type the port is - e.g. not present, optical, copper */
  enum PORT_BYPASS_MODES  portMode;       /* status of the current port mode e.g. Dual Monitor / End Station */
  bool_t        autoNegotiationEnabled;   /* status of whether port is set to autonegotiate automatically */
  bool_t        fullDuplex;
  bool_t        flowControl;
  bool_t        linkUp;
  enum LINK_QUALITY   linkQuality;        /* reflection of the link status LED on the 'S' card e.g. Green/Amber/Red */
  uint32_t        availableSpeedsBitmask; /* made up from the LINK_SPEED'S values, could be 1 or many e.g. 10/100/1000 RJ45 (=0x07) */
  uint32_t        actualSpeed;            /* LINK_SPEEDS values */
} ChannelStatusType1_t;

#define STID_CHANNEL_STATUS_TYPE1 \
    MAKE_STRUCTURE_ID(eChannelStatusType1, ChannelStatusType1_t)

typedef struct
{
  /* Auto-negotiation capabilities*/
  bool_t       advertise10000FullDuplex;
  bool_t       advertise10000HalfDuplex;
  bool_t       advertise1000FullDuplex;
  bool_t       advertise1000HalfDuplex;
  bool_t       advertise100FullDuplex;
  bool_t       advertise100HalfDuplex;
  bool_t       advertise10FullDuplex;
  bool_t       advertise10HalfDuplex;
  /* Manuel configuration capabilities*/
  bool_t       manualFullDuplex;
  bool_t       manualHalfDuplex;
  bool_t       manualSpeed10000;
  bool_t       manualSpeed1000;
  bool_t       manualSpeed100;
  bool_t       manualSpeed10;
  bool_t       manualMdiModeAuto;
  bool_t       manualMdiModeMDI;
  bool_t       manualMdiModeMDIX;
} PortConfigCapabilities_t;

typedef struct
{
  enum PORT_TYPES     portType;               /* status of what type the port is - e.g. not present, optical, copper */
  enum PORT_BYPASS_MODES  portMode;           /* status of the current port mode e.g. Dual Monitor / End Station */
  bool_t        autoNegotiationEnabled;       /* status of whether port is set to autonegotiate automatically */
  bool_t        fullDuplex;
  bool_t        flowControl;
  bool_t        linkUp;
  enum LINK_QUALITY   linkQuality;            /* reflection of the link status LED on the 'S' card e.g. Green/Amber/Red */
  uint32_t      availableSpeedsBitmask;       /* made up from the LINK_SPEED'S values, could be 1 or many e.g. 10/100/1000 RJ45 (=0x07) */
  uint32_t      actualSpeed;                  /* LINK_SPEEDS values */
  bool_t        configSuccess;
  PortConfigCapabilities_t configCapabilities;
} ChannelStatusType2_t;

#define STID_CHANNEL_STATUS_TYPE2 \
    MAKE_STRUCTURE_ID(eChannelStatusType2, ChannelStatusType2_t)

#define MANUAL_FULL_DUPLEX 0x0001
#define MANUAL_HALF_DUPLEX 0x0002

#define MANUAL_MDI_MODE_AUTO   0x0001
#define MANUAL_MDI_MODE_MDI    0x0002
#define MANUAL_MDI_MODE_MDIX   0x0004

//Defines for advertise bit mask
#define ADVERTISE_10M_HALFDUPLEX   0x0001
#define ADVERTISE_10M_FULLDUPLEX   0x0002
#define ADVERTISE_100M_HALFDUPLEX  0x0004
#define ADVERTISE_100M_FULLDUPLEX  0x0008
#define ADVERTISE_1G_HALFDUPLEX    0x0010
#define ADVERTISE_1G_FULLDUPLEX    0x0020
#define ADVERTISE_10G_FULLDUPLEX   0x0040
#define ADVERTISE_40G_FULLDUPLEX   0x0080
#define ADVERTISE_100G_FULLDUPLEX  0x0100

typedef struct
{
  enum PORT_TYPES         portType;           /* status of what type the port is - e.g. not present, optical, copper */
  enum PORT_BYPASS_MODES  portMode;           /* status of the current port mode e.g. Dual Monitor / End Station */
  bool_t        autoNegotiationEnabled;       /* status of whether port is set to autonegotiate automatically */
  bool_t        fullDuplex;
  bool_t        flowControl;
  bool_t        linkUp;
  enum LINK_QUALITY   linkQuality;            /* reflection of the link status LED on the 'S' card e.g. Green/Amber/Red */
  uint32_t      availableSpeedsBitmask;       /* made up from the LINK_SPEED'S values, could be 1 or many e.g. 10/100/1000 RJ45 (=0x07) */
  uint32_t      actualSpeed;                  /* LINK_SPEEDS values */
  bool_t        configSuccess;
  uint32_t      advertiseCapabilitiesdBitMask;/* Bitmask of advertised speeds and duplex*/
  uint32_t      manuelConfDuplexBitMask;      /* Bitmask of manual configuration Duplex capabilities */
  uint32_t      manuelConfSpeedBitMask;       /* Bitmask of manual configuration speed capabilities*/
  uint32_t      manuelMdiModeBitMask;         /* Bitmask of manual configuration mdi capabilities*/
} ChannelStatusType3_t;

#define STID_CHANNEL_STATUS_TYPE3 \
    MAKE_STRUCTURE_ID(eChannelStatusType3, ChannelStatusType3_t)

typedef struct
{
  ChannelStatusType3_t type3;
  bool_t               wisModeStatus;         /* WIS mode stataus 0: WIS mode disabled, 1: WIS mode enabled */
  uint32_t             linkDownEventCounter;
} ChannelStatusType4_t;

#define STID_CHANNEL_STATUS_TYPE4 \
    MAKE_STRUCTURE_ID(eChannelStatusType4, ChannelStatusType4_t)

#include "ntpackpop.h"

#endif // __NTAPI_CARDSTATUSSTRUCT_H__
