// $Id: extendeddescriptor03.h 17384 2012-02-16 14:48:40Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This file defines the format (structure definition) of the extended
 * descriptor with a type field of 0x03.
 * This structure definition can be overlaid onto the data that immeadiatly
 * follows the standard descriptor assuming that the user has created a packet
 * feed for which this extended descriptor has been requested.
 */

#ifndef  __NTAPI_EXTENDEDDESCRIPTOR03_H__
#define  __NTAPI_EXTENDEDDESCRIPTOR03_H__

#include "basetype.h"
#include "ntpackpush.h"
  
#define EXTENDED_DESCRIPTOR_03_KNOWN

/* The following are the values to use with CreatePacketFeed in order to request this
    full extended descriptor*/

/* The following definitions should be used when creating a packet feed with one of the
	'FeedConfigSampledType' structures
  For creating a feed with this structure you need to set the 'descriptorType' (if applicable)
   to EXTENDED_DESCRIPTOR
  Use the following with the 'extendedDescriptorFormat' attribute */
#define EXTENDED_DESCRIPTOR_03_TYPE	0x03

#define EXTENDED_DESCRIPTOR_03_LENGTH 0x01

/* Generic header. All ext03 has this */
typedef struct tagPacketExtDescriptorType03_00
{
  /* 32bit Word0. */
  uint32_t Reserved0:16;             /* Unused  - 15:00 - Reserved. */
  uint32_t extHeaderLength:8;        /* RX Only - 23:16 - Extended header length. Always 8. */
#ifndef NT_LEGACY_4_25
  uint32_t Reserved1:8;              /* Unused  - 21:24 - Reserved. */ 
#else //NT_LEGACY_4_25
  uint32_t userID:8;                 /* 21:24 - User ID.*/ 
#endif //NT_LEGACY_4_25
}PacketExtDescriptorType03_00_t;

typedef struct PacketClassificationDescriptorType_t
{
  /* 32bit Word1. */
  uint32_t reserved0:2;              /* Unused  - 01:00 - Reserved. */
  uint32_t protSmall:1;              /* RX Only -    02 - Frame is too small for the applied protocols. */
  uint32_t large:1;                  /* RX Only -    03 - Large frame. 1 when PktSz > MaxSz as configured with CardConfigMaxFrameSize() */
#ifdef WIN32                            
  uint32_t _small:1;                 /* RX Only -    04 - Small frame. 1 when PktSz < 64 (+ISL and/or VLAN if set via CardConfigDecoderSet()) */
#else                                   
  uint32_t small:1;                  /* RX Only -    04 - Small frame. 1 when PktSz < 64 (+ISL and/or VLAN if set via CardConfigDecoderSet()) */
#endif                                  
  uint32_t jumbo:1;                  /* RX Only -    05 - Jumbo frame. 1 when PktSz > 1518 (+ISL, VLAN, MPLS if set via CardConfigDecoderSet()) */
  uint32_t l3Fragmented:1;           /* RX Only -    06 - L3 fragmented frame. (More-bit=1 || offset!=0) Note: Only valid for IPv4. */
  uint32_t l3FirstFragment:1;        /* RX Only -    07 - L3 First fragment. (More-bit=1 && offset=0) Note: Only valid for IPv4. */
  uint32_t l4Length:4;               /* RX Only - 11:08 - Layer4 header length (in units of 32bits) */
  uint32_t l4Type:2;                 /* RX Only - 13:12 - Layer4 type. 0:TCP, 1:UDP, 2:ICMP, 3:Other*/
  uint32_t reserved1:2;              /* Unused  - 15:14 - Reserved. */
  uint32_t l3Length:4;               /* RX Only - 19:16 - Layer3 header length (in units of 32bits). Note: Valid for L3=IPv4 and IPv6. */
  uint32_t l3Type:2;                 /* RX Only - 21:20 - Layer3 type. 0:IPv4, 1:IPv6, 2:IPX, 3:Other*/
  uint32_t reserved2:1;              /* Unused  -    22 - Reserved. */
  uint32_t broadcastDest:1;          /* RX Only -    23 - Destination MAC address is broadcast. (DMAC are all 1's) */
  uint32_t mplsCount:3;              /* RX Only - 26:24 - Number of MPLS shim labels present. */
  uint32_t islPresent:1;             /* RX Only -    27 - ISL encapsulation present. */
  uint32_t vlanCount:2;              /* RX Only - 29:28 - Number of VLANs present. */
  uint32_t l2Type:2;                 /* RX Only - 31:30 - Layer2 type. (0:EtherII, 1:LLC, 2:SNAP, 3:Novell RAW) */
} PacketClassificationDescriptorType_t;

  
/* Layer 2 types. */
typedef enum
{
  EXT03_L2_ETHER_II=0,
  EXT03_L2_LLC,
  EXT03_L2_SNAP,
  EXT03_L2_NOVELL_RAW
} ext03_Layer2EncapsulationType_t;

/* Layer 3 types. */
typedef enum
{
  EXT03_L3_IPv4=0,
  EXT03_L3_IPv6,
  EXT03_L3_IPX,
  EXT03_L3_OTHER
} ext03_Layer3EncapsulationType_t;

/* Layer 4 types. */
typedef enum
{
  EXT03_L4_TCP=0,
  EXT03_L4_UDP,
  EXT03_L4_ICMP,
  EXT03_OTHER
} ext03_Layer4EncapsulationType_t;

#include "ntpackpop.h"

#endif // __NTAPI_EXTENDEDDESCRIPTOR03_H__
