// $Id: extendeddescriptor05.h 16051 2011-11-30 12:20:27Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This file defines the format (structure definition) of the extended
 * descriptor with a type field of 0x05.
 * This structure definition can be overlaid onto the data that immeadiatly
 * follows the standard descriptor assuming that the user has created a packet
 * feed for which this extended descriptor has been requested.
 */

#ifndef  __NTAPI_EXTENDEDDESCRIPTOR05_H__
#define  __NTAPI_EXTENDEDDESCRIPTOR05_H__

#include "basetype.h"
#include "ntpackpush.h"

#define EXTENDED_DESCRIPTOR_05_KNOWN

/* The following are the values to use with CreatePacketFeed in order to request this
    full extended descriptor*/

/* The following definitions should be used when creating a packet feed with one of the
	'FeedConfigSampledType' structures
  For creating a feed with this structure you need to set the 'descriptorType' (if applicable)
   to EXTENDED_DESCRIPTOR
  Use the following with the 'extendedDescriptorFormat' attribute */
#define EXTENDED_DESCRIPTOR_05_TYPE	0x05

#define EXTENDED_DESCRIPTOR_05_LENGTH 0x02

typedef struct PacketExtDescriptorType05
{
  /* 32bit Word0. */
  uint32_t hash:31;                  /* RX Only - 30:00 - 2-tuple or 5-tuple hash. (configurable via NTPL) */
  uint32_t hashValid:1;              /* RX Only -    31 - Hash valid. zero if hash-config is None or inbound frame invalid for hash-calc, one otherwise. */
  /* 32bit Word1. */
  uint32_t reserved0:1;              /* Unused  -    00 - Reserved. */
  uint32_t txTsInject:1;             /* TX Only -    01 - Inject TX timestamp.
                                                          Note: The offset where the timestamp should be injected is located
                                                                at txTsInjectOffset/layer5HeaderOffset. */
  uint32_t protSmall:1;              /* RX Only -    02 - Frame is too small for the applied protocols. */
  uint32_t large:1;                  /* RX Only -    03 - Large frame. 1 when PktSz > MaxSz as configured with CardConfigMaxFrameSize() */
#ifdef WIN32
  uint32_t _small:1;                 /* RX Only -    04 - Small frame. 1 when PktSz < 64 (+ISL and/or VLAN if set via CardConfigDecoderSet()) */
#else
  uint32_t small:1;                  /* RX Only -    04 - Small frame. 1 when PktSz < 64 (+ISL and/or VLAN if set via CardConfigDecoderSet()) */
#endif
  uint32_t jumbo:1;                  /* RX Only -    05 - Jumbo frame. 1 when PktSz > 1518 (+ISL, VLAN, MPLS if set via CardConfigDecoderSet()) */
  uint32_t l3Fragmented:1;           /* RX Only -    06 - L3 fragmented frame. (More-bit=1 || offset!=0) Note: Only valid for IPv4. */
  uint32_t l3FirstFragment:1;        /* RX Only -    07 - L3 First fragment. (More-bit=1 && offset=0) Note: Only valid for IPv4. */
  uint32_t l4Length:4;               /* RX Only - 11:08 - Layer4 header length (in units of 32bits) */
  uint32_t l4Type:2;                 /* RX Only - 13:12 - Layer4 type. 0:TCP, 1:UDP, 2:ICMP, 3:Other*/
  uint32_t reserved1:2;              /* Unused  - 15:14 - Reserved. */
  uint32_t l3Length:4;               /* RX Only - 19:16 - Layer3 header length (in units of 32bits). Note: Valid for L3=IPv4 and IPv6. */
  uint32_t l3Type:2;                 /* RX Only - 21:20 - Layer3 type. 0:IPv4, 1:IPv6, 2:IPX, 3:Other*/
  uint32_t reserved2:1;              /* Unused  -    22 - Reserved. */
  uint32_t broadcastDest:1;          /* RX Only -    23 - Destination MAC address is broadcast. (DMAC are all 1's) */
  uint32_t mplsCount:3;              /* RX Only - 26:24 - Number of MPLS shim labels present. */
  uint32_t islPresent:1;             /* RX Only -    27 - ISL encapsulation present. */
  uint32_t vlanCount:2;              /* RX Only - 29:28 - Number of VLANs present. */
  uint32_t l2Type:2;                 /* RX Only - 31:30 - Layer2 type. (0:EtherII, 1:LLC, 2:SNAP, 3:Novell RAW) */
  /* 32bit Word2. */
  uint32_t reserved3:8;              /* Unused  - 07:00 - Reserved. */
#define txTsInjectOffset layer5HeaderOffset
  uint32_t layer5HeaderOffset:8;     /* RX Only - 15:08 - RX: Layer5 offset in packet, offset is from end of extension header.
                                                              Note: Valid when Layer is TCP/UDP and L3 is IPv4, otherwise zero!
                                                          TX: TX timestamp inject offset. If bit1 of word1 is set this is the
                                                              offset where the adapter will inject a timestamp upon TX. */
  uint32_t layer4HeaderOffset:8;     /* RX Only - 23:16 - Layer4 offset in packet, offset is from end of extension header.
                                                          Note: Valid when Layer is TCP/UDP and L3 is IPv4, otherwise zero! */
  uint32_t layer3HeaderOffset:8;     /* RX Only - 31:24 - Layer3 offset in packet, offset is from end of extension header.
                                                          Note: Valid when Layer is IPv4, otherwise zero! */
  /* 32bit Word3: */
  uint32_t color:6;                  /* RX Only - 05:00 - Color */
  uint32_t reserved4:26;             /* Unused  - 31:06 - Reserved */
} PacketExtDescriptorType05_t;

/* Layer 2 types. */
typedef enum
{
  EXT05_L2_ETHER_II=0,
  EXT05_L2_LLC,
  EXT05_L2_SNAP,
  EXT05_L2_NOVELL_RAW
} ext05_Layer2EncapsulationType_t;

/* Layer 3 types. */
typedef enum
{
  EXT05_L3_IPv4=0,
  EXT05_L3_IPv6,
  EXT05_L3_IPX,
  EXT05_L3_OTHER
} ext05_Layer3EncapsulationType_t;

/* Layer 4 types. */
typedef enum
{
  EXT05_L4_TCP=0,
  EXT05_L4_UDP,
  EXT05_L4_ICMP,
  EXT05_OTHER
} ext05_Layer4EncapsulationType_t;

#include "ntpackpop.h"

#endif // __NTAPI_EXTENDEDDESCRIPTOR05_H__
