// $Id: extendeddescriptor06.h 16051 2011-11-30 12:20:27Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This file defines the format (structure definition) of the extended
 * descriptor with a type field of 0x06.
 * This structure definition can be overlaid onto the data that immeadiatly
 * follows the standard descriptor assuming that the user has created a packet
 * feed for which this extended descriptor has been requested.
 */

#ifndef  __NTAPI_EXTENDEDDESCRIPTOR06_H__
#define  __NTAPI_EXTENDEDDESCRIPTOR06_H__

#include "basetype.h"
#include "ntpackpush.h"

#define EXTENDED_DESCRIPTOR_06_KNOWN

/* The following are the values to use with CreatePacketFeed in order to request this
    full extended descriptor*/

/* The following definitions should be used when creating a packet feed with one of the
	'FeedConfigSampledType' structures
  For creating a feed with this structure you need to set the 'descriptorType' (if applicable)
   to EXTENDED_DESCRIPTOR
  Use the following with the 'extendedDescriptorFormat' attribute */
#define EXTENDED_DESCRIPTOR_06_TYPE	0x06

#define EXTENDED_DESCRIPTOR_06_LENGTH 0x02

typedef struct PacketExtDescriptorType06
{
  /* 32bit Word0. */
  uint32_t hash:24;                  /* RX Only - 23:00 - Hash value. */
  uint32_t hashType:5;               /* RX Only - 28:24 - Hash type. */
  uint32_t reserved0:2;              /* Unused  - 30:29 - Reserved. */
  uint32_t hashValid:1;              /* RX Only -    31 - Hash valid. zero if hash-config is None or inbound frame invalid for hash-calc, one otherwise. */
  /* 32bit Word1. */
  uint32_t l3Fragmented:1;           /* RX Only -    00 - L3 fragmented frame. (More-bit=1 || offset!=0) Note: Only valid for IPv4. */
  uint32_t l3FirstFragment:1;        /* RX Only -    01 - L3 First fragment. (More-bit=1 && offset=0) Note: Only valid for IPv4. */
  uint32_t broadcastDest:1;          /* RX Only -    02 - Destination MAC address is broadcast. (DMAC are all 1's) */
  uint32_t l4PortType:4;             /* RX Only - 06:03 - Layer4 port type.  See ext06_Layer4PortType_t*/
  uint32_t l4FrameType:4;            /* RX Only - 10:07 - Layer4 frame type. See ext06_Layer4FrameType_t; */
  uint32_t l3FrameType:3;            /* RX Only - 13:11 - Layer3 type. 0:IPv4, 1:IPv6, 2:IPX, 3:Other*/
  uint32_t l2FrameType:2;            /* RX Only - 15:14 - Layer2 type. (0:EtherII, 1:LLC, 2:SNAP, 3:Novell RAW) */
  uint32_t l4Size:4;                 /* RX Only - 19:16 - Layer4 header length (in units of 32bits) */
  uint32_t l3Size:6;                 /* RX Only - 25:20 - Layer3 header length (in units of 32bits). Note: Valid for L3=IPv4 and IPv6. */
  uint32_t mplsCount:3;              /* RX Only - 28:26 - Number of MPLS shim labels present. */
  uint32_t vlanCount:2;              /* RX Only - 30:29 - Number of VLANs present. */
  uint32_t islPresent:1;             /* RX Only -    31 - ISL encapsulation present. */
  /* 32bit Word2. */
  uint32_t reserved1:23;             /* Unused  - 22:00 - Reserved. */
  uint32_t txTsInject:1;             /* TX Only -    23 - Inject TX timestamp.
                                                          Note: The offset where the timestamp should be injected is located
                                                                at txTsInjectOffset/layer5HeaderOffset. */
  uint32_t reserved2:4;              /* Unused  - 27:24 - Reserved. */
  uint32_t frameProtSmall:1;         /* RX Only -    28 - Frame is "protocol small". */
  uint32_t frameLarge:1;             /* RX Only -    29 - Large frame. 1 when PktSz > MaxSz as configured with CardConfigMaxFrameSize() */
  uint32_t frameSmall:1;             /* RX Only -    30 - Small frame. 1 when PktSz < 64 (+ISL and/or VLAN if set via CardConfigDecoderSet()) */
  uint32_t jumbo:1;                  /* RX Only -    31 - Jumbo frame. 1 when PktSz > 1518 (+ISL, VLAN, MPLS if set via CardConfigDecoderSet()) */
  /* 32bit Word3. */
  uint32_t color:6;                  /* RX Only - 05:00 - Color */
  uint32_t reserved3:2;              /* Unused  - 07:06 - Reserved */
  #define txTsInjectOffset layer5HeaderOffset
  uint32_t layer5HeaderOffset:8;     /* RX & TX - 15:08 - RX: Layer5 offset in packet, offset is from end of extension header.
                                                              Note: Valid when Layer is TCP/UDP and L3 is IPv4, otherwise zero!
                                                          TX: TX timestamp inject offset. If bit1 of word1 is set this is the
                                                              offset where the adapter will inject a timestamp upon TX. */
  uint32_t udptcpHeaderOffset:8;     /* RX Only - 23:16 - Offset to UDP/TCP header. Offset is from end of extension header. */
  uint32_t ipHeaderOffset:8;         /* RX Only - 31:24 - Offset to the IP header. Offset is from end of extension header. */
} PacketExtDescriptorType06_t;

/* Layer 2 types. */
typedef enum
{
  EXT06_L2_ETHER_II=0,
  EXT06_L2_LLC,
  EXT06_L2_SNAP,
  EXT06_L2_NOVELL_RAW
} ext06_Layer2FrameType_t;

/* Layer 3 types. */
typedef enum
{
  EXT06_L3_IPv4=0,
  EXT06_L3_IPv6,
  EXT06_L3_IPX,
  EXT06_L3_OTHER,
} ext06_Layer3FrameType_t;

/* Layer 4 frame types. */
typedef enum
{
  EXT06_L4_TCP=0,
  EXT06_L4_UDP,
  EXT06_L4_ICMP,
  EXT06_L4_OTHER,
  EXT06_L4_GRE,
  EXT06_L4_SCTP,
} ext06_Layer4FrameType_t;

/* Layer 4 port types. */
typedef enum
{
  EXT06_L4_PORT_OTHER=0,
  EXT06_L4_PORT_GTPV0_U,
  EXT06_L4_PORT_GTPV1V2_C,
  EXT06_L4_PORT_GTPV1_U
} ext06_Layer4PortType_t;

#include "ntpackpop.h"

#endif // __NTAPI_EXTENDEDDESCRIPTOR06_H__
