// $Id: extendeddescriptor07.h 16051 2011-11-30 12:20:27Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This file defines the format (structure definition) of the extended
 * descriptor with a type field of 0x07.
 * This structure definition can be overlaid onto the data that immeadiatly
 * follows the standard descriptor assuming that the user has created a packet
 * feed for which this extended descriptor has been requested.
 */

#ifndef  __NTAPI_EXTENDEDDESCRIPTOR07_H__
#define  __NTAPI_EXTENDEDDESCRIPTOR07_H__

#include "basetype.h"
#include "ntpackpush.h"

#define EXTENDED_DESCRIPTOR_07_KNOWN

/* The following are the values to use with CreatePacketFeed in order to request this
    full extended descriptor*/

/* The following definitions should be used when creating a packet feed with one of the
	'FeedConfigSampledType' structures
  For creating a feed with this structure you need to set the 'descriptorType' (if applicable)
   to EXTENDED_DESCRIPTOR
  Use the following with the 'extendedDescriptorFormat' attribute */
#define EXTENDED_DESCRIPTOR_07_TYPE	0x07

#define EXTENDED_DESCRIPTOR_07_LENGTH 0x02

typedef struct PacketExtDescriptorType07
{
  /* 32bit Word0. */
  uint32_t hash:24;                  /* RX Only - 23:00 - Hash value. */
  uint32_t hashType:5;               /* RX Only - 28:24 - Hash type. */
  uint32_t reserved0:2;              /* Unused  - 30:29 - Reserved. */
  uint32_t hashValid:1;              /* RX Only -    31 - Hash valid. zero if hash-config is None or inbound frame invalid for hash-calc, one otherwise. */
  /* 32bit Word1. */
  uint32_t jumbo:1;                  /* RX Only -    00 - Jumbo frame. 1 when PktSz > 1518 (+ISL, VLAN, MPLS if set via CardConfigDecoderSet()) */
  uint32_t broadcastDest:1;          /* RX Only -    01 - Destination MAC address is broadcast. (DMAC are all 1's) */
  uint32_t l4PortType:4;             /* RX Only - 05:02 - Layer4 port type.  See ext07_Layer4PortType_t*/
  uint32_t l4FrameType:4;            /* RX & TX - 09:06 - Layer4 frame type. See ext07_Layer4FrameType_t; */
  uint32_t l3FrameType:3;            /* RX & TX - 12:10 - Layer3 type. 0:IPv4, 1:IPv6, 2:IPX, 3:Other*/
  uint32_t l2FrameType:2;            /* RX & TX - 14:13 - Layer2 type. (0:EtherII, 1:LLC, 2:SNAP, 3:Novell RAW) */
  uint32_t l4Size:4;                 /* RX & TX - 18:15 - Layer4 header length (in units of 32bits) */
  uint32_t l3Size:7;                 /* RX & TX - 25:19 - Layer3 header length (in units of 32bits). Note: Valid for L3=IPv4 and IPv6. */
  uint32_t mplsCount:3;              /* RX & TX - 28:26 - Number of MPLS shim labels present. */
  uint32_t vlanCount:2;              /* RX & TX - 30:29 - Number of VLANs present. */
  uint32_t islPresent:1;             /* RX & TX -    31 - ISL encapsulation present. */
  /* 32bit Word2. */
  uint32_t reserved1:14;             /* Unused  - 13:00 - Reserved */
  uint32_t txTsInject:1;             /* TX Only -    14 - Inject TX timestamp.
                                                          Note: The offset where the timestamp should be injected is located
                                                                at txTsInjectOffset/layer5HeaderOffset. */
  uint32_t udptcpChecksumOverride:1; /* TX Only -    15 - This bit requests that the Tx function overrides (recalculates) the UDP/TCP checksum (NT v41+ FPGA only) */
  uint32_t ipChecksumOverride:1;     /* TX Only -    16 - This bit requests that the Tx function overrides (recalculates) the IP checksum      (NT v41+ FPGA only) */
  uint32_t frameProtSmall:1;         /* RX Only -    17 - Frame is "protocol small". */
  uint32_t frameLarge:1;             /* RX Only -    18 - Large frame. 1 when PktSz > MaxSz as configured with CardConfigMaxFrameSize() */
  uint32_t frameSmall:1;             /* RX Only -    19 - Small frame. 1 when PktSz < 64 (+ISL and/or VLAN if set via CardConfigDecoderSet()) */
  uint32_t ipv6FragmentHeader:1;     /* RX Only -    20 - IPv6 Fragment header present */
  uint32_t ipv6RoutingHeader:1;      /* RX Only -    21 - IPv6 Routing header present */
  uint32_t l4ProtocolNumber:8;       /* RX Only - 29:22 - Layer 4 Protocol Number (TCP:, UDP, SCTP etc.)*/
  uint32_t l3Fragmented:1;           /* RX Only -    30 - L3 fragmented frame. (More-bit=1 || offset!=0) Note: Only valid for IPv4. */
  uint32_t l3FirstFragment:1;        /* RX Only -    31 - L3 First fragment. (offset=0) Note: Only valid for IPv4. This bit is always set on IPv6. IPV6 use the ipv6FragmentHeader. */
  /* 32bit Word3. */
  uint32_t color:6;                  /* RX Only - 05:00 - Color */
  uint32_t reserved3:1;              /* Unused  -    06 - Reserved */
  #define txTsInjectOffset layer5HeaderOffset
  uint32_t layer5HeaderOffset:9;     /* TX & RX - 15:07 - RX: Layer5 offset in packet, offset is from end of extension header.
                                                              Note: Valid when Layer is TCP/UDP and L3 is IPv4 or IPv6, otherwise zero!
                                                          TX: TX timestamp inject offset. If txTsInject is set this is the
                                                              offset where the adapter will inject a timestamp upon TX. */
  uint32_t udptcpHeaderOffset:9;     /* RX Only - 24:16 - Offset to the layer4 header. Offset is from end of extension header. */
  uint32_t ipHeaderOffset:7;         /* RX Only - 31:25 - Offset to the layer3 header. Offset is from end of extension header. */
} PacketExtDescriptorType07_t;

/* Layer 2 types. */
typedef enum
{
  EXT07_L2_ETHER_II=0,
  EXT07_L2_LLC,
  EXT07_L2_SNAP,
  EXT07_L2_NOVELL_RAW
} ext07_Layer2FrameType_t;

/* Layer 3 types. */
typedef enum
{
  EXT07_L3_IPv4=0,
  EXT07_L3_IPv6,
  EXT07_L3_IPX,
  EXT07_L3_OTHER,
} ext07_Layer3FrameType_t;

/* Layer 4 frame types. */
typedef enum
{
  EXT07_L4_TCP=0,
  EXT07_L4_UDP,
  EXT07_L4_ICMP,
  EXT07_L4_OTHER,
  EXT07_L4_GRE,
  EXT07_L4_SCTP,
} ext07_Layer4FrameType_t;

/* Layer 4 port types. */
typedef enum
{
  EXT07_L4_PORT_OTHER=0,
  EXT07_L4_PORT_GTPV0_U,
  EXT07_L4_PORT_GTPV1V2_C,
  EXT07_L4_PORT_GTPV1_U
} ext07_Layer4PortType_t;

#include "ntpackpop.h"

#endif // __NTAPI_EXTENDEDDESCRIPTOR07_H__
