// $Id: hwmonstructure.h 16051 2011-11-30 12:20:27Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Structure Definitions that are used by the external interface for
 * card Hardware Monitoring information
 */

#ifndef __NTAPI_HWMONSTRUCTURE_H__
#define __NTAPI_HWMONSTRUCTURE_H__

#include "basetype.h"
#include "ntpackpush.h"

enum
{
  eHardwareMonitorType1,
  eHardwareMonitorType2,  /* NT series only. */
  eHardwareMonitorType3,  /* NT series only. */
  eHardwareMonitorType99  /* NT series only. */
};

/* Sensor types. These types will report in:
 * SENSOR_TYPE_TEMPERATURE in 1/10th of a degree Celsius.
 * SENSOR_TYPE_VOLTAGE in 1/100th of a Volt.
 * SENSOR_TYPE_CURRENT in 1/100th of a Amapere
 * SENSOR_TYPE_FANSPEED in Revolutions Per Minute.
 *
 * These are also defined in hal/common/hardwaremonitor.h
 */
enum sensorType {
    TEMPERATURE,
    VOLTAGE,
    CURRENT,
    POWER,
    FANSPEED
};

typedef struct _HardwareMonitorType1
{
    uint8_t		StructureVersion;
    uint32_t  SensorType;
    int32_t   lowAlarmLimit;
    int32_t   highAlarmLimit;
    int32_t   SensorMeasurement;
    bool_t    SensorAlarm;
} HardwareMonitorType1_t;

#define STID_HARDWARE_MONITOR_TYPE1 \
    MAKE_STRUCTURE_ID(eHardwareMonitorType1, HardwareMonitorType1_t)

typedef struct _HardwareMonitorType2
{
  uint8_t		StructureVersion;   /* StructureVersion.(1)    */
  uint32_t  SensorType;         /* Sensor type.            */
  int32_t   lowAlarmLimit;      /* Low limit.              */
  int32_t   highAlarmLimit;     /* High limit.             */
  int32_t   sensorMeasurement;  /* Current value.          */
  bool_t    SensorAlarm;        /* Alarm status.           */
  int32_t   lowestValue;        /* Lowest value measured.  */
  int32_t   highestValue;       /* Highest value measured. */  
  uint8_t   SensorName[20];     /* Sensor name.            */
} HardwareMonitorType2_t;

#define STID_HARDWARE_MONITOR_TYPE2 \
    MAKE_STRUCTURE_ID(eHardwareMonitorType2, HardwareMonitorType2_t)

typedef struct _HardwareMonitorType3
{
  uint8_t		StructureVersion;   /* StructureVersion.(1)    */
  uint32_t  SensorType;         /* Sensor type.            */
  int32_t   reserved1;          /* Low limit.              */
  int32_t   reserved2;          /* High limit.             */
  int32_t   sensorMeasurement;  /* Current value.          */
  bool_t    boReserved;         /* Alarm status.           */
  int32_t   reserved3;          /* Lowest value measured.  */
  int32_t   reserved4;          /* Highest value measured. */  
  uint8_t   SensorName[20];     /* Sensor name.            */
} HardwareMonitorType3_t;

#define STID_HARDWARE_MONITOR_TYPE3 \
    MAKE_STRUCTURE_ID(eHardwareMonitorType3, HardwareMonitorType3_t)

typedef struct _HardwareMonitorType99
{
  uint8_t		StructureVersion;   /* StructureVersion.(1)    */
  uint32_t  SensorType;         /* Sensor type.            */
  int32_t   lowAlarmLimit;      /* Low limit.              */
  int32_t   highAlarmLimit;     /* High limit.             */
  int32_t   sensorMeasurement;  /* Current value.          */
  bool_t    SensorAlarm;        /* Alarm status.           */
  int32_t   lowestValue;        /* Lowest value measured.  */
  int32_t   highestValue;       /* Highest value measured. */  
  uint8_t   SensorName[20];     /* Sensor name.            */
} HardwareMonitorType99_t;

#define STID_HARDWARE_MONITOR_TYPE99 \
    MAKE_STRUCTURE_ID(eHardwareMonitorType99, HardwareMonitorType99_t)

#include "ntpackpop.h"

#endif // __NTAPI_HWMONSTRUCTURE_H__
