// $Id: packetclassificationinterface.h 14870 2011-10-13 07:23:23Z bk $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * This file contains the common types that can be used by any interface.
 */

#ifndef __PACKETCLASSIFICATIONINTERFACE_H__
#define __PACKETCLASSIFICATIONINTERFACE_H__

#include "ostype.h"
#include "basetype.h"

#define PACKET_CLASSIFICATION_INTERFACE_VERSION        2
#define PACKET_CLASSIFICATION_INTERFACE_VERSION_V3     3

typedef struct packetClassificationInterface
{
    uint16_t size;
    uint16_t version;
    void * context;

    NT_STATUS (*CreateExpression)(void * context,
                                      uint32_t *expressionId,
                                      StructureID componentType,
                                      void *component,
                                      uint8_t unaryFunction);

    NT_STATUS (*CombineExpression)(void * context,
                                      uint32_t expressionId,
                                      StructureID componentType,
                                      void *component,
                                      uint8_t unaryFunction,
                                      uint8_t binaryFunction);

    NT_STATUS (*DestroyExpression)(void * context,
                                      uint32_t expressionId);


    NT_STATUS (*CreateFunction)(void * context,
                                    uint32_t *functionId,
                                    uint32_t expressionId,
                                    StructureID actionType,
                                    void *action);

    NT_STATUS (*CombineFunction)(void * context,
                                      uint32_t functionId,
                                      StructureID actionType,
                                      void *action);

    NT_STATUS (*DestroyFunction)(void * context,
                                      uint32_t functionId);

    NT_STATUS (*AssignFunctionResources)(void * context,
                                             uint32_t functionId,
                                             uint32_t *classificationTag);

    NT_STATUS (*ReleaseFunctionResources)(void * context,
                                             uint32_t functionId);

    NT_STATUS (*EnableFunction)(void * context,
                                    uint32_t functionId,
                                    bool_t lastOperation);

    NT_STATUS (*DisableFunction)(void * context,
                                    uint32_t functionId,
                                    bool_t lastOperation);
    NT_STATUS (*AddSignatureMatchEntry)(void* context,
										 StructureID entryType,
										 void *entry,
										 StructureID entryActionType,
										 void *entryAction,
										 uint32_t *entryIdentifier,
										 bool_t writeNow);
	
    NT_STATUS (*RemoveSignatureMatchEntry)(void* context,
											uint32_t entryIdentifier,
											bool_t writeNow);

    NT_STATUS (*DestroyAllClassificationEntries)(void* context);

} PACKET_CLASSIFICATION_INTERFACE;

typedef struct packetClassificationInterface_v3
{
    uint16_t size;
    uint16_t version;
    void * context;

    NT_STATUS (*PacketClassification)(void *      context,
                                       StructureID structId,
                                       void *      pClassificationData);                              
} PACKET_CLASSIFICATION_INTERFACE_V3;  

#endif // __PACKETCLASSIFICATIONINTERFACE_H__
