// $Id: packetclassificationstructure.h 16051 2011-11-30 12:20:27Z ps $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Packet classification interface structure definitions
 */

#ifndef __NTAPI_PACKETCLASSIFICATIONSTRUCTURE_H__
#define __NTAPI_PACKETCLASSIFICATIONSTRUCTURE_H__

#include "basetype.h"
#include "commontypes.h"
#include "ntpackpush.h"
#include "packetfeedstructure.h"

#define UNDEFINED_TAG_ID           0xffffffff

#define MAC_ADDRESS_LENGTH                6
#define IPV4_ADDRESS_LENGTH               4
#define IPV6_ADDRESS_LENGTH               16
#define MAX_PATTERN_LENGTH                64
#define MAX_STRING_LENGTH                 128
#define MAX_DYNAMIC_OFFSETS               64
#define MAX_POSITION_FREE_PATTERN_LENGTH  16
#define MAX_SEARCH_INDEX                  8
#define MAX_COMMANDLINE_LEN               4096

enum UnaryFunction
{
  PCLASS_NONE,
  PCLASS_NOT
};

enum BinaryFunction
{
  PCLASS_AND,
  PCLASS_OR,
};

/* enums for structure type. enum is also the bit number used in
 * capabilities structure filters supported bitmask.
 */
enum {
  ePacketClassificationMacDestinationAddress,
  ePacketClassificationMacSourceAddress,
  ePacketClassificationIPv4DestinationAddress,
  ePacketClassificationIPv4SourceAddress,
  ePacketClassificationIPv6DestinationAddress,
  ePacketClassificationIPv6SourceAddress,
  ePacketClassificationDestinationPort,
  ePacketClassificationSourcePort,
  ePacketClassificationGeneralPattern,
  ePacketClassificationError,
  ePacketClassificationVLANTag,
  ePacketClassificationProtocol,
  ePacketClassificationSize,
  ePacketClassificationPositionIndependentPattern,
  ePacketClassificationSubExpression,
  ePacketClassificationSignature,
  ePacketClassificationIPv4Flow,
  ePacketClassificationIPv6Flow,
  
  ePacketClassificationActionChannelsInclude = 64,
  ePacketClassificationActionChannelsExclude,
  ePacketClassificationActionChannelsSlice,
  ePacketClassificationActionChannelsStartTriggerInclude ,
  ePacketClassificationActionChannelsStartTriggerExclude,
  ePacketClassificationActionChannelsStopTriggerInclude,
  ePacketClassificationActionChannelsStopTriggerExclude,
  ePacketClassificationActionChannelsDescriptorTag,
  ePacketClassificationActionPacketfeedInclude,
  ePacketClassificationActionPacketfeedExclude,
  ePacketClassificationActionPacketfeedSlice,
  ePacketClassificationActionPacketfeedStartTriggerInclude,
  ePacketClassificationActionPacketfeedStartTriggerExclude,
  ePacketClassificationActionPacketfeedStopTriggerInclude,
  ePacketClassificationActionPacketfeedStopTriggerExclude,
  ePacketClassificationActionPacketfeedDescriptorTag,
  ePacketClassificationActionChannelsRetransmitInclude,
  ePacketClassificationActionChannelsRetransmitExclude,
  ePacketClassificationActionPacketfeedRetransmitInclude,
  ePacketClassificationActionPacketfeedRetransmitExclude,
  ePacketClassificationActionFunction,
  ePacketClassificationActionChannelsCombined,
  ePacketClassificationPassFilter,
  ePacketClassificationGetFilterError,
  ePacketClassificationPassFilterType1,
};

/*
 * Match Types
 */
typedef struct MacAddressMatch_
{
    uint8_t Address[MAC_ADDRESS_LENGTH];
    uint8_t AddressMask[MAC_ADDRESS_LENGTH];
} MacAddressMatch_t;

#define STID_DESTINATION_MAC_ADDRESS_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationMacDestinationAddress, MacAddressMatch_t)

#define STID_SOURCE_MAC_ADDRESS_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationMacSourceAddress, MacAddressMatch_t)

typedef struct IPv4AddressMatch_
{
    /* Both address and mask should be specified in network byte order */
    uint8_t Address[IPV4_ADDRESS_LENGTH];
    uint8_t AddressMask[IPV4_ADDRESS_LENGTH];
} IPv4AddressMatch_t;

#define STID_DESTINATION_IPV4_ADDRESS_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationIPv4DestinationAddress, IPv4AddressMatch_t)

#define STID_SOURCE_IPV4_ADDRESS_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationIPv4SourceAddress, IPv4AddressMatch_t)

typedef struct IPv6AddressMatch_
{
    /* Both address and mask should be specified in network byte order */
    uint8_t Address[IPV6_ADDRESS_LENGTH];
    uint8_t AddressMask[IPV6_ADDRESS_LENGTH];
} IPv6AddressMatch_t;

#define STID_DESTINATION_IPV6_ADDRESS_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationIPv6DestinationAddress, IPv6AddressMatch_t)

#define STID_SOURCE_IPV6_ADDRESS_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationIPv6SourceAddress, IPv6AddressMatch_t)

enum PortL3Protocol
{
  PCLASS_PORTL3_IPV4,
  PCLASS_PORTL3_IPV6,
  PCLASS_PORTL3_BOTH
};

enum PortL4Protocol
{
  PCLASS_PORTL4_TCP,
  PCLASS_PORTL4_UDP,
  PCLASS_PORTL4_BOTH
};

typedef struct PortMatch_
{
  /* Port is in host byte order. The driver will convert it to network byte order */
  uint16_t Port;
  uint8_t Layer3Protocol;
  uint8_t Layer4Protocol;
} PortMatch_t;

#define STID_DESTINATION_PORT_ADDRESS_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationDestinationPort, PortMatch_t)

#define STID_SOURCE_PORT_ADDRESS_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationSourcePort, PortMatch_t)

enum PClassDynamicOffset {
  PCLASS_OFFSET_FRAME_START          = 0x00,
  PCLASS_OFFSET_LAYER2_FRAME         = 0x01,
  PCLASS_OFFSET_LAYER2_HEADER        = 0x02,
  PCLASS_OFFSET_LAYER2_DATA          = 0x03,
  PCLASS_OFFSET_ETHERTYPE_LENGTH     = 0x04,
  PCLASS_OFFSET_ETHERTYPE            = 0x05,
  PCLASS_OFFSET_IPV4_FRAME           = 0x08,
  PCLASS_OFFSET_IPV4_HEADER          = 0X09,
  PCLASS_OFFSET_IPV4_DATA            = 0X0a,
  PCLASS_OFFSET_IPV6_FRAME           = 0X10,
  PCLASS_OFFSET_IPV6_HEADER          = 0X11,
  PCLASS_OFFSET_IPV6_DATA            = 0X12,
  PCLASS_OFFSET_IPX_FRAME            = 0X18,
  PCLASS_OFFSET_TCP_FRAME            = 0X20,
  PCLASS_OFFSET_TCP_HEADER           = 0X21,
  PCLASS_OFFSET_TCP_DATA             = 0X22,
  PCLASS_OFFSET_UDP_FRAME            = 0X28,
  PCLASS_OFFSET_UDP_HEADER           = 0X29,
  PCLASS_OFFSET_UDP_DATA             = 0X2a,
  PCLASS_OFFSET_ICMP_FRAME           = 0X30
};

typedef struct GeneralPatternMatch_
{
  uint32_t FixedOffset;                 /* Always supported */
  uint32_t PatternLength;
  uint8_t DynamicOffset;                /* One of the dynamic offsets from the
                                         * supported dynamic offsets array in the
                                         * capabilities structure. Not all cards
                                         * will support this. */
  uint8_t Pattern[MAX_PATTERN_LENGTH];  /* Both pattern and mask are in the order the bytes appear in the frame.
                                         * The driver will do the necessary alignment and byte orderadjustment
                                         * before programming the filter. */
  uint8_t Mask[MAX_PATTERN_LENGTH];
} GeneralPatternMatch_t;

#define STID_GENERAL_PATTERN_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationGeneralPattern, GeneralPatternMatch_t)

enum Protocols
{
  PCLASS_L2_ETHERNET_II,
  PCLASS_L2_LLC,
  PCLASS_L2_SNAP,
  PCLASS_L2_RAW,
  PCLASS_L2_OTHER,
  
  PCLASS_L3_IPV6,
  PCLASS_L3_IPV4,
  PCLASS_L3_IPX,
  PCLASS_L3_IPV4_IPPROTO,
  PCLASS_L3_IPV6_NEXT_HEADER,
  PCLASS_L3_NOT_RECOGNISED,
  
  PCLASS_L4_TCP,
  PCLASS_L4_UDP,
  PCLASS_L4_ICMP,                 /* IPv4 or IPv6 */
  PCLASS_L4_IPV4_ICMP,
  PCLASS_L4_IPV6_ICMP,
  PCLASS_L4_NOT_RECOGNISED,
  
  PCLASS_OTHER_ISL,
  PCLASS_OTHER_VLAN,
  PCLASS_OTHER_JUMBO,
  PCLASS_OTHER_BROADCAST,
  PCLASS_OTHER_MULTICAST,
  PCLASS_OTHER_MPLS,
};

typedef struct ProtocolMatch_
{
  uint8_t Protocol;
  uint32_t ProtocolData; /* for PCLASS_L2_OTHER this is the value to match in
                          * the Ethernet type part of the layer2 frame
                          * for PCLASS_L3_IPPROTO this is the IP Protocol */
} ProtocolMatch_t;

#define STID_PROTOCOL_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationProtocol, ProtocolMatch_t)

enum Errors
{
  PCLASS_ERROR_CRC,
  PCLASS_ERROR_RUNT,
  PCLASS_ERROR_FRAGMENT,
  PCLASS_ERROR_JABBER,
  PCLASS_ERROR_OVERRUN,
  PCLASS_ERROR_IPCHECKSUM,  /* Bad IP checksum */
  PCLASS_ERROR_UDPCHECKSUM, /* Bad UDP checksum */
  PCLASS_ERROR_TCPCHECKSUM, /* Bad TCP checksum */
};

typedef struct ErrorMatch_
{
  uint8_t ErrorType;
} ErrorMatch_t;

#define STID_ERROR_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationError, ErrorMatch_t)

enum SizeComparisons
{
    PCLASS_SIZE_GREATER_THAN,
    PCLASS_SIZE_LESS_THAN,
    PCLASS_SIZE_EQUALS,
};

typedef struct SizeMatch_
{
    uint16_t frameSize;
    uint8_t comparisonType;
} SizeMatch_t;

#define STID_SIZE_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationSize, SizeMatch_t)

typedef struct VLANTagMatch_
{
    uint16_t VLANTag;
} VLANTagMatch_t;

#define STID_VLAN_TAG_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationVLANTag, VLANTagMatch_t)

enum PositionIndependentProtocolType
{
    PCLASS_PI_UDP,
    PCLASS_PI_TCP,
    PCLASS_PI_BOTH
};

typedef struct PositionIndependentPatternMatch_
{
    /* matches a general pattern any where in the TCP or UDP data of a frame.
     * Search may also be restricted to a subset of TCP or UDP destination ports. */
    uint8_t Pattern[MAX_POSITION_FREE_PATTERN_LENGTH];
    uint8_t Mask[MAX_POSITION_FREE_PATTERN_LENGTH];
    uint32_t Length;
    uint16_t DestinationPort;
    uint16_t DestinationPortMask;
    uint8_t ProtocolType;
} PositionIndependentPatternMatch_t;

#define STID_POSITION_INDEPENDENT_PATTERN_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationPositionIndependentPattern, PositionIndependentPatternMatch_t)

/* Allows modification of order of boolean evaluation. */
typedef struct SubExpression_
{
    uint32_t ExpressionId;
} SubExpression_t;

#define STID_SUB_EXPRESSION \
    MAKE_STRUCTURE_ID(ePacketClassificationSubExpression, SubExpression_t)


/* Allows a signature match entries to be made part of an expression.
 * Signature ID is a user defined ID which is used in the action type of
 * of the match entry along with the function identifier so that the signature
 * match entry is associated with the correct expression component */
typedef struct SignatureMatch_
{
    uint8_t SignatureId;
} SignatureMatch_t;

#define STID_SIGNATURE_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationSignature, SignatureMatch_t)

/*
 * Flow matches. Allow the user to describe a TCP or UDP connection in
 * terms of source and destination address, port number and protocol type
 */
enum FlowProtocolType
{
    PCLASS_FLOW_UDP,
    PCLASS_FLOW_TCP,
};

typedef struct IPv4FlowMatch_ {
    IPv4AddressMatch_t SourceAddress;
    IPv4AddressMatch_t DestinationAddress;
    uint16_t SourcePort;
    uint16_t SourcePortMask;
    uint16_t DestinationPort;
    uint16_t DestinationPortMask;
    uint8_t Protocol;
} IPv4FlowMatch_t;

#define STID_IPV4_FLOW_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationIPv4Flow, IPv4FlowMatch_t)

typedef struct IPv6FlowMatch_ {
    IPv6AddressMatch_t SourceAddress;
    IPv6AddressMatch_t DestinationAddress;
    uint16_t SourcePort;
    uint16_t SourcePortMask;
    uint16_t DestinationPort;
    uint16_t DestinationPortMask;
    uint8_t Protocol;
} IPv6FlowMatch_t;

#define STID_IPV6_FLOW_MATCH \
    MAKE_STRUCTURE_ID(ePacketClassificationIPv6Flow, IPv6FlowMatch_t)

/*
 * Classification Actions
 */
typedef struct ChannelsAction_
{
    uint32_t ChannelBitmask;
} ActionChannels_t;

#define STID_ACTION_CHANNELS_INCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionChannelsInclude, ActionChannels_t)
#define STID_ACTION_CHANNELS_EXCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionChannelsExclude, ActionChannels_t)

#define STID_ACTION_CHANNELS_TAG \
    MAKE_STRUCTURE_ID(ePacketClassificationActionChannelsDescriptorTag, ActionChannels_t)

#define STID_ACTION_CHANNELS_START_TRIGGER_INCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionChannelsStartTriggerInclude, ActionChannels_t)
#define STID_ACTION_CHANNELS_START_TRIGGER_EXCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionChannelsStartTriggerExclude, ActionChannels_t)

#define STID_ACTION_CHANNELS_STOP_TRIGGER_INCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionChannelsStopTriggerInclude, ActionChannels_t)
#define STID_ACTION_CHANNELS_STOP_TRIGGER_EXCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionChannelsStopTriggerExclude, ActionChannels_t)

#define STID_ACTION_CHANNELS_RETRANSMIT_INCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionChannelsRetransmitInclude, ActionChannels_t)
#define STID_ACTION_CHANNELS_RETRANSMIT_EXCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionChannelsRetransmitExclude, ActionChannels_t)

enum SlicePriority {
    PCLASS_SLICE_LOWEST_PRIORITY   = 0xffff,
    PCLASS_SLICE_HIGHEST_PRIORITY  = 0,
};

typedef struct ChannelsSliceAction_
{
    uint32_t ChannelBitmask;
    uint16_t SliceLength;
    uint16_t Priority;
} ActionChannelsSlice_t;

#define STID_ACTION_CHANNELS_SLICE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionChannelsSlice, ActionChannelsSlice_t)

typedef struct PacketfeedAction_
{
    FeedHandle feed;
} ActionPacketfeed_t;

#define STID_ACTION_PACKETFEED_INCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionPacketfeedInclude, ActionPacketfeed_t)
#define STID_ACTION_PACKETFEED_EXCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionPacketfeedExclude, ActionPacketfeed_t)

#define STID_ACTION_PACKETFEED_TAG \
    MAKE_STRUCTURE_ID(ePacketClassificationActionPacketfeedDescriptorTag, ActionPacketfeed_t)


#define STID_ACTION_PACKETFEED_START_TRIGGER_INCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionPacketfeedStartTriggerInclude, ActionPacketfeed_t)
#define STID_ACTION_PACKETFEED_START_TRIGGER_EXCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionPacketfeedStartTriggerExclude, ActionPacketfeed_t)

#define STID_ACTION_PACKETFEED_STOP_TRIGGER_INCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionPacketfeedStopTriggerInclude, ActionPacketfeed_t)
#define STID_ACTION_PACKETFEED_STOP_TRIGGER_EXCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionPacketfeedStopTriggerExclude, ActionPacketfeed_t)

#define STID_ACTION_PACKETFEED_RETRANSMIT_INCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionPacketfeedRetransmitInclude, ActionPacketfeed_t)
#define STID_ACTION_PACKETFEED_RETRANSMIT_EXCLUDE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionPacketfeedRetransmitExclude, ActionPacketfeed_t)

typedef struct PacketfeedSliceAction_
{
    FeedHandle feed;
    uint16_t SliceLength;
    uint16_t Priority;
} ActionPacketfeedSlice_t;

#define STID_ACTION_PACKETFEED_SLICE \
    MAKE_STRUCTURE_ID(ePacketClassificationActionPacketfeedSlice, ActionPacketfeedSlice_t)

typedef struct ActionFunction_
{
    uint32_t FunctionId; /* Must correspond to an enabled function containing
                          * a signatureMatch component somewhere in the
                          * expression */
    uint8_t SignatureId; /* This must also match the id specified in the
                          * expression component */
} ActionFunction_t;

#define STID_ACTION_FUNCTION \
    MAKE_STRUCTURE_ID(ePacketClassificationActionFunction, ActionFunction_t)


typedef struct ActionChannelsCombined_
{
    uint32_t FrameIncludeChannelBitmask;
    uint32_t FrameExcludeChannelBitmask;
    uint32_t RetransmitIncludeChannelBitmask;
    uint32_t RetransmitExcludeChannelBitmask;
} ActionChannelsCombined_t;

#define STID_ACTION_CHANNELS_COMBINED \
    MAKE_STRUCTURE_ID(ePacketClassificationActionChannelsCombined, ActionChannelsCombined_t)

typedef enum {
  eShadowPatternCompare,
  eShadowPatternOffset,
  eShadowPatternEndcheck,        
  eShadowSize,
  eShadowProtocol,
  eShadowError,
  eShadowRetransmitInclude,      /* Retired. Do not use */
  eShadowRetransmitExclude,      /* Retired. Do not use */
  eShadowRetransmitConfig,       /* Retired. Do not use */
  eShadowSearchLUT,              /* Retired. Do not use */
  eShadowSearch,                 /* Retired. Do not use */
  eShadowGroup4Plus,             
  eShadowGroup8,                 
  eShadowGroup,                  
  eShadowInclude,
  eShadowExclude,
  eShadowSlice,                  
  eShadowSliceLookup,            /* Retired. Do not use */
  eShadowSliceLength,            /* Retired. Do not use */
  eShadowDynoffset,
  eShadowFeedLUT,                 
  eShadowPriority,                
  eShadowDeduplication,           
  eShadowPortLUT,                 
  eShadowRetransmitPriority,      
  eShadowHashmode,                
  eShadowHashmask,                
  eShadowHashSplitNum,            
} ShadowSection_t;

typedef struct FilterParseSearchIndexStructure_
{
  uint16_t                            NumIndex;
  uint16_t                            Index[MAX_SEARCH_INDEX];
} FilterParseSearchIndexStructure_t;

typedef struct FilterParseIndexStructure_
{
  uint32_t                            FilterId;
  union {
    uint64_t                          GroupIndex;
    FilterParseSearchIndexStructure_t SearchIndex;
  } u;
} FilterParseIndexStructure_t;

typedef struct FilterParseFilterCounters_
{
    uint8_t SizeCount;
    uint8_t ProtocolCount;
    uint8_t ErrorCount;
    uint8_t PatternCount;
    uint8_t DynOffsetCount;
    uint8_t SearchCount;
    uint8_t Group4PlusCount;
    uint8_t Group8Count;
    uint8_t SliceLookupCount;
    uint8_t SliceLengthCount;
} FilterParseFilterCounters_t;

typedef struct FilterParseReturnStructure_
{
  uint32_t FilterReturnDataType;
    #define FILTER_RETURN_NONE                0
    #define FILTER_RETURN_GROUPID             1
    #define FILTER_RETURN_SEARCHID            2
    #define FILTER_RETURN_INFO                3
    #define FILTER_RETURN_FILTERID            4
  union {
    FilterParseIndexStructure_t       FilterIndex;
    FilterParseFilterCounters_t       FilterInfo;
  }u;
} FilterParseReturnStructure_t;

typedef struct PassFilter_
{
    FilterParseReturnStructure_t ReturnData;
    char achFilterString[MAX_COMMANDLINE_LEN];
    bool_t WriteHW;                               /* Set to one to submit command to Hardware. To stack commands, set all but last command to zero. */
} PassFilter_t;

#define STID_PASS_FILTER \
    MAKE_STRUCTURE_ID(ePacketClassificationPassFilter, PassFilter_t)

typedef struct FilterError_
{
    char achFilterError1[MAX_COMMANDLINE_LEN];
    char achFilterError2[MAX_COMMANDLINE_LEN];
    char achFilterError3[MAX_COMMANDLINE_LEN];
} FilterError_t;

#define STID_GET_FILTER_ERROR \
    MAKE_STRUCTURE_ID(ePacketClassificationGetFilterError, FilterError_t)

#include "ntpackpop.h"

#endif // __NTAPI_PACKETCLASSIFICATIONSTRUCTURE_H__
