// $Id: statisticsstructure.h 25153 2013-08-30 11:14:55Z ph $
/*
 * Copyright 2013 Napatech A/S. All Rights Reserved.
 * 
 *   The source code contained or described herein and all documents
 *   related to the source code ("Material") are owned by Napatech
 *   A/S or its suppliers or licensors.  Title to the Material
 *   remains with Napatech Inc. or its suppliers and licensors. The
 *   software is licensed under the Napatech A/S SCLA License.
 * 
 *   Unless otherwise agreed by Napatech A/S in writing, you may
 *   not remove or alter this notice or any other notice embedded
 *   in Materials by Napatech A/S or Napatech's suppliers or
 *   licensors in any way.
 */

/**
 * @file
 * Structure Definitions that are used by the external interface
 */

#ifndef __NTAPI_STATISTICSSTRUCTURE_H__
#define __NTAPI_STATISTICSSTRUCTURE_H__

#include "ostype.h"
#include "basetype.h"
#include "commontypes.h"
#include "packetfeedstatsstructure.h"
#include "memoryaccess.h"
#include "ntpackpush.h"

#ifndef _NT_FEED_HANDLE
#define _NT_FEED_HANDLE
typedef uint32_t FeedHandle;
#endif

/* The following enum block relates to the structure ID groups */
enum
{
  eStatisticsOptionsType1,  /* Used for S-Style SetStatisticsOptions (legacy) do not use for new code implementations */
  eStatisticsOptionsType2,
  eStatisticsOptionsType3,
  eStatisticsEtherType1,      /* Use for S-Style statistics structure block, also supported in X */
  eStatisticsEtherType2,      /* Use for X-Style statistics structure block - this structure should only be used in frame decoder mode 0 */
  eStatisticsEtherGB10Type1,  /* GB10S only */
  eStatisticsEther10GigType1, /* 10Gig (GB10000A) */
  eStatisticsOptionsType4,
  eStatisticsAdvEtherType1,   /* Use for X-Style statistics structure block - this structure should only be used in frame decoder mode 0 */
  eStatisticsEtherType3,      /* Use for X-Style statistics structure block - this structure should only be used in frame decoder mode 1 */
  eStatisticsEtherType4,      /* Used in "legacy" decoder mode and provides transmit statistics */
  eStatisticsEtherType5,      /* Used in "raw" decoder mode and provides transmit statistics */
  eStatisticsEtherType6,      /* Used for Rx statistics from the NT family cards */
  eStatisticsNtTxType1,       /* Used for NT series TX statistics. */
  eStatisticsBlockType1,      /* Used to get block statistics. (All statistics DMA'ed to a single buffer) */
  eStatisticsEtherType7,      /* Used for NT series only. */
  eStatisticsSimpleType1,     /* Used for NT series only. */
  eStatisticsMiscType1,       /* Used for NT series only. */
  eStatisticsRetransmit,      /***** NOTE: This will be obsoleted as of the "Longbeach" release. */
  eStatisticsEtherType8,      /* Used for NT series only. */
  eStatisticsIpFragType1      /* Used for NT series only. */
};

/* Statistics Options Structure - use this with SetStatisticsOptions in order to bind/unbind statistics with packet feed(s) */
typedef struct
{
  FeedHandle packetFeedHandle;     /* An optional packetFeed handle is required so that the options below can be linked to a packet feed. It is mandatory if FeedStart/Stop options are required */
  bool_t     startNow;
  bool_t     startWithFeedStart;
  bool_t     stopNow;
  bool_t     stopWithFeedStop;
  bool_t     resetNow;             /* Same as ResetStatistics function */
  bool_t     resetWithFeedStart;
} StatisticsOptionsType1_t;

typedef struct
{
  FeedHandle packetFeedHandle;     /* Mandatory */
  bool_t     startWithFeedStart;
  bool_t     stopWithFeedStop;
  bool_t     resetWithFeedStart;
} StatisticsOptionsType2_t;

typedef struct
{
  bool_t startNow;           /* Mutually exclusive with 'stopNow' */
  bool_t stopNow;            /* Mutually exclusive with 'startNow' */
  bool_t resetNow;           /* Same as ResetStatistics function */
} StatisticsOptionsType3_t;

/* **** NOTE that use of the following structure is no longer recommended
        The features have been enchanced and replaced by use of the SetCardConfiguration()
    function inconjunction with FrameDecoderConfigurationTypex_t */
typedef struct
{
  bool_t enableISL;          /* Enables detection of ISL frames. */
  bool_t enableVLAN;         /* Enables detection of VLAN tagged frames. */
  bool_t enableJUMBO;        /* Enables detection of Jumbo frames. */
  bool_t allowISLVLAN;       /* Enables VLAN tagged frame detection within ISL frames. */
  bool_t allowISLJUMBO;        /* Enables JUMBO frame detection within ISL frames. */
  bool_t allowJUMBOVLAN;       /* Enables JUMBO frame detection for VLAN tagged frames. */
  bool_t ISL64IsSmall;       /* Changes the "small" packet decode criteria for ISL frames. */
  bool_t VLAN64IsSmall;        /* Changes the "small" packet decode criteria for VLAN frames */
} StatisticsOptionsType4_t;
/* **** NOTE that use of the previous structure is no longer recommended
        The features have been enchanced and replaced by use of the SetCardConfiguration()
    function inconjunction with FrameDecoderConfigurationTypex_t */

/* Definition of the STRUCTID to be used at the application layer */
/* Type1 defines the S-Style Options Structure */
#define STID_STATISTICS_OPTIONS_TYPE1 \
    MAKE_STRUCTURE_ID(eStatisticsOptionsType1, StatisticsOptionsType1_t)

#define STID_STATISTICS_OPTIONS_TYPE2 \
    MAKE_STRUCTURE_ID(eStatisticsOptionsType2, StatisticsOptionsType2_t)

#define STID_STATISTICS_OPTIONS_TYPE3 \
    MAKE_STRUCTURE_ID(eStatisticsOptionsType3, StatisticsOptionsType3_t)

#define STID_STATISTICS_OPTIONS_TYPE4 \
    MAKE_STRUCTURE_ID(eStatisticsOptionsType4, StatisticsOptionsType4_t)

/* RMON1EtherStatistics_t defines the structure of the standard
 * statistics that will be supported by the majority of devices.
 */
typedef struct
{
  uint64_t dwlFramesReceivedOK;
  uint64_t dwlOctetsReceivedOK;                    // All octets received good and bad, bad variable name: OK !!
  uint64_t dwlGroupFramesReceivedOk;               // All frames received good and bad, bad variable name: OK !!
  uint64_t dwlGroupOctetsReceivedOk;
  uint64_t dwlFragmentsReceivedOk;                 // Fragments received, bad variable name: OK !!
  uint64_t dwlOversizeFramesReceivedOk;
  uint64_t dwlUnderSizedFramesReceivedOk;
  uint64_t dwlJabberFramesReceivedOk;              // Jabber frames recived, bad variable name: OK !!
  uint64_t dwlBroadcastFramesReceivedOK;
  uint64_t dwlUnicastFramesReceivedOK;
  uint64_t dwlMulticastFramesReceivedOK;
  uint64_t dwlCRCErrors;
  uint64_t dwlCharacterCVErrors;
  uint64_t dwlReceiveFrames64Octets;
  uint64_t dwlReceiveFrames65To127Octets;
  uint64_t dwlReceiveFrames128To255Octets;
  uint64_t dwlReceiveFrames256To511Octets;
  uint64_t dwlReceiveFrames512To1023Octets;
  uint64_t dwlReceiveFrames1024To1518Octets;
  uint64_t dwlBabyGiantVLANFramesReceivedOK;
  uint64_t dwlBabyGiantISLFramesReceivedOK;
  uint64_t dwlBabyGiantJumboFramesReceivedOK;
  uint64_t dwlJumboFrames1519To2047Octets;
  uint64_t dwlJumboFrames2048To4095Octets;
  uint64_t dwlJumboFrames4096To8191Octets;
  uint64_t dwlJumboFrames8192To9018Octets;
  uint64_t dwlVLANJumboFrames9019To9022Octets;
} RMON1EtherStatistics_t;

typedef struct
{
  uint64_t  dwlFramesReceivedOK;
  uint64_t  dwlOctetsReceivedOK;                    // All octets received good and bad, bad variable name: OK !!
  uint64_t  dwlGroupFramesReceivedOk;               // All frames received good and bad, bad variable name: OK !!
  uint64_t  dwlGroupOctetsReceivedOk;
  uint64_t  dwlFragmentsReceivedOk;                 // Fragments received, bad variable name: OK !!
  uint64_t  dwlOversizeFramesReceivedOk;
  uint64_t  dwlUnderSizedFramesReceivedOk;
  uint64_t  dwlJabberFramesReceivedOk;              // Jabber frames recived, bad variable name: OK !!
  uint64_t  dwlBroadcastFramesReceivedOK;
  uint64_t  dwlUnicastFramesReceivedOK;
  uint64_t  dwlMulticastFramesReceivedOK;
  uint64_t  dwlCRCErrors;
  uint64_t  dwlCharacterCVErrors;
  uint64_t  dwlReceiveFrames64Octets;
  uint64_t  dwlReceiveFrames65To127Octets;
  uint64_t  dwlReceiveFrames128To255Octets;
  uint64_t  dwlReceiveFrames256To511Octets;
  uint64_t  dwlReceiveFrames512To1023Octets;
  uint64_t  dwlReceiveFrames1024To1518Octets;
  uint64_t  dwlBabyGiantVLANFramesReceivedOK;
  uint64_t  dwlBabyGiantISLFramesReceivedOK;
  uint64_t  dwlJumboFrames1519To2047OctetsNotBabyGiant;
  uint64_t  dwlJumboFrames1519To2047Octets;             /* manufatured by software */
  uint64_t  dwlJumboFrames2048To4095Octets;
  uint64_t  dwlJumboFrames4096To8191Octets;
  uint64_t  dwlJumboFrames8192To9018Octets;
  uint64_t  dwlVLANJumboFrames9019To9022Octets;
  uint64_t  dwlJumboFrames9019ToHardSOctets;
  uint64_t  dwlJumboFrames8192ToHardSOctets;            /* manufatured by software */
} RMON1EtherStatisticsType2_t;

/* RMONStatisticsType1_t only defines the structure of the standard
 * RMON RX counters which are supported by the NT family cards.
 */
typedef struct
{
  uint64_t dwlOctetsReceived;
  uint64_t dwlFramesReceived;
  uint64_t dwlFragmentsReceived;
  uint64_t dwlUnderSizedFramesReceived;
  uint64_t dwlOversizeFramesReceived;
  uint64_t dwlJabberFramesReceived;
  uint64_t dwlBroadcastFramesReceived;
  uint64_t dwlMulticastFramesReceived;
  uint64_t dwlCRCAlignErrors;
  uint64_t dwlDropEvents;
  uint64_t dwlReceiveFrames64Octets;
  uint64_t dwlReceiveFrames65To127Octets;
  uint64_t dwlReceiveFrames128To255Octets;
  uint64_t dwlReceiveFrames256To511Octets;
  uint64_t dwlReceiveFrames512To1023Octets;
  uint64_t dwlReceiveFrames1024To1518Octets;
} RxRMONStatisticsType1_t;

/* RxHwStatisticsType1_t defines all the RX hardware counters
 * of the NT family cards.
 */
typedef struct
{
  uint64_t dwlRxFrameSize64;
  uint64_t dwlRxFrameSize65To127;
  uint64_t dwlRxFrameSize128To255;
  uint64_t dwlRxFrameSize256To511;
  uint64_t dwlRxFrameSize512To1023;
  uint64_t dwlRxFrameSize1024To1518;
  uint64_t dwlRxFrameSize1519To2047;
  uint64_t dwlRxFrameSize2048To4095;
  uint64_t dwlRxFrameSize4096To8191;
  uint64_t dwlRxFrameSize8192To9018;
  uint64_t dwlRxFrameSize9019ToMax;
  uint64_t dwlRxUndersizeFrames;
  uint64_t dwlRxFragments;
  uint64_t dwlRxOversizeFrames;
  uint64_t dwlRxJabberFrames;
  uint64_t dwlRxOversizeHardSlicedFrames;
  uint64_t dwlRxJabberHardSlicedFrames;
  uint64_t dwlRxGoodFrameBytes;
  uint64_t dwlRxGoodPauseUnsupportedMACControl;
  uint64_t dwlRxBadFrameBytes;
  uint64_t dwlRxUnicastFrames;
  uint64_t dwlRxMulticastFrames;
  uint64_t dwlRxBroadcastFrames;
  uint64_t dwlRxPauseFrames;
  uint64_t dwlRxUnsupportedMACControlFrames;
  uint64_t dwlRxCrcErrors;
  uint64_t dwlRxAlignmentErrors;
  uint64_t dwlRxCodeViolationErrors;
  uint64_t dwlRxTotalFrames;
  uint64_t dwlRxXGMIISmallFrameDrop;
  uint64_t dwlRxPacketMergeDrop;
  uint64_t dwlRxSmootherDrop;
} RxHwStatisticsType1_t;

/*
 * ExtendedStatisticsType1_t defines one type of additional statistics that are available
 */
typedef struct
{
  uint64_t dwlLayer3ChecksumErrors;
  uint64_t dwlLayer4ChecksumErrors;
} ExtendedStatisticsType1_t;

/*
 * ExtendedStatisticsType2_t defines one type of additional statistics that are available
 */
typedef struct
{
  uint64_t dwlLayer3ChecksumErrors;
  uint64_t dwlLayer4ChecksumErrors;
  uint64_t dwlTCPChecksumErrors;
  uint64_t dwlUDPChecksumErrors;
} ExtendedStatisticsType2_t;

/* Structure used for transmit statistics */
typedef struct
{
    uint64_t dwlRetransmitIncludedCount;
    uint64_t dwlRetransmitExcludedCount;
    uint64_t dwlRetransmitErrorCount;
} TransmitStatisticsType1_t;

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint64_t    dwlStringMatches DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } AdvancedStatisticsType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	RMON1EtherStatistics_t     RMON1Statistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	ExtendedStatisticsType1_t  ExtendedStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint32_t                   dwTimeValidAt DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } StatisticsEtherType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_ETHER_STATISTICS_TYPE1 MAKE_STRUCTURE_ID(eStatisticsEtherType1, StatisticsEtherType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

/* StatisticsEtherType2_t defines the data structure passed to the
 * application if using an X card.
 * This structure should only be used in frame decoder mode 0. The decoder mode
 * can be set using the SetCardConfigurationDetails API and the current
 * decoder mode read back useing the GetCardConfigurationDetails API.
*/
typedef struct
{
  RMON1EtherStatistics_t      RMON1Statistics;
  ExtendedStatisticsType2_t   ExtendedStatistics;
  PacketFeedStatisticsType1_t PacketFeedStatistics;

  TimestampType_t             timestampType;      /* Timestamp type that is supplied with 'TimeValidAt' */
  uint64_t                    dwlTimeValidAt;     /* Timestamp when these
                                                     values were extracted
                                                     from the hardware counters */
    bool_t            overflow;                   /* Set when HW counter overflows */
} StatisticsEtherType2_t;

#define STID_ETHER_STATISTICS_TYPE2 \
    MAKE_STRUCTURE_ID(eStatisticsEtherType2, StatisticsEtherType2_t)

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct{
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	RMON1EtherStatisticsType2_t			RMON1Statistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	ExtendedStatisticsType2_t			ExtendedStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	PacketFeedStatisticsType1_t			PacketFeedStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	TimestampType_t             timestampType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint64_t                    dwlTimeValidAt DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	bool_t						overflow DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } StatisticsEtherType3_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_ETHER_STATISTICS_TYPE3  MAKE_STRUCTURE_ID(eStatisticsEtherType3, StatisticsEtherType3_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct _statisticsEtherType4 {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     RMON1EtherStatistics_t                 RMON1Statistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     ExtendedStatisticsType2_t              ExtendedStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     TransmitStatisticsType1_t              TransmitStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     PacketFeedStatisticsType1_t            PacketFeedStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     TimestampType_t                        timestampType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint64_t                               dwlTimeValidAt DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     bool_t                                 overflow DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } StatisticsEtherType4_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_ETHER_STATISTICS_TYPE4  MAKE_STRUCTURE_ID(eStatisticsEtherType4, StatisticsEtherType4_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct _statisticsEtherType5 {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     RMON1EtherStatisticsType2_t            RMON1Statistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     ExtendedStatisticsType2_t              ExtendedStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     TransmitStatisticsType1_t              TransmitStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     PacketFeedStatisticsType1_t            PacketFeedStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     TimestampType_t                        timestampType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     uint64_t                               dwlTimeValidAt DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     bool_t                                 overflow DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } StatisticsEtherType5_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_ETHER_STATISTICS_TYPE5  MAKE_STRUCTURE_ID(eStatisticsEtherType5, StatisticsEtherType5_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25

/* This type is to be used for Rx statistics from the NT family cards */
typedef struct _statisticsEtherType6 {
  RxRMONStatisticsType1_t  RMONStatistics;            /* Includes the RMON counters supported */
  RxHwStatisticsType1_t    HardwareStatistics;        /* Holds all rx hardwarecounters        */

  TimestampType_t          timestampType;             /* Timestamp type that is supplied with 'TimeValidAt' */
  uint64_t                 dwlTimeValidAt;            /* Timestamp when these
                                                         values were extracted
                                                         from the hardware counters */
} StatisticsEtherType6_t;

#define STID_ETHER_STATISTICS_TYPE6 \
    MAKE_STRUCTURE_ID(eStatisticsEtherType6, StatisticsEtherType6_t)

/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #ifdef NT_LEGACY_4_25
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ typedef struct {
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	RMON1EtherStatistics_t      RMON1Statistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	ExtendedStatisticsType2_t   ExtendedStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	PacketFeedStatisticsType1_t	PacketFeedStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	AdvancedStatisticsType1_t	AdvancedStatistics DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	TimestampType_t             timestampType DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ 	uint64_t                    dwlTimeValidAt DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */     bool_t						overflow DEPRECATED_4_25;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ } StatisticsAdvEtherType1_t;
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #define STID_ETHER_ADV_STATISTICS_TYPE1  MAKE_STRUCTURE_ID(eStatisticsAdvEtherType1, StatisticsAdvEtherType1_t)
/* LEGACY SINCE v4.25 - PLEASE IGNORE THIS LINE - SEE DN-0059 */ #endif //NT_LEGACY_4_25


typedef struct {
  /* Bucket counters. */
  uint64_t dwlRxUndersizeFrames;
  uint64_t dwlRxFragments;
  uint64_t dwlRxUndersizeWithoutEncapsulation;
  uint64_t dwlRxFrameSize64;
  uint64_t dwlRxFrameSize65To127;
  uint64_t dwlRxFrameSize128To255;
  uint64_t dwlRxFrameSize256To511;
  uint64_t dwlRxFrameSize512To1023;
  uint64_t dwlRxFrameSize1024To1518;
  uint64_t dwlRxBabyGiantFrames;
  uint64_t dwlRxFrameSize1519To2047;
  uint64_t dwlRxFrameSize2048To4095;
  uint64_t dwlRxFrameSize4096To8191;
  uint64_t dwlRxFrameSize8192ToMax;
  uint64_t dwlRxOversizeFrames;
  uint64_t dwlRxJabberFrames;
  uint64_t dwlRxOversizeHardSlicedFrames;
  uint64_t dwlRxJabberHardSlicedFrames;
  /* Counter 18;19 reserved. */
  uint64_t reserved0[2];
  /* Byte counters. */
  uint64_t dwlRxGoodFrameBytes;
  uint64_t dwlRxGoodPauseMacFramesBytes;
  uint64_t dwlRxBadFrameBytes;
  /* Counter 23..31 reserved. */
  uint64_t reserved1[9];
} _StatSetType1_t;

typedef struct {
  /* Delivery counters. */
  uint64_t dwlRxUnicastFrames;
  uint64_t dwlRxMulticastFrames;
  uint64_t dwlRxBroadcastFrames;
  /* Counter 3 reserved. */
  uint64_t reserved0;
  /* Pause counters. */
  uint64_t dwlRxUnicastPauseFrames;
  uint64_t dwlRxMulticastPauseFrames;
  /* Counter 6;7 reserved. */
  uint64_t reserved1[2];
  /* MAC control counters. */
  uint64_t dwlRxMacUnicastFrames;
  uint64_t dwlRxMacMulticastFrames;
  uint64_t dwlRxMacBroadcastFrames;
  /* Counter 11..15 reserved. */
  uint64_t reserved2[5];
  /* Decode counters. */
  uint64_t dwlRxNotIslVlanMplsFrames;
  uint64_t dwlRxIslFrames;
  uint64_t dwlRxVlanFrames;
  uint64_t dwlRxIslVlanFrames;
  uint64_t dwlRxMplsFrames;
  uint64_t dwlRxIslMplsFrames;
  uint64_t dwlRxVlanMplsFrames;
  uint64_t dwlRxIslVlanMplsFrames;
  uint64_t dwlRxCrcErrors;
  uint64_t dwlRxAlignmentErrors;
  uint64_t dwlRxCodeViolationErrors;
  uint64_t dwlRxSmallFrames;
  uint64_t dwlRxLargePackets;
  uint64_t reserved3[3];
} _StatSetType2_t;

/* This type is to be used for Rx statistics from the NT family cards */
typedef struct _statisticsEtherType7 {
  _StatSetType1_t          Set1;           /* Counter set 1. Bucket and byte counters.              */
  _StatSetType2_t          Set2;           /* Counter set 2. Decode counters.                       */
  TimestampType_t          timestampType;  /* Timestamp type that is supplied with 'TimeValidAt'    */
  uint64_t                 dwlTimeValidAt; /* Timestamp when these values were sampled by hardware. */
} StatisticsEtherType7_t;

#define STID_ETHER_STATISTICS_TYPE7 \
    MAKE_STRUCTURE_ID(eStatisticsEtherType7, StatisticsEtherType7_t)

typedef struct {
  uint64_t dwlGoodFrames;
  uint64_t dwlBadFrames;
  uint64_t dwlGoodBytes;
  uint64_t dwlBadBytes;
} StatSetSimpleType1_t;

/* Simple statistics structure used to return good/bad frames/bytes received. */
typedef struct _statisticsSimpleType1 {
  StatSetSimpleType1_t simple;    /* Simple counter set.                                   */
  TimestampType_t timestampType;  /* Timestamp type that is supplied with 'TimeValidAt'    */
  uint64_t        dwlTimeValidAt; /* Timestamp when these values were sampled by hardware. */
} StatisticsSimpleType1_t;

#define STID_STATISTICS_SIMPLE_TYPE1 \
    MAKE_STRUCTURE_ID(eStatisticsSimpleType1, StatisticsSimpleType1_t)

/*
 * BLOCK MODE statistics.
 */
typedef struct {
  uint32_t waitForNewData:1; /* Should we wait for data or just get latest?    */
  uint32_t clearAfterRead:1; /* Should the statistics be cleared after read.
                                NOTE: This will have global effect on all apps
                                      getting statistics.                      */
  uint32_t reserved:30;      /* Reserved.                                      */
  uint32_t numDescriptors;
  ExternalBufferInfoType1_t dmaBuffer;
} StatisticsBlockType1_t;

/* Used to get block statistics. (All statistics DMA'ed to a single buffer) */
#define STID_STATISTICS_BLOCK_TYPE1 \
    MAKE_STRUCTURE_ID(eStatisticsBlockType1, StatisticsBlockType1_t)

/**
 * Misc statistics counter per port.
 */
typedef struct {
  uint64_t dwlRxXGMIISmallFrameDrop; //<! Packets discarded because they are too small
  uint64_t dwlRxPacketMergeDrop;     //<! Packets discarded because of adapter bandwidth issues
  uint64_t reserved[30];             //<! Reserved for future use
  TimestampType_t timestampType;     //<! Timestamp type that is supplied with 'TimeValidAt'
  uint64_t dwlTimeValidAt;           //<! Sampling time
} StatisticsMiscType1_t;

#define STID_STATISTICS_MISC_TYPE1 \
    MAKE_STRUCTURE_ID(eStatisticsMiscType1, StatisticsMiscType1_t)

/**
 * IPFG statistics counter per port.
 */
typedef struct {
  uint64_t dwlIpFragFirstHit;       //<! Number of first fragments which could be stored in table
  uint64_t dwlIpFragFirstNoHit;     //<! Number of first fragments which could not be stored in table.
  uint64_t dwlIpFragMidHit;         //<! Number of middle fragments which hit in table.
  uint64_t dwlIpFragMidNoHit;       //<! Number of middle fragments which missed in table.
  uint64_t dwlIpFragLastHit;        //<! Number of last fragments which hit in table.
  uint64_t dwlIpFragLastNoHit;      //<! Number of last fragments which missed in table.
  uint64_t dwlIpFragFirstLast;      //<! Number of fragments that are both first fragment and last fragment.
  uint64_t dwlIpFragNotConsidered;  //<! Number of IPv4 and IPv6 frames not considered by the IP fragment block.
  TimestampType_t timestampType;    //<! Timestamp type that is supplied with TimeValidAt.
  uint64_t dwlTimeValidAt;          //<! Sampling time.
} StatisticsIpFragType1_t;

#define STID_STATISTICS_IPFRAG_TYPE1 \
    MAKE_STRUCTURE_ID(eStatisticsIpFragType1, StatisticsIpFragType1_t)

/**
* If the adapter support TX statistics this structure is obsoleted and is replaced
* by the TX statistics counters. FPGAv40+ will not provide this counterset.
*/
typedef struct {
  struct {
    uint64_t packetsDiscard; //<! Packets being discarded from port 0 because of MAC bandwidth problems.
    uint64_t packetsSent;    //<! Packets retransmitted onto port 0;
  } Port0;
  struct {
    uint64_t packetsDiscard; //<! Packets being discarded from port 1 because of MAC bandwidth problems.
    uint64_t packetsSent;    //<! Packets retransmitted onto port 1;
  } Port1;
} StatisticsRetransmit_t;

#define STID_STATISTICS_RETRANSMIT \
    MAKE_STRUCTURE_ID(eStatisticsRetransmit, StatisticsRetransmit_t)

typedef enum {
  STAT_TYPE_PORT = 0,
  STAT_TYPE_PORT_FILTERED,       // Discarded by filter
  STAT_TYPE_FILTER_FORWARD,      // Forwarded by filter
  STAT_TYPE_FILTER_DISCARD,
  STAT_TYPE_FEED_FORWARD,
  STAT_TYPE_FEED_INVALID_DROP,   // Droped because no feed is active
  STAT_TYPE_FEED_BANDWIDTH_DROP, // Droped because of on-board buffer overflow
  STAT_TYPE_DEDUP,
  STAT_TYPE_MISC,
  STAT_TYPE_RETRANSMIT, /***** NOTE: This will be obsoleted as of the "Longbeach" release. */
  STAT_TYPE_PORT_TX,
  STAT_TYPE_FEED_IP_FRAGMENT,
  STAT_TYPE_PORT_IP_FRAGMENT,
  STAT_TYPE_DROP_IP_FRAGMENT
} eStatSubType;

typedef struct _statistics_block_descriptor_type1 {
  uint8_t  headerVersion;/* Version = 1.                             */
  uint8_t  type;         /* Types of eStatSubType.                   */
  uint8_t  index;        /* The stat index ie. channel, feed, filter.*/
  uint8_t  last:1;       /* Set to 1 if this is the last descriptor. */
  uint8_t  reserved0:7;  /* Reserved field.                          */
  uint16_t storedLength; /* The length of the statistics incl. this
                            header.                                  */
  uint16_t reserved1;    /* Reserved field.                          */
  uint32_t reserved2;    /* Reserved field.                          */
  uint32_t structureId;  /* The statistics structure.                */
} StatBlockDescriptorType1_t;

typedef struct {
  /* Bucket counters. */
  uint64_t dwlUndersizeFrames;
  uint64_t dwlFragments;
  uint64_t dwlUndersizeWithoutEncapsulation; //!< Doesn't count for TX
  uint64_t dwlFrameSize64;
  uint64_t dwlFrameSize65To127;
  uint64_t dwlFrameSize128To255;
  uint64_t dwlFrameSize256To511;
  uint64_t dwlFrameSize512To1023;
  uint64_t dwlFrameSize1024To1518;
  uint64_t dwlBabyGiantFrames;               //!< Doesn't count for TX
  uint64_t dwlFrameSize1519To2047;
  uint64_t dwlFrameSize2048To4095;
  uint64_t dwlFrameSize4096To8191;
  uint64_t dwlFrameSize8192ToMax;
  uint64_t dwlOversizeFrames;
  uint64_t dwlJabberFrames;
  uint64_t dwlOversizeHardSlicedFrames;
  uint64_t dwlJabberHardSlicedFrames;
  /* Counter 18;19 reserved. */
  uint64_t reserved0[2];
  /* Byte counters. */
  uint64_t dwlGoodFrameBytes;
  uint64_t dwlGoodPauseMacFramesBytes;
  uint64_t dwlBadFrameBytes;
  /* Counter 23..31 reserved. */
  uint64_t reserved1[9];
} StatSetType1_t;

typedef struct {
  /* Delivery counters. */
  uint64_t dwlUnicastFrames;
  uint64_t dwlMulticastFrames;
  uint64_t dwlBroadcastFrames;
  /* Counter 3 reserved. */
  uint64_t reserved0;
  /* Pause counters. */
  uint64_t dwlUnicastPauseFrames;
  uint64_t dwlMulticastPauseFrames;
  /* Counter 6;7 reserved. */
  uint64_t reserved1[2];
  /* MAC control counters. */
  uint64_t dwlMacUnicastFrames;
  uint64_t dwlMacMulticastFrames;
  uint64_t dwlMacBroadcastFrames;
  /* Counter 11..15 reserved. */
  uint64_t reserved2[5];
  /* Decode counters. */
  uint64_t dwlNotIslVlanMplsFrames;    //!< Doesn't count for TX
  uint64_t dwlIslFrames;               //!< Doesn't count for TX
  uint64_t dwlVlanFrames;              //!< Doesn't count for TX
  uint64_t dwlIslVlanFrames;           //!< Doesn't count for TX
  uint64_t dwlMplsFrames;              //!< Doesn't count for TX
  uint64_t dwlIslMplsFrames;           //!< Doesn't count for TX
  uint64_t dwlVlanMplsFrames;          //!< Doesn't count for TX
  uint64_t dwlIslVlanMplsFrames;       //!< Doesn't count for TX
  uint64_t dwlCrcErrors;               //!< TX: Counted based on information in the standard descriptor
  uint64_t dwlAlignmentErrors;         //!< Doesn't count for TX
  uint64_t dwlCodeViolationErrors;     //!< TX: Counted based on information in the standard descriptor
  uint64_t dwlSmallFrames;
  uint64_t dwlLargePackets;
  /* ONLY SUPPORTED FOR FPGA VERSION 40+ */
  uint64_t dwlIpChecksumErrors;        //!< TX: Counted based on information in the standard descriptor
  uint64_t dwlTcpChecksumErrors;       //!< TX: Counted based on information in the standard descriptor
  uint64_t dwlUdpChecksumErrors;       //!< TX: Counted based on information in the standard descriptor
} StatSetType2_t;

/* This type is to be used for Rx/Tx statistics from the NT family cards */
typedef struct _statisticsEtherType8 {
  StatSetType1_t          Set1;            /* Counter set 1. Bucket and byte counters.              */
  StatSetType2_t          Set2;            /* Counter set 2. Decode counters.                       */
  TimestampType_t          timestampType;  /* Timestamp type that is supplied with 'TimeValidAt'    */
  uint64_t                 dwlTimeValidAt; /* Timestamp when these values were sampled by hardware. */
} StatisticsEtherType8_t;

#define STID_ETHER_STATISTICS_TYPE8 \
    MAKE_STRUCTURE_ID(eStatisticsEtherType8, StatisticsEtherType8_t)

#include "ntpackpop.h"

void ConvertEtherType7ToEtherType2(StatisticsEtherType2_t *d, StatisticsEtherType7_t *s);
void ConvertEtherType7ToEtherType6(StatisticsEtherType6_t *d, StatisticsEtherType7_t *s);
void ConvertEtherType8ToEtherType2(StatisticsEtherType2_t *d, StatisticsEtherType8_t *s);
void ConvertEtherType8ToEtherType6(StatisticsEtherType6_t *d, StatisticsEtherType8_t *s);

#endif // __NTAPI_STATISTICSSTRUCTURE_H__

#ifdef _STAT_CONVERT_FUNCTIONS_
/* CONVERT FUNCTIONS. */
void ConvertEtherType7ToEtherType2(StatisticsEtherType2_t *d, StatisticsEtherType7_t *s)
{
  d->RMON1Statistics.dwlFramesReceivedOK              += (s->Set1.dwlRxUndersizeFrames +
                                                          s->Set1.dwlRxFragments +
                                                          s->Set1.dwlRxUndersizeWithoutEncapsulation +
                                                          s->Set1.dwlRxFrameSize64 +
                                                          s->Set1.dwlRxFrameSize65To127 +
                                                          s->Set1.dwlRxFrameSize128To255 +
                                                          s->Set1.dwlRxFrameSize256To511 +
                                                          s->Set1.dwlRxFrameSize512To1023 +
                                                          s->Set1.dwlRxFrameSize1024To1518 +
                                                          s->Set1.dwlRxBabyGiantFrames +
                                                          s->Set1.dwlRxFrameSize1519To2047 +
                                                          s->Set1.dwlRxFrameSize2048To4095 +
                                                          s->Set1.dwlRxFrameSize4096To8191 +
                                                          s->Set1.dwlRxFrameSize8192ToMax +
                                                          s->Set1.dwlRxOversizeFrames +
                                                          s->Set1.dwlRxJabberFrames +
                                                          s->Set1.dwlRxOversizeHardSlicedFrames +
                                                          s->Set1.dwlRxJabberHardSlicedFrames);
  d->RMON1Statistics.dwlOctetsReceivedOK              += (s->Set1.dwlRxGoodFrameBytes+
                                                          s->Set1.dwlRxGoodPauseMacFramesBytes+
                                                          s->Set1.dwlRxBadFrameBytes);
  d->RMON1Statistics.dwlGroupFramesReceivedOk         += (0);
  d->RMON1Statistics.dwlGroupOctetsReceivedOk         += (0);
  d->RMON1Statistics.dwlFragmentsReceivedOk           += (s->Set1.dwlRxFragments);
  d->RMON1Statistics.dwlOversizeFramesReceivedOk      += (s->Set1.dwlRxOversizeFrames +
                                                          s->Set1.dwlRxOversizeHardSlicedFrames);
  d->RMON1Statistics.dwlUnderSizedFramesReceivedOk    += (s->Set1.dwlRxUndersizeFrames);
  d->RMON1Statistics.dwlJabberFramesReceivedOk        += (s->Set1.dwlRxJabberFrames);
  d->RMON1Statistics.dwlBroadcastFramesReceivedOK     += (s->Set2.dwlRxBroadcastFrames);
  d->RMON1Statistics.dwlUnicastFramesReceivedOK       += (s->Set2.dwlRxUnicastFrames);
  d->RMON1Statistics.dwlMulticastFramesReceivedOK     += (s->Set2.dwlRxMulticastFrames);
  d->RMON1Statistics.dwlCRCErrors                     += (s->Set2.dwlRxCrcErrors);
  d->RMON1Statistics.dwlCharacterCVErrors             += (s->Set2.dwlRxCodeViolationErrors);
  d->RMON1Statistics.dwlReceiveFrames64Octets         += s->Set1.dwlRxFrameSize64;
  d->RMON1Statistics.dwlReceiveFrames65To127Octets    += s->Set1.dwlRxFrameSize65To127;
  d->RMON1Statistics.dwlReceiveFrames128To255Octets   += s->Set1.dwlRxFrameSize128To255;
  d->RMON1Statistics.dwlReceiveFrames256To511Octets   += s->Set1.dwlRxFrameSize256To511;
  d->RMON1Statistics.dwlReceiveFrames512To1023Octets  += s->Set1.dwlRxFrameSize512To1023;
  d->RMON1Statistics.dwlReceiveFrames1024To1518Octets += s->Set1.dwlRxFrameSize1024To1518;
  d->RMON1Statistics.dwlBabyGiantVLANFramesReceivedOK += 0;
  d->RMON1Statistics.dwlBabyGiantISLFramesReceivedOK  += 0;
  d->RMON1Statistics.dwlBabyGiantJumboFramesReceivedOK+= 0;
  d->RMON1Statistics.dwlJumboFrames1519To2047Octets   += s->Set1.dwlRxFrameSize1519To2047;
  d->RMON1Statistics.dwlJumboFrames2048To4095Octets   += s->Set1.dwlRxFrameSize2048To4095;
  d->RMON1Statistics.dwlJumboFrames4096To8191Octets   += s->Set1.dwlRxFrameSize4096To8191;
  d->RMON1Statistics.dwlJumboFrames8192To9018Octets   += s->Set1.dwlRxFrameSize8192ToMax;
  d->RMON1Statistics.dwlVLANJumboFrames9019To9022Octets= 0;
  d->ExtendedStatistics.dwlLayer3ChecksumErrors       += 0;
  d->ExtendedStatistics.dwlLayer4ChecksumErrors       += 0;
  d->ExtendedStatistics.dwlTCPChecksumErrors          += 0;
  d->ExtendedStatistics.dwlUDPChecksumErrors          += 0;
  d->dwlTimeValidAt                                    = s->dwlTimeValidAt;
  d->timestampType                                     = s->timestampType;
  d->overflow                                          = 0;
}

void ConvertEtherType7ToEtherType6(StatisticsEtherType6_t *d, StatisticsEtherType7_t *s)
{
  d->RMONStatistics.dwlOctetsReceived                      += (s->Set1.dwlRxGoodFrameBytes+
                                                               s->Set1.dwlRxGoodPauseMacFramesBytes+
                                                               s->Set1.dwlRxBadFrameBytes);
  d->RMONStatistics.dwlFramesReceived                      += (s->Set1.dwlRxUndersizeFrames +
                                                               s->Set1.dwlRxFragments +
                                                               s->Set1.dwlRxUndersizeWithoutEncapsulation +
                                                               s->Set1.dwlRxFrameSize64 +
                                                               s->Set1.dwlRxFrameSize65To127 +
                                                               s->Set1.dwlRxFrameSize128To255 +
                                                               s->Set1.dwlRxFrameSize256To511 +
                                                               s->Set1.dwlRxFrameSize512To1023 +
                                                               s->Set1.dwlRxFrameSize1024To1518 +
                                                               s->Set1.dwlRxBabyGiantFrames +
                                                               s->Set1.dwlRxFrameSize1519To2047 +
                                                               s->Set1.dwlRxFrameSize2048To4095 +
                                                               s->Set1.dwlRxFrameSize4096To8191 +
                                                               s->Set1.dwlRxFrameSize8192ToMax +
                                                               s->Set1.dwlRxOversizeFrames +
                                                               s->Set1.dwlRxJabberFrames +
                                                               s->Set1.dwlRxOversizeHardSlicedFrames +
                                                               s->Set1.dwlRxJabberHardSlicedFrames);
  d->RMONStatistics.dwlFragmentsReceived                    += (s->Set1.dwlRxFragments);
  d->RMONStatistics.dwlUnderSizedFramesReceived             += (s->Set1.dwlRxUndersizeFrames);
  d->RMONStatistics.dwlOversizeFramesReceived               += (s->Set1.dwlRxOversizeFrames +
                                                                s->Set1.dwlRxOversizeHardSlicedFrames);
  d->RMONStatistics.dwlJabberFramesReceived                 += (s->Set1.dwlRxJabberFrames);
  d->RMONStatistics.dwlBroadcastFramesReceived              += (s->Set2.dwlRxBroadcastFrames +
                                                                s->Set2.dwlRxMacBroadcastFrames);
  d->RMONStatistics.dwlMulticastFramesReceived              += (s->Set2.dwlRxMulticastFrames +
                                                                s->Set2.dwlRxMacMulticastFrames);
  d->RMONStatistics.dwlCRCAlignErrors                       += (s->Set2.dwlRxCrcErrors +
                                                                s->Set2.dwlRxAlignmentErrors +
                                                                s->Set2.dwlRxCodeViolationErrors);
  d->RMONStatistics.dwlDropEvents                           += (0);
  d->RMONStatistics.dwlReceiveFrames64Octets                += (s->Set1.dwlRxFrameSize64);
  d->RMONStatistics.dwlReceiveFrames65To127Octets           += (s->Set1.dwlRxFrameSize65To127);
  d->RMONStatistics.dwlReceiveFrames128To255Octets          += (s->Set1.dwlRxFrameSize128To255);
  d->RMONStatistics.dwlReceiveFrames256To511Octets          += (s->Set1.dwlRxFrameSize256To511);
  d->RMONStatistics.dwlReceiveFrames512To1023Octets         += (s->Set1.dwlRxFrameSize512To1023);
  d->RMONStatistics.dwlReceiveFrames1024To1518Octets        += (s->Set1.dwlRxFrameSize1024To1518);

  d->HardwareStatistics.dwlRxFrameSize64                    += (s->Set1.dwlRxFrameSize64);
  d->HardwareStatistics.dwlRxFrameSize65To127               += (s->Set1.dwlRxFrameSize65To127);
  d->HardwareStatistics.dwlRxFrameSize128To255              += (s->Set1.dwlRxFrameSize128To255);
  d->HardwareStatistics.dwlRxFrameSize256To511              += (s->Set1.dwlRxFrameSize256To511);
  d->HardwareStatistics.dwlRxFrameSize512To1023             += (s->Set1.dwlRxFrameSize512To1023);
  d->HardwareStatistics.dwlRxFrameSize1024To1518            += (s->Set1.dwlRxFrameSize1024To1518);
  d->HardwareStatistics.dwlRxFrameSize1519To2047            += (s->Set1.dwlRxFrameSize1519To2047);
  d->HardwareStatistics.dwlRxFrameSize2048To4095            += (s->Set1.dwlRxFrameSize2048To4095);
  d->HardwareStatistics.dwlRxFrameSize4096To8191            += (s->Set1.dwlRxFrameSize4096To8191);
  d->HardwareStatistics.dwlRxFrameSize8192To9018            += (s->Set1.dwlRxFrameSize8192ToMax);
  d->HardwareStatistics.dwlRxFrameSize9019ToMax             += (0);
  d->HardwareStatistics.dwlRxUndersizeFrames                += (s->Set1.dwlRxUndersizeFrames);
  d->HardwareStatistics.dwlRxFragments                      += (s->Set1.dwlRxFragments);
  d->HardwareStatistics.dwlRxOversizeFrames                 += (s->Set1.dwlRxOversizeFrames);
  d->HardwareStatistics.dwlRxJabberFrames                   += (s->Set1.dwlRxJabberFrames);
  d->HardwareStatistics.dwlRxOversizeHardSlicedFrames       += (s->Set1.dwlRxOversizeHardSlicedFrames);
  d->HardwareStatistics.dwlRxJabberHardSlicedFrames         += (s->Set1.dwlRxJabberHardSlicedFrames);
  d->HardwareStatistics.dwlRxGoodFrameBytes                 += (s->Set1.dwlRxGoodFrameBytes);
  d->HardwareStatistics.dwlRxGoodPauseUnsupportedMACControl += (s->Set2.dwlRxUnicastPauseFrames +
                                                                s->Set2.dwlRxMulticastPauseFrames +
                                                                s->Set2.dwlRxMacUnicastFrames +
                                                                s->Set2.dwlRxMacMulticastFrames +
                                                                s->Set2.dwlRxMacBroadcastFrames);
  d->HardwareStatistics.dwlRxBadFrameBytes                  += (s->Set1.dwlRxBadFrameBytes);
  d->HardwareStatistics.dwlRxUnicastFrames                  += (s->Set2.dwlRxUnicastFrames);
  d->HardwareStatistics.dwlRxMulticastFrames                += (s->Set2.dwlRxMulticastFrames);
  d->HardwareStatistics.dwlRxBroadcastFrames                += (s->Set2.dwlRxBroadcastFrames);
  d->HardwareStatistics.dwlRxPauseFrames                    += (s->Set2.dwlRxUnicastPauseFrames +
                                                               s->Set2.dwlRxMulticastPauseFrames);

  d->HardwareStatistics.dwlRxUnsupportedMACControlFrames    += (s->Set2.dwlRxMacUnicastFrames +
                                                               s->Set2.dwlRxMacMulticastFrames +
                                                               s->Set2.dwlRxMacBroadcastFrames);

  d->HardwareStatistics.dwlRxCrcErrors                      += (s->Set2.dwlRxCrcErrors);
  d->HardwareStatistics.dwlRxAlignmentErrors                += (s->Set2.dwlRxAlignmentErrors);
  d->HardwareStatistics.dwlRxCodeViolationErrors            += (s->Set2.dwlRxCodeViolationErrors);
  d->HardwareStatistics.dwlRxTotalFrames                    += (s->Set1.dwlRxUndersizeFrames +
                                                               s->Set1.dwlRxFragments +
                                                               s->Set1.dwlRxUndersizeWithoutEncapsulation +
                                                               s->Set1.dwlRxFrameSize64 +
                                                               s->Set1.dwlRxFrameSize65To127 +
                                                               s->Set1.dwlRxFrameSize128To255 +
                                                               s->Set1.dwlRxFrameSize256To511 +
                                                               s->Set1.dwlRxFrameSize512To1023 +
                                                               s->Set1.dwlRxFrameSize1024To1518 +
                                                               s->Set1.dwlRxBabyGiantFrames +
                                                               s->Set1.dwlRxFrameSize1519To2047 +
                                                               s->Set1.dwlRxFrameSize2048To4095 +
                                                               s->Set1.dwlRxFrameSize4096To8191 +
                                                               s->Set1.dwlRxFrameSize8192ToMax +
                                                               s->Set1.dwlRxOversizeFrames +
                                                               s->Set1.dwlRxJabberFrames +
                                                               s->Set1.dwlRxOversizeHardSlicedFrames +
                                                               s->Set1.dwlRxJabberHardSlicedFrames);
  d->HardwareStatistics.dwlRxXGMIISmallFrameDrop            += (0);
  d->HardwareStatistics.dwlRxPacketMergeDrop                += (0);
  d->HardwareStatistics.dwlRxSmootherDrop                   += (0);
  d->dwlTimeValidAt                                          = s->dwlTimeValidAt;
  d->timestampType                                           = s->timestampType;
}

void ConvertEtherType8ToEtherType2(StatisticsEtherType2_t *d, StatisticsEtherType8_t *s)
{
  d->RMON1Statistics.dwlFramesReceivedOK              += (s->Set1.dwlUndersizeFrames +
                                                          s->Set1.dwlFragments +
                                                          s->Set1.dwlUndersizeWithoutEncapsulation +
                                                          s->Set1.dwlFrameSize64 +
                                                          s->Set1.dwlFrameSize65To127 +
                                                          s->Set1.dwlFrameSize128To255 +
                                                          s->Set1.dwlFrameSize256To511 +
                                                          s->Set1.dwlFrameSize512To1023 +
                                                          s->Set1.dwlFrameSize1024To1518 +
                                                          s->Set1.dwlBabyGiantFrames +
                                                          s->Set1.dwlFrameSize1519To2047 +
                                                          s->Set1.dwlFrameSize2048To4095 +
                                                          s->Set1.dwlFrameSize4096To8191 +
                                                          s->Set1.dwlFrameSize8192ToMax +
                                                          s->Set1.dwlOversizeFrames +
                                                          s->Set1.dwlJabberFrames +
                                                          s->Set1.dwlOversizeHardSlicedFrames +
                                                          s->Set1.dwlJabberHardSlicedFrames);
  d->RMON1Statistics.dwlOctetsReceivedOK              += (s->Set1.dwlGoodFrameBytes+
                                                          s->Set1.dwlGoodPauseMacFramesBytes+
                                                          s->Set1.dwlBadFrameBytes);
  d->RMON1Statistics.dwlGroupFramesReceivedOk         += (0);
  d->RMON1Statistics.dwlGroupOctetsReceivedOk         += (0);
  d->RMON1Statistics.dwlFragmentsReceivedOk           += (s->Set1.dwlFragments);
  d->RMON1Statistics.dwlOversizeFramesReceivedOk      += (s->Set1.dwlOversizeFrames +
                                                          s->Set1.dwlOversizeHardSlicedFrames);
  d->RMON1Statistics.dwlUnderSizedFramesReceivedOk    += (s->Set1.dwlUndersizeFrames);
  d->RMON1Statistics.dwlJabberFramesReceivedOk        += (s->Set1.dwlJabberFrames);
  d->RMON1Statistics.dwlBroadcastFramesReceivedOK     += (s->Set2.dwlBroadcastFrames);
  d->RMON1Statistics.dwlUnicastFramesReceivedOK       += (s->Set2.dwlUnicastFrames);
  d->RMON1Statistics.dwlMulticastFramesReceivedOK     += (s->Set2.dwlMulticastFrames);
  d->RMON1Statistics.dwlCRCErrors                     += (s->Set2.dwlCrcErrors);
  d->RMON1Statistics.dwlCharacterCVErrors             += (s->Set2.dwlCodeViolationErrors);
  d->RMON1Statistics.dwlReceiveFrames64Octets         += s->Set1.dwlFrameSize64;
  d->RMON1Statistics.dwlReceiveFrames65To127Octets    += s->Set1.dwlFrameSize65To127;
  d->RMON1Statistics.dwlReceiveFrames128To255Octets   += s->Set1.dwlFrameSize128To255;
  d->RMON1Statistics.dwlReceiveFrames256To511Octets   += s->Set1.dwlFrameSize256To511;
  d->RMON1Statistics.dwlReceiveFrames512To1023Octets  += s->Set1.dwlFrameSize512To1023;
  d->RMON1Statistics.dwlReceiveFrames1024To1518Octets += s->Set1.dwlFrameSize1024To1518;
  d->RMON1Statistics.dwlBabyGiantVLANFramesReceivedOK += 0;
  d->RMON1Statistics.dwlBabyGiantISLFramesReceivedOK  += 0;
  d->RMON1Statistics.dwlBabyGiantJumboFramesReceivedOK+= 0;
  d->RMON1Statistics.dwlJumboFrames1519To2047Octets   += s->Set1.dwlFrameSize1519To2047;
  d->RMON1Statistics.dwlJumboFrames2048To4095Octets   += s->Set1.dwlFrameSize2048To4095;
  d->RMON1Statistics.dwlJumboFrames4096To8191Octets   += s->Set1.dwlFrameSize4096To8191;
  d->RMON1Statistics.dwlJumboFrames8192To9018Octets   += s->Set1.dwlFrameSize8192ToMax;
  d->RMON1Statistics.dwlVLANJumboFrames9019To9022Octets= 0;
  d->ExtendedStatistics.dwlLayer3ChecksumErrors       += (s->Set2.dwlIpChecksumErrors);
  d->ExtendedStatistics.dwlLayer4ChecksumErrors       += (s->Set2.dwlTcpChecksumErrors+
                                                          s->Set2.dwlUdpChecksumErrors);
  d->ExtendedStatistics.dwlTCPChecksumErrors          += (s->Set2.dwlTcpChecksumErrors);
  d->ExtendedStatistics.dwlUDPChecksumErrors          += (s->Set2.dwlUdpChecksumErrors);
  d->dwlTimeValidAt                                    = s->dwlTimeValidAt;
  d->timestampType                                     = s->timestampType;
  d->overflow                                          = 0;
}

void ConvertEtherType8ToEtherType6(StatisticsEtherType6_t *d, StatisticsEtherType8_t *s)
{
  d->RMONStatistics.dwlOctetsReceived                      += (s->Set1.dwlGoodFrameBytes+
                                                               s->Set1.dwlGoodPauseMacFramesBytes+
                                                               s->Set1.dwlBadFrameBytes);
  d->RMONStatistics.dwlFramesReceived                      += (s->Set1.dwlUndersizeFrames +
                                                               s->Set1.dwlFragments +
                                                               s->Set1.dwlUndersizeWithoutEncapsulation +
                                                               s->Set1.dwlFrameSize64 +
                                                               s->Set1.dwlFrameSize65To127 +
                                                               s->Set1.dwlFrameSize128To255 +
                                                               s->Set1.dwlFrameSize256To511 +
                                                               s->Set1.dwlFrameSize512To1023 +
                                                               s->Set1.dwlFrameSize1024To1518 +
                                                               s->Set1.dwlBabyGiantFrames +
                                                               s->Set1.dwlFrameSize1519To2047 +
                                                               s->Set1.dwlFrameSize2048To4095 +
                                                               s->Set1.dwlFrameSize4096To8191 +
                                                               s->Set1.dwlFrameSize8192ToMax +
                                                               s->Set1.dwlOversizeFrames +
                                                               s->Set1.dwlJabberFrames +
                                                               s->Set1.dwlOversizeHardSlicedFrames +
                                                               s->Set1.dwlJabberHardSlicedFrames);
  d->RMONStatistics.dwlFragmentsReceived                    += (s->Set1.dwlFragments);
  d->RMONStatistics.dwlUnderSizedFramesReceived             += (s->Set1.dwlUndersizeFrames);
  d->RMONStatistics.dwlOversizeFramesReceived               += (s->Set1.dwlOversizeFrames +
                                                                s->Set1.dwlOversizeHardSlicedFrames);
  d->RMONStatistics.dwlJabberFramesReceived                 += (s->Set1.dwlJabberFrames);
  d->RMONStatistics.dwlBroadcastFramesReceived              += (s->Set2.dwlBroadcastFrames +
                                                                s->Set2.dwlMacBroadcastFrames);
  d->RMONStatistics.dwlMulticastFramesReceived              += (s->Set2.dwlMulticastFrames +
                                                                s->Set2.dwlMacMulticastFrames);
  d->RMONStatistics.dwlCRCAlignErrors                       += (s->Set2.dwlCrcErrors +
                                                                s->Set2.dwlAlignmentErrors +
                                                                s->Set2.dwlCodeViolationErrors);
  d->RMONStatistics.dwlDropEvents                           += (0);
  d->RMONStatistics.dwlReceiveFrames64Octets                += (s->Set1.dwlFrameSize64);
  d->RMONStatistics.dwlReceiveFrames65To127Octets           += (s->Set1.dwlFrameSize65To127);
  d->RMONStatistics.dwlReceiveFrames128To255Octets          += (s->Set1.dwlFrameSize128To255);
  d->RMONStatistics.dwlReceiveFrames256To511Octets          += (s->Set1.dwlFrameSize256To511);
  d->RMONStatistics.dwlReceiveFrames512To1023Octets         += (s->Set1.dwlFrameSize512To1023);
  d->RMONStatistics.dwlReceiveFrames1024To1518Octets        += (s->Set1.dwlFrameSize1024To1518);

  d->HardwareStatistics.dwlRxFrameSize64                    += (s->Set1.dwlFrameSize64);
  d->HardwareStatistics.dwlRxFrameSize65To127               += (s->Set1.dwlFrameSize65To127);
  d->HardwareStatistics.dwlRxFrameSize128To255              += (s->Set1.dwlFrameSize128To255);
  d->HardwareStatistics.dwlRxFrameSize256To511              += (s->Set1.dwlFrameSize256To511);
  d->HardwareStatistics.dwlRxFrameSize512To1023             += (s->Set1.dwlFrameSize512To1023);
  d->HardwareStatistics.dwlRxFrameSize1024To1518            += (s->Set1.dwlFrameSize1024To1518);
  d->HardwareStatistics.dwlRxFrameSize1519To2047            += (s->Set1.dwlFrameSize1519To2047);
  d->HardwareStatistics.dwlRxFrameSize2048To4095            += (s->Set1.dwlFrameSize2048To4095);
  d->HardwareStatistics.dwlRxFrameSize4096To8191            += (s->Set1.dwlFrameSize4096To8191);
  d->HardwareStatistics.dwlRxFrameSize8192To9018            += (s->Set1.dwlFrameSize8192ToMax);
  d->HardwareStatistics.dwlRxFrameSize9019ToMax             += (0);
  d->HardwareStatistics.dwlRxUndersizeFrames                += (s->Set1.dwlUndersizeFrames);
  d->HardwareStatistics.dwlRxFragments                      += (s->Set1.dwlFragments);
  d->HardwareStatistics.dwlRxOversizeFrames                 += (s->Set1.dwlOversizeFrames);
  d->HardwareStatistics.dwlRxJabberFrames                   += (s->Set1.dwlJabberFrames);
  d->HardwareStatistics.dwlRxOversizeHardSlicedFrames       += (s->Set1.dwlOversizeHardSlicedFrames);
  d->HardwareStatistics.dwlRxJabberHardSlicedFrames         += (s->Set1.dwlJabberHardSlicedFrames);
  d->HardwareStatistics.dwlRxGoodFrameBytes                 += (s->Set1.dwlGoodFrameBytes);
  d->HardwareStatistics.dwlRxGoodPauseUnsupportedMACControl += (s->Set2.dwlUnicastPauseFrames +
                                                                s->Set2.dwlMulticastPauseFrames +
                                                                s->Set2.dwlMacUnicastFrames +
                                                                s->Set2.dwlMacMulticastFrames +
                                                                s->Set2.dwlMacBroadcastFrames);
  d->HardwareStatistics.dwlRxBadFrameBytes                  += (s->Set1.dwlBadFrameBytes);
  d->HardwareStatistics.dwlRxUnicastFrames                  += (s->Set2.dwlUnicastFrames);
  d->HardwareStatistics.dwlRxMulticastFrames                += (s->Set2.dwlMulticastFrames);
  d->HardwareStatistics.dwlRxBroadcastFrames                += (s->Set2.dwlBroadcastFrames);
  d->HardwareStatistics.dwlRxPauseFrames                    += (s->Set2.dwlUnicastPauseFrames +
                                                               s->Set2.dwlMulticastPauseFrames);

  d->HardwareStatistics.dwlRxUnsupportedMACControlFrames    += (s->Set2.dwlMacUnicastFrames +
                                                               s->Set2.dwlMacMulticastFrames +
                                                               s->Set2.dwlMacBroadcastFrames);

  d->HardwareStatistics.dwlRxCrcErrors                      += (s->Set2.dwlCrcErrors);
  d->HardwareStatistics.dwlRxAlignmentErrors                += (s->Set2.dwlAlignmentErrors);
  d->HardwareStatistics.dwlRxCodeViolationErrors            += (s->Set2.dwlCodeViolationErrors);
  d->HardwareStatistics.dwlRxTotalFrames                    += (s->Set1.dwlUndersizeFrames +
                                                               s->Set1.dwlFragments +
                                                               s->Set1.dwlUndersizeWithoutEncapsulation +
                                                               s->Set1.dwlFrameSize64 +
                                                               s->Set1.dwlFrameSize65To127 +
                                                               s->Set1.dwlFrameSize128To255 +
                                                               s->Set1.dwlFrameSize256To511 +
                                                               s->Set1.dwlFrameSize512To1023 +
                                                               s->Set1.dwlFrameSize1024To1518 +
                                                               s->Set1.dwlBabyGiantFrames +
                                                               s->Set1.dwlFrameSize1519To2047 +
                                                               s->Set1.dwlFrameSize2048To4095 +
                                                               s->Set1.dwlFrameSize4096To8191 +
                                                               s->Set1.dwlFrameSize8192ToMax +
                                                               s->Set1.dwlOversizeFrames +
                                                               s->Set1.dwlJabberFrames +
                                                               s->Set1.dwlOversizeHardSlicedFrames +
                                                               s->Set1.dwlJabberHardSlicedFrames);
  d->HardwareStatistics.dwlRxXGMIISmallFrameDrop            += (0);
  d->HardwareStatistics.dwlRxPacketMergeDrop                += (0);
  d->HardwareStatistics.dwlRxSmootherDrop                   += (0);
  d->dwlTimeValidAt                                          = s->dwlTimeValidAt;
  d->timestampType                                           = s->timestampType;
}

#endif // _STAT_CONVERT_FUNCTIONS_
